<?php

namespace application\personal_calendar\upgrade\v4_0_0;

use common\libraries\CommonDataManager;
use common\libraries\DataClassRetrievesParameters;
use common\libraries\EqualityCondition;
use common\libraries\package\Package;
use common\libraries\PropertyConditionVariable;
use common\libraries\Registration;
use common\libraries\Setting;
use common\libraries\StaticConditionVariable;
use common\libraries\Translation;
use common\libraries\Utilities;
use core\home\HomeBlockRegistration;
use tracking\TrackingDataManager;

/**
 * Database schema & content upgrader for Chamilo 3.x to 4.0.0.
 *
 * Instead of putting everything in the run() method, this class makes calls to
 * upgrade_db() and upgrade_registration(). Since the only thing that really differs
 * across Upgraders is upgrade_db() implementation, the other methods could/should
 * be moved to the \common\libraries\package\Upgrader.
 */
class Upgrader extends \common\libraries\package\Upgrader
{

    /**
     * Constructor
     */
    public function __construct()
    {
        parent :: __construct(\application\personal_calendar\DataManager :: get_instance());
    }

    /*
     * (non-PHPdoc) @see \common\libraries\package\Upgrader::run()
     */
    public function run()
    {
        $data_manager = $this->get_data_manager();
        $upgrader = $this;

        $success = $data_manager->transactional(
            function ($c) use($data_manager, $upgrader)
            {
                $success = $upgrader->upgrade_db($data_manager);

                if($success)
                {
                    $upgrader->add_message(\common\libraries\package\Upgrader :: TYPE_NORMAL, Translation :: get('UpgradeDatabaseSuccessful', null, Utilities :: COMMON_LIBRARIES));
                }
                else
                {
                    $upgrader->add_message(\common\libraries\package\Upgrader :: TYPE_ERROR, Translation :: get('UpgradeDatabaseFailed', null, Utilities :: COMMON_LIBRARIES));
                }

                if(!$success)
                {
                    return false;
                }

                $success = $upgrader->update_db_content($data_manager);

                if($success)
                {
                    $upgrader->add_message(\common\libraries\package\Upgrader :: TYPE_NORMAL, Translation :: get('UpdateDatabaseContentSuccessful', null, Utilities :: COMMON_LIBRARIES));
                }
                else
                {
                    $upgrader->add_message(\common\libraries\package\Upgrader :: TYPE_ERROR, Translation :: get('UpdateDatabaseContentFailed', null, Utilities :: COMMON_LIBRARIES));
                }

                if(!$success)
                {
                    return false;
                }

                $success = $upgrader->upgrade_registration($data_manager);

                if($success)
                {
                    $upgrader->add_message(\common\libraries\package\Upgrader :: TYPE_NORMAL, Translation :: get('UpgradeRegistrationSuccessful', null, Utilities :: COMMON_LIBRARIES));
                }
                else
                {
                    $upgrader->add_message(\common\libraries\package\Upgrader :: TYPE_ERROR, Translation :: get('UpgradeRegistrationFailed', null, Utilities :: COMMON_LIBRARIES));
                }

                return $success;
            }
        );

        return $success;
    }

    /**
     * Performs the database upgrade.
     *
     * @param type $data_manager
     * @return boolean
     */
    public function upgrade_db($data_manager)
    {
        // Remove old tables
        if(TrackingDataManager :: get_instance()->storage_unit_exist('personal_calendar_publication_changes'))
        {
            if (!TrackingDataManager :: get_instance()->drop_storage_unit('personal_calendar_publication_changes'))
            {
                return false;
            }
        }
        if($data_manager->storage_unit_exist('test'))
        {
            if (!$data_manager->drop_storage_unit('test'))
            {
                return false;
            }
        }
        return true;
    }

    /**
     * Performs the database content upgrade.
     *
     * @param type $data_manager
     * @return boolean
     */
    public function update_db_content($data_manager)
    {
        // Home blocks
        $condition = new EqualityCondition(
            new PropertyConditionVariable(
                HomeBlockRegistration :: class_name(),
                HomeBlockRegistration :: PROPERTY_CONTEXT),
            new StaticConditionVariable('application\personal_calendar'));
        $home_block_registrations = \core\home\DataManager :: retrieves(
            HomeBlockRegistration :: class_name(),
            new DataClassRetrievesParameters($condition));

        while ($home_block_registration = $home_block_registrations->next_result())
        {
            $home_block_registration->set_context('application\personal_calendar\integration\core\home');
            if(!$home_block_registration->update())
            {
                return false;
            }
        }

        return true;
    }

    /**
     * Upgrades the package registration.
     *
     * @param type $data_manager
     * @return boolean
     */
    public function upgrade_registration($data_manager)
    {
        $package = Package :: get('application\personal_calendar');
        $registration = CommonDataManager :: get_registration('application\personal_calendar');
        $registration->set_version($package->get_version());
        if (! $registration->update())
        {
            return false;
        }

        // New subpackage registrations
        $subpackages = array(
            'application\personal_calendar\integration\core\home'
        );

        foreach($subpackages as $subpackage)
        {
            $package_info = Package :: get($subpackage);
            $application_registration = new Registration();

            $parent_namespace = Utilities :: get_namespace_parent($subpackage);
            $application_registration->set_type((! $parent_namespace ? 'core' : $parent_namespace));

            $application_registration->set_name($package_info->get_code());
            $application_registration->set_category($package_info->get_category());
            $application_registration->set_version($package_info->get_version());
            $application_registration->set_status(Registration :: STATUS_ACTIVE);
            $application_registration->set_context($subpackage);

            if(!$application_registration->create())
            {
                return false;
            }
        }

        return true;
    }
}
