<?php

namespace application\personal_messenger;

use common\libraries\EqualityCondition;
use common\libraries\Session;
use common\libraries\Theme;
use common\libraries\Translation;
use common\libraries\Utilities;
use group\Group;
use group\DataManager as GroupDataManager;
use user\User;

/** * ***************************************************************************
 * This class represents a personal message (PM).
 *
 * PM objects have a number of default properties:
 * - id: the numeric ID of the PM;
 * - parent_id: the numeric ID identifying the root of the 'chain' in case of
 *              a discussion;
 * - reply_id: the numeric ID identifying the PM on which is replied should it
 *              not be the root (indenting in discussion possible);
 * - author_id: the ID of the author of the PM;
 * - creation_time: the time of creation of the PM;
 * - modification_time: the time of the last modification of the PM;
 * - title: the title of the PM;
 * - content: the content of the PM;
 *
 * @name Chamilo Personal Messenger > Message
 * @version 1.0.0
 * @author Stijn Van Hoecke
 * **************************************************************************** */
class Message extends PersonalMessengerDataClass
{
    //**************************************************************************
    // GENERAL CONSTANTS
    //**************************************************************************
    const CLASS_NAME = __CLASS__;
    
    //**************************************************************************
    // PROPERTY CONSTANTS
    //**************************************************************************
    const PROPERTY_MESSAGE_PARENT_ID = 'parent_id';
    const PROPERTY_MESSAGE_REPLY_ID = 'reply_id';
    const PROPERTY_MESSAGE_AUTHOR_ID = 'author_id';
    const PROPERTY_MESSAGE_CREATION_TIME = 'creation_time';
    const PROPERTY_MESSAGE_MODIFICATION_TIME = 'modification_time';
    const PROPERTY_MESSAGE_TITLE = 'title';
    const PROPERTY_MESSAGE_CONTENT = 'content';
    
    //**************************************************************************
    // VARIABLES
    //**************************************************************************
    /**
     * Cache to store the user ids before creating the links.
     * @var array
     */
    private $target_users;
    /**
     * Cache to store the group ids before creating the links.
     * @var array
     */
    private $target_groups;
    /**
     * Cache to store the content object id before creating the links.
     * @var array
     */
    private $attachments;
    
    //**************************************************************************
    // CONSTRUCTOR
    //**************************************************************************
    /**
     * Constructor.
     * @param array $default_roperties The default properties of the message
     * @param array $optional_roperties The optional properties of the message
     */
    public function __construct($default_roperties = array(), $optional_roperties = array())
    {
        parent :: __construct($default_roperties, $optional_roperties);
    }
    
    //**************************************************************************
    // GENERAL GETTERS
    //**************************************************************************
    /**
     * Get the default properties of all messages.
     * @return array The property names
     */
    public static function get_default_property_names()
    {
        return parent :: get_default_property_names(array(self :: PROPERTY_MESSAGE_PARENT_ID, 
                self :: PROPERTY_MESSAGE_REPLY_ID, self :: PROPERTY_MESSAGE_AUTHOR_ID, 
                self :: PROPERTY_MESSAGE_CREATION_TIME, self :: PROPERTY_MESSAGE_MODIFICATION_TIME, 
                self :: PROPERTY_MESSAGE_TITLE, self :: PROPERTY_MESSAGE_CONTENT));
    }
    
    //**************************************************************************
    // PROPERTY GETTERS
    //**************************************************************************
    /**
     * Returns the numeric ID of the parent of this PM.
     * @return int The parent ID
     */
    public function get_parent_id()
    {
        return $this->get_default_property(self :: PROPERTY_MESSAGE_PARENT_ID);
    }

    /**
     * Returns the numeric ID of the PM on which is replied.
     * This structure allows to reply on a separate message inside a discussion
     * rather than only allowing to reply on the root.
     * @return int The reply ID
     */
    public function get_reply_id()
    {
        return $this->get_default_property(self :: PROPERTY_MESSAGE_REPLY_ID);
    }

    /**
     * Returns the numeric ID of the author of this PM.
     * @return int The author ID
     */
    public function get_author_id()
    {
        return $this->get_default_property(self :: PROPERTY_MESSAGE_AUTHOR_ID);
    }

    /**
     * Returns the creation timestamp of this PM.
     * @return Timestamp the creation time
     */
    public function get_creation_time()
    {
        return $this->get_default_property(self :: PROPERTY_MESSAGE_CREATION_TIME);
    }

    /**
     * Returns the modification timestamp of this PM.
     * @return Timestamp the modification time
     */
    public function get_modification_time()
    {
        return $this->get_default_property(self :: PROPERTY_MESSAGE_MODIFICATION_TIME);
    }

    /**
     * Returns the title of this PM.
     * @return String the title of the message
     */
    public function get_title()
    {
        return $this->get_default_property(self :: PROPERTY_MESSAGE_TITLE);
    }

    /**
     * Returns the content of this PM.
     * @return String the content of the message
     */
    public function get_content()
    {
        return $this->get_default_property(self :: PROPERTY_MESSAGE_CONTENT);
    }
    
    //**************************************************************************
    // PROPERTY SETTERS
    //**************************************************************************
    /**
     * Sets the numeric ID of the parent of this PM.
     * @param int $parent_id The parent ID
     */
    public function set_parent_id($parent_id)
    {
        $this->set_default_property(self :: PROPERTY_MESSAGE_PARENT_ID, $parent_id);
    }

    /**
     * Sets the numeric ID of the reply of this PM.
     * This structure allows to reply on a separate message inside a discussion
     * rather than only allowing to reply on the root.
     * @param int $reply_id The reply ID
     */
    public function set_reply_id($reply_id)
    {
        $this->set_default_property(self :: PROPERTY_MESSAGE_REPLY_ID, $reply_id);
    }

    /**
     * Sets the numeric ID of the author of this PM.
     * @param int $author_id The author ID
     */
    public function set_author_id($author_id)
    {
        $this->set_default_property(self :: PROPERTY_MESSAGE_AUTHOR_ID, $author_id);
    }

    /**
     * Sets the creation time of this PM.
     * @param int $creation_time The timestamp of the creation date
     */
    public function set_creation_time($creation_time)
    {
        $this->set_default_property(self :: PROPERTY_MESSAGE_CREATION_TIME, $creation_time);
    }

    /**
     * Sets the modification time of this PM.
     * @param int $modification_time The timestamp of the modification date
     */
    public function set_modification_time($modification_time)
    {
        $this->set_default_property(self :: PROPERTY_MESSAGE_MODIFICATION_TIME, $modification_time);
    }

    /**
     * Sets the title of this PM.
     * @param String $title The title
     */
    public function set_title($title)
    {
        $this->set_default_property(self :: PROPERTY_MESSAGE_TITLE, $title);
    }

    /**
     * Sets the content of this PM.
     * @param String $content The content
     */
    public function set_content($content)
    {
        $this->set_default_property(self :: PROPERTY_MESSAGE_CONTENT, $content);
    }
    
    //**************************************************************************
    // VARIABLE GETTERS
    //**************************************************************************
    /**
     * Gets the list of target users of this message.
     * @return array An array of user ids.
     */
    public function get_target_users()
    {
        if (! isset($this->target_users))
        {
            $dm = PersonalMessengerDataManager :: get_instance();
            $condition = new EqualityCondition(MessageUserRecipient :: PROPERTY_MESSAGE_RECIPIENT_MESSAGE_ID, $this->get_id());
            $users = $dm->retrieve_all_message_user_recipients($condition);
            
            $target_users = array();
            while ($user = $users->next_result())
            {
                // exclude your own account
                if ($user->get_user_id() != Session :: get_user_id())
                {
                    $target_users[] = $user->get_user_id();
                }
            }
            $this->set_target_users($target_users);
        }
        
        return $this->target_users;
    }

    /**
     * Gets the list of target groups of this message.
     * @return array An array of group ids.
     */
    public function get_target_groups()
    {
        if (! isset($this->target_groups))
        {
            $dm = PersonalMessengerDataManager :: get_instance();
            $condition = new EqualityCondition(MessageGroupRecipient :: PROPERTY_MESSAGE_RECIPIENT_MESSAGE_ID, $this->get_id());
            $groups = $dm->retrieve_all_message_group_recipients($condition);
            
            $target_groups = array();
            while ($group = $groups->next_result())
            {
                $target_groups[] = $group->get_group_id();
            }
            $this->set_target_groups($target_groups);
        }
        
        return $this->target_groups;
    }

    /**
     * Gets the list of ids of attachments of this message.
     * @return array An array of content_object_ids.
     */
    public function get_attachments()
    {
        if (! isset($this->attachments))
        {
            $attachment_ids = array();
            foreach ($this->get_message_attachments() as $attachment)
            {
                $attachments[] = $attachment->get_content_object_id();
            }
            $this->set_attachments($attachments);
        }
        
        return $this->attachments;
    }

    /**
     * Gets the list of attachments of this message that still exist in the owners repository.
     * @return array An array of message-attachment objects.
     */
    public function get_message_attachments()
    {
        $dm = PersonalMessengerDataManager :: get_instance();
        $condition = new EqualityCondition(MessageAttachment :: PROPERTY_MESSAGE_ATTACHMENT_MESSAGE_ID, $this->get_id());
        $attachment_objects = $dm->retrieve_all_message_attachments($condition);
        
        $attachments = array();
        while ($attachment = $attachment_objects->next_result())
        {
            if ($attachment->ensure())
            {
                $attachments[] = $attachment;
            }
        }
        
        return $attachments;
    }
    
    //**************************************************************************
    // VARIABLE SETTERS
    //**************************************************************************
    /**
     * Sets the list of target users of this message
     * @param array $target_users An array of user_ids.
     */
    public function set_target_users($target_users)
    {
        $this->target_users = $target_users;
    }

    /**
     * Sets the list of target groups of this message
     * @param array $target_groups An array of group_ids.
     */
    public function set_target_groups($target_groups)
    {
        $this->target_groups = $target_groups;
    }

    /**
     * Sets the list of attachments of this message
     * @param array $attachments An array of content_object_ids.
     */
    public function set_attachments($attachments)
    {
        $this->attachments = $attachments;
    }
    
    //**************************************************************************
    // GENERAL FUNCTIONS
    //**************************************************************************
    /**
     * Returns the status of this message.
     * @return int
     */
    public function get_status()
    {
        return PersonalMessengerDataManager :: get_user_message_status($this->get_id());
    }

    /**
     * Determines the folder of the message.
     */
    public function get_folder()
    {
        // ignored / trashed ???
        switch ($this->get_status())
        {
            case UserMessage :: STATUS_IGNORE :
                return PersonalMessengerFolder :: IGNORED;
            case UserMessage :: STATUS_TRASH :
                return PersonalMessengerFolder :: TRASH;
        }
        
        // discussion ???
        if ($this->is_discussion())
        {
            return PersonalMessengerFolder :: DISCUSSIONS;
        }
        
        // outbox ???
        if ($this->get_author_id() == Session :: get_user_id())
        {
            return PersonalMessengerFolder :: OUTBOX;
        }
        
        // inbox (by elimination)
        return PersonalMessengerFolder :: INBOX;
    }

    /**
     * Returns if the author of this message is ignored.
     * @return boolean
     */
    public function is_author_ignored()
    {
        return PersonalMessengerDataManager :: is_user_ignored_by_me($this->get_author_id());
    }

    /**
     * Returns true if this message is part of a discussion.
     * @return boolean
     */
    public function is_discussion()
    {
        return ! is_null($this->get_parent_id()) ? ($this->get_parent_id() != 0) : false;
    }

    /**
     * Returns true if this message is the root of a discussion.
     * @return boolean
     */
    public function is_discussion_root()
    {
        return $this->get_id() == $this->get_parent_id();
    }

    /**
     * Returns the name of the author of this message.
     * @return String The name of the author
     */
    public function get_author_name()
    {
        // retrieve the user using the existing UserDataManager
        $user = \user\DataManager :: retrieve(User :: class_name(), (int) $this->get_author_id());
        
        // return its name
        if ($user)
        {
            return $user->get_firstname() . ' ' . $user->get_lastname();
        }
        else
        {
            return Translation :: get('SenderUnknown');
        }
    }

    /**
     * Returns an array of strings representing the names of all recipients.
     * @return Array An array containting all recipient names
     */
    public function get_recipient_names()
    {
        return PersonalMessengerDataManager :: get_recipient_names($this->get_id(), array($this->get_author_id()));
    }

    /**
     * Checks if the given user has a link in his/her inbox for this message.
     * @param int $user_id
     * @return boolean
     */
    public function has_user_message_relation($user_id)
    {
        return PersonalMessengerDataManager :: user_message_relation_exists($this->get_id(), $user_id);
    }

    /**
     * Returns the number of attachments linked to this message.
     * @return int The number of attachments
     */
    public function count_attachments()
    {
        return PersonalMessengerDataManager :: count_attachments($this->get_id());
    }

    /**
     * Checks if the attachment link is already present, creates a new attachment link for the given message and attachment otherwise.
     * @param int $content_object_id The attachment ID
     * @return boolean
     */
    public function add_attachment($content_object_id)
    {
        // check if the attachment already exists
        $existing_attachments = $this->get_attachments();
        if (! in_array($content_object_id, $existing_attachments))
        {
            // create new attachment
            $new_attachment = PersonalMessengerDataManager :: message_add_attachment($this, $content_object_id);
            
            if ($new_attachment)
            {
                $this->attachments[] = $new_attachment;
                return true;
            }
        }
        return false;
    }

    /**
     * Checks if the attachment link is present, and deletes it.
     * @param int $content_object_id The attachment ID
     * @return boolean
     */
    public function delete_attachment($content_object_id)
    {
        // check if the attachment exists
        $existing_attachments = $this->get_attachments();
        if (in_array($content_object_id, $existing_attachments))
        {
            // retrieve the attachment
            $deleted_attachment = PersonalMessengerDataManager :: message_delete_attachment($this, $content_object_id);
            
            if ($deleted_attachment)
            {
                $this->attachments = Utilities :: remove_item_by_value($this->attachments, $content_object_id, false);
                return true;
            }
        }
        return false;
    }

    /**
     * Checks if the group link is already present, creates a new group recipient link for the given message and group otherwise.
     * @param int $group_id The group ID
     * @return boolean
     */
    public function add_group_recipient($group_id, $create_user_message_link = true)
    {
        // check if the group link already exists
        $existing_groups = $this->get_target_groups();
        if (! in_array($group_id, $existing_groups))
        {
            // create new group
            $new_group = PersonalMessengerDataManager :: message_add_group_recipient($this, $group_id);
            
            if ($new_group)
            {
                $this->target_groups[] = $new_group;
                
                // creates the user message links if needed
                if ($create_user_message_link)
                {
                    $group = GroupDataManager :: retrieve_by_id(Group :: class_name(), $group_id);
                    if (! $group)
                    {
                        return false;
                    }
                    foreach ($group->get_users(true, true) as $user_id)
                    {
                        if (! $this->add_user_message_relation($user_id))
                        {
                            return false;
                        }
                    }
                }
                return true;
            }
        }
        elseif ($create_user_message_link)
        {
            $group = GroupDataManager :: retrieve_by_id(Group :: class_name(), $group_id);
            if (! $group)
            {
                return false;
            }
            foreach ($group->get_users(true, true) as $user_id)
            {
                if (! $this->add_user_message_relation($user_id))
                {
                    return false;
                }
            }
            return true;
        }
        return false;
    }

    /**
     * Checks if the user link is already present, creates a new user recipient link for the given message and user otherwise.
     * @param int $user_id The user ID
     * @return boolean
     */
    public function add_user_recipient($user_id, $create_user_message_link = true)
    {
        // check if the user link already exists
        $existing_users = $this->get_target_users();
        if (! in_array($user_id, $existing_users))
        {
            // create new group
            $new_user = PersonalMessengerDataManager :: message_add_user_recipient($this, $user_id);
            
            if ($new_user)
            {
                $this->target_users[] = $new_user;
                
                // creates the user message links if needed
                if ($create_user_message_link)
                {
                    return $this->add_user_message_relation($user_id);
                }
                return true;
            }
        }
        elseif ($create_user_message_link)
        {
            return $this->add_user_message_relation($user_id);
        }
        return false;
    }

    /**
     * Checks if the user message link is already present, creates a new user message link for the given message and user otherwise.
     * @param int $user_id The user ID
     * @return boolean
     */
    public function add_user_message_relation($user_id)
    {
        // check if the relation already exists
        if (! $this->has_user_message_relation($user_id))
        {
            return PersonalMessengerDataManager :: message_add_user_message_relation($this, $user_id);
        }
        return true;
    }

    /**
     * Deletes the user message based on the given user ID.
     * @param int $user_id
     * @return boolean
     */
    public function delete_user_message_relation($user_id)
    {
        return PersonalMessengerDataManager :: message_delete_user_message_relation($this, $user_id);
    }

    /**
     * Returns the path to the image related to the type of this message.
     * @return String Path to the image related to this type of message
     */
    public function get_icon_path()
    {
        $icon_path = Theme :: get_image_path();
        $icon_path .= 'type_';
        if ($this->is_discussion())
        {
            $icon_path .= 'discussion';
            
            if (! $this->is_discussion_root())
            {
                $icon_path .= '_reply';
            }
        }
        else
        {
            $icon_path .= 'personal_message';
        }
        $icon_path .= '.png';
        
        return $icon_path;
    }
    
    //**************************************************************************
    // DATABASE FUNCTIONS
    //**************************************************************************
    /**
     * Gets the table name for this class.
     */
    public static function get_table_name()
    {
        return Utilities :: get_classname_from_namespace(self :: CLASS_NAME, true);
    }

    /**
     * Instructs the data manager to create the PM, making it
     * persistent. Also assigns a unique ID to the PM and sets
     * the PM's creation date to the current time.
     * @return boolean True if creation succeeded, false otherwise.
     */
    public function create($own_read = true)
    {
        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~//
        // (0) Cache to store each user to avoid double linking, afterwards   //
        //          this cache will be used to store the individual links for //
        //          users of groups, so we can itterate it for the creation   //
        //          of inbox-links.                                           //
        //     Cache to store each group to avoid double linking.             //
        //     Cache to store each attachment to avoid double linking.        //
        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~//
        $users_to_link = array();
        $group_to_link = array();
        $attachments_to_link = array();
        
        // (1) set the current time as creation and modification time
        //------------------------------------------------------------
        $now = time();
        $this->set_creation_time($now);
        $this->set_modification_time($now);
        
        // (2) store the message
        //-----------------------
        if (! parent :: create($this))
        {
            return false;
        }
        
        // (3) store the links to the attachments
        //----------------------------------------
        foreach ($this->get_attachments() as $index => $content_object_id)
        {
            if (! in_array($content_object_id, $attachments_to_link))
            {
                if (! PersonalMessengerDataManager :: message_add_attachment($this, $content_object_id))
                {
                    return false;
                }
                
                // avoid double linking
                $attachments_to_link[] = $content_object_id;
            }
        }
        
        // direct database functions below
        $dm = PersonalMessengerDataManager :: get_instance();
        
        // ONLY STORE LINKS IF THE MESSAGE IS NOT A REPLY ON A DISCUSSION.
        if (! $this->is_discussion())
        {
            // (4a) store the links to the users
            //-----------------------------------
            $target_users_incl_author = $this->get_target_users();
            $target_users_incl_author[] = $this->get_author_id();
            foreach ($target_users_incl_author as $index => $user_id)
            {
                if (! in_array($user_id, $users_to_link))
                {
                    if (! PersonalMessengerDataManager :: message_add_user_recipient($this, $user_id))
                    {
                        return false;
                    }
                    
                    // avoid double linking
                    $users_to_link[] = $user_id;
                }
            }
            
            // (4b) store the links to the groups
            //------------------------------------
            foreach ($this->get_target_groups() as $index => $group_id)
            {
                if (! in_array($group_id, $group_to_link))
                {
                    if (! PersonalMessengerDataManager :: message_add_group_recipient($this, $group_id))
                    {
                        return false;
                    }
                    
                    // avoid double linking
                    $group_to_link[] = $group_id;
                    
                    // inbox linking afterwards
                    $group = GroupDataManager :: retrieve_by_id(Group :: class_name(), $group_id);
                    foreach ($group->get_users(true, true) as $user_id)
                    {
                        $users_to_link[] = $user_id;
                    }
                }
            }
            
            // (4c) store the links to the inbox of each user
            //------------------------------------------------
            $linked_users = array();
            
            // process users_to_link
            foreach ($users_to_link as $user_id)
            {
                // if the user doesn't already have a link
                if (! in_array($user_id, $linked_users))
                {
                    if (! PersonalMessengerDataManager :: message_add_user_message_relation($this, $user_id, $own_read))
                    {
                        return false;
                    }
                    
                    // avoid double linking
                    $linked_users[] = $user_id;
                }
            }
        }
        else
        {
            // (4) reset the status of the parent (root of the discussion) from read to unread for all other recipients
            //----------------------------------------------------------------------------------------------------------
            PersonalMessengerDataManager :: change_user_message_status($this->get_parent_id(), UserMessage :: STATUS_READ, UserMessage :: STATUS_UNREAD, ! $own_read);
        }
        
        // (5) return true if no errors occured
        //--------------------------------------
        return true;
    }

    public function update()
    {
        $now = time();
        $this->set_modification_time($now);
        
        if (parent :: update($this))
        {
            if ($this->is_discussion())
            {
                // reset the status of the parent (root of the discussion) from read to unread for all other recipients
                //------------------------------------------------------------------------------------------------------
                // the defaults are preset to the correct values, we only need to give the ID of the message to change.
                PersonalMessengerDataManager :: change_user_message_status($this->get_parent_id());
            }
        }
        else
        {
            return false;
        }
        
        return true;
    }

    public function delete()
    {
        // delete the message
        $succes = parent :: delete();
        
        if ($succes)
        {
            $dm = PersonalMessengerDataManager :: get_instance();
            
            // delete all attachments
            $condition = new EqualityCondition(MessageAttachment :: PROPERTY_MESSAGE_ATTACHMENT_MESSAGE_ID, $this->get_id());
            $attachments = $dm->retrieve_all_message_attachments($condition);
            while ($attachment = $attachments->next_result())
            {
                $attachment->delete();
            }
            
            // delete all group recipients
            $condition = new EqualityCondition(MessageGroupRecipient :: PROPERTY_MESSAGE_RECIPIENT_MESSAGE_ID, $this->get_id());
            $group_recipients = $dm->retrieve_all_message_group_recipients($condition);
            while ($group_recipient = $group_recipients->next_result())
            {
                $group_recipient->delete();
            }
            
            // delete all user recipients
            $condition = new EqualityCondition(MessageUserRecipient :: PROPERTY_MESSAGE_RECIPIENT_MESSAGE_ID, $this->get_id());
            $user_recipients = $dm->retrieve_all_message_user_recipients($condition);
            while ($user_recipient = $user_recipients->next_result())
            {
                $user_recipient->delete();
            }
            
            // in case of discussion root also delete all replies
            if ($this->is_discussion_root())
            {
                $condition = new EqualityCondition(Message :: PROPERTY_MESSAGE_PARENT_ID, $this->get_id());
                $replies = $dm->retrieve_all_messages($condition);
                while ($reply = $replies->next_result())
                {
                    $reply->delete();
                }
            }
        }
        
        return $succes;
    }

}
