<?php
namespace application\personal_messenger;

use common\libraries\EqualityCondition;
use common\libraries\Utilities;
use repository\ContentObject;

/**
 * * *************************************************************************** This class represents a personal
 * message attachment (PMA). PMA objects only have 2 default properties, because they link a message to an object in the
 * repository. Direct linking was not possible because messages are no longer repository content objects. - message_id:
 * the numeric ID of the PM; - content_object_id: the numeric ID identifying the content object in the repository;
 * 
 * @name Chamilo Personal Messenger > MessageAttachment
 * @version 1.0.0
 * @author Stijn Van Hoecke ****************************************************************************
 */
class MessageAttachment extends PersonalMessengerDataClass
{
    // **************************************************************************
    // GENERAL CONSTANTS
    // **************************************************************************
    const CLASS_NAME = __CLASS__;
    
    /**
     * Indicates if invalid attachments should be removed.
     */
    const REMOVE_INVALID_ATTACHMENT = true;
    
    // **************************************************************************
    // PROPERTY CONSTANTS
    // **************************************************************************
    const PROPERTY_MESSAGE_ATTACHMENT_MESSAGE_ID = 'message_id';
    const PROPERTY_MESSAGE_ATTACHMENT_CONTENT_OBJECT_ID = 'content_object_id';
    
    // **************************************************************************
    // GENERAL GETTERS
    // **************************************************************************
    /**
     * Get the default properties of all message attachments.
     * 
     * @return array The property names
     */
    public static function get_default_property_names()
    {
        return array(self :: PROPERTY_MESSAGE_ATTACHMENT_MESSAGE_ID, 
                self :: PROPERTY_MESSAGE_ATTACHMENT_CONTENT_OBJECT_ID);
    }
    
    // **************************************************************************
    // PROPERTY GETTERS
    // **************************************************************************
    /**
     * Returns the numeric ID of the PM to which this attachment is attached.
     * 
     * @return int The message ID
     */
    public function get_message_id()
    {
        return $this->get_default_property(self :: PROPERTY_MESSAGE_ATTACHMENT_MESSAGE_ID);
    }

    /**
     * Returns the numeric ID of the content object in the repository to which this attachment is attached.
     * 
     * @return int The ID of the content object in the repository
     */
    public function get_content_object_id()
    {
        return $this->get_default_property(self :: PROPERTY_MESSAGE_ATTACHMENT_CONTENT_OBJECT_ID);
    }
    
    // **************************************************************************
    // PROPERTY SETTERS
    // **************************************************************************
    /**
     * Sets the numeric ID of the PM to which this attachment is attached.
     * 
     * @param int $message_id The ID of the content object in the repository
     */
    public function set_message_id($message_id)
    {
        $this->set_default_property(self :: PROPERTY_MESSAGE_ATTACHMENT_MESSAGE_ID, $message_id);
    }

    /**
     * Sets the numeric ID of the content object in the repository to which this attachment is attached.
     * 
     * @param int $content_object_id The ID of the content object in the repository
     */
    public function set_content_object_id($content_object_id)
    {
        $this->set_default_property(self :: PROPERTY_MESSAGE_ATTACHMENT_CONTENT_OBJECT_ID, $content_object_id);
    }
    
    // **************************************************************************
    // GENERAL FUNCTIONS
    // **************************************************************************
    /**
     * This function ensures the availability of the attachment. If the global constant REMOVE_INVALID_ATTACHMENT is put
     * to true, the attachment will be removed if it cannot be found.
     * 
     * @return boolean
     */
    public function ensure()
    {
        $condition = new EqualityCondition(ContentObject :: PROPERTY_ID, $this->get_content_object_id());
        if (\repository\DataManager :: count_active_content_objects(ContentObject :: class_name(), $condition) > 0)
        {
            return true;
        }
        elseif (self :: REMOVE_INVALID_ATTACHMENT)
        {
            $this->delete();
        }
        return false;
    }

    /**
     * Retrieves the content object, based on the ID stored in this MessageAttachment.
     * 
     * @return ContentObject
     */
    public function retieve_attachment_content_object()
    {
        if ($this->ensure())
        {
            return \repository\DataManager :: retrieve_content_object($this->get_content_object_id());
        }
        else
        {
            return null;
        }
    }
    
    // **************************************************************************
    // DATABASE FUNCTIONS
    // **************************************************************************
    /**
     * Gets the table name for this class.
     */
    public static function get_table_name()
    {
        return Utilities :: get_classname_from_namespace(self :: CLASS_NAME, true);
    }
}
