<?php

namespace application\personal_messenger;

use common\libraries\Application;
use common\libraries\BreadcrumbTrail;
use common\libraries\Breadcrumb;
use common\libraries\DatetimeUtilities;
use common\libraries\Request;
use common\libraries\Translation;
use common\libraries\Utilities;

/** * ***************************************************************************
 * This component offers the functionality to forward a message.
 *
 * @name Chamilo Personal Messenger > PersonalMessengerManagerForwarderComponent
 * @version 1.0.0
 * @author Stijn Van Hoecke
 * @uses ForwardMessageForm
 * **************************************************************************** */
class PersonalMessengerManagerForwarderComponent extends PersonalMessengerManager
{
    //**************************************************************************
    // GENERAL CONSTANTS
    //**************************************************************************
    /**
     * String put in front of the parent title to indicate it's a reply.
     */
    const FIXED_FORWARD_TITLE_PREFIX = 'FW: ';

    /**
     * String put in front of the original message making it indent etc.
     */
    const FIXED_FORWARD_MESSAGE_PREFIX = '<p></p><hr/><div>';
    const FIXED_FORWARD_MESSAGE_PREFIX_INDENT = '<p></p><hr/><div STYLE="padding-left: 2em;">';

    /**
     * Close string depending on the FIXED_FORWARD_MESSAGE_PREFIX.
     */
    const FIXED_FORWARD_MESSAGE_SUFFIX = '</div>';

    //**************************************************************************
    // VARIABLES
    //**************************************************************************
    /**
     * The message we want to forward.
     * @var Message
     */
    private $message;
    /**
     * The original message.
     * @var Message
     */
    private $original_message;
    /**
     * Indicates if the original message should be indenten.
     */
    private $indent_original_message;

    //**************************************************************************
    // CONSTRUCTOR
    //**************************************************************************
    /**
     * Constructor.
     */
    public function __construct($user, $indent_original_message = true)
    {
        parent :: __construct($user);
        $this->indent_original_message = $indent_original_message;
    }

    //**************************************************************************
    // RUN
    //**************************************************************************
    /**
     * Runs this component.
     */
    public function run()
    {
        // create a new message
        $this->message = new Message();

        // get the message on which we reply
        $this->original_message = $this->get_message_by_id(Request :: get(PersonalMessengerManager :: PARAM_MESSAGE_ID));

        // preset the title, content, reply- and parent-id of the message
        $this->message->set_title($this->get_forward_title());
        $this->message->set_content($this->get_forward_message());
        $this->message->set_reply_id($this->original_message->get_reply_id());
        $this->message->set_parent_id($this->original_message->get_parent_id());

        // include the original attachments
        $this->message->set_attachments($this->original_message->get_attachments());

        // create the form
        $form = new MessageForm(
                        MessageForm :: TYPE_FORWARD,
                        $this->message,
                        $this->get_url(),
                        $this->get_user(),
                        $this,
                        false
        );

        if ($form->validate())
        {
            $success = $form->create_message();

            // go back
            $go_back_parameters = array();
            if (Request :: get(PersonalMessengerManager :: PARAM_REFERER))
            {
                $go_back_parameters[Application :: PARAM_ACTION] = Request :: get(self :: PARAM_REFERER);
                // in case of forward, if you have a referer it is the (viewer) of the original message -> return to the original message
                $go_back_parameters[self :: PARAM_MESSAGE_ID] = Request :: get(self :: PARAM_MESSAGE_ID);
            }
            else
            {
                $go_back_parameters[Application :: PARAM_ACTION] = self :: DEFAULT_ACTION;
            }

            $this->redirect(
                    Translation :: get($success ? 'MessageForwarded' : 'MessageNotForwarded'),
                    ($success ? false : true),
                    $go_back_parameters
            );
        }
        else
        {
            $this->display_header();
            $form->display();
            $this->display_footer();
        }
    }

    //**************************************************************************
    // GENERAL FUNCTIONS
    //**************************************************************************
    /**
     * Constructs a forward title based on the parent title.
     * @param Message $message
     * @return String
     */
    public function get_forward_title()
    {
        $forward_title = $this->original_message->get_title();
        if (substr($forward_title, 0, strlen(self :: FIXED_FORWARD_TITLE_PREFIX)) != self :: FIXED_FORWARD_TITLE_PREFIX)
        {
            $forward_title = self :: FIXED_FORWARD_TITLE_PREFIX . $forward_title;
        }

        return $forward_title;
    }

    /**
     * Constructs a forward message based on the parent message.
     * @param Message $message
     * @return String
     */
    public function get_forward_message()
    {
        if ($this->indent_original_message)
        {
            $forward_message .= self :: FIXED_FORWARD_MESSAGE_PREFIX_INDENT;
        }
        else
        {
            $forward_message .= self :: FIXED_FORWARD_MESSAGE_PREFIX;
        }

        $forward_message .= '<b>' . Translation :: get('OriginalMessage') . ': <b><i>';
        $forward_message .= $this->original_message->get_title() . '</i><br/>';
        $forward_message .= '<b>' . Translation :: get('MessageFrom') . ': </b><i>';
        $forward_message .= $this->original_message->get_author_name() . '</i><br/>';
        $forward_message .= '<b>' . Translation :: get('MessageTo') . ': </b><i>';
        $forward_message .= implode(', ', $this->original_message->get_recipient_names()) . '</i><br/>';
        $forward_message .= '<b>' . Translation :: get('MessageDate') . ': </b><i>';
        $forward_message .= DatetimeUtilities :: format_locale_date(
                        Translation :: get('DateFormatShort', null, Utilities :: COMMON_LIBRARIES) .
                        ', ' . Translation :: get('TimeNoSecFormat', null, Utilities :: COMMON_LIBRARIES),
                        $this->original_message->get_modification_time());
        $forward_message .= '</i><br/>';
        $forward_message .= $this->original_message->get_content();
        $forward_message .= self :: FIXED_FORWARD_MESSAGE_SUFFIX;

        return $forward_message;
    }

    //**************************************************************************
    // ADDITIONAL FUNCTIONS
    //**************************************************************************
    /**
     * Adds the additional parameters to the URL.
     * @return array
     */
    public function get_additional_parameters()
    {
        return array(self :: PARAM_REFERER, self :: PARAM_MESSAGE_ID);
    }

    /**
     * Adds the additional breadcrumbs for navigation.
     * @param BreadcrumbTrail $breadcrumbtrail
     */
    public function add_additional_breadcrumbs(BreadcrumbTrail $breadcrumbtrail)
    {
        $browser_folder_url = $this->get_browser_url($this->get_folder(), $this->get_category_id());

        // browser
        $breadcrumbtrail->add(new Breadcrumb(
                        $browser_folder_url,
                        Translation :: get('PersonalMessengerManagerBrowserComponent')
        ));

        // folder
        $breadcrumbtrail->add(new Breadcrumb(
                        $browser_folder_url,
                        Translation :: get(Utilities:: underscores_to_camelcase(
                                        PersonalMessengerFolder :: get_folder_name($this->get_folder())
                        ))));

        // message
        $message = $this->retrieve_message_by_id(Request :: get(PersonalMessengerManager :: PARAM_MESSAGE_ID));
        if ($message)
        {
            $message_url = $this->get_message_view_url($message);
            $breadcrumbtrail->add(new Breadcrumb(
                            $message_url,
                            Translation :: get(
                                    $message->is_discussion() ? 'PersonalMessengerManagerViewerDiscussionComponent' : 'PersonalMessengerManagerViewerMessageComponent'
                            )
            ));
        }
    }

}
