<?php
namespace application\personal_messenger;

use common\libraries\Path;
use common\libraries\AndCondition;
use common\libraries\Application;
use common\libraries\DatetimeUtilities;
use common\libraries\EqualityCondition;
use common\libraries\NotCondition;
use common\libraries\ObjectTableOrder;
use common\libraries\Request;
use common\libraries\ResourceManager;
use common\libraries\Theme;
use common\libraries\ToolbarItem;
use common\libraries\Translation;
use common\libraries\Utilities;
use common\libraries\WebApplication;

/**
 * * *************************************************************************** Specific implementation of Viewer to
 * view discussions.
 *
 * @name Chamilo Personal Messenger > PersonalMessengerManagerViewerDiscussionComponent
 * @version 1.0.0
 * @author Stijn Van Hoecke
 * @see Viewer ****************************************************************************
 */
class PersonalMessengerManagerViewerDiscussionComponent extends PersonalMessengerManagerViewerComponent
{

    // **************************************************************************
    // CONSTRUCTOR
    // **************************************************************************
    /**
     * Constructor.
     */
    public function __construct($user = null)
    {
        parent :: __construct($user);
        Request :: set_get(self :: PARAM_REFERER, self :: ACTION_VIEW_DISCUSSION);
    }

    // **************************************************************************
    // INHERITED FUNCTIONS
    // **************************************************************************
    /**
     * Adds additional features specific to the viewer implementation.
     */
    public function add_viewer_additionals()
    {
        // AJAX functions
        // -- meta data for the AJAX postback
        echo '<script type="text/javascript">';
        echo 'var lastRetrieved = ' . time() . '; ';
        echo 'var messageId = ' . $this->message->get_id() . '; ';
        echo 'var ajaxUri = getPath("WEB_PATH") + "ajax.php";';
        echo '</script>';

        // -- retrieve function
        echo ResourceManager :: get_instance()->get_resource_html(
            Path :: namespace_to_full_path(__NAMESPACE__, true) . 'resources/javascript/retrieve_new_replies.js');

        // -- polling function
        echo ResourceManager :: get_instance()->get_resource_html(
            Path :: namespace_to_full_path(__NAMESPACE__, true) . 'resources/javascript/has_new_replies.js');

        // REPLY
        // display the replies for a discussion
        $this->display_replies();

        // also display a quick reply form
        $this->display_quick_reply_form($this->get_quick_reply_form());
    }

    /**
     * Returns the root message.
     *
     * @return Message
     */
    public function get_root_message($message_id)
    {
        if (! $this->root_message)
        {
            // retrieve the message
            $message = $this->retrieve_message_by_id($message_id);

            if ($message->get_parent_id() != $message->get_id() && $message->get_parent_id() != 0)
            {
                // retrieve the root
                $message = $this->retrieve_message_by_id($message->get_parent_id());
            }

            $this->root_message = $message;
        }

        return $this->root_message;
    }

    /**
     * Returns the global division declaration
     *
     * @return String HTML
     */
    public function get_global_div()
    {
        $global_div = '<div class="discussion" id="message_' . $this->message->get_id() .
             '" style="background-image: url(' . Theme :: get_image_path() . 'view_discussion.png);">';
        return $global_div;
    }

    // **************************************************************************
    // ADDITIONAL FUNCTIONS
    // **************************************************************************
    /**
     * Fetches and displays the replies on the given message.
     *
     * @param Message $message The root message
     */
    public function display_replies()
    {
        $root_message_id = $this->message->get_id();

        $html = array();
        $dm = PersonalMessengerDataManager :: get_instance();

        // construct conditions
        $conditions = array();
        $conditions[] = new EqualityCondition(Message :: PROPERTY_MESSAGE_PARENT_ID, $root_message_id);
        $conditions[] = new NotCondition(
            new EqualityCondition(Message :: PROPERTY_ID, $root_message_id));
        $condition = new AndCondition($conditions);

        // create sort order
        $order = array(
            new ObjectTableOrder(
                Message :: PROPERTY_MESSAGE_CREATION_TIME,
                SORT_ASC,
                PersonalMessengerDataManager :: get_instance()->get_alias(Message :: get_table_name())));

        // retrieve replies
        $replies = $dm->retrieve_all_messages($condition, $order);

        // display replies
        while ($reply = $replies->next_result())
        {
            $html[] = $this->get_reply_html($reply);
        }

        echo implode("\n", $html);
    }

    /**
     * Renders a reply and returns its HTML.
     *
     * @param Message $reply
     * @return String HTML
     */
    public function get_reply_html($reply)
    {
        $html = array();

        $author = $reply->get_author_name();

        // header
        // - start DIV reply
        $html[] = '<div class="reply" style="background-image: url(' . Theme :: get_image_path() . 'view_reply.png);">';

        // - start DIV title
        $html[] = '<div class="title"><b>' . $reply->get_author_name() . '</b>';
        $title = $reply->get_title();
        if ($title)
        {
            $html[] = ' - ' . $title;
        }
        // - end DIV title
        $html[] = '</div>';

        // - start DIV message
        $html[] = '<div class="message">' . $reply->get_content();
        // -- attachments
        if ($reply->count_attachments() > 0)
        {
            $html[] = '<table><tr>';
            $html[] = '<td class="header_title">';
            $html[] = '<img src="' . Theme :: get_image_path() . 'has_attachments.png"';
            $html[] = 'alt="' . Translation :: get('Attachments') . '"/>';
            $html[] = '</td><td class="header_value">';
            $html[] = $this->render_attachments($reply);
            $html[] = '</td></tr></table>';
        }
        // - end DIV message
        $html[] = '</div>';

        // - start DIV modification
        $creation = $reply->get_creation_time();
        $modification = $reply->get_modification_time();
        $html[] = '<div class="modification">';
        $html[] = Translation :: get('CreatedOn') . ': ';
        $html[] = DatetimeUtilities :: format_locale_date(
            Translation :: get('DateFormatShort', null, Utilities :: COMMON_LIBRARIES) . ', ' .
                 Translation :: get('TimeNoSecFormat', null, Utilities :: COMMON_LIBRARIES),
                $creation);
        if ($creation != $modification)
        {
            $html[] = '<br/>';
            $html[] = Translation :: get('LastModified') . ': ';
            $html[] = DatetimeUtilities :: format_locale_date(
                Translation :: get('DateFormatShort', null, Utilities :: COMMON_LIBRARIES) . ', ' .
                     Translation :: get('TimeNoSecFormat', null, Utilities :: COMMON_LIBRARIES),
                    $modification);
        }

        /*
         * TO DISCUSS => EDIT AND DELETE LINK if ($reply->get_author_id() == $this->get_user_id()) { $html[] = '<br/><a
         * href="' . $this->get_message_update_url($reply, self :: ACTION_VIEW_DISCUSSION) . '"><img src="' . Theme ::
         * get_image_path() . 'action_edit.png' . '" alt="" title="' . Translation :: get('Edit') . '" /></a>'; $html[]
         * = ' <a href="' . $this->get_message_delete_url($reply, self :: ACTION_VIEW_DISCUSSION) . '"><img src="' .
         * Theme :: get_image_path() . 'action_delete.png' . '" alt="" title="' . Translation :: get('Delete') . '"
         * /></a>'; }
         */

        // - end DIV modification
        $html[] = '</div>';

        // - end DIV reply
        $html[] = '</div>';

        return implode("\n", $html);
    }

    /**
     * Constructs a quick reply form.
     */
    public function get_quick_reply_form()
    {
        // get the message on which we reply
        $original_message = $this->get_message_by_id($this->root_message->get_id());

        // create the form
        if (self :: USE_AJAX_QUICK_REPLY)
        {
            return new QuickReplyFormAjax($original_message);
        }
        else
        {
            return new QuickReplyForm($original_message, $this->get_url(), $this->get_user(), $this);
        }
    }

    /**
     * Displays the given form.
     *
     * @param QuickReplyForm $form
     */
    public function display_quick_reply_form($form)
    {
        // use same layout as actual reply
        // - start DIV reply
        $html[] = '<div class="reply" style="background-image: url(' . Theme :: get_image_path() . 'view_reply.png);">';

        // - start DIV title
        $html[] = '<div class="title"><b>' . Translation :: get('QuickReply') . '</b>';
        // - end DIV title
        $html[] = '</div>';

        // - start DIV message
        $html[] = '<div class="message" style="margin: 0;">' . $form->toHtml();
        // - end DIV message
        $html[] = '</div>';

        // - start DIV modification
        $html[] = '<div class="modification">';
        // - end DIV modification
        $html[] = '</div>';

        // - end DIV reply
        $html[] = '</div>';

        echo implode("\n", $html);
    }

    /**
     * Processes the forms and redirect before headers are sent.
     */
    public function process_forms()
    {
        // process any previous quick reply form
        $form = $this->get_quick_reply_form();
        if ($form->validate())
        {
            $success = $form->create_quick_reply();
            $this->redirect(
                Translation :: get(
                    $success ? 'ObjectCreated' : 'ObjectNotCreated',
                    array('OBJECT' => Translation :: get('Reply')),
                    Utilities :: COMMON_LIBRARIES),
                $success ? false : true,
                array(Application :: PARAM_ACTION => PersonalMessengerManager :: ACTION_VIEW_DISCUSSION));
        }
    }

    /**
     * Override. Returns the action bar.
     *
     * @return ActionBar
     */
    public function get_action_bar()
    {
        // default bar
        $action_bar = parent :: get_action_bar();

        // MANAGE DISCUSSION USERS
        $manage_discussion_users_url = $this->get_discussion_user_manager_url(
            $this->message->get_id(),
            PersonalMessengerManager :: ACTION_VIEW_DISCUSSION);
        $manage_discussion_users_item = new ToolbarItem(
            Translation :: get('ManageDiscussionUsers'),
            Theme :: get_image_path() . 'action_manage_discussion_users.png',
            $manage_discussion_users_url,
            ToolbarItem :: DISPLAY_ICON_AND_LABEL,
            false);
        $action_bar->add_tool_action($manage_discussion_users_item);

        return $action_bar;
    }
}
