<?php

namespace application\personal_messenger;

use common\libraries\DatetimeUtilities;
use common\libraries\ObjectTableCellRenderer;
use common\libraries\Redirect;
use common\libraries\Translation;
use common\libraries\Utilities;
use user\User;

/** * ***************************************************************************
 * This class is the default cell renderer for messages.
 *
 * @name Chamilo Personal Messenger > DefaultMessageTableCellRenderer
 * @version 1.0.0
 * @author Stijn Van Hoecke
 * **************************************************************************** */
class DefaultMessageTableCellRenderer extends ObjectTableCellRenderer
{

    //**************************************************************************
    // VARIABLES
    //**************************************************************************
    /**
     * Holds the folder.
     * Used to create the links containing the folder.
     * @var int
     */
    protected $folder;
    /**
     * Holds the category_id.
     * @var int
     */
    protected $category_id;

    //**************************************************************************
    // CONSTRUCTOR
    //**************************************************************************
    /**
     * Constructor
     */
    public function __construct($folder = null, $category_id = 0)
    {
        $this->folder = $folder;
        $this->category_id = $category_id;
    }

    //**************************************************************************
    // GENERAL FUNCTIONS
    //**************************************************************************
    /**
     * Renders a table cell based on the given parameters column and message.
     * The message contains additional parameters obtained by a join between the
     * user message and actual message in the database data manager.
     * @param MessageTableColumn $column The column which should be rendered
     * @param Message $message The message to render
     * @return String A HTML representation of the rendered table cell
     */
    public function render_cell($column, $message_with_optional_properties)
    {
        switch ($column->get_name())
        {
            // status column gets the status from the optional parameters
            case UserMessage :: PROPERTY_USER_MESSAGE_STATUS :
                return $message_with_optional_properties->get_optional_property(UserMessage :: PROPERTY_USER_MESSAGE_STATUS);

            // author column gets the name of the author
            case Message :: PROPERTY_MESSAGE_AUTHOR_ID :
                $return = $message_with_optional_properties->get_optional_property(User :: PROPERTY_FIRSTNAME);
                $return .= ' ' . $message_with_optional_properties->get_optional_property(User :: PROPERTY_LASTNAME);
                return $return;

            // recipients column gets the recipients
            case MessageRecipient :: RECIPIENT_NAME :
                return implode(', ', $message_with_optional_properties->get_recipient_names());

            // attachment column
            case DefaultMessageTableColumnModel :: TABLE_HEADER_ATTACHMENT :
                return $message_with_optional_properties->count_attachments();

            // title column returns the title of the message using the optional parameters
            case Message :: PROPERTY_MESSAGE_TITLE :
                $link = '<a href="';
                $link .= $this->get_message_viewing_link($message_with_optional_properties, $this->folder);
                $link .= '">';
                if ($message_with_optional_properties->get_title())
                {
                    $link.= $message_with_optional_properties->get_title();
                }
                else
                {
                    $link.= '< ' . Translation :: get('NoTitle') . ' >';
                }
                $link .= '</a>';
                return $link;

            // date column returns the modification date of the message using the optional parameters
            case Message :: PROPERTY_MESSAGE_MODIFICATION_TIME :
                return DatetimeUtilities :: format_locale_date(
                        Translation :: get('DateFormatShort', null, Utilities :: COMMON_LIBRARIES) .
                        ', ' . Translation :: get('TimeNoSecFormat', null, Utilities :: COMMON_LIBRARIES),
                        $message_with_optional_properties->get_modification_time()
                );

            // type column returns if this is a discussion or a personal message
            case DefaultMessageTableColumnModel :: TABLE_HEADER_TYPE :
                return $message_with_optional_properties->is_discussion();

            // default return blank
            default :
                return '&nbsp;';
        }
    }

    public function render_id_cell($object)
    {
        return $object->get_id();
    }

    /**
     * Returns a link to the viewer of the message.
     * @param Message $message
     * @return String The link to the viewer of the message
     */
    public function get_message_viewing_link($message)
    {
        // construct viewing parameters
        $params = array();
        $params[PersonalMessengerManager :: PARAM_MESSAGE_ID] = $message->get_id();
        $params[PersonalMessengerManager :: PARAM_FOLDER] = $this->folder;
        $params[PersonalMessengerManager :: PARAM_CATEGORY_ID] = $this->category_id;

        if ($message->is_discussion())
        {
            $params[PersonalMessengerManager :: PARAM_ACTION] = PersonalMessengerManager :: ACTION_VIEW_DISCUSSION;
        }
        else
        {
            $params[PersonalMessengerManager :: PARAM_ACTION] = PersonalMessengerManager :: ACTION_VIEW_MESSAGE;
        }

        // return a link for the Personal Messenger application, with the given parameters
        return Redirect :: get_link(
                PersonalMessengerManager :: APPLICATION_NAME,
                $params,
                array(),
                false,
                Redirect :: TYPE_APPLICATION
        );
    }

}
