<?php

namespace application\personal_messenger;

use common\libraries\EqualityCondition;
use common\libraries\Session;
use common\libraries\Theme;
use common\libraries\Utilities;

/** * ***************************************************************************
 * This class represents a personal message (PM) for one specific user holding
 * some extra data.
 *
 * UserMessage objects have a number of default properties:
 * - message_id: the numeric ID of the PM;
 * - user_id: the numeric ID identifying the user;
 * - category_id: the numeric ID idenitifying the user defined category for
 *                  this message;
 * - status: indicates if the message is (un)read, trashed,... ;
 *
 * @name Chamilo Personal Messenger > UserMessage
 * @version 1.0.0
 * @author Stijn Van Hoecke
 * **************************************************************************** */
class UserMessage extends PersonalMessengerDataClass
{
    //**************************************************************************
    // GENERAL CONSTANTS
    //**************************************************************************
    const CLASS_NAME = __CLASS__;

    //**************************************************************************
    // PROPERTY CONSTANTS
    //**************************************************************************
    const PROPERTY_USER_MESSAGE_MESSAGE_ID = 'message_id';
    const PROPERTY_USER_MESSAGE_USER_ID = 'user_id';
    const PROPERTY_USER_MESSAGE_CATEGORY_ID = 'category_id';
    const PROPERTY_USER_MESSAGE_STATUS = 'status';

    //**************************************************************************
    // STATUS CONSTANTS
    //**************************************************************************
    const STATUS_READ = 1;
    const STATUS_UNREAD = 2;
    const STATUS_IGNORE = 3;
    const STATUS_TRASH = 4;

    //**************************************************************************
    // GENERAL GETTERS
    //**************************************************************************
    /**
     * Get the default properties of all user defined categories.
     * @return array The property names
     */
    public static function get_default_property_names()
    {
        return array(
            self :: PROPERTY_USER_MESSAGE_MESSAGE_ID,
            self :: PROPERTY_USER_MESSAGE_USER_ID,
            self :: PROPERTY_USER_MESSAGE_CATEGORY_ID,
            self :: PROPERTY_USER_MESSAGE_STATUS);
    }

    //**************************************************************************
    // PROPERTY GETTERS
    //**************************************************************************
    /**
     * Returns the numeric ID of the PM to which this user is linked.
     * @return int The message ID
     */
    public function get_message_id()
    {
        return $this->get_default_property(self :: PROPERTY_USER_MESSAGE_MESSAGE_ID);
    }

    /**
     * Returns the numeric ID of the user to which this message is linked.
     * @return int The ID of the user
     */
    public function get_user_id()
    {
        return $this->get_default_property(self :: PROPERTY_USER_MESSAGE_USER_ID);
    }

    /**
     * Returns the numeric ID of the catagory in which the PM is stored.
     * @return int The category ID
     */
    public function get_category_id()
    {
        return $this->get_default_property(self :: PROPERTY_USER_MESSAGE_CATEGORY_ID);
    }

    /**
     * Returns the status of the message in the users inbox.
     * @return boolean The read status of the message
     */
    public function get_status()
    {
        return $this->get_default_property(self :: PROPERTY_USER_MESSAGE_STATUS);
    }

    //**************************************************************************
    // PROPERTY SETTERS
    //**************************************************************************
    /**
     * Sets the numeric ID of the PM to which this user is linked.
     * @param int $message_id The ID of the message
     */
    public function set_message_id($message_id)
    {
        $this->set_default_property(self :: PROPERTY_USER_MESSAGE_MESSAGE_ID, $message_id);
    }

    /**
     * Sets the numeric ID of the user to which this message is linked.
     * @param int $user_id The ID of the user
     */
    public function set_user_id($user_id)
    {
        $this->set_default_property(self :: PROPERTY_USER_MESSAGE_USER_ID, $user_id);
    }

    /**
     * Sets the numeric ID of the catagory in which the PM is stored.
     * @param int $category_id The category ID
     */
    public function set_category_id($category_id)
    {
        $this->set_default_property(self :: PROPERTY_USER_MESSAGE_CATEGORY_ID, $category_id);
    }

    /**
     * Sets the status of the message in the users inbox.
     * @param boolean $status The read status of the message
     */
    public function set_status($status)
    {
        $this->set_default_property(self :: PROPERTY_USER_MESSAGE_STATUS, $status);
    }

    //**************************************************************************
    // STATUS FUNCTIONS
    //**************************************************************************
    /**
     * Returns the name according to the specified folder.
     * @param int $folder The folder
     * @return String The name of the folder
     */
    public static function get_status_name($status)
    {
        switch ($status)
        {
            case self :: STATUS_READ :
                return 'read';
            case self :: STATUS_UNREAD :
                return 'unread';
            case self :: STATUS_IGNORE :
                return 'ignored';
            case self :: STATUS_TRASH :
                return 'trashed';
            default :
                return 'unknown';
        }
    }

    /**
     * Checks if the constant for the given folder exists.
     * @param int $folder The folder
     * @return boolean Indicating if the folder exists
     */
    public static function is_valid_status($status)
    {
        return!($status != self :: STATUS_READ
        && $status != self :: STATUS_UNREAD
        && $status != self :: STATUS_IGNORE
        && $status != self :: STATUS_TRASH);
    }

    /**
     * Returns the link to the icon related to a given status.
     * @param int $status The status
     * @param boolean $discussion Indicating if it is a discussion
     * @return String HTML representing the icon related to the given status
     */
    public static function get_status_icon_path($status, $discussion)
    {
        $icon_path = Theme :: get_image_path();
        $icon_path .= 'status';
        if (self :: is_valid_status($status))
        {
            $icon_path .= '_';
            $icon_path .= $discussion ? 'discussion' : 'message';
            $icon_path .= '_';
            $icon_path .= self :: get_status_name($status);
        }
        $icon_path .= '.png';

        return $icon_path;
    }

    //**************************************************************************
    // DATABASE FUNCTIONS
    //**************************************************************************
    /**
     * Gets the table name for this class.
     */
    public static function get_table_name()
    {
        return Utilities :: get_classname_from_namespace(self :: CLASS_NAME, true);
    }

    public function create()
    {
        // set status to (un)read if there is no status previously set.
        // There is the possibility to set the status to ignored BEFORE creating
        // the link, thus we need to check for an existing status.
        if (!$this->get_status())
        {
            // in case the link is for me, I read the message, otherwise it's unread
            if ($this->get_user_id() == Session :: get_user_id())
            {
                $this->set_status(self :: STATUS_READ);
            }
            else
            {
                $this->set_status(self :: STATUS_UNREAD);
            }
        }

        // create the message
        return parent :: create($this);
    }

    public function delete()
    {
        // delete the user message
        $succes = parent :: delete($this);

        if ($succes)
        {
            // check if any user links remain to the original message, if not delete this original message
            $user_message_condition = new EqualityCondition(
                            UserMessage :: PROPERTY_USER_MESSAGE_MESSAGE_ID,
                            $this->get_message_id()
            );
            $dm = PersonalMessengerDataManager :: get_instance();
            $remaining_links = $dm->count_user_message($user_message_condition);
            if ($remaining_links == 0)
            {
                $dm = PersonalMessengerDataManager :: get_instance();
                $message_condition = new EqualityCondition(
                                Message :: PROPERTY_ID,
                                $this->get_message_id()
                );
                $message = $dm->retrieve_message($message_condition);
                $message->delete();
            }
        }

        return $succes;
    }

}
