<?php

namespace application\portfolio;

use rights\RightsUtil;

/**
 * class to handle the different rights in the portfolio application following logic will be used to set rights for the
 * portfolio application: 1) rights for user_id 1 = rights for anonymous users & every platform-user 2) rights for
 * user_id 0 = rights for every platform-user 3) no rights = right for portfolio(-item) owner only 4) rights defined for
 * specific users and/or groups rights can be inherited from parent item or set for item specifically. if rights are set
 * for item specifically they are NOT inherited from the parent item. Top-level portfolio's can inherit default rights
 * 
 * @author Nathalie Blocry
 */
class PortfolioRights extends RightsUtil
{

    const VIEW_RIGHT = '1';
    const EDIT_RIGHT = '2';
    const VIEW_FEEDBACK_RIGHT = '3';
    const GIVE_FEEDBACK_RIGHT = '4';
    const SET_PERMISSIONS_RIGHT = '5';
    
    const TYPE_PORTFOLIO_FOLDER = 1;
    const TYPE_PORTFOLIO_SUB_FOLDER = 2;
    const TYPE_PORTFOLIO_ITEM = 3;
    
    const RADIO_OPTION_SET_SPECIFIC = 'set';
    const RADIO_OPTION_DEFAULT = 'SystemDefaults';
    const RADIO_OPTION_INHERIT = 'inheritFromParent';
    const RADIO_OPTION_ANONYMOUS = 'AnonymousUsers';
    const RADIO_OPTION_ALLUSERS = 'SystemUsers';
    const RADIO_OPTION_ME = 'OnlyMe';
    const ANONYMOUS_USERS_ID = '1';
    const ALL_USERS_ID = '0';

    /**
     * create a portfolio_tree for a specific user. The portfolio-tree is a tree of locations that represent the
     * structure of the portfolio and can be used to have items inherit rights
     * 
     * @param $user_id: id of the user that owns the portfolio to identify it (tree-identifier)
     * @return true or false
     */
    public static function create_portfolio_root($user_id)
    {
        return parent :: get_root(WeblcmsManager :: context());
    }

    /**
     * get the root of a specific user's portfolio-tree.
     * 
     * @param $user_id: id of the user the portfolio belongs to, this is the tree_identifier
     * @return returns the id of the root location or "false" when no root location is found
     */
    public static function get_portfolio_root_id($user_id)
    {
        return parent :: get_root_id(PortfolioManager :: context());
    }

    /**
     * create a location for a portfolio or portfolio-item in the portfolio-tree
     * 
     * @param $type: type of object the location refers to, can be "portfolio" or "portfolio-item"
     * @param $identifier: id of the object the location refers to
     * @param $user_id: id of the user that owns the portfolio to identify in wich portfolio-tree to look
     * @param $name: name for the location
     * @param $parent: parent of the location
     * @param $inherit: true or false --> does the location inherit rights from it's parents
     * @param $locked: true of false --> can children override rights set for this location?
     * @param children: this item may contain children that need locations to (import from portfolio from repository)
     * @param $return_location <bool> = return location (true) or only boolean (false)
     * @return location when location has been created or true or false
     */
    public static function create_location_in_portfolio_tree($name, $type, $identifier, $parent, $user_id, $inherit, $locked, $children, $return_location = true)
    {

        $location = parent::create_location(
                        PortfolioManager::context(), $type = $type, $identifier, $inherit, $parent, $locked, $user_id, $tree_type, true);

        if ($location != false)
        {
            $success = true;
        }

        if ($success && $children)
        {
            $success &= self::create_locations_for_children($type, $location, $identifier, $user_id);
        }

        if ($return_location && $success)
        {
            return $location;
        }
        else
        {
            return $success;
        }
    }

    public static function create_locations_for_children($type, $parent_location, $identifier, $owner_id)
    {
        if (($type == self :: TYPE_PORTFOLIO_FOLDER) || ($type == self :: TYPE_PORTFOLIO_SUB_FOLDER))
        {
            //TODO: check type sub-folder = ok?????
            //portfolio publication
            $children_set = PortfolioManager :: get_portfolio_children($identifier, true, false);
        }
        else
        {
            //portfolio item
            $children_set = PortfolioManager :: get_portfolio_children($identifier, false, true);
        }

        if ($children_set)
        {
            $location_id = $parent_location->get_id();
            while ($complex_child = $children_set->next_result())
            {
                $object_id = PortfolioManager :: get_co_id_from_complex_wrapper($complex_child->get_id(), $complex_child);
                if ($object_id)
                {
                    $object = \repository\DataManager :: retrieve_content_object($object_id);
                    if ($object instanceof \repository\content_object\portfolio\Portfolio)
                    {
                        $type = self :: TYPE_PORTFOLIO_SUB_FOLDER;
                        $children = true;
                    }
                    else
                    {
                        $type = self :: TYPE_PORTFOLIO_ITEM;
                        $children = false;
                    }
                    $child_id = $complex_child->get_id();
                    $success = self :: create_location_in_portfolio_tree(null, $type, $child_id, $location_id, $owner_id, true, false, true, false);
                }
            }
        }
        return $success;
    }
    
    /**
     * this method returns the location that is linked to a specific portfolio item
     * 
     * @param $portfolio_identifier: the id of the portfolio-item
     * @param $type: the type of the portfolio-item this is an array so can be more then one type
     * @param $user_id: id of the user that owns the portfolio to identify in wich portfolio-tree to look
     * @param $fix: create the location if the location does not exist (default = false)
     * @return $location: object or false if no location has been found
     */
    public static function get_portfolio_location($portfolio_identifier, $type, $user_id, $fix = false)
    {
        $location = parent :: retrieve_rights_location_by_identifier(PortfolioManager::context(), $type, $identifier, $user_id);
        
        if($location == false && $fix)
        {
            $location = self::fix_non_existing_location();
        }
        return $location;
    }
    
    
    public static function fix_non_existing_location($owner_id)
    {
        
            // NO LOCATION FOR ITEM! item probably added in repository -> create location at correct place
            $parent_id = Request :: get(PortfolioManager :: PARAM_PARENT);
           
            
            $item_type = 
            
            //1. Parent might be a publication
            $parent_type = self::TYPE_PORTFOLIO_FOLDER;
            $parent_location = PortfolioRights :: get_portfolio_location($parent_type, $parent_id, $owner_id);
            
            if($parent_location == false)
            {
                //2. Parent might be a subfolder
                $parent_type = self::TYPE_PORTFOLIO_SUB_FOLDER;
                $parent_location = PortfolioRights :: get_portfolio_location($parent_type, $parent_id, $owner_id);
            }    
           
            if ($parent_location)
            {
                $parent_location_id = $parent_location->get_id();
                //create location
                $location = PortfolioRights :: create_location_in_portfolio_tree(PortfolioRights :: TYPE_PORTFOLIO_ITEM, $item_type, $identifier, $parent_location_id, $owner_id, true, false, true, true);
            }
            else
            {
                //TODO no parent found: fix recursive???
                $location = false;
            }
        
    }

}
