<?php
namespace application\weblcms\course;

use common\libraries\SubManager;
use application\weblcms\WeblcmsManager;
use common\libraries\Request;
use common\libraries\Translation;
use common\libraries\InCondition;
use common\libraries\ObjectNotExistException;
use common\libraries\NoObjectSelectedException;

/**
 * This class describes the submanager for course management
 *
 * @package \application\weblcms\course
 * @author Sven Vanpoucke - Hogeschool Gent
 */
class CourseManager extends SubManager
{
    /**
     * **************************************************************************************************************
     * PARAMATERS *
     * **************************************************************************************************************
     */
    const PARAM_ACTION = 'course_action';
    const PARAM_COURSE_ID = 'course_id';

    /**
     * **************************************************************************************************************
     * ACTIONS *
     * **************************************************************************************************************
     */
    const ACTION_BROWSE = 'browse';
    const ACTION_BROWSE_UNSUBSCRIBED_COURSES = 'browse_unsubscribed_courses';
    const ACTION_BROWSE_SUBSCRIBED_COURSES = 'browse_subscribed_courses';
    const ACTION_CREATE = 'create';
    const ACTION_DELETE = 'delete';
    const ACTION_QUICK_CREATE = 'quick_create';
    const ACTION_QUICK_UPDATE = 'quick_update';
    const ACTION_SUBSCRIBE = 'subscribe';
    const ACTION_UNSUBSCRIBE = 'unsubscribe';
    const ACTION_UPDATE = 'update';
    const ACTION_VIEW = 'view';
    const DEFAULT_ACTION = self :: ACTION_BROWSE;

    /**
     * **************************************************************************************************************
     * Constructor Functionality *
     * **************************************************************************************************************
     */

    /**
     * Constructor Check if the parent component inherits the required marker interface
     *
     * @param $parent
     */
    public function __construct($parent)
    {
        if (! $parent instanceof CourseSubManagerSupport)
        {
            throw new \Exception(
                'Components that use the course submanager support need to implement the CourseSubManagerSupport');
        }

        parent :: __construct($parent);
    }

    /**
     * **************************************************************************************************************
     * Common Functionality *
     * **************************************************************************************************************
     */

    /**
     * Retrieves the first selected course
     *
     * @return Course
     */
    protected function get_selected_course()
    {
        return $this->get_selected_courses()->next_result();
    }

    /**
     * Retrieves the selected course Use this function if you want to retrieve the selected course as a resultset
     *
     * @return \common\libraries\ResultSet <Course>
     */
    protected function get_selected_courses()
    {
        $course_ids = $this->get_selected_course_ids();

        $condition = new InCondition(Course :: PROPERTY_ID, $course_ids);
        $result_set = DataManager :: retrieves(Course :: class_name(), $condition);

        if ($result_set->size() == 0)
        {
            throw new ObjectNotExistException(Translation :: get('Course'), $course_ids);
        }

        return $result_set;
    }

    /**
     * Returns the selected course ids as an array
     *
     * @return string[]
     */
    protected function get_selected_course_ids()
    {
        $course_ids = Request :: get(self :: PARAM_COURSE_ID);

        if (! isset($course_ids))
        {
            throw new NoObjectSelectedException(Translation :: get('Course'));
        }

        if (! is_array($course_ids))
        {
            $course_ids = array($course_ids);
        }

        return $course_ids;
    }

    /**
     * **************************************************************************************************************
     * URL Building *
     * **************************************************************************************************************
     */

    /**
     * Returns the url to the course browse component for the given course id
     *
     * @return String
     */
    public function get_browse_course_url()
    {
        return $this->get_action_url(self :: ACTION_BROWSE);
    }

    /**
     * Returns the url to the subscribed courses browse component for the given course id
     *
     * @return String
     */
    public function get_browse_subscribed_courses_url()
    {
        return $this->get_action_url(self :: ACTION_BROWSE_SUBSCRIBED_COURSES);
    }

    /**
     * Returns the url to the unsubscribed courses browse component for the given course id
     *
     * @return String
     */
    public function get_browse_unsubscribed_courses_url()
    {
        return $this->get_action_url(self :: ACTION_BROWSE_UNSUBSCRIBED_COURSES);
    }

    /**
     * Returns the url to the course create component for the given course id
     *
     * @param $course_id int
     *
     * @return String
     */
    public function get_create_course_url()
    {
        return $this->get_url(array(self :: PARAM_ACTION => self :: ACTION_CREATE));
    }

    /**
     * Returns the url to the course viewing component for the given course id
     *
     * @param $course_id int
     *
     * @return String
     */
    public function get_view_course_url($course_id)
    {
        return $this->get_course_url(self :: ACTION_VIEW, $course_id);
    }

    /**
     * Returns the url to the course delete component for the given course id
     *
     * @param $course_id int
     *
     * @return String
     */
    public function get_delete_course_url($course_id)
    {
        return $this->get_course_url(self :: ACTION_DELETE, $course_id);
    }

    /**
     * Returns the url to the course update component for the given course id
     *
     * @param $course_id int
     *
     * @return String
     */
    public function get_update_course_url($course_id)
    {
        return $this->get_course_url(self :: ACTION_UPDATE, $course_id);
    }

    /**
     * Returns the url to subscribe a user to a course
     *
     * @param $course_id int
     *
     * @return string
     */
    public function get_subscribe_to_course_url($course_id)
    {
        return $this->get_course_url(self :: ACTION_SUBSCRIBE, $course_id);
    }

    /**
     * Returns the url to unsubscribe a user from a course
     *
     * @param $course_id int
     *
     * @return string
     */
    public function get_unsubscribe_from_course_url($course_id)
    {
        return $this->get_course_url(self :: ACTION_UNSUBSCRIBE, $course_id);
    }

    /**
     * Builds the view course home url
     *
     * @param int $course_id
     *
     * @return string
     */
    public function get_view_course_home_url($course_id)
    {
        return $this->get_url(
            array(
                WeblcmsManager :: PARAM_ACTION => WeblcmsManager :: ACTION_VIEW_COURSE,
                WeblcmsManager :: PARAM_COURSE => $course_id),
            array(CourseManager :: PARAM_ACTION));
    }

    /**
     * **************************************************************************************************************
     * Helper Functionality *
     * **************************************************************************************************************
     */

    /**
     * Returns a url for an action based on 1 specific course
     *
     * @param $action string
     * @param $course_id int
     * @param $parameters string[] - Optional parameters
     * @return string
     */
    protected function get_course_url($action, $course_id, $parameters = array())
    {
        $parameters[self :: PARAM_COURSE_ID] = $course_id;

        return $this->get_action_url($action, $parameters);
    }

    /**
     * Returns a url for a given action
     *
     * @param $action string
     * @param $parameters string[] - Optional parameters
     * @return string
     */
    protected function get_action_url($action, $parameters = array())
    {
        $parameters[self :: PARAM_ACTION] = $action;

        return $this->get_url($parameters);
    }

    /**
     * **************************************************************************************************************
     * Inherited Functionality *
     * **************************************************************************************************************
     */

    /**
     * Launches the submanager
     *
     * @param $application Application
     */
    public static function launch($application)
    {
        parent :: launch(__CLASS__, $application);
    }

    /**
     * **************************************************************************************************************
     * Helper Inherited Functionality *
     * **************************************************************************************************************
     */

    /**
     * Helper function for the SubManager class, pending access to class constants via variables in PHP 5.3 e.g. $name =
     * $class :: DEFAULT_ACTION DO NOT USE IN THIS SUBMANAGER'S CONTEXT Instead use: - self :: DEFAULT_ACTION in the
     * context of this class - YourSubManager :: DEFAULT_ACTION in all other application classes
     */
    public static function get_default_action()
    {
        return self :: DEFAULT_ACTION;
    }

    /**
     * Helper function for the SubManager class, pending access to class constants via variables in PHP 5.3 e.g. $name =
     * $class :: PARAM_ACTION DO NOT USE IN THIS SUBMANAGER'S CONTEXT Instead use: - self :: PARAM_ACTION in the context
     * of this class - YourSubManager :: PARAM_ACTION in all other application classes
     */
    public static function get_action_parameter()
    {
        return self :: PARAM_ACTION;
    }
}
