<?php
namespace application\weblcms\integration\reporting;

use application\weblcms\integration\tracking\AssessmentAttempt;
use common\libraries\DataClassRetrievesParameters;
use application\weblcms\ContentObjectPublication;
use application\weblcms\tool\reporting\ReportingTool;
use common\libraries\EqualityCondition;
use common\libraries\Theme;
use common\libraries\Translation;
use common\libraries\Utilities;
use reporting\ReportingManager;
use reporting\Reporting;
use reporting\ReportingData;
use reporting\ReportingFormatter;
use repository\ComplexContentObjectItem;
use repository\ContentObject;

/**
 *
 * @package application.weblcms.php.reporting.blocks Reporting block with an overview of the scores of each question in
 *          the assessment
 * @author Joris Willems <joris.willems@gmail.com>
 * @author Alexander Van Paemel
 */
class AssessmentQuestionsBlock extends AssessmentBlock
{

    public function count_data()
    {
        $reporting_data = new ReportingData();

        $rows = array(
            Translation :: get('QuestionTitle'),
            Translation :: get('QuestionType')
        );

        $rows = array_merge($rows, $this->get_question_reporting_info_headers());
        $rows[] = Translation :: get('QuestionDetails');

        $reporting_data->set_rows($rows);

        $pid = $this->get_publication_id();

        $publication = \application\weblcms\DataManager :: retrieve(ContentObjectPublication :: class_name(), $pid);

        $condition = new EqualityCondition(
            ComplexContentObjectItem :: PROPERTY_PARENT,
            $publication->get_content_object_id()
        );

        $questions = \repository\DataManager :: retrieve_complex_content_object_items(
            ComplexContentObjectItem :: class_name(),
            new DataClassRetrievesParameters($condition)
        );

        $img = '<img src="' . Theme :: get_common_image_path() . 'action_reporting.png" title="' .
            Translation :: get('Details') . '" />';

        $count = 0;

        while ($question = $questions->next_result())
        {
            $count++;

            $reporting_data->add_category($count);

            $reporting_info = $this->get_question_reporting_info($question);
            $this->add_row_from_array($count, $reporting_info, $reporting_data);

            $attempt_count = $reporting_info[Translation :: get('NumberOfAttempts')];
            if ($attempt_count > 0)
            {
                $params = $this->get_parent()->get_parameters();
                $params[ReportingManager :: PARAM_TEMPLATE_ID] = AssessmentQuestionUsersTemplate :: class_name();
                $params[ReportingTool :: PARAM_QUESTION] = $question->get_id();
                $link = '<a href="' . $this->get_parent()->get_url($params) . '">' . $img . '</a>';
                $reporting_data->add_data_category_row($count, Translation :: get('QuestionDetails'), $link);
            }
        }

        $reporting_data->hide_categories();

        return $reporting_data;
    }

    public function retrieve_data()
    {
        return $this->count_data();
    }

    public function get_views()
    {
        return array(\reporting\viewer\HtmlBlockRendition :: VIEW_TABLE);
    }

    /**
     * Returns the question reporting headers
     *
     * @return string[]
     */
    protected function get_question_reporting_info_headers()
    {
        return array(
            Translation :: get('NumberOfAttempts'),
            Translation :: get('AverageScore'),
            Translation :: get('MinScoreAchieved'),
            Translation :: get('MaxScoreAchieved'),
            Translation :: get('DifficultyIndex')
        );
    }

    /**
     * Returns the question reporting info for a given question
     *
     * @param ComplexContentObjectItem $complex_question
     *
     * @return mixed[]
     */
    protected function get_question_reporting_info($complex_question)
    {
        $weight = $complex_question->get_weight();

        $question_attempts = $this->get_question_attempts_from_publication_and_question(
            $this->get_publication_id(), $complex_question->get_id()
        );

        $score = $min = $max = null;
        $correct_answers = $finished_attempts = 0;

        while($question_attempt = $question_attempts->next_result())
        {
            $assessment_attempt = $question_attempt->get_optional_property(self::PROPERTY_ASSESSMENT_ATTEMPT);

            if($assessment_attempt->get_status() == AssessmentAttempt :: STATUS_COMPLETED)
            {
                if ($question_attempt->get_score() == $weight)
                {
                    $correct_answers ++;
                }

                if (is_null($min) || $min > $question_attempt->get_score())
                {
                    $min = $question_attempt->get_score();
                }
                if (is_null($max) || $max < $question_attempt->get_score())
                {
                    $max = $question_attempt->get_score();
                }

                $score += $question_attempt->get_score();

                $finished_attempts++;
            }
        }

        if($finished_attempts > 0)
        {
            $score = $this->get_score_bar($score / $finished_attempts / $complex_question->get_weight() * 100);
            $difficulty_index = $this->render_difficulty_index($correct_answers, $finished_attempts);
        }

        if (! is_null($min))
        {
            $min = $this->get_score_bar($min / $weight * 100);
        }

        if (! is_null($max))
        {
            $max = $this->get_score_bar($max / $weight * 100);
        }

        $object = $complex_question->get_ref_object();

        $type = Translation :: get(
            'TypeName',
            array(),
            Utilities :: get_namespace_from_classname($object->get_type())
        );

        $reporting_info = array();

        $reporting_info[Translation :: get('QuestionTitle')] = $object->get_title();
        $reporting_info[Translation :: get('QuestionType')] = $type;
        $reporting_info[Translation :: get('NumberOfAttempts')] = $question_attempts->size();
        $reporting_info[Translation :: get('DifficultyIndex')] = $difficulty_index;
        $reporting_info[Translation :: get('AverageScore')] = $score;
        $reporting_info[Translation :: get('MinScoreAchieved')] = $min;
        $reporting_info[Translation :: get('MaxScoreAchieved')] = $max;


        return $reporting_info;
    }
}
