<?php
namespace application\weblcms\integration\reporting;

use common\libraries\DataClassRetrievesParameters;
use common\libraries\Utilities;
use application\weblcms\ContentObjectPublication;
use reporting\Reporting;
use reporting\ReportingData;
use reporting\ReportingFormatter;
use common\libraries\Application;
use common\libraries\AndCondition;
use common\libraries\EqualityCondition;
use common\libraries\Translation;
use common\libraries\Request;
use common\libraries\ObjectTableOrder;
use repository\content_object\assessment\Assessment;
use common\libraries\PlatformSetting;
use application\weblcms\course\DataManager as CourseDataManager;

/**
 *
 * @package application.weblcms.php.reporting.blocks Reporting block with an overview of scores of each assessment per
 *          user
 * @author Joris Willems <joris.willems@gmail.com>
 * @author Alexander Van Paemel
 */
class AssessmentUserScoresBlock extends ToolBlock
{

    public function count_data()
    {
        $reporting_data = new ReportingData();
        $course_id = $this->get_course_id();

        $weblcms_data_manager = CourseDataManager :: get_instance();
        $users = $weblcms_data_manager->retrieve_all_course_users($course_id)->as_array();

        $conditions = array();
        $conditions[] = new EqualityCondition(ContentObjectPublication :: PROPERTY_COURSE_ID, $course_id);
        $conditions[] = new EqualityCondition(
            ContentObjectPublication :: PROPERTY_TOOL,
            Utilities :: get_classname_from_namespace(Assessment :: class_name(), true));
        $condition = new AndCondition($conditions);

        $order_by = array(new ObjectTableOrder(ContentObjectPublication :: PROPERTY_MODIFIED_DATE, SORT_DESC));

        $publication_resultset = \application\weblcms\DataManager :: retrieves(
            ContentObjectPublication :: class_name(),
            new DataClassRetrievesParameters($condition, null, null, $order_by)
        );

        $publications = array();
        $headings = array();
        $headings[] = Translation :: get('Name');
        $headings[] = Translation :: get('OfficialCode', null, 'user');
        while ($publication = $publication_resultset->next_result())
        {
            $publications[] = $publication;

            $content_object = $publication->get_content_object();

            if ($publication_resultset->size() > 5)
            {
                $headings[] = substr($content_object->get_title(), 0, 14);
            }
            else
            {
                $headings[] = $content_object->get_title();
            }
        }

        $reporting_data->set_rows($headings);

        foreach ($users as $key => $user)
        {
            $reporting_data->add_category($key);
            $reporting_data->add_data_category_row(
                $key,
                Translation :: get('Name'),
                \user\User :: fullname($user[\user\User :: PROPERTY_FIRSTNAME], $user[\user\User :: PROPERTY_LASTNAME])
            );

            $reporting_data->add_data_category_row(
                $key, Translation :: get('OfficialCode', null, 'user'), $user[\user\User :: PROPERTY_OFFICIAL_CODE]
            );

            foreach ($publications as $publication)
            {
                $content_object = $publication->get_content_object();

                if ($publication_resultset->size() > 5)
                {
                    $title = substr($content_object->get_title(), 0, 14);
                }
                else
                {
                    $title = $content_object->get_title();
                }

                $attempts_tracker = new \application\weblcms\integration\tracking\AssessmentAttempt();
                $conditions = array();

                $conditions[] = new EqualityCondition(
                    \application\weblcms\integration\tracking\AssessmentAttempt :: PROPERTY_ASSESSMENT_ID,
                    $publication->get_id()
                );

                $conditions[] = new EqualityCondition(
                    \application\weblcms\integration\tracking\AssessmentAttempt :: PROPERTY_USER_ID,
                    $user[\user\User :: PROPERTY_ID]);
                $condition = new AndCondition($conditions);
                $attempts_by_user = $attempts_tracker->retrieve_tracker_items($condition);

                if (count($attempts_by_user) == 0)
                {
                    if (\application\weblcms\DataManager :: is_publication_target_user(
                        $user[\user\User :: PROPERTY_ID],
                        $publication->get_id(),
                        $course_id
                    )
                    )
                    {
                        $reporting_data->add_data_category_row($key, $title, null);
                        continue;
                    }

                    $reporting_data->add_data_category_row($key, $title, 'X');
                    continue;
                }

                $score = $this->get_score($attempts_by_user);
                if ($score < PlatformSetting :: get('passing_percentage'))
                {
                    $score = '<span style="color:red">' . $score . '</span>';
                }
                else
                {
                    $score = '<span style="color:green">' . $score . '</span>';
                }
                $reporting_data->add_data_category_row($key, $title, $score);
            }
        }
        $reporting_data->hide_categories();
        return $reporting_data;
    }

    private function get_score($attempts)
    {
        $score_type = (Request :: post('sel')) ? Request :: post('sel') : Request :: get('sel');
        if (is_null($score_type))
        {
            $score_type = Request :: get('sel');
        }
        $score = null;

        switch ($score_type)
        {
            case self :: SCORE_TYPE_AVG :
                foreach ($attempts as $attempt)
                {
                    $score += $attempt->get_total_score();
                }
                return number_format($score / count($attempts), 1);
            case self :: SCORE_TYPE_MIN :
                foreach ($attempts as $attempt)
                {
                    if (is_null($score) || $attempt->get_total_score() < $score)
                    {
                        $score = $attempt->get_total_score();
                    }
                }
                return $score;
            case self :: SCORE_TYPE_MAX :
                foreach ($attempts as $attempt)
                {
                    if (is_null($score) || $attempt->get_total_score() > $score)
                    {
                        $score = $attempt->get_total_score();
                    }
                }
                return $score;
            case self :: SCORE_TYPE_FIRST :
                $date = null;
                foreach ($attempts as $attempt)
                {
                    if (is_null($score) || $attempt->get_start_time() < $date)
                    {
                        $date = $attempt->get_start_time();
                        $score = $attempt->get_total_score();
                    }
                }
                return $score;
            case self :: SCORE_TYPE_LAST :
                $date = null;
                foreach ($attempts as $attempt)
                {
                    if (is_null($score) || $attempt->get_start_time() > $date)
                    {
                        $date = $attempt->get_start_time();
                        $score = $attempt->get_total_score();
                    }
                }
                return $score;
            default :
                foreach ($attempts as $attempt)
                {
                    $score += $attempt->get_total_score();
                }
                return number_format($score / count($attempts), 1);
        }
    }

    public function retrieve_data()
    {
        return $this->count_data();
    }

    public function get_views()
    {
        return array(\reporting\viewer\HtmlBlockRendition :: VIEW_TABLE);
    }
}
