<?php

namespace application\weblcms;

use common\libraries\RecordTableCellRenderer;
use core\admin\AdminDataManager;
use application\weblcms\tool\course_group\CourseGroup;
use rights\RightsDataManager;
use user\UserDataManager;
use common\libraries\DatetimeUtilities;
use common\libraries\Theme;
use common\libraries\Translation;
use common\libraries\Utilities;
use common\libraries\ComplexContentObjectSupport;
use common\libraries\TableCellRendererActionsColumnSupport;

use repository\ContentObject;

use group\DataManager as GroupDataManager;
use application\weblcms\tool\course_group\DataManager as CourseGroupDataManager;

use group\Group;

use Exception;
/**
 * Cell renderer for the object publication table
 *
 * @package application.weblcms
 *
 * @author Original Author Unknown
 * @author Sven Vanpoucke - Hogeschool Gent - Refactoring to record table
 */
class ObjectPublicationTableCellRenderer extends RecordTableCellRenderer implements
    TableCellRendererActionsColumnSupport
{
    /****************************************************************************************************************
     * Inherited Functionality                                                                                      *
     ****************************************************************************************************************/

    /**
     * Renders a cell for a given object
     *
     * @param $column \common\libraries\ObjectTableColumn
     *
     * @param mixed $publication
     *
     * @return String
     */
    public function render_cell($column, $publication)
    {
        $content_object = $this->get_component()->get_content_object_from_publication($publication);

        switch ($column->get_name())
        {
            case ObjectPublicationTableColumnModel :: COLUMN_STATUS :
                $title = Translation :: get('TypeName', array(), $content_object->context());

                $icon_suffix = '';
                if ($publication[ContentObjectPublication :: PROPERTY_HIDDEN])
                {
                    $icon_suffix = '_na';
                    $title .= ' ' . Translation :: get('NotAvailable') . ')';
                }
                else
                {
                    $last_visit_date = $this->get_component()->get_tool_browser()->get_last_visit_date();
                    if ($publication[ContentObjectPublication :: PROPERTY_PUBLICATION_DATE] >= $last_visit_date)
                    {
                        $icon_suffix = '_new';
                        $title .= ' (' . Translation :: get('New') . ')';
                    }
                    else
                    {
                        $feedbacks = AdminDataManager :: get_instance()->retrieve_feedback_publications(
                            $publication[ContentObjectPublication :: PROPERTY_ID], null,
                            WeblcmsManager :: APPLICATION_NAME
                        );

                        while ($feedback = $feedbacks->next_result())
                        {
                            if ($feedback->get_modification_date() >= $last_visit_date)
                            {
                                $icon_suffix = '_new';
                                $title .= ' (' . Translation :: get('New') . ')';
                                break;
                            }
                        }
                    }
                }

                return '<img title="' . $title . '" src="' . str_replace(
                    '.png', $icon_suffix . '.png',
                    $content_object->get_icon_path(Theme :: ICON_MINI)
                )
                    . '" />';

                break;
            case ContentObject :: PROPERTY_TITLE :
                if ($content_object instanceof ComplexContentObjectSupport)
                {
                    $details_url = $this->get_component()->get_url(
                        array(
                            Tool :: PARAM_PUBLICATION_ID => $publication[ContentObjectPublication :: PROPERTY_ID],
                            Tool :: PARAM_ACTION => Tool :: ACTION_DISPLAY_COMPLEX_CONTENT_OBJECT
                        )
                    );

                    return '<a href="' . $details_url . '">' . parent :: render_cell(
                        $column, $publication
                    ) . '</a>';
                }

                $details_url = $this->get_component()->get_url(
                    array(
                        Tool :: PARAM_PUBLICATION_ID => $publication[ContentObjectPublication :: PROPERTY_ID],
                        Tool :: PARAM_ACTION => Tool :: ACTION_VIEW
                    )
                );

                return '<a href="' . $details_url . '">'
                    . parent :: render_cell($column, $publication) . '</a>';

                break;
            case ContentObjectPublication :: PROPERTY_PUBLICATION_DATE :
                $date_format = Translation :: get('DateTimeFormatLong', null, Utilities :: COMMON_LIBRARIES);
                $data = DatetimeUtilities :: format_locale_date(
                    $date_format,
                    $publication[ContentObjectPublication :: PROPERTY_PUBLICATION_DATE]
                );
                break;
            case ContentObjectPublication :: PROPERTY_PUBLISHER_ID :
                $user = $this->retrieve_user($publication[ContentObjectPublication :: PROPERTY_PUBLISHER_ID]);
                if (!$user)
                {
                    $data = '<i>' . Translation :: get('UserUnknown') . '</i>';
                }
                else
                {
                    $data = $user->get_fullname();
                }
                break;
            case 'published_for' :
                if ($publication[ContentObjectPublication :: PROPERTY_EMAIL_SENT])
                {
                    $email_icon = ' - <img src="' . Theme :: get_common_image_path() . 'action_email.png" alt=""
                        style="vertical-align: middle;" title="' . Translation :: get('SentByEmail') . '"/>';
                }
                $data = '<div style="float: left;">'
                    . $this->render_publication_targets($publication) . '</div>' . $email_icon;
                break;
            case ContentObject :: PROPERTY_DESCRIPTION :
                $data = $publication[ContentObject :: PROPERTY_DESCRIPTION];
                $data = Utilities :: truncate_string($data, 100);
        }

        if ($data)
        {
            if ($publication[ContentObjectPublication :: PROPERTY_HIDDEN])
            {
                return '<span style="color: gray">' . $data . '</span>';
            }
            else
            {
                return $data;
            }
        }

        return parent :: render_cell($column, $publication);
    }

    /**
     * Returns the actions toolbar
     *
     * @param mixed $publication
     *
     * @return string
     */
    public function get_actions($publication)
    {
        return $this->get_component()->get_publication_actions(
            $publication, $this->get_table()->get_column_model()->is_display_order_column()
        )->as_html();
    }

    /****************************************************************************************************************
     * Helper Functionality                                                                                         *
     ****************************************************************************************************************/

    /**
     * Renders the publication targets
     *
     * @param mixed$publication
     *
     * @return string
     */
    public function render_publication_targets($publication)
    {
        try{
            $target_entities = WeblcmsRights :: get_instance()->get_target_entities(
                WeblcmsRights :: VIEW_RIGHT, WeblcmsManager :: context(),
                $publication[ContentObjectPublication :: PROPERTY_ID], WeblcmsRights :: TYPE_PUBLICATION,
                $this->get_component()->get_tool_browser()->get_course_id(), WeblcmsRights :: TREE_TYPE_COURSE
            );
        }
        catch(Exception $exception)
        {
            error_log($exception->getMessage());
            $target_entities = array();
        }
        
        $rdm = RightsDataManager :: get_instance();

        $target_list = array();

        if (array_key_exists(0, $target_entities[0]))
        {
            $target_list[] = Translation :: get('Everybody', null, Utilities :: COMMON_LIBRARIES);
        }
        else
        {
            $target_list[] = '<select>';

            foreach ($target_entities as $entity_type => $entity_ids)
            {
                switch ($entity_type)
                {
                    case CoursePlatformGroupEntity :: ENTITY_TYPE :
                        foreach ($entity_ids as $group_id)
                        {
                            $group = GroupDataManager :: retrieve_by_id(Group :: class_name(), $group_id);
                            if ($group)
                            {
                                $target_list[] = '<option>' . $group->get_name() . '</option>';
                            }
                        }
                        break;
                    case CourseUserEntity :: ENTITY_TYPE :
                        foreach ($entity_ids as $user_id)
                        {
                            $user = $this->retrieve_user($user_id);
                            if ($user)
                            {
                                $target_list[] = '<option>' . $user->get_fullname() . '</option>';
                            }
                        }
                        break;
                    case CourseGroupEntity :: ENTITY_TYPE :
                        foreach ($entity_ids as $course_group_id)
                        {
                            $course_group = CourseGroupDataManager :: retrieve_by_id(
                                CourseGroup :: class_name(), $course_group_id
                            );

                            if ($course_group)
                            {
                                $target_list[] = '<option>' . $course_group->get_name() . '</option>';
                            }
                        }

                        break;
                    case TemplateEntity :: ENTITY_TYPE :
                        foreach ($entity_ids as $template_id)
                        {
                            $template = $rdm->retrieve_template($template_id);
                            if ($template)
                            {
                                $target_list[] = '<option>' . $template->get_name() . '</option>';
                            }
                        }
                        break;
                    case 0 :
                        $target_list[] = '<option>Everyone</option>';
                        break;
                }
            }
            $target_list[] = '</select>';
        }

        return implode("\n", $target_list);
    }

    /**
     * Retrieves a user by id
     *
     * @param int $user_id
     *
     * @return \user\User
     */
    public function retrieve_user($user_id)
    {
        return UserDataManager :: get_instance()->retrieve_user($user_id);
    }
}
