<?php

namespace application\weblcms;

use common\libraries\FormValidator;
use common\libraries\Utilities;
use common\libraries\DynamicFormTabsRenderer;
use common\libraries\DynamicFormTab;
use common\libraries\Translation;
use common\libraries\Theme;
use common\libraries\NotCondition;
use common\libraries\InCondition;
use common\libraries\EqualityCondition;
use common\libraries\SubselectCondition;
use common\libraries\AndCondition;
use common\libraries\FormDynamicContentMenu;
use common\libraries\FormDynamicContentMenuItem;
use common\libraries\Path;
use common\libraries\Application;
use common\libraries\AdvancedElementFinderElementTypes;
use common\libraries\ResourceManager;
use common\libraries\DataClass;
use common\libraries\Session;
use common\libraries\AdvancedElementFinderElements;
use common\libraries\DynamicContentMenuItem;

use application\weblcms\WeblcmsManager;

use application\weblcms\Tool;
use application\weblcms\CourseTool;
use application\weblcms\CourseSetting;
use application\weblcms\CourseSection;

use rights\NewUserEntity;
use rights\NewPlatformGroupEntity;

/**
 * This class describes a form for the course object
 *
 * @package \application\weblcms
 *
 * @author Yannick & Tristan
 * @author Sven Vanpoucke - Hogeschool Gent - Refactoring
 */
abstract class CommonCourseForm extends FormValidator implements CourseSettingsXmlFormParserSupport
{
    const PROPERTY_LOCKED_PREFIX = 'locked_';

    /**
     * The name for this form (used for tabs / menus)
     *
     * @var String
     */
    private $form_name;
    
    /**
     * The base object (course / course type) for this form
     *
     * @var DataClass
     */
    private $base_object;
    
    /**
     * Caching of the entities for default values
     *
     * @var RightsEntity[]
     */
    private $entities;
    
    /*****************************************************************************************************************
     * Tabs                                                                                                          *
     ****************************************************************************************************************/
    
    const TAB_GENERAL = 'general';
    const TAB_SETTINGS = 'settings';
    const TAB_TOOLS = 'tools';
    const TAB_RIGHTS = 'rights';

    /*****************************************************************************************************************
     * Inherited Functionality                                                                                       *
     ****************************************************************************************************************/
    
    /**
     * Constructor
     *
     * @param String $action
     * @param DataClass $base_object
     */
    public function __construct($action, DataClass $base_object)
    {
        $this->form_name = Utilities :: get_classname_from_namespace(get_class($this), true);
        $this->base_object = $base_object;
        
        parent :: __construct($this->form_name, self :: FORM_METHOD_POST, $action);
        
        $this->create_tabs();
        
        $buttons = array();

        $buttons[] = $this->createElement(
            'style_submit_button',
            'submit',
            Translation :: get('Save', array(), Utilities :: COMMON_LIBRARIES),
            array('class' => 'positive')
        );

        $buttons[] = $this->createElement(
            'style_reset_button',
            'reset',
            Translation :: get('Reset', array(), Utilities :: COMMON_LIBRARIES),
            array('class' => 'normal empty')
        );

        $this->addGroup($buttons, 'buttons', null, '&nbsp;', false);
        
        $this->set_default_values();
        $this->add_checkbox_javascript();
    }

    /*****************************************************************************************************************
     * Defaults Functionality                                                                                        *
     ****************************************************************************************************************/
    
    /**
     * Sets the default values
     * If the selected base object identified than we use the values for that base object
     *
     * @param string[] $default_values
     */
    public function set_default_values()
    {
        $this->setDefaults($this->get_base_object_default_values($this->base_object));
        $this->set_default_values_for_rights_form();
        
        $this->setDefaults(
            $this->multi_dimensional_array_to_single_dimensional_array(
                CourseSettingsController :: get_instance()->convert_course_settings_to_values_array($this->base_object)
            )
        );
    }

    /**
     * Sets the default values for the rights form (based on the simple rights form)
     */
    public function set_default_values_for_rights_form()
    {
        $available_rights = $this->get_available_rights();
        
        if ($this->base_object->is_identified())
        {
            $location = $this->base_object->get_rights_location();
        }
        else
        {
            $location = $this->base_object->get_parent_rights_location();
        }
        
        if (!$location)
        {
            return;
        }
        
        $selected_entities = CourseManagementRights :: get_instance()->retrieve_rights_location_rights_for_location(
            $location, $available_rights
        );
        
        $selected_entities_per_right = array();
        while ($selected_entity = $selected_entities->next_result())
        {
            $selected_entities_per_right[$selected_entity->get_right_id()][] = $selected_entity;
        }
        
        foreach ($available_rights as $right_id)
        {
            $option_name = CourseManagementRights :: PARAM_RIGHT_OPTION . '[' . $right_id . ']';

            if (count($selected_entities_per_right[$right_id]) == 0)
            {
                $defaults[$option_name] = CourseManagementRights :: RIGHT_OPTION_NOBODY;
                continue;
            }

            if (count($selected_entities_per_right[$right_id]) == 1)
            {
                $selected_entity = $selected_entities_per_right[$right_id][0];

                if ($selected_entity->get_entity_type() == 0 && $selected_entity->get_entity_id() == 0)
                {
                    $defaults[$option_name] = CourseManagementRights :: RIGHT_OPTION_ALL;
                    continue;
                }
                
                if ($selected_entity->get_entity_type() == 1 && $selected_entity->get_entity_id() ==
                    Session :: get_user_id()
                )
                {
                    $defaults[$option_name] = CourseManagementRights :: RIGHT_OTPION_ME;
                    continue;
                }
            
            }
            
            $targets_name = CourseManagementRights :: PARAM_RIGHT_TARGETS . '[' . $right_id . ']';
            
            $defaults[$option_name] = CourseManagementRights :: RIGHT_OPTION_SELECT;
            
            $default_elements = new AdvancedElementFinderElements();
            
            foreach ($selected_entities_per_right[$right_id] as $selected_entity)
            {
                $entity = $this->entities[$selected_entity->get_entity_type()];
                if ($entity)
                {
                    $default_elements->add_element(
                        $entity->get_element_finder_element(
                            $selected_entity->get_entity_id()
                        )
                    );
                }
            }
            
            $element = $this->getElement($targets_name);

            if (!$element instanceof \HTML_QuickForm_Error)
            {
                $element->setDefaultValues($default_elements);
            }
        }
        
        $this->setDefaults($defaults);
    }

    /****************************************************************************************************************
     * Tabs Building Functionality                                            *
     ****************************************************************************************************************/
    
    /**
     * Creates the dynamic tabs for the form
     *
     * @param String $form_name
     */
    private function create_tabs()
    {
        $tabs_renderer = new DynamicFormTabsRenderer($this->form_name, $this);

        $tabs_renderer->add_tab(
            new DynamicFormTab(
                self :: TAB_GENERAL,
                Utilities :: underscores_to_camelcase(self :: TAB_GENERAL),
                null,
                'build_general_tab_form_elements'
            )
        );
        
        $tabs_renderer->add_tab(
            new DynamicFormTab(
                self :: TAB_SETTINGS,
                Utilities :: underscores_to_camelcase(self :: TAB_SETTINGS),
                null,
                'build_settings_tab_form_elements'
            )
        );
        
        $tabs_renderer->add_tab(
            new DynamicFormTab(
                self :: TAB_TOOLS,
                Utilities :: underscores_to_camelcase(self :: TAB_TOOLS),
                null,
                'build_tools_tab_form_elements'
            )
        );
        
        $tabs_renderer->add_tab(
            new DynamicFormTab(
                self :: TAB_RIGHTS,
                Utilities :: underscores_to_camelcase(self :: TAB_RIGHTS),
                null,
                'build_rights_tab_form_elements'
            )
        );
        
        $tabs_renderer->render();
    }

    /****************************************************************************************************************
     * Tabs Functionality                                                *
     ****************************************************************************************************************/
    
    /**
     * Builds the elements for the settings tab
     */
    public function build_settings_tab_form_elements()
    {
        $this->add_settings_from_xml(
            Path :: get_application_path() . 'weblcms/php/settings/course_settings.xml',
            Application :: determine_namespace(WeblcmsManager :: APPLICATION_NAME),
            new \application\weblcms\CourseSettingsConnector(),
            CourseSettingsController :: SETTING_PARAM_COURSE_SETTINGS
        );
    }

    /**
     * Builds the elements for the tools tab
     */
    public function build_tools_tab_form_elements()
    {
        $dynamic_content_menu = new FormDynamicContentMenu($this->form_name);
        
        $dynamic_content_menu->add_menu_item(
            new FormDynamicContentMenuItem(
                self :: TAB_GENERAL,
                Utilities :: underscores_to_camelcase(self :: TAB_GENERAL),
                null,
                'build_tools_tab_general_content_form_elements',
                true
            )
        );
        
        $settings_conditions = array();
        
        $settings_conditions[] = new EqualityCondition(
            CourseSetting :: PROPERTY_GLOBAL_SETTING,
            0,
            CourseSetting :: get_table_name()
        );

        $settings_conditions[] = new InCondition(
            CourseSetting :: PROPERTY_NAME,
            CourseSetting :: get_static_tool_settings(),
            CourseSetting :: get_table_name()
        );

        $settings_condition = new NotCondition(new AndCondition($settings_conditions));
        
        $tools_condition = new SubselectCondition(
            CourseTool :: PROPERTY_ID,
            CourseSetting :: PROPERTY_TOOL_ID,
            CourseSetting :: get_table_name(),
            $settings_condition
        );

        $tools = DataManager :: retrieves(CourseTool :: class_name(), $tools_condition);
        
        while ($tool = $tools->next_result())
        {
            $tool_namespace = Tool :: get_tool_type_namespace($tool->get_name());
            $tool_title = Translation :: get('TypeName', null, $tool_namespace);
            
            $dynamic_content_menu->add_menu_item(
                new FormDynamicContentMenuItem(
                    $tool->get_name() . '-' . $tool->get_id(),
                    $tool_title,
                    null,
                    'build_tools_tab_detail_content_form_elements'
                )
            );
        }
        
        $dynamic_content_menu->add_to_form($this);
    }

    /**
     * Builds the elements for the rights tab
     */
    public function build_rights_tab_form_elements()
    {
        $available_rights = $this->get_available_rights();

        $this->addElement('html', '<div class="specific_rights_selector_box">');
        
        $rights_form_added = false;
        
        foreach ($available_rights as $right_name => $right_id)
        {
            if ($this->base_object->can_change_course_management_right($right_id))
            {
                $this->build_right_form($right_name, $right_id);
                $rights_form_added = true;
            }
        }
        
        if (!$rights_form_added)
        {
            $this->addElement(
                'html', '<div class="normal-message">' .
                Translation :: get('NoRightsAvailable') . '</div>'
            );
        }
        
        $this->addElement('html', '</div>');
        $this->addElement(
            'html', ResourceManager :: get_instance()->get_resource_html(
                Path :: get_application_web_path() . 'weblcms/resources/javascript/rights_form.js'
            )
        );
    }

    /****************************************************************************************************************
     * Tool Tab - Menu Content Functionality                                        *
     ****************************************************************************************************************/
    
    /**
     * Builds the elements for the tools tab - general menu item
     *
     * @param DynamicContentMenuItem $menu_item
     */
    public function build_tools_tab_general_content_form_elements(DynamicContentMenuItem $menu_item)
    {
        $locked_settings_supported = ($this instanceof FormLockedSettingsSupport);

        $table_header = array();
        $table_header[] = '<table class="data_table">';
        $table_header[] = '<thead>';
        $table_header[] = '<tr>';
        $table_header[] = '<th class="checkbox"></th>';
        $table_header[] = '<th>' . Translation :: get('Tool') . '</th>';
        $table_header[] = '<th class="checkbox">' . Translation :: get('IsToolAvailable') . '</th>';
        
        if ($locked_settings_supported)
        {
            $table_header[] = '<th class="checkbox">' . Translation :: get('ToolAvailableLocked') . '</th>';
        }
        
        $table_header[] = '<th class="checkbox">' . Translation :: get('IsToolVisible') . '</th>';
        
        if ($locked_settings_supported)
        {
            $table_header[] = '<th class="checkbox">' . Translation :: get('ToolVisibleLocked') . '</th>';
        }
        
        $table_header[] = '</tr>';
        $table_header[] = '</thead>';
        $table_header[] = '<tbody>';
        
        $this->addElement('html', implode("\n", $table_header));
        
        $tools_condition = new NotCondition(
            new EqualityCondition(
                CourseTool :: PROPERTY_SECTION_TYPE,
                CourseSection :: TYPE_CUSTOM
            )
        );

        $tools = DataManager :: retrieves(CourseTool :: class_name(), $tools_condition);
        
        while ($tool = $tools->next_result())
        {
            $tool_name = $tool->get_name();
            
            // Filter out the course settings so this can not be disabled (would cause a deadlock otherwise)
            if ($tool_name == 'course_settings')
            {
                continue;
            }
            
            $tool_namespace = Tool :: get_tool_type_namespace($tool_name);
            $tool_title = Translation :: get('TypeName', null, $tool_namespace);
            $tool_image_src = Theme :: get_image_path($tool_namespace) . '/logo/' . Theme :: ICON_MINI . '.png';
            $tool_image = $tool_name . "_image";
            
            $table_body = array();
            $table_body[] = '<tr>';
            $table_body[] = '<td class="center"><img class="' . $tool_image . '" width="16px" height="16px" src="';
            $table_body[] = $tool_image_src . '" style="vertical-align: middle;" alt="' . $tool_title . '"/></td>';
            
            $table_body[] = '<td>' . $tool_title . '</td>';
            
            $this->addElement('html', implode("\n", $table_body));
            
            $tool_element_name = CourseSettingsController :: SETTING_PARAM_TOOL_SETTINGS . '[' . $tool_name . ']';
            $active_element_name = $tool_element_name . '[' . CourseSetting :: COURSE_SETTING_TOOL_ACTIVE . ']';
            $visible_element_name = $tool_element_name . '[' . CourseSetting :: COURSE_SETTING_TOOL_VISIBLE . ']';
            $locked_prefix = CourseSettingsController :: SETTING_PARAM_LOCKED_PREFIX;
            
            $this->addElement('html', '<td><div style="margin: 0 auto; width: 75px;">');
            $active_element = $this->addElement(
                'checkbox', $active_element_name, Translation :: get('Active'), '',
                null, '1', '0'
            );
            
            if (!$this->can_change_course_setting(CourseSetting :: COURSE_SETTING_TOOL_ACTIVE, $tool->get_id()))
            {
                $active_element->freeze();
            }
            
            $this->addElement('html', '</div></td>');
            
            if ($locked_settings_supported)
            {
                $this->addElement('html', '<td><div style="margin: 0 auto; width: 38px;">');
                $this->addElement(
                    'checkbox', $locked_prefix . $active_element_name,
                    '', '', array('class' => 'no-iphone-style'), '1', '0'
                );
                $this->addElement('html', '</div></td>');
            }
            
            $this->addElement('html', '<td><div style="margin: 0 auto; width: 75px;">');
            $visible_element = $this->addElement(
                'checkbox', $visible_element_name, Translation :: get('Visible'), '',
                null, '1', '0'
            );

            if (!$this->can_change_course_setting(CourseSetting :: COURSE_SETTING_TOOL_VISIBLE, $tool->get_id()))
            {
                $visible_element->freeze();
            }
            
            $this->addElement('html', '</div></td>');
            
            if ($locked_settings_supported)
            {
                $this->addElement('html', '<td><div style="margin: 0 auto; width: 38px;">');
                $this->addElement(
                    'checkbox', $locked_prefix . $visible_element_name,
                    '', '', array('class' => 'no-iphone-style'), '1', '0'
                );
                $this->addElement('html', '</div></td>');
            }
            
            $this->addElement('html', '</tr>');
            
            $this->get_renderer()->setElementTemplate('{element}', $active_element_name);
            $this->get_renderer()->setElementTemplate('{element}', $visible_element_name);
        }
        
        $table_footer = array();
        $table_footer[] = '</tbody>';
        $table_footer[] = '</table>';
        
        $this->addElement('html', implode("\n", $table_footer));
    }

    /**
     * Builds the elements for the tools tab - specific menu item (settings form for tool)
     *
     * @param DynamicContentMenuItem $menu_item
     */
    public function build_tools_tab_detail_content_form_elements(DynamicContentMenuItem $menu_item)
    {
        $menu_item_id = $menu_item->get_id();
        
        $menu_item_id = explode('-', $menu_item_id);
        $tool = $menu_item_id[0];
        $tool_id = $menu_item_id[1];
        
        $tool_namespace = Tool :: get_tool_type_namespace($tool);
        $tool_path = Path :: namespace_to_full_path($tool_namespace);
        $settings_xml_path = $tool_path . 'php/settings/course_settings.xml';
        
        $this->add_settings_from_xml(
            $settings_xml_path, $tool_namespace, null,
            CourseSettingsController :: SETTING_PARAM_TOOL_SETTINGS . '[' . $tool . ']', $tool_id
        );
    }

    /****************************************************************************************************************
     * Single Right Form Builder                                                *
     ****************************************************************************************************************/
    
    /**
     * Builds the form for a given right
     *
     * @param String $right_name
     * @param int $right_id
     */
    private function build_right_form($right_name, $right_id)
    {
        $name = CourseManagementRights :: PARAM_RIGHT_OPTION . '[' . $right_id . ']';
        $targets_name = CourseManagementRights :: PARAM_RIGHT_TARGETS . '[' . $right_id . ']';
        $locked_name = CourseManagementRights :: PARAM_RIGHT_LOCKED . '[' . $right_id . ']';
        
        $this->addElement('category', $right_name);
        $this->addElement('html', '<div class="right">');
        
        $group = array();
        
        $group[] = & $this->createElement(
            'radio', null, null, Translation :: get('Nobody'),
            CourseManagementRights :: RIGHT_OPTION_NOBODY, array('class' => 'other_option_selected')
        );
        $group[] = & $this->createElement(
            'radio', null, null, Translation :: get('Everyone'),
            CourseManagementRights :: RIGHT_OPTION_ALL, array('class' => 'other_option_selected')
        );
        $group[] = & $this->createElement(
            'radio', null, null, Translation :: get('OnlyForMe'),
            CourseManagementRights :: RIGHT_OTPION_ME, array('class' => 'other_option_selected')
        );
        $group[] = & $this->createElement(
            'radio', null, null, Translation :: get('SelectSpecificEntities'),
            CourseManagementRights :: RIGHT_OPTION_SELECT, array('class' => 'entity_option_selected')
        );
        
        $this->addGroup($group, $name, Translation :: get('Target'), '<br />');
        
        // Add the advanced element finder
        $types = new AdvancedElementFinderElementTypes();
        
        $entities = array();
        $entities[NewUserEntity :: ENTITY_TYPE] = new NewUserEntity();
        $entities[NewPlatformGroupEntity :: ENTITY_TYPE] = new NewPlatformGroupEntity();
        
        foreach ($entities as $entity)
        {
            $types->add_element_type($entity->get_element_finder_type());
        }
        
        $this->entities = $entities;
        
        $this->addElement('html', '<div style="margin-left:25px; display:none;" class="entity_selector_box">');
        $this->addElement('advanced_element_finder', $targets_name, null, $types);
        
        $this->addElement('html', '</div></div>');
        
        $defaults = array();
        $defaults[$name] = CourseManagementRights :: RIGHT_OPTION_NOBODY;
        
        if ($this instanceof FormLockedSettingsSupport)
        {
            $this->addElement(
                'checkbox', $locked_name, Translation :: get('Locked'), '',
                array('class' => 'no-iphone-style')
            );
            
            if (CourseManagementRights :: get_instance()->is_right_locked_for_base_object(
                $this->base_object,
                $right_id
            )
            )
            {
                $defaults[$locked_name] = 1;
            }
        }
        
        $this->addElement('category');
        
        $this->setDefaults($defaults);
    }

    /****************************************************************************************************************
     * Form Generation Helper Functionality                                         *
     ****************************************************************************************************************/

    /**
     * Checks whether or not the base object can change a course setting
     *
     * @param string $name
     * @param int $tool_id
     *
     * @return boolean
     */
    public function can_change_course_setting($name, $tool_id = 0)
    {
        $course_settings_controller = CourseSettingsController :: get_instance();
        
        $course_setting = $course_settings_controller->get_course_setting_object_from_name_and_tool($name, $tool_id);

        if (!$course_setting)
        {
            return false;
        }
        
        return $this->base_object->can_change_course_setting($course_setting);
    }

    /****************************************************************************************************************
     * Helper Functionality                                                *
     ****************************************************************************************************************/
    
    /**
     * Adds xml defined settings to the form
     *
     * @param String $xml_file_path
     * @param String $context - [OPTIONAL] The context - Default common\libraries
     * @param Object $connector_class - [OPTIONAL] The connector class to retrieve the dynamic options
     * @param String $prefix - [OPTIONAL] The prefix for the elements
     */
    protected function add_settings_from_xml(
        $xml_file_path, $context = null, $connector_class = null, $prefix = null,
        $tool_id = 0
    )
    {
        $xml_form_parser = new CourseSettingsXmlFormParser($this, $tool_id);
        $xml_form_parser_result = $xml_form_parser->build_elements($xml_file_path, $context, $connector_class, $prefix);
        $xml_form_parser_result->add_result_to_form($this);
    }

    /**
     * Returns the available rights for this form
     *
     * @return string[]
     */
    protected function get_available_rights()
    {
        return $this->base_object->get_available_course_management_rights();
    }

    /****************************************************************************************************************
     * Getters and setters                                                *
     ****************************************************************************************************************/
    
    /**
     * Returns the base object for this form
     *
     * @return DataClass
     */
    public function get_base_object()
    {
        return $this->base_object;
    }

    /**
     * Sets the base object for this form
     *
     * @param DataClass $base_object
     */
    public function set_base_object(DataClass $base_object)
    {
        $this->base_object = $base_object;
    }

    /****************************************************************************************************************
     * Abstract Functionality                                                *
     ****************************************************************************************************************/
    
    /**
     * Builds the elements for the general tab
     */
    abstract public function build_general_tab_form_elements();

    /**
     * Returns the defaults for the selected base object (course_type)
     *
     * @param DataClass $base_object
     *
     * @return string[]
     */
    abstract public function get_base_object_default_values(DataClass $base_object);
}
