<?php
namespace application\weblcms\tool\assessment;

use application\weblcms\ContentObjectPublication;
use application\weblcms\integration\tracking\AssessmentAttempt;
use application\weblcms\integration\tracking\QuestionAttempt;
use application\weblcms\WeblcmsRights;
use common\libraries\AndCondition;
use common\libraries\Breadcrumb;
use common\libraries\BreadcrumbTrail;
use common\libraries\Display;
use common\libraries\EqualityCondition;
use common\libraries\NotAllowedException;
use common\libraries\PropertyConditionVariable;
use common\libraries\Request;
use common\libraries\StaticConditionVariable;
use repository\ComplexDisplay;
use application\weblcms\integration\tracking\DataManager as WeblcmsTrackingDataManager;
use common\libraries\Translation;
use common\libraries\DataClassRetrieveParameters;

/**
 * This class displays the result of a single attempt
 *
 * @package application.weblcms.tool.assessmet
 * @author Sven Vanpoucke
 */
class AssessmentToolAttemptResultViewerComponent extends AssessmentTool
{
    const PARAM_SHOW_FULL = 'show_full';

    /**
     * The assessment object
     *
     * @var Assessment
     */
    private $assessment;

    /**
     * The assessment attempt
     *
     * @var AssessmentAttempt
     */
    private $assessment_attempt;

    /**
     * The content object publication
     *
     * @var ContentObjectPublication
     */
    private $assessment_publication;

    /**
     * Runs this component
     *
     * @throws \common\libraries\NotAllowedException
     */
    public function run()
    {
        $assessment_attempt_id = Request :: get(AssessmentTool :: PARAM_USER_ASSESSMENT);

        $condition = new EqualityCondition(
            new PropertyConditionVariable(AssessmentAttempt :: class_name(), AssessmentAttempt :: PROPERTY_ID),
            new StaticConditionVariable($assessment_attempt_id));

        $this->assessment_attempt = WeblcmsTrackingDataManager :: retrieve(
            AssessmentAttempt :: class_name(),
            $condition);
        if (!$this->assessment_attempt)
        {
            $this->redirect(
                    Translation :: get("NotAllowed", null, Utilities :: COMMON_LIBRARIES),
                    true,
                    array(),
                    array(Tool :: PARAM_ACTION, Tool :: PARAM_PUBLICATION_ID));
        }
        $this->assessment_publication = \application\weblcms\DataManager :: retrieve(
            ContentObjectPublication :: class_name(),
            $this->assessment_attempt->get_assessment_id());

        $parameters = new DataClassRetrieveParameters(
            new EqualityCondition(
                new PropertyConditionVariable(Publication :: class_name(), Publication :: PROPERTY_PUBLICATION_ID),
                new StaticConditionVariable($this->assessment_publication->get_id())));
        $assessment_publication = DataManager :: retrieve(Publication :: class_name(), $parameters);

        if (! $this->is_allowed(WeblcmsRights :: VIEW_RIGHT, $this->assessment_publication))
        {
            throw new NotAllowedException();
        }
        elseif (! $this->is_allowed(WeblcmsRights :: EDIT_RIGHT))
        {
            if ($this->get_user_id() != $this->assessment_attempt->get_user_id())
            {
                throw new NotAllowedException();
            }
            elseif ($this->assessment_attempt->get_status() == AssessmentAttempt :: STATUS_NOT_COMPLETED)
            {
                throw new NotAllowedException();
            }
            elseif (! $assessment_publication->get_feedback_configuration()->get_feedback_summary() &&
                 ! $assessment_publication->get_feedback_configuration()->get_feedback_per_page())
            {
                throw new NotAllowedException();
            }
        }

        $assessment = $this->assessment_publication->get_content_object();
        $this->assessment = $assessment;

        $this->add_assessment_title_breadcrumb($assessment);

        Request :: set_get(ComplexDisplay :: PARAM_DISPLAY_ACTION, AssessmentTool :: ACTION_VIEW_RESULTS);

        ComplexDisplay :: launch($assessment->get_type(), $this);
    }

    /**
     * Displays the header, depending on the parameters
     */
    public function display_header()
    {
        if (Request :: get(self :: PARAM_SHOW_FULL))
        {
            parent :: display_header();
        }
        else
        {
            Display :: small_header();
        }

        if ($this->assessment_attempt->get_status() == AssessmentAttempt :: STATUS_NOT_COMPLETED)
        {
            echo '<div class="warning-message">' . Translation :: get('AttemptNotCompleted') . '</div>';
        }
    }

    /**
     * Displays the footer, depending on the parameters
     */
    public function display_footer()
    {
        if (Request :: get(self :: PARAM_SHOW_FULL))
        {
            return parent :: display_footer();
        }

        Display :: small_footer();
    }

    /**
     * Retrieves the assessment results
     *
     * @return array
     */
    public function retrieve_assessment_results()
    {
        $question_attempt_id = Request :: get(self :: PARAM_QUESTION_ATTEMPT);
        if ($question_attempt_id)
        {
            $condition = new EqualityCondition(
                new PropertyConditionVariable(QuestionAttempt :: class_name(), QuestionAttempt :: PROPERTY_ID),
                new StaticConditionVariable($question_attempt_id));
        }
        else
        {
            $condition = new EqualityCondition(
                new PropertyConditionVariable(
                    QuestionAttempt :: class_name(),
                    QuestionAttempt :: PROPERTY_ASSESSMENT_ATTEMPT_ID),
                new StaticConditionVariable($this->assessment_attempt->get_id()));
        }

        $question_attempts = WeblcmsTrackingDataManager :: retrieves(QuestionAttempt :: class_name(), $condition);

        $results = array();

        while ($question_attempt = $question_attempts->next_result())
        {
            $results[$question_attempt->get_question_cid()] = array(
                'answer' => $question_attempt->get_answer(),
                'feedback' => $question_attempt->get_feedback(),
                'score' => $question_attempt->get_score(),
                'hint' => $question_attempt->get_hint());
        }

        return $results;
    }

    /**
     * Changes the answer data
     *
     * @param int $question_cid
     * @param int $score
     * @param string $feedback
     */
    public function change_answer_data($question_cid, $score, $feedback)
    {
        $conditions = array();

        $conditions[] = new EqualityCondition(
            new PropertyConditionVariable(
                QuestionAttempt :: class_name(),
                QuestionAttempt :: PROPERTY_ASSESSMENT_ATTEMPT_ID),
            new StaticConditionVariable($this->assessment_attempt->get_id()));

        $conditions[] = new EqualityCondition(
            new PropertyConditionVariable(QuestionAttempt :: class_name(), QuestionAttempt :: PROPERTY_QUESTION_CID),
            new StaticConditionVariable($question_cid));

        $condition = new AndCondition($conditions);

        $question_attempt = WeblcmsTrackingDataManager :: retrieve(QuestionAttempt :: class_name(), $condition);

        $question_attempt->set_score($score);
        $question_attempt->set_feedback($feedback);
        $question_attempt->update();
    }

    /**
     * Changes the total score
     *
     * @param int $total_score
     */
    public function change_total_score($total_score)
    {
        $this->assessment_attempt->set_total_score($total_score);
        $this->assessment_attempt->update();
    }

    /**
     * Returns whether or not the answer data can be changed
     *
     * @return bool
     */
    public function can_change_answer_data()
    {
        if (Request :: get(self :: PARAM_SHOW_FULL))
        {
            return $this->assessment_attempt->get_status() == AssessmentAttempt :: STATUS_COMPLETED &&
                 $this->is_allowed(WeblcmsRights :: EDIT_RIGHT);
        }

        return false;
    }

    /**
     * Returns the root content object
     *
     * @return Assessment
     */
    public function get_root_content_object()
    {
        return $this->assessment;
    }

    /**
     * Returns the assessment parameters
     *
     * @return array
     */
    public function get_assessment_parameters()
    {
        return array();
    }

    /**
     * Returns the parameters for automatic registration
     *
     * @return array
     */
    public function get_additional_parameters()
    {
        return array(
            AssessmentTool :: PARAM_ASSESSMENT,
            AssessmentTool :: PARAM_USER_ASSESSMENT,
            self :: PARAM_SHOW_FULL);
    }

    /**
     * Add a breadcrumb with the title of the assessment
     *
     * @param $assessment Assessment
     */
    protected function add_assessment_title_breadcrumb($assessment)
    {
        $breadcrumb_trail = BreadcrumbTrail :: get_instance();
        $breadcrumbs = $breadcrumb_trail->get_breadcrumbs();

        $breadcrumbs[$breadcrumb_trail->size() - 1] = new Breadcrumb(
            $this->get_url(
                array(AssessmentTool :: PARAM_ACTION => AssessmentTool :: ACTION_VIEW_RESULTS),
                array(AssessmentTool :: PARAM_USER_ASSESSMENT, self :: PARAM_SHOW_FULL)),
            Translation :: get('ViewResultsForAssessment', array('TITLE' => $assessment->get_title())));

        $breadcrumb_trail->set_breadcrumbtrail($breadcrumbs);

        $user_fullname = \user\DataManager :: get_fullname_from_user($this->assessment_attempt->get_user_id());

        $breadcrumb_trail->add(
            new Breadcrumb($this->get_url(), Translation :: get('ViewResultsForUser', array('USER' => $user_fullname))));
    }

    public function get_assessment_feedback_configuration($complex_content_object_item)
    {
        $parameters = new DataClassRetrieveParameters(
            new EqualityCondition(
                new PropertyConditionVariable(Publication :: class_name(), Publication :: PROPERTY_PUBLICATION_ID),
                new StaticConditionVariable($this->assessment_publication->get_id())));
        $assessment_publication = DataManager :: retrieve(Publication :: class_name(), $parameters);

        return $assessment_publication->get_feedback_configuration();
    }
}
