<?php
namespace application\weblcms\tool\assessment;

use application\weblcms\ContentObjectPublication;
use application\weblcms\Tool;
use application\weblcms\WeblcmsRights;
use application\weblcms\integration\tracking\AssessmentAttempt;
use common\libraries\ActionBarRenderer;
use common\libraries\AndCondition;
use common\libraries\Breadcrumb;
use common\libraries\BreadcrumbTrail;
use common\libraries\EqualityCondition;
use common\libraries\NotAllowedException;
use common\libraries\PropertyConditionVariable;
use common\libraries\Request;
use common\libraries\StaticConditionVariable;
use common\libraries\TableSupport;
use common\libraries\Theme;
use common\libraries\ToolbarItem;
use common\libraries\Translation;
use common\libraries\Utilities;
use repository\content_object\assessment\Assessment;
use user\User;

/**
 * Views the results of an assessment
 *
 * @author Sven Vanpoucke - Hogeschool Gent
 */
class AssessmentToolResultsViewerComponent extends AssessmentTool implements TableSupport
{

    /**
     * The action bar
     *
     * @var ActionBarRenderer
     */
    private $action_bar;

    private $publication;

    /**
     * Runs this component
     *
     * @throws \common\libraries\NotAllowedException
     */
    public function run()
    {
        $pid = Request :: get(AssessmentTool :: PARAM_ASSESSMENT);

        $this->publication = \application\weblcms\DataManager :: retrieve(
            ContentObjectPublication :: class_name(),
            $pid);

        $assessment = $this->publication->get_content_object();

        $this->add_assessment_title_breadcrumb($assessment);

        if (! $this->is_allowed(WeblcmsRights :: VIEW_RIGHT, $this->publication))
        {
            throw new NotAllowedException();
        }

        $this->display_header();

        $this->action_bar = $this->get_toolbar();
        if ($this->action_bar)
        {
            echo $this->action_bar->as_html();
        }

        echo '<div class="content_object" style="background-image: url(' .
             Theme :: get_image_path(Assessment :: context()) . 'logo/22.png);">';
        echo '<div class="title">';
        echo $assessment->get_title();
        echo '</div>';
        echo $assessment->get_description();
        echo '</div>';

        $table = new AssessmentAttemptTable($this);

        echo $table->as_html();

        $this->display_footer();
    }

    /**
     * Returns the action bar
     *
     * @return ActionBarRenderer
     */
    public function get_toolbar()
    {
        if ($this->is_allowed(WeblcmsRights :: EDIT_RIGHT))
        {
            $action_bar = new ActionBarRenderer(ActionBarRenderer :: TYPE_HORIZONTAL);
            $action_bar->set_search_url($this->get_url());

            $action_bar->add_common_action(
                new ToolbarItem(
                    Translation :: get('ShowAll', null, Utilities :: COMMON_LIBRARIES),
                    Theme :: get_common_image_path() . 'action_browser.png',
                    $this->get_url(),
                    ToolbarItem :: DISPLAY_ICON_AND_LABEL));

            $aid = Request :: get(AssessmentTool :: PARAM_ASSESSMENT);
            $action_bar->add_common_action(
                new ToolbarItem(
                    Translation :: get('DownloadDocuments'),
                    Theme :: get_common_image_path() . 'action_download.png',
                    $this->get_url(
                        array(
                            Tool :: PARAM_ACTION => AssessmentTool :: ACTION_SAVE_DOCUMENTS,
                            AssessmentTool :: PARAM_ASSESSMENT => $aid)),
                    ToolbarItem :: DISPLAY_ICON_AND_LABEL));

            $action_bar->add_common_action(
                new ToolbarItem(
                    Translation :: get('DeleteAllResults'),
                    Theme :: get_common_image_path() . 'action_delete.png',
                    $this->get_url(
                        array(
                            Tool :: PARAM_ACTION => AssessmentTool :: ACTION_DELETE_RESULTS,
                            AssessmentTool :: PARAM_ASSESSMENT => $aid)),
                    ToolbarItem :: DISPLAY_ICON_AND_LABEL,
                    true));

            $action_bar->add_common_action(
                new ToolbarItem(
                    Translation :: get('RawExportResults'),
                    Theme :: get_common_image_path() . 'action_export.png',
                    $this->get_url(
                        array(
                            Tool :: PARAM_ACTION => AssessmentTool :: ACTION_RAW_EXPORT_RESULTS,
                            AssessmentTool :: PARAM_ASSESSMENT => Request :: get(AssessmentTool :: PARAM_ASSESSMENT))),
                    ToolbarItem :: DISPLAY_ICON_AND_LABEL));
        }
        return $action_bar;
    }

    /**
     * Add a breadcrumb with the title of the assessment
     *
     * @param $assessment Assessment
     */
    protected function add_assessment_title_breadcrumb($assessment)
    {
        $breadcrumb_trail = BreadcrumbTrail :: get_instance();
        $breadcrumbs = $breadcrumb_trail->get_breadcrumbs();

        $breadcrumbs[$breadcrumb_trail->size() - 1] = new Breadcrumb(
            $this->get_url(array(AssessmentTool :: PARAM_ACTION => AssessmentTool :: ACTION_VIEW_RESULTS)),
            Translation :: get('ViewResultsForAssessment', array('TITLE' => $assessment->get_title())));

        $breadcrumb_trail->set_breadcrumbtrail($breadcrumbs);
    }

    /**
     * Returns the additional parameters needed for registration
     *
     * @return array
     */
    public function get_additional_parameters()
    {
        return array(AssessmentTool :: PARAM_ASSESSMENT);
    }

    /**
     * Returns the condition
     *
     * @param string $table_class_name
     *
     * @return \common\libraries\Condition
     */
    public function get_table_condition($table_class_name)
    {
        $conditions = array();

        $conditions[] = new EqualityCondition(
            new PropertyConditionVariable(AssessmentAttempt :: class_name(), AssessmentAttempt :: PROPERTY_ASSESSMENT_ID),
            new StaticConditionVariable(Request :: get(self :: PARAM_ASSESSMENT)));

        if (! $this->is_allowed(WeblcmsRights :: EDIT_RIGHT))
        {
            $conditions[] = new EqualityCondition(
                new PropertyConditionVariable(AssessmentAttempt :: class_name(), AssessmentAttempt :: PROPERTY_USER_ID),
                new StaticConditionVariable($this->get_user_id()));
        }

        $action_bar = $this->action_bar;

        if ($this->action_bar)
        {
            $search_properties = array();

            $search_properties[] = new PropertyConditionVariable(User :: class_name(), User :: PROPERTY_FIRSTNAME);
            $search_properties[] = new PropertyConditionVariable(User :: class_name(), User :: PROPERTY_LASTNAME);
            $search_properties[] = new PropertyConditionVariable(User :: class_name(), User :: PROPERTY_OFFICIAL_CODE);

            $search_conditions = $action_bar->get_conditions($search_properties);
            if ($search_conditions)
            {
                $conditions[] = $search_conditions;
            }
        }

        return new AndCondition($conditions);
    }

    public function get_publication()
    {
        return $this->publication;
    }
}
