<?php
namespace application\weblcms\tool\assessment;

use common\libraries\StaticConditionVariable;
use common\libraries\PropertyConditionVariable;
use common\libraries\EqualityCondition;
use common\libraries\DataClassRetrieveParameters;
use common\libraries\FormValidator;
use repository\content_object\assessment\FeedbackDisplayConfiguration;
use common\libraries\Translation;
use application\weblcms\ContentObjectPublicationForm;
use common\libraries\ResourceManager;
use common\libraries\Path;

/**
 * Extension of the default Content to allow for the feedback-functionality
 *
 * @author Hans De Bisschop
 */
class PublicationForm extends ContentObjectPublicationForm
{

    public function setDefaults()
    {
        $defaults = array();
        $defaults[Publication :: PROPERTY_HINTS_ALLOWED] = 1;
        $defaults[Publication :: PROPERTY_FEEDBACK_SUMMARY] = 1;
        $defaults[Publication :: PROPERTY_FEEDBACK_PER_PAGE] = 0;
        $defaults[Publication :: PROPERTY_FEEDBACK_TYPE] = FeedbackDisplayConfiguration :: TYPE_BOTH;

        $publications = $this->get_publications();

        if (count($publications) == 1 && $this->get_form_type() == self :: TYPE_UPDATE)
        {
            $first_publication = $publications[0];

            $parameters = new DataClassRetrieveParameters(
                new EqualityCondition(
                    new PropertyConditionVariable(Publication :: class_name(), Publication :: PROPERTY_PUBLICATION_ID),
                    new StaticConditionVariable($first_publication->get_id())));
            $assessment_publication = DataManager :: retrieve(Publication :: class_name(), $parameters);

            $defaults[Publication :: PROPERTY_HINTS_ALLOWED] = $assessment_publication->get_hints_allowed();
            $defaults[Publication :: PROPERTY_FEEDBACK_SUMMARY] = $assessment_publication->get_feedback_summary();
            $defaults[Publication :: PROPERTY_FEEDBACK_PER_PAGE] = $assessment_publication->get_feedback_per_page();
            $defaults[Publication :: PROPERTY_FEEDBACK_TYPE] = $assessment_publication->get_feedback_type();
        }

        FormValidator :: setDefaults($defaults);
        parent :: setDefaults();
    }

    /**
     * Builds the basic create form (without buttons)
     */
    public function build_basic_create_form()
    {
        $this->addElement('category', Translation :: get('DefaultProperties'));
        parent :: build_basic_create_form();
        $this->addElement('category');

        $this->build_publication_properties_form();
    }

    /**
     * Builds the basic update form (without buttons)
     */
    public function build_basic_update_form()
    {
        $this->addElement('category', Translation :: get('DefaultProperties'));
        parent :: build_basic_update_form();
        $this->addElement('category');

        $this->build_publication_properties_form();
    }

    /**
     * Builds the form by adding the necessary form elements.
     */
    protected function build_publication_properties_form()
    {
        $this->addElement('category', Translation :: get('PublicationProperties'));
        $this->addElement('checkbox', Publication :: PROPERTY_HINTS_ALLOWED, Translation :: get('HintsAllowed'));
        $this->addElement('checkbox', Publication :: PROPERTY_FEEDBACK_SUMMARY, Translation :: get('FeedbackSummary'));
        $this->addElement('checkbox', Publication :: PROPERTY_FEEDBACK_PER_PAGE, Translation :: get('FeedbackPerPage'));

        $feedback_type_options = array();
        $feedback_type_options[FeedbackDisplayConfiguration :: TYPE_TEXT] = Translation :: get('TextFeedback');
        $feedback_type_options[FeedbackDisplayConfiguration :: TYPE_NUMERIC] = Translation :: get('NumericFeedback');
        $feedback_type_options[FeedbackDisplayConfiguration :: TYPE_BOTH] = Translation :: get('TextAndNumericFeedback');

        $this->addElement(
            'select',
            Publication :: PROPERTY_FEEDBACK_TYPE,
            Translation :: get('FeedbackType'),
            $feedback_type_options);
        $this->addElement('category');

        $this->addElement(
            'html',
            ResourceManager :: get_instance()->get_resource_html(
                Path :: namespace_to_full_path(__NAMESPACE__, true) . 'resources/javascript/publication_form.js'));
    }

    /**
     * Handles the submit of the form for both create and edit
     *
     * @return boolean
     */
    public function handle_form_submit()
    {
        $values = $this->exportValues();
        $success = parent :: handle_form_submit();

        $hints_allowed = isset($values[Publication :: PROPERTY_HINTS_ALLOWED]) ? 1 : 0;
        $feedback_summary = isset($values[Publication :: PROPERTY_FEEDBACK_SUMMARY]) ? 1 : 0;
        $feedback_per_page = isset($values[Publication :: PROPERTY_FEEDBACK_PER_PAGE]) ? 1 : 0;

        if ($success)
        {
            $publications = $this->get_publications();
            $succes = true;

            foreach ($publications as $publication)
            {
                switch ($this->get_form_type())
                {
                    case self :: TYPE_CREATE :
                        $assessment_publication = new Publication();
                        $assessment_publication->set_publication_id($publication->get_id());
                        $assessment_publication->set_hints_allowed($hints_allowed);
                        $assessment_publication->set_feedback_summary($feedback_summary);
                        $assessment_publication->set_feedback_per_page($feedback_per_page);
                        $assessment_publication->set_feedback_type($values[Publication :: PROPERTY_FEEDBACK_TYPE]);
                        $succes &= $assessment_publication->create();
                        break;
                    case self :: TYPE_UPDATE :
                        $parameters = new DataClassRetrieveParameters(
                            new EqualityCondition(
                                new PropertyConditionVariable(
                                    Publication :: class_name(),
                                    Publication :: PROPERTY_PUBLICATION_ID),
                                new StaticConditionVariable($publication->get_id())));
                        $assessment_publication = DataManager :: retrieve(Publication :: class_name(), $parameters);

                        $assessment_publication->set_hints_allowed($hints_allowed);
                        $assessment_publication->set_feedback_summary($feedback_summary);
                        $assessment_publication->set_feedback_per_page($feedback_per_page);
                        $assessment_publication->set_feedback_type($values[Publication :: PROPERTY_FEEDBACK_TYPE]);
                        $succes &= $assessment_publication->update();
                        break;
                }
            }
            return $succes;
        }
        else
        {
            return false;
        }
    }
}
