<?php
namespace application\weblcms\tool\assessment;

use application\weblcms\ContentObjectPublication;
use application\weblcms\Tool;
use application\weblcms\WeblcmsRights;
use application\weblcms\integration\tracking\AssessmentAttempt;
use common\libraries\DataClassPropertyTableColumn;
use common\libraries\DatetimeUtilities;
use common\libraries\RecordTableCellRenderer;
use common\libraries\TableCellRendererActionsColumnSupport;
use common\libraries\Theme;
use common\libraries\Toolbar;
use common\libraries\ToolbarItem;
use common\libraries\Translation;
use repository\content_object\hotpotatoes\Hotpotatoes;
use common\libraries\DataClassRetrieveParameters;
use common\libraries\EqualityCondition;
use common\libraries\PropertyConditionVariable;
use common\libraries\StaticConditionVariable;

/**
 * This class is a cell renderer for the attempts of an assessment
 *
 * @author Sven Vanpoucke - Hogeschool Gent
 */
class AssessmentAttemptTableCellRenderer extends RecordTableCellRenderer implements
    TableCellRendererActionsColumnSupport
{

    /**
     * **************************************************************************************************************
     * Inherited Functionality *
     * **************************************************************************************************************
     */

    /**
     * Renders a cell for a given object
     *
     * @param $column \common\libraries\ObjectTableColumn
     *
     * @param mixed $assessment_attempt
     *
     * @return String
     */
    public function render_cell($column, $assessment_attempt)
    {
        if ($column instanceof DataClassPropertyTableColumn)
        {
            switch ($column->get_class_name())
            {
                case AssessmentAttempt :: class_name() :
                    {
                        switch ($column->get_name())
                        {
                            case AssessmentAttempt :: PROPERTY_START_TIME :
                                return DatetimeUtilities :: format_locale_date(
                                    null,
                                    $assessment_attempt[AssessmentAttempt :: PROPERTY_START_TIME]);
                            case AssessmentAttempt :: PROPERTY_END_TIME :
                                if ($assessment_attempt[AssessmentAttempt :: PROPERTY_END_TIME])
                                {
                                    return DatetimeUtilities :: format_locale_date(
                                        null,
                                        $assessment_attempt[AssessmentAttempt :: PROPERTY_END_TIME]);
                                }
                                return null;
                            case AssessmentAttempt :: PROPERTY_TOTAL_TIME :
                                if ($assessment_attempt[AssessmentAttempt :: PROPERTY_STATUS] ==
                                     AssessmentAttempt :: STATUS_COMPLETED)
                                {
                                    return DatetimeUtilities :: convert_seconds_to_hours(
                                        $assessment_attempt[AssessmentAttempt :: PROPERTY_TOTAL_TIME]);
                                }
                                return null;
                            case AssessmentAttempt :: PROPERTY_TOTAL_SCORE :
                                if ($assessment_attempt[AssessmentAttempt :: PROPERTY_STATUS] ==
                                     AssessmentAttempt :: STATUS_COMPLETED)
                                {
                                    $total = $assessment_attempt[AssessmentAttempt :: PROPERTY_TOTAL_SCORE];
                                    return $total . '%';
                                }
                                return null;
                            case AssessmentAttempt :: PROPERTY_STATUS :
                                return $assessment_attempt[AssessmentAttempt :: PROPERTY_STATUS] ==
                                     AssessmentAttempt :: STATUS_COMPLETED ? Translation :: get(
                                        'Completed',
                                        null,
                                        'application\weblcms\integration\tracking') : Translation :: get(
                                        'NotCompleted',
                                        null,
                                        'application\weblcms\integration\tracking');
                        }
                    }
            }
        }

        return parent :: render_cell($column, $assessment_attempt);
    }

    /**
     * Returns the actions toolbar
     *
     * @param mixed $assessment_attempt
     *
     * @return String
     */
    public function get_actions($assessment_attempt)
    {
        $pub = \application\weblcms\DataManager :: retrieve(
            ContentObjectPublication :: class_name(),
            $assessment_attempt[AssessmentAttempt :: PROPERTY_ASSESSMENT_ID]);

        $assessment_attempt_status = $assessment_attempt[AssessmentAttempt :: PROPERTY_STATUS];
        $assessment_attempt_id = $assessment_attempt[AssessmentAttempt :: PROPERTY_ID];

        $assessment = $pub->get_content_object();

        $parameters = new DataClassRetrieveParameters(
            new EqualityCondition(
                new PropertyConditionVariable(Publication :: class_name(), Publication :: PROPERTY_PUBLICATION_ID),
                new StaticConditionVariable($pub->get_id())));
        $assessment_publication = DataManager :: retrieve(Publication :: class_name(), $parameters);

        $toolbar = new Toolbar();

        if ($assessment->get_type() != Hotpotatoes :: TYPE_HOTPOTATOES && (($assessment_attempt_status ==
             AssessmentAttempt :: STATUS_COMPLETED &&
             ($assessment_publication->get_feedback_configuration()->get_feedback_summary() ||
             $assessment_publication->get_feedback_configuration()->get_feedback_per_page())) ||
             $this->get_component()->is_allowed(WeblcmsRights :: EDIT_RIGHT)))
        {
            $toolbar->add_item(
                new ToolbarItem(
                    Translation :: get('ViewResults'),
                    Theme :: get_common_image_path() . 'action_browser.png',
                    $this->get_component()->get_url(
                        array(
                            Tool :: PARAM_ACTION => AssessmentTool :: ACTION_ATTEMPT_RESULT_VIEWER,
                            AssessmentTool :: PARAM_USER_ASSESSMENT => $assessment_attempt_id,
                            AssessmentToolAttemptResultViewerComponent :: PARAM_SHOW_FULL => 1)),
                    ToolbarItem :: DISPLAY_ICON));
        }
        else
        {
            $toolbar->add_item(
                new ToolbarItem(
                    Translation :: get('ViewResultsNA'),
                    Theme :: get_common_image_path() . 'action_browser_na.png',
                    null,
                    ToolbarItem :: DISPLAY_ICON));
        }

        if ($this->get_component()->is_allowed(WeblcmsRights :: DELETE_RIGHT))
        {
            $toolbar->add_item(
                new ToolbarItem(
                    Translation :: get('DeleteResult'),
                    Theme :: get_common_image_path() . 'action_delete.png',
                    $this->get_component()->get_url(
                        array(
                            Tool :: PARAM_ACTION => AssessmentTool :: ACTION_DELETE_RESULTS,
                            AssessmentTool :: PARAM_USER_ASSESSMENT => $assessment_attempt_id)),
                    ToolbarItem :: DISPLAY_ICON,
                    true));
        }

        return $toolbar->as_html();
    }
}
