<?php

namespace application\weblcms\tool\assignment;

use group\DataManager as GroupDataManager;
use group\Group;

use application\weblcms\tool\course_group\CourseGroup;
use application\weblcms\ContentObjectPublication;

use application\weblcms\WeblcmsManager;
use repository\content_object\description\Description;

use application\weblcms\WeblcmsRights;
use application\weblcms\Tool;
use common\libraries\Breadcrumb;
use common\libraries\Translation;
use common\libraries\EqualityCondition;
use common\libraries\Request;
use common\libraries\Utilities;

use repository\content_object\feedback\Feedback;
use tracking\Event;
use user\UserDataManager;

/**
 *
 * @package application.weblcms.tool.assignment.php.component
 *
 *          Component for the submission feedback form for assignment
 *          publications.
 *          Uses repoviewer
 *
 * @author Joris Willems <joris.willems@gmail.com>
 * @author Alexander Van Paemel
 * @author Bert De Clercq (Hogeschool Gent)
 */
class AssignmentToolSubmissionFeedbackComponent extends AssignmentTool implements \repository\viewer\ViewerInterface
{

    /**
     * ID of the submission
     *
     * @var int
     */
    private $submission_id;

    /**
     * The assignment content object
     *
     * @var Assignment
     */
    private $assignment;

    public function run()
    {
        $publication_id = Request :: get(AssignmentTool :: PARAM_PUBLICATION_ID);
        $target_id = Request :: get(AssignmentTool :: PARAM_TARGET_ID);
        $submitter_type = Request :: get(AssignmentTool :: PARAM_SUBMITTER_TYPE);

        // display form when content object created in RepoViewer
        if (\repository\viewer\Manager :: is_ready_to_be_published())
        {
            $this->create_tracker();
            $this->redirect(
                Translation :: get('FeedbackCreated'), false, array(
                    Tool :: PARAM_ACTION => AssignmentTool :: ACTION_VIEW_SUBMISSION,
                    Tool :: PARAM_PUBLICATION_ID => $publication_id, AssignmentTool :: PARAM_TARGET_ID => $target_id,
                    AssignmentTool :: PARAM_SUBMITTER_TYPE => $submitter_type,
                    AssignmentTool :: PARAM_SUBMISSION => $this->submission_id
                )
            );
        }

        // construct RepoViewer when no content object created yet
        if (! \repository\viewer\Manager :: is_ready_to_be_published())
        {
            $repository_viewer = \repository\viewer\Manager :: construct($this);
            $repository_viewer->set_maximum_select(\repository\viewer\Manager :: SELECT_SINGLE);
            $repository_viewer->set_parameter(AssignmentTool :: PARAM_SUBMISSION, $this->submission_id);
            $repository_viewer->run();
        }
    }

    /**
     * Creates a submission feedback tracker
     *
     * @param $values array
     *            exported values of the form
     */
    public function create_tracker()
    {
        $arguments = array(
            \application\weblcms\integration\tracking\SubmissionFeedback :: PROPERTY_SUBMISSION_ID => $this->submission_id,
            \application\weblcms\integration\tracking\SubmissionFeedback :: PROPERTY_CREATED => time(),
            \application\weblcms\integration\tracking\SubmissionFeedback :: PROPERTY_MODIFIED => time(),
            \application\weblcms\integration\tracking\SubmissionFeedback :: PROPERTY_USER_ID => $this->get_user_id(),
            \application\weblcms\integration\tracking\SubmissionFeedback :: PROPERTY_CONTENT_OBJECT_ID => $this->get_repo_object(
            )
        );
        Event :: trigger('feedback_submission', WeblcmsManager :: context(), $arguments);
    }

    public function add_additional_breadcrumbs(BreadcrumbTrail $breadcrumbtrail)
    {
        $publication_id = Request :: get(Tool :: PARAM_PUBLICATION_ID);
        $this->submission_id = Request :: get(AssignmentTool :: PARAM_SUBMISSION);

        $tracker = new \application\weblcms\integration\tracking\AssignmentSubmission();
        $condition =
            new EqualityCondition(\application\weblcms\integration\tracking\AssignmentSubmission :: PROPERTY_ID, $this->submission_id);
        $submission_trackers = $tracker->retrieve_tracker_items($condition);
        $submission_tracker = $submission_trackers[0];

        $submissions = \application\weblcms\integration\tracking\AssignmentSubmission :: get_data(
            \application\weblcms\integration\tracking\AssignmentSubmission :: CLASS_NAME,
            WeblcmsManager :: APPLICATION_NAME, $condition
        )->as_array();
        $submission = $submissions[0]->get_content_object();

        $pub = \application\weblcms\DataManager :: retrieve(
            ContentObjectPublication :: class_name(),
            $publication_id
        );

        if (! $this->is_allowed(WeblcmsRights :: EDIT_RIGHT, $pub))
        {
            $this->redirect(
                Translation :: get("NotAllowed", null, Utilities :: COMMON_LIBRARIES), true, array(), array(
                    Tool :: PARAM_ACTION, Tool :: PARAM_PUBLICATION_ID, AssignmentTool :: PARAM_TARGET_ID,
                    AssignmentTool :: PARAM_SUBMITTER_TYPE, AssignmentTool :: PARAM_SUBMISSION
                )
            );
        }
        $this->assignment = $pub->get_content_object();

        $breadcrumbtrail->add(
            new Breadcrumb(
                $this->get_url(
                    array(
                        Tool :: PARAM_ACTION => AssignmentTool :: ACTION_BROWSE_SUBMITTERS,
                        Tool :: PARAM_PUBLICATION_ID => $publication_id
                    )
                ), $this->assignment->get_title()
            )
        );
        $breadcrumbtrail->add(
            new Breadcrumb(
                $this->get_url(
                    array(
                        Tool :: PARAM_ACTION => AssignmentTool :: ACTION_BROWSE_SUBMISSIONS,
                        Tool :: PARAM_PUBLICATION_ID => $submission_tracker->get_publication_id(),
                        AssignmentTool :: PARAM_TARGET_ID => $submission_tracker->get_submitter_id(),
                        AssignmentTool :: PARAM_SUBMITTER_TYPE => $submission_tracker->get_submitter_type()
                    )
                ), $this->get_submitter_name($submission_tracker)
            )
        );
        $breadcrumbtrail->add(
            new Breadcrumb(
                $this->get_url(
                    array(
                        Tool :: PARAM_ACTION => AssignmentTool :: ACTION_VIEW_SUBMISSION,
                        Tool :: PARAM_PUBLICATION_ID => $submission_tracker->get_publication_id(),
                        AssignmentTool :: PARAM_TARGET_ID => $submission_tracker->get_submitter_id(),
                        AssignmentTool :: PARAM_SUBMITTER_TYPE => $submission_tracker->get_submitter_type(),
                        AssignmentTool :: PARAM_SUBMISSION => $this->submission_id
                    )
                ), $submission->get_title() . ' - ' . Translation :: get('Detail')
            )
        );
    }

    public function get_allowed_content_object_types()
    {
        $types = explode(',', $this->assignment->get_allowed_types());
        $types[] = Description :: CLASS_NAME;
        $types[] = Feedback :: class_name();
        return $types;
    }

    /**
     * Gets the selected object id of the RepoViewer
     *
     * @return int
     */
    public function get_repo_object()
    {
        return \repository\viewer\Manager :: get_selected_objects();
    }

    public function get_additional_parameters()
    {
        return array(
            Tool :: PARAM_PUBLICATION_ID, AssignmentTool :: PARAM_TARGET_ID,
            AssignmentTool :: PARAM_SUBMITTER_TYPE, AssignmentTool :: PARAM_SUBMISSION
        );
    }

    /**
     * Returns the name of the submitter as a string.
     * When submitted as a group, it will return the name of the user who
     * submitted followed by the group name.
     *
     * @return string The name of the submitter
     */
    private function get_submitter_name($submission_tracker)
    {
        // name of the user who submitted
        $user_name = UserDataManager :: get_instance()->retrieve_user($submission_tracker->get_user_id());

        switch ($submission_tracker->get_submitter_type())
        {
            case \application\weblcms\integration\tracking\AssignmentSubmission :: SUBMITTER_TYPE_USER :
                return $user_name->get_fullname();
            case \application\weblcms\integration\tracking\AssignmentSubmission :: SUBMITTER_TYPE_COURSE_GROUP :
                return $user_name->get_fullname() . ' - ' .
                    \application\weblcms\DataManager :: retrieve(
                        CourseGroup :: class_name(),
                        $submission_tracker->get_submitter_id()
                    )->get_name();
            case \application\weblcms\integration\tracking\AssignmentSubmission :: SUBMITTER_TYPE_PLATFORM_GROUP :
                return $user_name->get_fullname() . ' - ' .
                    GroupDataManager :: retrieve_by_id(Group :: class_name(), $submission_tracker->get_submitter_id())
                        ->get_name();
        }
    }
}
