<?php

namespace application\weblcms\tool\assignment;

use application\weblcms\ObjectPublicationTableCellRenderer;
use application\weblcms\Tool;
use application\weblcms\WeblcmsRights;
use common\libraries\Theme;
use common\libraries\EqualityCondition;
use application\weblcms\ContentObjectPublication;
use common\libraries\Utilities;
use common\libraries\DatetimeUtilities;
use common\libraries\Translation;
use repository\content_object\assignment\Assignment;
use repository\ContentObject;

/**
 * Extension on the content object publication table cell renderer for this tool
 *
 * @author Sven Vanpoucke - Hogeschool Gent
 */
class PublicationTableCellRenderer extends ObjectPublicationTableCellRenderer
{
    /****************************************************************************************************************
     * Inherited Functionality                                                                                      *
     ****************************************************************************************************************/

    /**
     * Renders a cell for a given object
     *
     * @param $column \common\libraries\ObjectTableColumn
     *
     * @param mixed $publication
     *
     * @return String
     */
    public function render_cell($column, $publication)
    {
        $content_object = $this->get_component()->get_content_object_from_publication($publication);

        switch ($column->get_name())
        {
            case ContentObject :: PROPERTY_TITLE :
                return $this->generate_title_link($publication);
            case Assignment :: PROPERTY_END_TIME :
                $time = $content_object->get_end_time();
                $date_format = Translation :: get('DateTimeFormatLong', null, Utilities :: COMMON_LIBRARIES);
                $time = DatetimeUtilities :: format_locale_date($date_format, $time);
                if ($publication[ContentObjectPublication :: PROPERTY_HIDDEN])
                {
                    return '<span style="color: gray">' . $time . '</span>';
                }
                return $time;
            case AssignmentTool :: PROPERTY_NUMBER_OF_SUBMISSIONS :
                $tracker = new \application\weblcms\integration\tracking\AssignmentSubmission();
                $condition = new EqualityCondition(
                    \application\weblcms\integration\tracking\AssignmentSubmission :: PROPERTY_PUBLICATION_ID,
                    $publication[ContentObjectPublication :: PROPERTY_ID]
                );
                return $tracker->count_tracker_items($condition);
            case Assignment :: PROPERTY_ALLOW_GROUP_SUBMISSIONS :
                if ($content_object->get_allow_group_submissions())
                {
                    return '<img src="' . Theme :: get_image_path() . 'type/group.png" alt="' .
                        Translation :: get('GroupAssignment') . '" title="' . Translation :: get('GroupAssignment') .
                        '"/>';
                }
                return '<img src="' . Theme :: get_image_path() . 'type/individual.png" alt="' .
                    Translation :: get('IndividualAssignment') . '" title="' .
                    Translation :: get('IndividualAssignment') . '"/>';
        }

        return parent :: render_cell($column, $publication);
    }

    /**
     * Generated the HTML for the title column, including link, depending on the
     * status of the current browsing user.
     *
     * @param $publication type
     *            The publication for which the title link is to be generated.
     * @return string The HTML for the link in the title column.
     */
    private function generate_title_link($publication)
    {
        if ($this->get_component()->is_allowed(WeblcmsRights :: EDIT_RIGHT))
        {
            return $this->generate_teacher_title_link($publication);
        }
        return $this->generate_student_title_link($publication);
    }

    /**
     * Generates the link applicable for the current browsing user being a
     * teacher or admin.
     *
     * @param $publication type
     *            The publication for which the link is being generated.
     * @return string The HTML anchor elemnt that represents the link.
     */
    private function generate_teacher_title_link($publication)
    {
        $url = $this->get_component()->get_url(
            array(
                Tool :: PARAM_PUBLICATION_ID => $publication[ContentObjectPublication :: PROPERTY_ID],
                Tool :: PARAM_ACTION => AssignmentTool :: ACTION_BROWSE_SUBMITTERS
            )
        );
        return '<a href="' . $url . '">' .
            Utilities :: truncate_string($publication[ContentObject :: PROPERTY_TITLE], 50) . '</a>';
    }

    /**
     * Generates the link applicable for the current browsing user being a
     * student.
     *
     * @param $publication type
     *            The publication for which the link is being generated.
     * @return string The HTML anchor element that represents the link.
     */
    private function generate_student_title_link($publication)
    {
        $url = $this->get_component()->get_url(
            array(
                Tool :: PARAM_ACTION => AssignmentTool :: ACTION_STUDENT_BROWSE_SUBMISSIONS,
                Tool :: PARAM_PUBLICATION_ID => $publication[ContentObjectPublication :: PROPERTY_ID]
            )
        );
        return '<a href="' . $url . '">' .
            Utilities :: truncate_string($publication[ContentObject :: PROPERTY_TITLE], 50) . '</a>';
    }
}