<?php

namespace application\weblcms\tool\learning_path;

use repository\ExportParameters;

use application\weblcms\Tool;


use application\weblcms\WeblcmsManager;
use application\weblcms\WeblcmsRights;
use common\libraries\AndCondition;
use common\libraries\EqualityCondition;
use common\libraries\InCondition;
use common\libraries\Request;
use common\libraries\Translation;
use common\libraries\Utilities;
use repository\ComplexBuilder;
use repository\ComplexContentObjectItem;
use repository\content_object\assessment_open_question\AssessmentOpenQuestion;
use repository\ContentObjectExport;
use repository\ContentObjectExportController;


/**
 * Provides functionality to download documents included with an assessment.
 *
 * @author Bert De Clercq (Hogeschool Gent)
 */
class LearningPathToolDocumentSaverComponent extends LearningPathTool
{

    /**
     * Launches this component.
     */
    public function run()
    {
        if (!$this->is_allowed(WeblcmsRights :: EDIT_RIGHT))
        {
            $this->redirect(
                Translation :: get('NotAllowed', null, Utilities :: COMMON_LIBRARIES), true, array(), array(
                    Tool :: PARAM_ACTION, Tool :: PARAM_PUBLICATION_ID
                )
            );
        }

        $this->retrieve_assessment_documents(
            Request :: get(LearningPathTool :: PARAM_ATTEMPT_ID),
            Request :: get(ComplexBuilder :: PARAM_SELECTED_COMPLEX_CONTENT_OBJECT_ITEM_ID),
            Request :: get(LearningPathTool :: PARAM_ASSESSMENT_ID)
        );
    }

    /**
     * Retrieve assessment documents
     *
     * @param $attempt_id int
     *            The attempt it for the learning path
     * @param $ccoi_id int
     *            The complex content object item id
     * @param $assessment_id int
     *            The assessment id
     */
    protected function retrieve_assessment_documents($attempt_id, $ccoi_id, $assessment_id)
    {
        $document_open_question_ids = $this->retrieve_document_open_question_ids($assessment_id);

        $conditions = array();
        $conditions[] = new EqualityCondition(\application\weblcms\integration\tracking\LpiAttempt :: PROPERTY_LP_ITEM_ID, $ccoi_id);
        $conditions[] = new EqualityCondition(\application\weblcms\integration\tracking\LpiAttempt :: PROPERTY_LP_VIEW_ID, $attempt_id);
        $condition = new AndCondition($conditions);

        $assessment_attempts = \application\weblcms\integration\tracking\LpiAttempt :: get_data(
            \application\weblcms\integration\tracking\LpiAttempt :: CLASS_NAME, WeblcmsManager :: APPLICATION_NAME, $condition
        )->as_array();

        $assessment_attempt_ids = array();

        foreach ($assessment_attempts as $assessment_attempt)
        {
            $assessment_attempt_ids[] = $assessment_attempt->get_id();
        }

        $this->retrieve_assessment_attempts_documents($document_open_question_ids, $assessment_attempt_ids);
    }

    /**
     * Downloads the documents from the open questions in the assessment
     * attempts.
     *
     * @param $document_open_question_ids array
     *            The ids of the open questions that can contain documents
     * @param $assessment_attempt_ids array
     *            The ids of the assessment attempts
     */
    protected function retrieve_assessment_attempts_documents($document_open_question_ids, $assessment_attempt_ids)
    {
        if (!is_array($assessment_attempt_ids))
        {
            $assessment_attempt_ids = array($assessment_attempt_ids);
        }

        if (count($document_open_question_ids) < 1)
        {
            $this->redirect_to_previous('NoOpenDocumentQuestions');
        }

        $conditions = array();

        $conditions[] = new InCondition(
            \application\weblcms\integration\tracking\LearningPathQuestionAttempt :: PROPERTY_QUESTION_CID,
            $document_open_question_ids
        );

        $conditions[] = new InCondition(
            \application\weblcms\integration\tracking\LearningPathQuestionAttempt :: PROPERTY_LPI_ATTEMPT_ID,
            $assessment_attempt_ids
        );

        $condition = new AndCondition($conditions);

        $question_attempt_trackers = \application\weblcms\integration\tracking\LearningPathQuestionAttempt :: get_data(
            \application\weblcms\integration\tracking\LearningPathQuestionAttempt :: CLASS_NAME, WeblcmsManager :: APPLICATION_NAME, $condition
        )->as_array();

        $document_ids = array();

        foreach ($question_attempt_trackers as $question_attempt_tracker)
        {
            $answer = unserialize($question_attempt_tracker->get_answer());
            if (!is_null($answer[2]) && strlen($answer[2]) > 0)
            {
                // Assign key to get ids without duplicates.
                $document_ids[$answer[2]] = $answer[2];
            }
        }

        if (count($document_ids) < 1)
        {
            $this->redirect_to_previous('NoDocumentsForAssessment');
        }

        $parameters = new ExportParameters($this->get_user_id(), ContentObjectExport :: FORMAT_ZIP, $document_ids);
        $exporter = ContentObjectExportController :: factory($parameters);
        $exporter->download();
    }

    /**
     * Returns the ids of the open questions that contain a document.
     *
     * @param $assessment_id int
     *            The assessment id
     *
     * @return array The ids of the open questions that can contain a document
     */
    protected function retrieve_document_open_question_ids($assessment_id)
    {
        $condition = new EqualityCondition(ComplexContentObjectItem :: PROPERTY_PARENT, $assessment_id);
        $complex_questions = \repository\DataManager :: retrieve_complex_content_object_items(
                ComplexContentObjectItem :: class_name(),
                $condition)->as_array();

        $open_document_question_ids = array();

        foreach ($complex_questions as $complex_question)
        {
            if ($complex_question->get_ref_object()->get_type() == AssessmentOpenQuestion :: class_name() &&
                $this->is_open_question_document_allowed($complex_question->get_ref_object())
            )
            {
                $open_document_question_ids[] = $complex_question->get_id();
            }
        }

        return $open_document_question_ids;
    }

    /**
     * Returns true if the open question can contain documents.
     *
     * @param $open_question AssessmentOpenQuestion
     *
     * @return boolean True if the open question can contain documents
     */
    protected function is_open_question_document_allowed($open_question)
    {
        switch ($open_question->get_question_type())
        {
            case AssessmentOpenQuestion :: TYPE_OPEN_WITH_DOCUMENT :
                return true;

            case AssessmentOpenQuestion :: TYPE_DOCUMENT :
                return true;
        }

        return false;
    }

    /**
     * Redirects to the previous page show the given message.
     *
     * @param $message string
     *            The message to display
     */
    protected function redirect_to_previous($message)
    {
        $params = array();

        $params[Tool :: PARAM_ACTION] = LearningPathTool :: ACTION_VIEW_ASSESSMENT_RESULTS;
        $params[Tool :: PARAM_PUBLICATION_ID] = Request :: get(Tool :: PARAM_PUBLICATION_ID);
        $params[WeblcmsManager :: PARAM_USERS] = Request :: get(WeblcmsManager :: PARAM_USERS);
        $params[LearningPathTool :: PARAM_ATTEMPT_ID] = Request :: get(LearningPathTool :: PARAM_ATTEMPT_ID);
        $params[ComplexBuilder :: PARAM_SELECTED_COMPLEX_CONTENT_OBJECT_ITEM_ID] =
            Request :: get(ComplexBuilder :: PARAM_SELECTED_COMPLEX_CONTENT_OBJECT_ITEM_ID);
        $params[LearningPathTool :: PARAM_ASSESSMENT_ID] = Request :: get(LearningPathTool :: PARAM_ASSESSMENT_ID);
        $params[LearningPathTool :: PARAM_LEARNING_PATH_ITEM_ATTEMPT_ID] =
            Request :: get(LearningPathTool :: PARAM_LEARNING_PATH_ITEM_ATTEMPT_ID);

        $this->redirect(Translation :: get($message), false, $params);
    }

}
