<?php
namespace application\weblcms\tool\user;

use application\weblcms\WeblcmsRights;
use common\libraries\TableCellRendererActionsColumnSupport;
use common\libraries\RecordTableCellRenderer;
use application\weblcms\WeblcmsManager;
use application\weblcms\Tool;
use common\libraries\Toolbar;
use common\libraries\ToolbarItem;
use user\User;
use common\libraries\PlatformSetting;
use common\libraries\Theme;
use common\libraries\Request;
use common\libraries\Translation;
use common\libraries\WebApplication;
use application\weblcms\course\CourseUserRelation;
use application\weblcms\CourseManagementRights;

/**
 * Cell renderer for a direct subscribed course user browser table, or users
 * in a direct subscribed group.
 *
 * @author Stijn Van Hoecke
 * @author Sven Vanpoucke - Hogeschool Gent - Refactoring to RecordTable
 */
class SubscribedUserBrowserTableCellRenderer extends RecordTableCellRenderer implements
    TableCellRendererActionsColumnSupport
{

    /****************************************************************************************************************
     * Inherited Functionality                                                                                      *
     ****************************************************************************************************************/

    /**
     * Renders a given cell.
     *
     * @param $column type
     * @param mixed $user_with_subscription_status
     *            User from the advanced join query in weblcms database class
     *            that includes his subscription status.
     *
     * @return string
     */
    public function render_cell($column, $user_with_subscription_status)
    {
        // Add special features here
        switch ($column->get_name())
        {
            // Exceptions that need post-processing go here ...
            case CourseUserRelation :: PROPERTY_STATUS :
                switch ($user_with_subscription_status[CourseUserRelation :: PROPERTY_STATUS])
                {
                    case CourseUserRelation :: STATUS_TEACHER :
                        return Translation :: get('CourseAdmin');
                    case CourseUserRelation :: STATUS_STUDENT :
                        return Translation :: get('Student');
                    default :
                        return Translation :: get('Unknown');
                }
            case User :: PROPERTY_PLATFORMADMIN :
                if ($user_with_subscription_status[User :: PROPERTY_PLATFORMADMIN] == '1')
                {
                    return Translation :: get('PlatformAdministrator');
                }
                else
                {
                    return '';
                }
            case User :: PROPERTY_EMAIL :
                $email = $user_with_subscription_status[User :: PROPERTY_EMAIL];

                if (PlatformSetting :: get('active_online_email_editor'))
                {
                    $parameters = array();
                    $parameters[Tool :: PARAM_ACTION] = UserTool :: ACTION_EMAIL;
                    $parameters[UserTool :: PARAM_OBJECTS] = $user_with_subscription_status[User :: PROPERTY_ID];
                    $email_url = $this->get_component()->get_url($parameters);
                }
                else
                {
                    $email_url = 'mailto:' . $email;
                }
                return '<a href="' . $email_url . '">' . $email . '</a>';
        }

        return parent :: render_cell($column, $user_with_subscription_status);
    }

    /**
     * Gets the action links to display
     *
     * @param mixed $user_with_subscription_status
     *            The user for which the
     *            action links should be returned
     * @return string A HTML representation of the action links
     */
    public function get_actions($user_with_subscription_status)
    {
        $user_id = $user_with_subscription_status[User :: PROPERTY_ID];

        // construct the toolbar
        $toolbar = new Toolbar(Toolbar :: TYPE_HORIZONTAL);

        // always show details
        $parameters = array();
        $parameters[Tool :: PARAM_ACTION] = UserTool :: ACTION_USER_DETAILS;
        $parameters[UserTool :: PARAM_TAB] = Request :: get(UserTool :: PARAM_TAB);
        $parameters[WeblcmsManager :: PARAM_USERS] = $user_id;
        $details_url = $this->get_component()->get_url($parameters);

        $toolbar->add_item(
            new ToolbarItem(
                Translation :: get('Details'),
                Theme :: get_common_image_path() . 'action_details.png',
                $details_url,
                ToolbarItem :: DISPLAY_ICON
            )
        );

        // display the actions to change the individual status and unsubscribe
        // if:
        // (1) the user is platform or course admin
        // AND
        // (2) the row is not the current user
        // AND
        // (3) we are not editing groups
        if ($this->get_component()->is_allowed(WeblcmsRights :: EDIT_RIGHT))
        {
            $group_id = Request :: get(WeblcmsManager :: PARAM_GROUP);
            if ($user_id != $this->get_component()->get_user()->get_id() &&
                !isset($group_id)
            )
            {
                if ($this->get_component()->get_user()->is_platform_admin() ||
                    CourseManagementRights :: get_instance()->is_allowed(
                        CourseManagementRights :: TEACHER_UNSUBSCRIBE_RIGHT,
                        $this->get_component()->get_course_id(),
                        CourseManagementRights :: TYPE_COURSE,
                        $user_id
                    )
                )

                {
                    $parameters = array();
                    $parameters[Tool :: PARAM_ACTION] = UserTool :: ACTION_UNSUBSCRIBE;
                    $parameters[UserTool :: PARAM_TAB] = Request :: get(UserTool :: PARAM_TAB);
                    $parameters[UserTool :: PARAM_OBJECTS] = $user_id;
                    $unsubscribe_url = $this->get_component()->get_url($parameters);

                    $toolbar->add_item(
                        new ToolbarItem(
                            Translation :: get('Unsubscribe'),
                            Theme :: get_common_image_path() . 'action_unsubscribe.png',
                            $unsubscribe_url,
                            ToolbarItem :: DISPLAY_ICON
                        )
                    );
                }
                else
                {
                    $toolbar->add_item(
                        new ToolbarItem(
                            Translation :: get('UnsubscribeNotAvailable'),
                            Theme :: get_common_image_path() . 'action_unsubscribe_na.png',
                            null,
                            ToolbarItem :: DISPLAY_ICON
                        )
                    );
                }

                $weblcms_manager_namespace = WebApplication :: determine_namespace(WeblcmsManager :: APPLICATION_NAME);

                switch ($user_with_subscription_status[CourseUserRelation :: PROPERTY_STATUS])
                {
                    case CourseUserRelation :: STATUS_TEACHER :
                        $status_change_url = $this->get_component()->get_status_changer_url(
                            $user_id, CourseUserRelation :: STATUS_STUDENT
                        );

                        $toolbar->add_item(
                            new ToolbarItem(
                                Translation :: get('MakeStudent'),
                                Theme :: get_image_path($weblcms_manager_namespace) . 'action_subscribe_student.png',
                                $status_change_url,
                                ToolbarItem :: DISPLAY_ICON
                            )
                        );
                        break;
                    case CourseUserRelation :: STATUS_STUDENT :
                        $status_change_url = $this->get_component()->get_status_changer_url(
                            $user_id, CourseUserRelation :: STATUS_TEACHER
                        );

                        $toolbar->add_item(
                            new ToolbarItem(
                                Translation :: get('MakeTeacher'),
                                Theme :: get_image_path($weblcms_manager_namespace). 'action_subscribe_teacher.png',
                                $status_change_url,
                                ToolbarItem :: DISPLAY_ICON
                            )
                        );
                        break;
                }
            }
            else
            {
                $toolbar->add_item(
                    new ToolbarItem(
                        Translation :: get('UnsubscribeNotAvailable'),
                        Theme :: get_common_image_path() .
                        'action_unsubscribe_na.png',
                        null,
                        ToolbarItem :: DISPLAY_ICON
                    )
                );
            }

            // if we have editing rights, display the reporting action
            $params = array();
            $params[WeblcmsManager :: PARAM_USERS] = $user_id;
            $params[Tool :: PARAM_ACTION] = UserTool :: ACTION_REPORTING;
            $parameters[UserTool :: PARAM_TAB] = Request :: get(UserTool :: PARAM_TAB);
            $reporting_url = $this->get_component()->get_url($params);

            $toolbar->add_item(
                new ToolbarItem(
                    Translation :: get('Report'),
                    Theme :: get_common_image_path() . 'action_reporting.png',
                    $reporting_url,
                    ToolbarItem :: DISPLAY_ICON
                )
            );
        }

        // add action for view as user
        if ($this->get_component()->is_allowed(WeblcmsRights :: EDIT_RIGHT)) // get_parent()->is_teacher())
        {
            if ($user_id != $this->get_component()->get_user()->get_id())
            {
                $parameters = array();
                $parameters[Tool :: PARAM_ACTION] = UserTool :: ACTION_VIEW_AS;
                $parameters[WeblcmsManager :: PARAM_USERS] = $user_id;
                $view_as_url = $this->get_component()->get_url($parameters);

                $toolbar->add_item(
                    new ToolbarItem(
                        Translation :: get('ViewAsUser'),
                        Theme :: get_common_image_path() . 'action_login.png',
                        $view_as_url,
                        ToolbarItem :: DISPLAY_ICON
                    )
                );
            }
        }

        // return
        return $toolbar->as_html();
    }
}
