<?php
namespace application\weblcms\tool\user;

use user\DefaultUserTableCellRenderer;
use application\weblcms\WeblcmsRights;
use application\weblcms\WeblcmsManager;
use application\weblcms\Tool;
use common\libraries\Toolbar;
use common\libraries\ToolbarItem;
use user\User;
use common\libraries\Theme;
use common\libraries\Request;
use common\libraries\Translation;
use common\libraries\WebApplication;
use application\weblcms\course\CourseUserRelation;
use application\weblcms\CourseManagementRights;


/** * ***************************************************************************
 * Cell renderer for an unsubscribed course user browser table.
 *
 * @author Stijn Van Hoecke
 * **************************************************************************** */
class UnsubscribedUserBrowserTableCellRenderer extends DefaultUserTableCellRenderer
{


    //**************************************************************************
    // CONSTRUCTOR
    //**************************************************************************
    /**
     * Constructor
     * @param WeblcmsBrowserComponent $browser
     */
    public function __construct($browser)
    {
        parent :: __construct($browser);
    }

    //**************************************************************************
    // GENERAL FUNCTIONS
    //**************************************************************************
    // Inherited
    /**
     * Renders a given cell.
     * @param type $column
     * @param type $user_with_subscription_status User from the advanced join query in
     *      weblcms database class that includes his subscription status.
     * @return type
     */
    public function render_cell($column, $user_with_subscription_status)
    {
        // Add special features here
        switch ($column->get_name())
        {
            // Exceptions that need post-processing go here ...
            case User :: PROPERTY_STATUS :
                switch ($user_with_subscription_status->get_status())
                {
                    case CourseUserRelation :: STATUS_TEACHER :
                        return Translation :: get('CourseAdmin');
                    case CourseUserRelation :: STATUS_STUDENT :
                        return Translation :: get('Student');
                    default :
                        return Translation :: get('Unknown');
                }
            case User :: PROPERTY_PLATFORMADMIN :
                if ($user_with_subscription_status->get_platformadmin() == '1')
                {
                    return Translation :: get('PlatformAdministrator');
                }
                else
                {
                    return '';
                }
            case User :: PROPERTY_EMAIL :
                $email_url = 'mailto:' . $user_with_subscription_status->get_email();
                return '<a href="' . $email_url . '">' . $user_with_subscription_status->get_email() . '</a>';
        }
        return parent :: render_cell($column, $user_with_subscription_status);
    }

    /**
     * Gets the action links to display
     * @param User $user The user for which the
     * action links should be returned
     * @return string A HTML representation of the action links
     */
    public function get_object_actions($user_with_subscription_status)
    {
        // construct
        $toolbar = new Toolbar(Toolbar :: TYPE_HORIZONTAL);

        if ($this->get_component()->get_user()->is_platform_admin() || (
            $this->get_component()->is_allowed(WeblcmsRights :: EDIT_RIGHT) &&
                CourseManagementRights :: get_instance()->is_allowed(
                    CourseManagementRights :: TEACHER_DIRECT_SUBSCRIBE_RIGHT, $this->get_component()->get_course_id(),
                    CourseManagementRights :: TYPE_COURSE, $user_with_subscription_status->get_id()
                ))
        )

        {
            // subscribe regular student
            $parameters = array();
            $parameters[UserTool :: PARAM_OBJECTS] = $user_with_subscription_status->get_id();
            $parameters[Tool :: PARAM_ACTION] = UserTool :: ACTION_SUBSCRIBE;
            $parameters[UserTool :: PARAM_TAB] = Request :: get(UserTool :: PARAM_TAB);
            $subscribe_url = $this->get_component()->get_url($parameters);

            $weblcms_manager_namespace = WebApplication :: determine_namespace(WeblcmsManager :: APPLICATION_NAME);

            $toolbar->add_item(
                new ToolbarItem(
                    Translation :: get('SubscribeAsStudent'),
                    Theme :: get_image_path($weblcms_manager_namespace) . 'action_subscribe_student.png',
                    $subscribe_url,
                    ToolbarItem :: DISPLAY_ICON
                )
            );

            // subscribe as course admin
            $parameters = array();
            $parameters[UserTool :: PARAM_OBJECTS] = $user_with_subscription_status->get_id();
            $parameters[Tool :: PARAM_ACTION] = UserTool :: ACTION_SUBSCRIBE_AS_ADMIN;
            $parameters[UserTool :: PARAM_TAB] = Request :: get(UserTool :: PARAM_TAB);
            $subscribe_url = $this->get_component()->get_url($parameters);

            $toolbar->add_item(
                new ToolbarItem(
                    Translation :: get('SubscribeAsTeacher'),
                    Theme :: get_image_path($weblcms_manager_namespace) . 'action_subscribe_teacher.png',
                    $subscribe_url,
                    ToolbarItem :: DISPLAY_ICON
                )
            );
        }
        elseif ($this->get_component()->get_user()->is_platform_admin() || (
            $this->get_component()->is_allowed(WeblcmsRights :: EDIT_RIGHT) &&
                CourseManagementRights :: get_instance()->is_allowed(
                    CourseManagementRights :: TEACHER_REQUEST_SUBSCRIBE_RIGHT, $this->get_component()->get_course_id(),
                    CourseManagementRights :: TYPE_COURSE, $user_with_subscription_status->get_id()
                ))
        )

        {
            if (!\application\weblcms\DataManager :: is_user_requested_for_course(
                $user_with_subscription_status->get_id(),
                $this->get_component()->get_course_id()
            )
            )
            {
                $parameters = array();
                $parameters[UserTool :: PARAM_OBJECTS] = $user_with_subscription_status->get_id();
                $parameters[Tool :: PARAM_ACTION] = UserTool :: ACTION_REQUEST_SUBSCRIBE_USER;
                $parameters[UserTool :: PARAM_TAB] = Request :: get(UserTool :: PARAM_TAB);
                $subscribe_request_url = $this->get_component()->get_url($parameters);

                $toolbar->add_item(
                    new ToolbarItem(
                        Translation :: get('RequestUser'),
                        Theme :: get_image_path() . 'action_request_subscribe_user.png',
                        $subscribe_request_url,
                        ToolbarItem :: DISPLAY_ICON
                    )
                );
            }
            else
            {
                $toolbar->add_item(
                    new ToolbarItem(
                        Translation :: get('UserRequestPending'),
                        Theme :: get_common_image_path() . 'action_period.png',
                        null,
                        ToolbarItem :: DISPLAY_ICON
                    )
                );
            }
        }
        else
        {
            $toolbar->add_item(
                new ToolbarItem(
                    Translation :: get('SubscribeNA'),
                    Theme :: get_common_image_path() . 'action_subscribe_na.png',
                    null,
                    ToolbarItem :: DISPLAY_ICON
                )
            );
        }

        // return
        return $toolbar->as_html();
    }
}
