<?php

namespace common\extensions\new_rights_editor_manager;

use common\libraries\DynamicVisualTabsRenderer;
use common\libraries\StaticConditionVariable;
use common\libraries\DynamicTabsRenderer;
use common\libraries\DynamicContentTab;
use common\libraries\DynamicVisualTab;

use common\libraries\Translation;
use common\libraries\Theme;

use common\libraries\EqualityCondition;
use common\libraries\AndCondition;
use common\libraries\ConditionProperty;

use common\libraries\ActionBarRenderer;
use common\libraries\ToolbarItem;

use common\libraries\Utilities;
use common\libraries\ResourceManager;
use common\libraries\Path;

/**
 * Simple interface to edit rights
 *
 * @author Sven Vanpoucke
 * @package application.common.rights_editor_manager.component
 */
class RightsEditorManagerAdvancedRightsEditorComponent extends RightsEditorManagerRightsEditorComponent
{
    const TABS_NAME = 'rights_editor';
    const TABS_NAME_CATEGORY = 'rights_editor_category';

    const TAB_ENTITY = 'entity';
    const TAB_SUB_ENTITIES = 'sub_entities';

    /**
     * The action bar
     * @var ActionBarRenderer
     */
    private $action_bar;

    /**
     * Runs the component
     */
    public function run()
    {
        $this->action_bar = $this->get_action_bar();

        $this->display_header();

        echo '<br />';
        echo $this->action_bar->as_html();

        echo '<div id="action_bar_browser" style="width:100%;">';
        echo $this->display_entities();
        echo '</div>';

        $this->display_footer();
    }

    /**
     * Displays the entities tabs
     */
    public function display_entities()
    {
        $entities = $this->get_entities();
        $selected_entity = $this->get_selected_entity();

        $tabs = new DynamicVisualTabsRenderer(self :: TABS_NAME, $this->display_entity($selected_entity));

        foreach ($entities as $entity_type => $entity)
        {
            $name = $entity->get_entity_translated_name();
            $link = $this->get_entity_url($entity_type);
            $selected = ($selected_entity->get_entity_type() == $entity_type);

            $tab = new DynamicVisualTab($entity_type, $name, $entity->get_entity_icon(), $link, $selected);
            $tabs->add_tab($tab);
        }

        return $tabs->render();
    }

    /**
     * Displays the content of the selected entity
     */
    public function display_entity($selected_entity)
    {
        $html = array();

        if ($selected_entity instanceof NestedRightsEntity)
        {
            $html[] = '<div style="float: left; width: 18%; overflow: auto;">';
            $html[] = $this->display_entity_menu();
            $html[] = '</div>';

            $html[] = '<div style="float: right; width: 80%; overflow:auto;">';
            $html[] = $this->display_entity_table($selected_entity);
            $html[] = '</div>';
        }
        else
        {
            $html[] = $this->display_entity_table($selected_entity);
        }

        return implode("\n", $html);
    }

    /**
     * Displays the location entity table of the selected entity
     */
    public function display_entity_table($selected_entity)
    {
        $html = array();

        $search_query = $this->action_bar->get_query();

        $html[] = '<div style="overflow: auto;">';

        if ($selected_entity instanceof NestedRightsEntity)
        {
            $selected_id = $this->get_selected_entity_id();

            $tabs = new DynamicTabsRenderer(self :: TABS_NAME_CATEGORY);

            // Get the rights table for the current entity item
            $condition = $this->get_condition($selected_entity, $search_query, $selected_id);
            $table = new LocationEntityBrowserTable($this, $this->get_parameters(), $condition);

            $tabs->add_tab(
                new DynamicContentTab(self :: TAB_ENTITY, Translation :: get('Rights'), null, $table->as_html())
            );

            // Get the rights table for the children entity items
            $condition = $this->get_condition($selected_entity, $search_query, null, $selected_id);
            $table = new LocationEntityBrowserTable($this, $this->get_parameters(), $condition);

            $tabs->add_tab(
                new DynamicContentTab(self :: TAB_SUB_ENTITIES, Translation :: get('Children'), null, $table->as_html())
            );

            $html[] = $tabs->render();
        }
        else
        {
            $condition = $this->get_condition($selected_entity, $search_query);
            $table = new LocationEntityBrowserTable($this, $this->get_parameters(), $condition);

            $html[] = $table->as_html();
        }

        $location_ids = array();
        foreach ($this->get_locations() as $location)
        {
            $location_ids[] = $location->get_id();
        }

        $html[] = '</div>';
        $html[] = '<script type="text/javascript">';
        $html[] = '  var locations = \'' . json_encode($location_ids) . '\';';
        $html[] = '</script>';

        $html[] = ResourceManager :: get_instance()->get_resource_html(
            Path::namespace_to_path(__NAMESPACE__) . '/resources/javascript/configure_entity.js'
        );

        return implode("\n", $html);
    }

    /**
     * Displays the location entity tree menu of the selected entity
     */
    public function display_entity_menu()
    {
        $html = array();

        $tree = new LocationEntityBrowserTreeMenu($this, $this->get_selected_entity());
        $html[] = $tree->render_as_tree();

        return implode("\n", $html);
    }

    /**
     * Builds the actionbar;
     * @return ActionBarRenderer
     */
    public function get_action_bar()
    {
        $action_bar = new ActionBarRenderer(ActionBarRenderer :: TYPE_HORIZONTAL);
        $action_bar->set_search_url($this->get_url());

        // Add the simple rights editor button
        $action_bar->add_common_action(
            new ToolbarItem(Translation :: get('SimpleRightsEditor'),
                Theme :: get_common_image_path() . 'action_config.png', $this->get_url(
                    array(
                        self :: PARAM_ACTION => self :: ACTION_EDIT_SIMPLE_RIGHTS
                    )
                ), ToolbarItem :: DISPLAY_ICON_AND_LABEL)
        );

        // Add the show all button
        $action_bar->add_common_action(
            new ToolbarItem(Translation :: get('ShowAll', null, Utilities :: COMMON_LIBRARIES),
                Theme :: get_common_image_path() . 'action_browser.png', $this->get_url(
                ), ToolbarItem :: DISPLAY_ICON_AND_LABEL)
        );

        // Add the inherit button
        $locations = $this->get_locations();
        if (count($locations) == 1)
        {
            $location = $locations[0];
            if ($location != null && $location->get_parent())
            {
                $url = $this->get_url(array(self :: PARAM_ACTION => self :: ACTION_CHANGE_INHERIT));

                if ($location->inherits())
                {
                    //Disable inherit
                    $action_bar->add_common_action(
                        new ToolbarItem(Translation :: get('NoInherit'), Theme :: get_common_image_path() .
                            'action_setting_false_inherit.png', $url, ToolbarItem :: DISPLAY_ICON_AND_LABEL)
                    );
                }
                else
                {
                    // Enable the inherit
                    $action_bar->add_common_action(
                        new ToolbarItem(Translation :: get('Inherit'), Theme :: get_common_image_path() .
                            'action_setting_true_inherit.png', $url, ToolbarItem :: DISPLAY_ICON_AND_LABEL)
                    );
                }
            }
        }

        return $action_bar;
    }

    /**
     * Builds the condition for the entity table
     *
     * @param RightsEntity selected_entity
     * @param String $search_query
     * @param int $entity_id
     * @param int $parent_id
     *
     * @return Condition
     */
    public function get_condition($selected_entity, $search_query, $entity_id = null, $parent_id = null)
    {
        $conditions = array();

        if (isset($entity_id))
        {
            $conditions[] = new EqualityCondition(
                new PropertyConditionVariable(
                    $selected_entity->data_class_class_name(), $selected_entity->get_id_property()
                ),
                new StaticConditionVariable($entity_id)
            );
        }

        if (isset($parent_id))
        {

            $conditions[] = new EqualityCondition(
                new PropertyConditionVariable(
                    $selected_entity->data_class_class_name(), $selected_entity->get_parent_property()
                ),
                new StaticConditionVariable($parent_id)
            );
        }

        if (isset($search_query) && $search_query != '')
        {
            $condition_properties = array();

            foreach ($selected_entity->get_search_properties() as $property)
            {
                $condition_properties[] = new ConditionProperty($property);
            }

            $conditions[] = $this->action_bar->get_conditions($condition_properties);
        }

        $count = count($conditions);
        if ($count > 1)
        {
            return new AndCondition($conditions);
        }
        elseif ($count == 1)
        {
            return $conditions[0];
        }
    }

    /**
     * Register additional parameters for the breadcrumbs etc
     * @return Array
     */
    public function get_additional_parameters()
    {
        return array(self :: PARAM_ENTITY_TYPE, self :: PARAM_ENTITY_ID);
    }

    public function get_selected_entity_id()
    {
        $current_node_id = $this->get_parameter(self :: PARAM_ENTITY_ID);

        if (! $current_node_id)
        {
            $root_ids = $this->get_selected_entity()->get_root_ids();
            if (empty($root_ids))
            {
                return 0;
            }

            return $root_ids[0];
        }

        return $current_node_id;
    }
}
