<?php
namespace common\extensions\video_conferencing_manager;

use common\libraries\CommonDataManager;

use common\libraries\Filesystem;
use common\libraries\SubManager;
use common\libraries\Utilities;
use common\libraries\EqualityCondition;
use common\libraries\AndCondition;
use common\libraries\Path;
use common\libraries\Theme;
use common\libraries\Request;
use common\libraries\Application;
use common\libraries\Translation;
use common\libraries\DynamicVisualTabsRenderer;
use common\libraries\DynamicVisualTab;
use common\libraries\Registration;
use repository\ExternalSetting;

abstract class Manager extends SubManager
{
    const PARAM_VIDEO_CONFERENCING_MANAGER_ACTION = 'conferencing_action';
    const ACTION_CREATE_MEETING = 'creator';
    const ACTION_JOIN_MEETING = 'joiner';
    const ACTION_CONFIGURE_VIDEO_CONFERENCING = 'configurer';
    const ACTION_BROWSER_VIDEO_CONFERENCING = 'browser';
    const ACTION_VIEW_VIDEO_CONFERENCING = 'viewer';
    const ACTION_END_VIDEO_CONFERENCING = 'ender';
    const DEFAULT_ACTION = self :: ACTION_CREATE_MEETING;
    const PARAM_VIDEO_CONFERENCING_ID = 'video_conferencing_id';
    const PARAM_VIDEO_CONFERENCING = 'external_instance';
    const PARAM_QUERY = 'query';
    const PARAM_RENDERER = 'renderer';
    const NAMESPACE_NAME = __NAMESPACE__;
    const CLASS_NAME = __CLASS__;

    private $video_conferencing;

    /**
     *
     * @param $application Application
     */
    public function __construct($application)
    {
        parent :: __construct($application);
        $this->video_conferencing = $application->get_external_instance();

        $video_conferencing_manager_action = Request :: get(self :: PARAM_VIDEO_CONFERENCING_MANAGER_ACTION);
        if ($video_conferencing_manager_action)
        {
            $this->set_parameter(self :: PARAM_VIDEO_CONFERENCING_MANAGER_ACTION, $video_conferencing_manager_action);
        }

        $this->set_optional_parameters();

        if ($this->validate_settings($this->video_conferencing))
        {
            $this->initialize_video_conferencing($this);
        }
    }

    /**
     *
     * @return VideoConferencing
     */
    public function get_video_conferencing()
    {
        return $this->video_conferencing;
    }

    /**
     *
     * @param $video_conferencing VideoConferencing
     */
    public function set_video_conferencing(VideoConferencing $video_conferencing)
    {
        $this->video_conferencing = $video_conferencing;
    }

    /**
     *
     * @return DataConnector
     */
    public function get_video_conferencing_manager_connector()
    {
        return DataConnector :: get_instance($this->get_video_conferencing());
    }

    public function set_optional_parameters()
    {
        $this->set_parameter(self :: PARAM_RENDERER, $this->get_renderer());
    }

    /**
     *
     * @return boolean
     */
    public function is_stand_alone()
    {
        return $this->get_parent() instanceof \repository\VideoConferencingComponent;
    }

    /**
     *
     * @param $application Application
     */
    public static function launch($application)
    {
        $classname = static :: class_name();
        if (is_subclass_of($classname, __CLASS__))
        {
            parent :: launch(null, $application);
        }
        else
        {
            $external_instance = $application->get_external_instance();
            $type = $external_instance->get_type();

            $class = $type . '\\Manager';

            $settings_validated = call_user_func(array($class, 'validate_settings'), $external_instance);

            if (! $settings_validated)
            {
                if ($application->get_user()->is_platform_admin())
                {
                    Request :: set_get(Application :: PARAM_ERROR_MESSAGE, Translation :: get('PleaseReviewSettings'));
                    Request :: set_get(
                        self :: PARAM_VIDEO_CONFERENCING_MANAGER_ACTION,
                        self :: ACTION_CONFIGURE_VIDEO_CONFERENCING);
                }
                else
                {
                    parent :: display_header();
                    $application->display_warning_message('TemporarilyUnavailable');
                    parent :: display_footer();
                    exit();
                }
            }

            $class :: launch($application);
        }
    }

    /**
     *
     * @return string
     */
    public function is_ready_to_be_used()
    {
        return false;
    }

    public function any_object_selected()
    {
    }

    /**
     *
     * @return string
     */
    public function get_action()
    {
        return $this->get_parameter(self :: PARAM_VIDEO_CONFERENCING_MANAGER_ACTION);
    }

    public function display_header()
    {
        $action = $this->get_action();
        parent :: display_header();

        $html = array();
        $video_conferencing_actions = $this->get_video_conferencing_actions();

        if ($action == self :: ACTION_VIEW_VIDEO_CONFERENCING)
        {
            $video_conferencing_actions[] = self :: ACTION_VIEW_VIDEO_CONFERENCING;
        }

        $this->tabs = new DynamicVisualTabsRenderer(Utilities :: get_classname_from_object($this, true));

        foreach ($video_conferencing_actions as $video_conferencing_action)
        {
            if ($action == $video_conferencing_action)
            {
                $selected = true;
            }
            else
            {
                $selected = false;
            }

            $parameters = $this->get_parameters();
            $parameters[self :: PARAM_VIDEO_CONFERENCING_MANAGER_ACTION] = $video_conferencing_action;

            if ($video_conferencing_action == self :: ACTION_VIEW_VIDEO_CONFERENCING)
            {
                $parameters[self :: PARAM_VIDEO_CONFERENCING_ID] = Request :: get(self :: PARAM_VIDEO_CONFERENCING_ID);
            }

            $label = htmlentities(
                Translation :: get(Utilities :: underscores_to_camelcase($video_conferencing_action) . 'Title'));
            $link = $this->get_url($parameters);

            $this->tabs->add_tab(
                new DynamicVisualTab(
                    $video_conferencing_action,
                    $label,
                    Theme :: get_common_image_path() . 'place_tab_' . $video_conferencing_action . '.png',
                    $link,
                    $selected));
        }

        $html[] = $this->tabs->header();
        $html[] = $this->tabs->body_header();

        echo implode("\n", $html);
    }

    /**
     *
     * @return array
     */
    public function get_video_conferencing_actions()
    {
        $actions = array();
        $actions[] = self :: ACTION_CREATE_MEETING;

        $is_platform = $this->get_user()->is_platform_admin() &&
             (count(ExternalSetting :: get_all($this->get_video_conferencing()->get_id())) > 0);

        if ($is_platform)
        {
            $actions[] = self :: ACTION_CONFIGURE_VIDEO_CONFERENCING;
        }

        return $actions;
    }

    public function display_footer()
    {
        $html = array();
        $html[] = $this->tabs->body_footer();
        $html[] = $this->tabs->footer();
        echo implode("\n", $html);

        parent :: display_footer();
    }

    /**
     *
     * @param $condition mixed
     */
    public function count_video_conferencing_objects($condition)
    {
        return $this->get_video_conferencing_manager_connector()->count_video_conferencing_objects($condition);
    }

    /**
     *
     * @param $condition mixed
     * @param $order_property ObjectTableOrder
     * @param $offset int
     * @param $count int
     * @return ArrayResultSet
     */
    public function retrieve_video_conferencing_objects($condition, $order_property, $offset, $count)
    {
        return $this->get_video_conferencing_manager_connector()->retrieve_video_conferencing_objects(
            $condition,
            $order_property,
            $offset,
            $count);
    }

    /**
     *
     * @param $video_conferencing_manager \common\extensions\video_conferencing_manager\Manager
     */
    public function initialize_video_conferencing(Manager $video_conferencing_manager)
    {
        $this->get_video_conferencing_manager_connector();
    }

    /**
     *
     * @return boolean
     */
    abstract public function validate_settings($video_conferencing);

    /**
     *
     * @return string
     */
    public function support_sorting_direction()
    {
        return true;
    }

    /**
     *
     * @param $query mixed
     */
    public function translate_search_query($query)
    {
        return $this->get_video_conferencing_manager_connector()->translate_search_query($query);
    }

    /**
     *
     * @return array
     */
    abstract public function get_menu_items();

    /**
     *
     * @param $object ConferenceObject
     * @return string
     */
    // abstract function get_video_conferencing_object_viewing_url(ConferenceObject $object);

    /**
     *
     * @param $id string
     * @return ConferenceObject
     */
    public function retrieve_video_conferencing_object($id)
    {
        return $this->get_video_conferencing_manager_connector()->retrieve_video_conferencing_object($id);
    }

    /**
     *
     * @param $id string
     * @return boolean
     */
    public function delete_video_conferencing_object($id)
    {
        return $this->get_video_conferencing_manager_connector()->delete_video_conferencing_object($id);
    }

    // /**
    // * @param string $id
    // * @return boolean
    // */
    // function export_external_repository_object($id)
    // {
    // return $this->get_external_repository_manager_connector()->export_external_repository_object($id);
    // }

    /**
     *
     * @param $object ConferenceObject
     * @return array
     */
    public function get_video_conferencing_object_actions(ConferenceObject $object)
    {
        $toolbar_items = array();

        return $toolbar_items;
    }

    /**
     *
     * @return string
     */
    public function get_renderer()
    {
        $renderer = Request :: get(self :: PARAM_RENDERER);

        if ($renderer && in_array($renderer, $this->get_available_renderers()))
        {
            return $renderer;
        }
        else
        {
            $renderers = $this->get_available_renderers();
            return $renderers[0];
        }
    }

    /**
     *
     * @return array
     */
    public function get_available_renderers()
    {
        return array(VideoConferencingRenderer :: TYPE_TABLE);
    }

    /**
     *
     * @return Condition
     */
    // abstract function get_content_object_type_conditions();

    /**
     *
     * @param $type string
     * @return boolean
     */
    public static function exists($type)
    {
        $path = Path :: get_common_extensions_path() . 'video_conferencing_manager/implementation';
        $video_conferencing_path = $path . '/' . $type;
        $video_conferencing_manager_path = $video_conferencing_path . '/php/' . $type .
             '_video_conferencing_manager.class.php';

        if (file_exists($video_conferencing_path) && is_dir($video_conferencing_path) &&
             file_exists($video_conferencing_manager_path))
        {
            return true;
        }
        else
        {
            return false;
        }
    }

    public static function get_namespace($type = null)
    {
        if ($type)
        {
            return __NAMESPACE__ . '\implementation\\' . $type;
        }
        else
        {
            return __NAMESPACE__;
        }
    }

    public static function get_object_viewing_parameters($external_instance_sync)
    {
        return array(
            self :: PARAM_VIDEO_CONFERENCING_MANAGER_ACTION => self :: ACTION_VIEW_VIDEO_CONFERENCING,
            self :: PARAM_VIDEO_CONFERENCING_ID => $external_instance_sync->get_id());
    }

    public static function get_registered_types($status = Registration :: STATUS_ACTIVE)
    {
        $conditions = array();
        $conditions[] = new EqualityCondition(Registration :: PROPERTY_TYPE, __NAMESPACE__ . '\implementation');
        $conditions[] = new EqualityCondition(Registration :: PROPERTY_STATUS, $status);
        $condition = new AndCondition($conditions);

        return CommonDataManager :: retrieves(Registration :: class_name(), $condition);
    }

    public static function get_repository_menu_parameter()
    {
        return array(self :: PARAM_VIDEO_CONFERENCING_MANAGER_ACTION => self :: ACTION_BROWSER_VIDEO_CONFERENCING);
    }

    public static function get_packages_from_filesystem()
    {
        $external_repository_managers = array();

        $path = Path :: get_common_extensions_path() . 'video_conferencing_manager/implementation';
        $directories = Filesystem :: get_directory_content($path, Filesystem :: LIST_DIRECTORIES, false);

        foreach ($directories as $directory)
        {
            $manager_name = basename($directory);
            $external_repository_managers[] = self :: get_namespace($manager_name);
        }

        return $external_repository_managers;
    }

    public static function get_installable_packages($include_installed = false)
    {
        $package_list = new \common\libraries\package\PackageList(
            __NAMESPACE__ . '\implementation',
            Translation :: get('TypeName', null, __NAMESPACE__),
            Theme :: get_image_path() . 'logo/16.png');

        foreach (self :: get_packages_from_filesystem() as $implementation)
        {
            if (! CommonDataManager :: get_registration($implementation) || $include_installed)
            {
                $package_list->add_package($implementation);
            }
        }

        return $package_list;
    }
}
