<?php
namespace common\libraries\package;

use common\libraries\CommonDataManager;
use common\libraries\Filesystem;
use common\libraries\Setting;
use common\libraries\Translation;
use common\libraries\Utilities;
use DOMDocument;

abstract class Remover extends Action
{

    /**
     * Removes the package
     *
     * @return boolean
     */
    public function run()
    {
        if (! $this->verify_dependencies())
        {
            return false;
        }

        if (! $this->deregister_package())
        {
            return false;
        }

        if (method_exists($this, 'extra'))
        {
            $this->add_message(
                self :: TYPE_NORMAL,
                '<span class="subtitle">' . Translation :: get('Various', null, 'core\install') . '</span>');
            if (! $this->extra())
            {
                return $this->failed(Translation :: get('VariousFailed', null, 'core\install'));
            }
            else
            {
                $this->add_message(self :: TYPE_NORMAL, Translation :: get('VariousFinished', null, 'core\install'));
            }
            $this->add_message(self :: TYPE_NORMAL, '');
        }

        if (! $this->deconfigure_package())
        {
            return false;
        }

        if (! $this->uninstall_storage_units())
        {
            return false;
        }

        PackageList :: reset();

        return $this->successful();
    }

    /**
     * Verifies the package dependencies
     *
     * @param $package_attributes
     * @return boolean
     */
    public function verify_dependencies()
    {
        $verifier = new DependencyVerifier(Package :: get(static :: context()));
        $success = $verifier->is_removable();

        $this->add_message(self :: TYPE_NORMAL, $verifier->get_logger()->render());

        if (! $success)
        {
            return $this->failed(Translation :: get('PackageDependenciesFailed'));
        }
        else
        {
            $this->add_message(self :: TYPE_NORMAL, Translation :: get('PackageDependenciesVerified'));
            return true;
        }
    }

    /**
     * Scans for the available storage units and removes them
     *
     * @return boolean
     */
    public function uninstall_storage_units()
    {
        $dir = $this->get_path() . 'php/package/install/';
        $files = Filesystem :: get_directory_content($dir, Filesystem :: LIST_FILES);

        foreach ($files as $file)
        {
            if ((substr($file, - 3) == 'xml'))
            {
                if (! $this->delete_storage_unit($file))
                {
                    return false;
                }
            }
        }

        return true;
    }

    /**
     *
     * @param string $file
     * @return string
     */
    public static function parse_xml_file($file)
    {
        $doc = new DOMDocument();
        $doc->load($file);
        $object = $doc->getElementsByTagname('object')->item(0);
        return $object->getAttribute('name');
    }

    /**
     * Parses an XML file and sends the request to the database manager
     *
     * @param $path String
     */
    public function delete_storage_unit($path)
    {
        $storage_unit_name = self :: parse_xml_file($path);

        $this->add_message(
            self :: TYPE_NORMAL,
            Translation :: get_instance()->get('StorageUnitRemoval', null, 'core\install') . ': <em>' .
                 $storage_unit_name . '</em>');

        $data_manager = static :: context() . '\DataManager';

        if (! $data_manager :: drop_storage_unit($storage_unit_name))
        {
            return $this->failed(
                Translation :: get_instance()->get('StorageUnitRemovalFailed', null, 'core\install') . ': <em>' .
                     $storage_unit_name . '</em>');
        }
        else
        {
            return true;
        }
    }

    public function parse_application_settings($file)
    {
        $doc = new DOMDocument();

        $doc->load($file);
        $object = $doc->getElementsByTagname('package')->item(0);

        $setting_elements = $doc->getElementsByTagname('setting');
        $settings = array();

        foreach ($setting_elements as $index => $setting_element)
        {
            $settings[$setting_element->getAttribute('name')] = array(
                'default' => $setting_element->getAttribute('default'),
                'user_setting' => $setting_element->getAttribute('user_setting'));
        }

        return $settings;
    }

    public function deconfigure_package()
    {
        $settings_file = static :: get_path() . 'php/settings/settings.xml';

        if (file_exists($settings_file))
        {
            $xml = $this->parse_application_settings($settings_file);

            foreach ($xml as $name => $parameters)
            {
                $setting = CommonDataManager :: retrieve_setting_from_variable_name($name, static :: context());

                if (! $setting instanceof Setting || ! $setting->delete())
                {
                    $message = Translation :: get('PackageDeconfigurationFailed', null, 'core\install');
                    return $this->failed($message);
                }
            }

            $this->add_message(self :: TYPE_NORMAL, Translation :: get('PackageSettingsRemoved', null, 'core\install'));
        }

        return true;
    }

    public function deregister_package()
    {
        $registration = CommonDataManager :: get_registration(static :: context());

        if (! $registration->delete())
        {
            return $this->failed(Translation :: get('PackageDeregistrationFailed', null, 'core\install'));
        }
        else
        {
            return true;
        }
    }

    /**
     *
     * @param string $context
     * @return \common\libraries\package\Remover
     */
    public static function factory($context)
    {
        $class = $context . '\Remover';
        return new $class();
    }

    /**
     * Returns the list with extra installable packages that are connected to this package
     *
     * @return multitype:string
     */
    public function get_additional_packages()
    {
        return array();
    }

    public function extract_xml_file($file)
    {
        return Utilities :: extract_xml_file($file);
    }
}
