<?php

namespace common\libraries\package\upgrade\v4_0_0;

use common\libraries\AndCondition;
use common\libraries\DataClassProperties;
use common\libraries\DataClassProperty;
use common\libraries\DataClassRetrievesParameters;
use common\libraries\EqualityCondition;
use common\libraries\PropertyConditionVariable;
use common\libraries\StaticConditionVariable;
use common\libraries\Translation;
use common\libraries\Utilities;
use core\home\HomeBlockRegistration;

/**
 * Basic upgrader script for upgrades from v3_0_0 to v4_0_0
 *
 * @author Sven Vanpoucke - Hogeschool Gent
 */
abstract class Upgrader extends \common\libraries\package\Upgrader
{
    /*
     * (non-PHPdoc) @see \common\libraries\package\Upgrader::run()
     *
     * @return bool
     */
    public function run()
    {
        $data_manager = $this->get_data_manager();
        $upgrader = $this;

        return $data_manager->transactional(
            function ($c) use ($data_manager, $upgrader)
            {
                try
                {
                    return $upgrader->upgrade_package($data_manager);
                }
                catch (\Exception $ex)
                {
                    echo $ex->getMessage();

                    return false;
                }
            }
        );
    }

    /**
     * Upgrades the package. Executes basic upgrade procedures and calls the upgrade function
     *
     * @param mixed $data_manager
     *
     * @return bool
     */
    public function upgrade_package($data_manager)
    {
        if (! $this->upgrade_package_registration($this->get_package_name()))
        {
            return false;
        }

        return $this->upgrade($data_manager);
    }

    /**
     * Deduces the package name by the giving context of the called class
     *
     * @return string
     */
    protected function get_package_name()
    {
        $class = get_class($this);

        return Utilities :: get_namespace_parent(
            Utilities :: get_namespace_parent(Utilities :: get_namespace_from_classname($class))
        );
    }

    /**
     * Upgrades the home block registrations for a given package
     *
     * @param string $package
     *
     * @return bool
     */
    protected function upgrade_home_block_registrations($package)
    {
        $condition = new EqualityCondition(
            new PropertyConditionVariable(
                HomeBlockRegistration :: class_name(),
                HomeBlockRegistration :: PROPERTY_CONTEXT
            ),
            new StaticConditionVariable($package)
        );

        $home_block_registrations = \core\home\DataManager :: retrieves(
            HomeBlockRegistration :: class_name(),
            new DataClassRetrievesParameters($condition)
        );

        while ($home_block_registration = $home_block_registrations->next_result())
        {
            $home_block_registration->set_context($package . '\integration\core\home');
            if (! $home_block_registration->update())
            {
                $this->add_message(
                    \common\libraries\package\Upgrader :: TYPE_ERROR,
                    Translation :: get('UpgradeHomeBlockRegistrationsFailed')
                );

                return false;
            }
        }

        $this->add_message(
            \common\libraries\package\Upgrader :: TYPE_NORMAL,
            Translation :: get('UpgradeHomeBlockRegistrationsSuccessful')
        );

        return true;
    }

    /**
     * Upgrades the tracking application's databases to the latest version
     *
     * @param string $application_name
     * @param string $context
     * @param array $tracker_names
     *
     * @return bool
     */
    protected function upgrade_tracking($application_name, $context, $tracker_names = array())
    {
        return $this->add_package_registration($context) &&
            $this->upgrade_event_context($application_name, $context) &&
            $this->upgrade_tracking_registrations($application_name, $context, $tracker_names) &&
            $this->rename_tracking_storage_units($application_name, $tracker_names);
    }

    /**
     * Upgrades the tracking registrations from the application name to the context and changes the given tracker names
     *
     * @param string $application_name
     * @param string $context
     * @param array $tracker_names
     *
     * @return bool
     */
    protected function upgrade_tracking_registrations($application_name, $context, $tracker_names = array())
    {
        foreach ($tracker_names as $old_tracker => $new_tracker)
        {
            $properties = new DataClassProperties();

            $properties->add(
                new DataClassProperty(
                    new PropertyConditionVariable(
                        \tracking\TrackerRegistration :: class_name(),
                        \tracking\TrackerRegistration :: PROPERTY_TRACKER),
                    new StaticConditionVariable($new_tracker)
                )
            );

            $properties->add(
                new DataClassProperty(
                    new PropertyConditionVariable(
                        \tracking\TrackerRegistration :: class_name(),
                        \tracking\TrackerRegistration :: PROPERTY_CONTEXT),
                    new StaticConditionVariable($context)
                )
            );

            $conditions = array();

            $conditions[] = new EqualityCondition(
                new PropertyConditionVariable(
                    \tracking\TrackerRegistration :: class_name(),
                    \tracking\TrackerRegistration :: PROPERTY_TRACKER),
                new StaticConditionVariable($old_tracker)
            );

            $conditions[] = new EqualityCondition(
                new PropertyConditionVariable(
                    \tracking\TrackerRegistration :: class_name(),
                    \tracking\TrackerRegistration :: PROPERTY_CONTEXT),
                new StaticConditionVariable($application_name)
            );

            $condition = new AndCondition($conditions);

            if (! \tracking\TrackingDataManager :: get_instance()->updates(
                \tracking\TrackerRegistration :: class_name(), $properties, $condition
            )
            )
            {
                $this->add_message(
                    \common\libraries\package\Upgrader :: TYPE_ERROR,
                    Translation :: get(
                        'UpgradeTrackerRegistrationFailed', array('CONTEXT' => $context, 'TRACKER' => $old_tracker)
                    )
                );
            }

        }

        $this->add_message(
            \common\libraries\package\Upgrader :: TYPE_NORMAL,
            Translation :: get('UpgradeTrackerRegistrationSuccessful', array('CONTEXT' => $context))
        );

        return true;
    }

    /**
     * Renames the storage units of the trackers
     *
     * @param string $application_name
     * @param array $tracker_names
     *
     * @return bool
     */
    protected function rename_tracking_storage_units($application_name, $tracker_names)
    {
        foreach ($tracker_names as $old_tracker => $new_tracker)
        {
            $new_tracker = $application_name . '_' . $new_tracker;

            if (! \tracking\TrackingDataManager :: get_instance()->rename_storage_unit(
                $old_tracker,
                $new_tracker
            ))
            {
                $this->add_message(
                    \common\libraries\package\Upgrader :: TYPE_ERROR,
                    Translation :: get(
                        'TrackingStorageUnitRenameFailed',
                        array('OLD_TRACKER' => $old_tracker, 'NEW_TRACKER' => $new_tracker)
                    )
                );
            }
        }

        $this->add_message(
            \common\libraries\package\Upgrader :: TYPE_NORMAL,
            Translation :: get(
                'TrackingStorageUnitRenameSuccessful',
                array('OLD_TRACKER' => $old_tracker, 'NEW_TRACKER' => $new_tracker)
            )
        );

        return true;
    }

    /**
     * Upgrades the context from the tracking events from the application name to the real context
     *
     * @param string $application_name
     * @param string $context
     *
     * @return bool
     */
    protected function upgrade_event_context($application_name, $context)
    {
        $condition = new EqualityCondition(
            new PropertyConditionVariable(\tracking\Event :: class_name(), \tracking\Event :: PROPERTY_CONTEXT),
            new StaticConditionVariable($application_name)
        );

        $properties = new DataClassProperties();

        $properties->add(
            new DataClassProperty(
                new PropertyConditionVariable(
                    \tracking\Event :: class_name(),
                    \tracking\Event::PROPERTY_CONTEXT),
                new StaticConditionVariable($context)
            )
        );

        if (! \tracking\TrackingDataManager :: get_instance()->updates(
            \tracking\Event :: class_name(), $properties, $condition
        )
        )
        {
            $this->add_message(
                \common\libraries\package\Upgrader :: TYPE_ERROR,
                Translation :: get('UpgradeEventsContextFailed')
            );
        }

        $this->add_message(
            \common\libraries\package\Upgrader :: TYPE_NORMAL,
            Translation :: get('UpgradeEventsContextSuccessful')
        );

        return true;
    }

    /**
     * Upgrades the packages
     *
     * @param $data_manager
     *
     * @return bool
     */
    abstract function upgrade($data_manager);
}