<?php
namespace common\libraries;

use user\User;

/**
 * Basic AjaxManager class to handle AJAX calls
 *
 * @author Hans De Bisschop
 * @package common.libraries
 */
abstract class AjaxManager
{
    const PARAM_USER_ID = 'user_id';

    /**
     * The user making the AJAX request
     *
     * @var User
     */
    private $user;

    /**
     * An array of parameters as passed by the POST-request
     *
     * @var array
     */
    private $parameters = array();

    /**
     * AjaxManager constructor
     *
     * @param User $user
     */
    public function __construct()
    {
        $this->validate_request();
    }

    /**
     * Run the AJAX component
     */
    abstract public function run();

    /**
     * Return the current user's unique identifier
     *
     * @return int
     */
    public function get_user_id()
    {
        return $this->user->get_id();
    }

    /**
     * Gets the user.
     *
     * @return User
     */
    public function get_user()
    {
        return $this->user;
    }

    /**
     * Validate the AJAX call, if not validated, trigger an HTTP 400 (Bad request) error
     */
    public function validate_request()
    {
        if (! $this instanceof NoAuthenticationSupport && ! Authentication :: is_valid())
        {
            JsonAjaxResult :: not_allowed();
        }

        if ($this instanceof NoAuthenticationSupport)
        {
            $user_id = Request :: post(self :: PARAM_USER_ID);

            if (is_null($user_id))
            {
                JsonAjaxResult :: bad_request();
            }
            else
            {
                $this->set_parameter(self :: PARAM_USER_ID, $user_id);
            }

            $this->user = \user\DataManager :: retrieve(User :: class_name(), $user_id);
        }
        else
        {
            $this->user = \user\DataManager :: retrieve(User :: class_name(), (int) Session :: get_user_id());
        }

        foreach ($this->required_parameters() as $parameter)
        {
            $value = Request :: post($parameter);
            if (! is_null($value))
            {
                $this->set_parameter($parameter, $value);
            }
            else
            {
                JsonAjaxResult :: bad_request();
            }
        }
    }

    /**
     * Get the parameters
     *
     * @return array
     */
    public function get_parameters()
    {
        return $this->parameters;
    }

    /**
     * Set the parameters
     *
     * @param array $parameters
     */
    public function set_parameters($parameters)
    {
        $this->parameters = $parameters;
    }

    /**
     * Returns the value of the given parameter.
     *
     * @param string $name The parameter name.
     * @return string The parameter value.
     */
    public function get_parameter($name)
    {
        if (array_key_exists($name, $this->parameters))
        {
            return $this->parameters[$name];
        }
    }

    /**
     * Sets the value of a parameter.
     *
     * @param string $name The parameter name.
     * @param string $value The parameter value.
     */
    public function set_parameter($name, $value)
    {
        $this->parameters[$name] = $value;
    }

    /**
     * Get an array of parameters which should be set for this call to work
     *
     * @return array
     */
    abstract public function required_parameters();

    /**
     * Create an instance of an AjaxManager and run it
     *
     * @param User $user
     * @param string $context
     * @param string $method
     */
    public static function launch($context, $method)
    {
        self :: construct($context, $method)->run();
    }

    public static function construct($context, $method)
    {
        $class = $context . '\\' . Utilities :: get_package_name_from_namespace($context, true) . 'Ajax' .
             Utilities :: underscores_to_camelcase($method);

        if (! class_exists($class))
        {
            JsonAjaxResult :: bad_request($context . ' > ' . $class);
        }

        return new $class();
    }
}
