<?php
namespace common\libraries;

class NewObjectTable
{
    const DEFAULT_ROW_COUNT = 20;
    
    /**
     * Application or submanager component calling the ObjectTable
     *
     * @var Ambigous <Application, SubManager>
     */
    private $component;

    /**
     * Suffix for checkbox name when using actions on selected learning objects.
     */
    const CHECKBOX_NAME_SUFFIX = '_id';

    /**
     * The column model assigned to this table
     */
    private $column_model;

    /**
     * The data provider assigned to this table
     */
    private $data_provider;

    /**
     * The cell renderer assigned to this table
     */
    private $cell_renderer;

    /**
     * Caching of form actions
     *
     * @var ObjectTableFormActions
     */
    private $form_actions;

    public function __construct($component)
    {
        if (! $component instanceof NewObjectTableSupport)
        {
            throw new \Exception(Utilities :: get_classname_from_object($component) . " doesn't seem to support object tables, please implement the NewObjectTableSupport interface");
        }

        $interface_class = $this->get_class('Interface');
        if(interface_exists($interface_class))
        {
            if(!$component instanceof $interface_class)
            {
                throw new \Exception(Utilities :: get_classname_from_object($component) . ' must implement ' . $interface_class);
            }
        }

        $this->component = $component;
    }

    public function get_class($type = null)
    {
        $class_name = get_class($this);

        if (! is_null($type))
        {
            $class_name .= $type;
        }

        return $class_name;
    }

    /**
     * Creates an HTML representation of the table.
     *
     * @return string The HTML.
     */
    public function as_html()
    {

        $table = new SortableTable($this->get_name(), array($this, 'get_object_count'), array($this, 'get_objects'), $this->get_column_model()->get_default_order_column() + ($this->has_form_actions() ? 1 : 0), $this->get_default_row_count(), $this->get_column_model()->get_default_order_direction(), $this->supports_ajax(), !$this->prohibits_page_selection());
        $table->set_additional_parameters($this->get_parameters());

        if ($this->has_form_actions())
        {
            $table->set_form_actions($this->get_form_actions(), $this->get_checkbox_name());
            $table->set_header(0, '', false);
        }

        $column_count = $this->get_column_model()->get_column_count();
        for($i = 0; $i < $column_count; $i ++)
        {
            $column = $this->get_column_model()->get_column($i);
            $table->set_header(($this->has_form_actions() ? $i + 1 : $i), Security :: remove_XSS($column->get_title()), $column->is_sortable());
        }

        return $table->as_html();
    }

    /**
     *
     * @return boolean
     */
    public function prohibits_page_selection()
    {
        return $this instanceof NewObjectTablePageSelectionProhibition;
    }

    /**
     *
     * @return boolean
     */
    public function supports_ajax()
    {
        return $this instanceof NewObjectTableAjaxSupport;
    }

    /**
     * Gets the name of the HTML table element
     *
     * @return string The name
     */
    public static function get_name()
    {
        return Utilities :: get_classname_from_namespace(get_called_class(), true);
    }

    /**
     * Gets the default row count of the table
     *
     * @return int The number of rows
     */
    public function get_default_row_count()
    {
        return static :: DEFAULT_ROW_COUNT;
    }

    /**
     * Gets the table's data provider
     *
     * @return NewObjectTableDataProvider The data provider
     */
    public function get_data_provider()
    {
        if (! isset($this->data_provider))
        {
            $classname = $this->get_class('DataProvider');
            $this->data_provider = new $classname($this);
        }
        return $this->data_provider;
    }

    /**
     * Sets the data provider
     *
     * @param NewObjectTableDataProvider $data_provider
     */
    public function set_data_provider($data_provider)
    {
        $this->data_provider = $data_provider;
    }

    /**
     * Gets the table's column model
     *
     * @return NewObjectTableColumnModel The column model
     */
    public function get_column_model()
    {
        if (! isset($this->column_model))
        {
            $classname = $this->get_class('ColumnModel');
            $this->column_model = new $classname($this);
        }
        return $this->column_model;
    }

    /**
     * Sets the column model
     *
     * @param NewObjectTableColumnModel $column_model
     */
    public function set_column_model($column_model)
    {
        $this->column_model = $column_model;
    }

    /**
     * Gets the table's cell renderer
     *
     * @return NewObjectTableCellRenderer The cell renderer
     */
    public function get_cell_renderer()
    {
        if (! isset($this->cell_renderer))
        {
            $classname = $this->get_class('CellRenderer');
            $this->cell_renderer = new $classname($this);
        }
        return $this->cell_renderer;
    }

    /**
     * Sets the cell renderer
     *
     * @param NewObjectTableCellRenderer $cell_renderer
     */
    public function set_cell_renderer($cell_renderer)
    {
        $this->cell_renderer = $cell_renderer;
    }

    /**
     * Gets the actions for the mass-update form at the bottom of the table.
     *
     * @return ObjectTableFormActions The actions as an associative array.
     */
    public function get_form_actions()
    {
        if (! isset($this->form_actions))
        {
            $this->form_actions = $this->get_implemented_form_actions();
        }

        return $this->form_actions;
    }

    /**
     * Returns whether or not the table has form actions
     *
     * @return boolean
     */
    public function has_form_actions()
    {
        return ($this instanceof NewObjectTableFormActionsSupport && $this->get_form_actions()->has_form_actions());
    }

    /**
     *
     * @return string
     */
    public function get_checkbox_name()
    {
        return $this->get_name() . static :: CHECKBOX_NAME_SUFFIX;
    }

    public function get_objects($offset, $count, $order_column, $order_direction)
    {
        $column_model = $this->get_column_model();

        $objects = $this->get_data_provider()->get_objects($offset, $count, $column_model->get_order_column($order_column - ($this->has_form_actions() ? 1 : 0), $order_direction));
        $table_data = array();
        $column_count = $column_model->get_column_count();
        $column_model->set_default_order_column($order_column - ($this->has_form_actions() ? 1 : 0));

        while ($objects != null && $object = $objects->next_result())
        {
            $row = array();
            if ($this->has_form_actions())
            {
                $row[] = $this->get_cell_renderer()->render_id_cell($object);
            }
            for($i = 0; $i < $column_count; $i ++)
            {
                $row[] = $this->get_cell_renderer()->render_cell($this->get_column_model()->get_column($i), $object);
            }
            $table_data[] = $row;
        }
        return $table_data;
    }

    public function get_object_count()
    {
        return $this->get_data_provider()->get_object_count();
    }

    /**
     *
     * @param $table_name unknown_type
     * @return multitype:int
     */
    public static function get_selected_ids()
    {
        $selected_ids = Request :: post(static :: get_name() . self :: CHECKBOX_NAME_SUFFIX);

        if (empty($selected_ids))
        {
            $selected_ids = array();
        }
        elseif (! is_array($selected_ids))
        {
            $selected_ids = array($selected_ids);
        }

        return $selected_ids;
    }

    public static function handle_table_action()
    {
        $ids = static :: get_selected_ids();
        Request :: set_get(static :: TABLE_IDENTIFIER, $ids);
    }

    /**
     *
     * @return \common\libraries\Application
     */
    public function get_component()
    {
        return $this->component;
    }

    public function get_condition()
    {
        return $this->get_component()->get_object_table_condition(get_called_class());
    }

    public function get_parameters()
    {
        return $this->get_component()->get_parameters();
    }

    /**
     * Get the fully qualified class name of the ObjectTable object
     *
     * @return string
     */
    public static function class_name()
    {
        return get_called_class();
    }

    /**
     * Get the namespace of the ObjectTable object
     *
     * @return string
     */
    public static function context()
    {
        return Utilities :: get_namespace_from_classname(get_called_class());
    }

}
