<?php
namespace common\libraries;

use Exception;
use MDB2;

/**
 * $Id: database.class.php 229 2009-11-16 09:02:34Z scaramanga $
 *
 * @package common.database
 */

/**
 * This class provides basic functionality for database connections Create Table, Get next id, Insert, Update, Delete,
 * Select(with use of conditions), Count(with use of conditions)
 *
 * @author Sven Vanpoucke
 */
class DoctrineDatabase
{
    const STORAGE_TYPE = 'doctrine';
    const ALIAS_MAX_SORT = 'max_sort';

    /**
     *
     * @var \Doctrine\DBAL\Connection
     */
    private $connection;

    private $prefix;

    private $aliases;

    /**
     * Static error log so we don't open the error log every time an error is written
     *
     * @var FileLogger
     */
    private static $error_log;

    /**
     * Used for debug
     *
     * @var int
     */
    private static $query_counter;

    public function __call($name, $arguments)
    {
        $class = static :: context() . '\\' . Utilities :: get_package_name_from_namespace(static :: context(), true) .
             'DataManager';
        return call_user_func_array($class . '::' . $name, $arguments);
    }

    /**
     * Constructor
     */
    public function __construct($aliases = array())
    {
        $this->aliases = $aliases;
        $this->initialize();
    }

    /**
     * Initialiser, creates the connection and sets the database to UTF8
     */
    public function initialize()
    {
        $this->connection = DoctrineConnection :: get_instance()->get_connection();

        $package = Utilities :: get_package_name_from_namespace(static :: context());
        $this->set_prefix($package . '_');
    }

    public function set_aliases($aliases = array())
    {
        $this->aliases = $aliases;
    }

    public function get_aliases()
    {
        return $this->aliases;
    }

    public function get_storage_type()
    {
        return self :: STORAGE_TYPE;
    }

    /**
     * Returns the prefix
     *
     * @return String the prefix
     */
    public function get_prefix()
    {
        return $this->prefix;
    }

    /**
     * Sets the prefix
     *
     * @param $prefix String
     */
    public function set_prefix($prefix)
    {
        $this->prefix = $prefix;
    }

    /**
     * Returns the connection
     *
     * @return Doctrine\DBAL\Connection
     */
    public function get_connection()
    {
        return $this->connection;
    }

    /**
     * Sets the connection
     *
     * @param $connection Doctrine\DBAL\Connection
     */
    public function set_connection($connection)
    {
        $this->connection = $connection;
    }

    /**
     * Debug function Uncomment the lines if you want to debug
     */
    public static function debug()
    {
        $args = func_get_args();
        // Do something with the arguments
        if ($args[1] == 'query' || $args[1] == 'prepare')
        {
            // echo '<pre>';
            // echo($args[2]);
            // echo self :: $query_counter;
            // echo '</pre>';
            // self :: $query_counter++;
        }
    }

    /*
     * A workaround to the fact that MDB2 hardcodes a default PEAR_Error error mode of PEAR_ERROR_RETURN
     */
    public function error_handling($error)
    {
        // I don't care about performance, this should not happen more than once per request
        if (! self :: $error_log)
        {
            $logfile = Path :: get_log_path() . '/doctrine_errors.log';
            self :: $error_log = new FileLogger($logfile, true);
        }

        $message = "[Message: {$error->getMessage()}] [Information: { USER INFO GOES HERE}]";
        self :: $error_log->log_message($message);

        // throw new Exception("[Message: ".$err->getMessage()."] [Information: ".$err->getUserInfo()."]",
        // $err->getCode());
    }

    /**
     * Escapes a column name in accordance with the database type.
     *
     * @param $name string The column name.
     * @param $table_alias String The alias of the table the coloumn is in
     * @return string The escaped column name.
     */
    public static function escape_column_name($name, $table_alias = null, $connection = null)
    {
        if (is_null($connection))
        {
            $connection = DoctrineConnection :: get_instance()->get_connection();
        }

        $quoted_name = $connection->quoteIdentifier($name);
        if (! empty($table_alias))
        {
            return $connection->quoteIdentifier($table_alias) . '.' . $quoted_name;
        }
        else
        {
            return $quoted_name;
        }
    }

    /**
     * Expands a table identifier to the real table name. Currently, this method prefixes the given table name with the
     * user-defined prefix, if any.
     *
     * @param $name string The table identifier.
     * @return string The actual table name.
     */
    public function get_table_name($name)
    {
        return $this->prefix . $name;
    }

    /**
     * Escapes a table name in accordance with the database type.
     *
     * @param $name string The table identifier as provided by the data class.
     * @return string The escaped table name INCLUDING the application prefix.
     */
    public function escape_table_name($name)
    {
        return $this->connection->quoteIdentifier($this->get_table_name($name));
    }

    /**
     * Maps a record to an object
     *
     * @param $record Record a record from the database
     * @param $class String Class to create new object
     * @return new object from type Class
     * @deprecated Use DataClass :: factory()
     */
    public function record_to_object($record, $class_name)
    {
        return DataClass :: factory($class_name, $record);
    }

    public function storage_unit_exist($name)
    {
        return $this->get_connection()->getSchemaManager()->tablesExist(array($this->get_table_name($name)));
    }

    public function database_exists($name)
    {
        return in_array($name, $this->get_connection()->getSchemaManager()->listDatabases());
    }

    /**
     * Creates a storage unit in the system
     *
     * @param $name String the table name
     * @param $properties Array the table properties
     * @param $indexes Array the table indexes
     * @return true if the storage unit is succesfully created
     */
    public function create_storage_unit($name, $properties, $indexes)
    {
        try
        {
            $table_name = $this->get_table_name($name);

            if ($this->get_connection()->getSchemaManager()->tablesExist(array($table_name)))
            {
                $this->drop_storage_unit($name);
            }

            $schema = new \Doctrine\DBAL\Schema\Schema();
            $table = $schema->createTable($table_name);

            foreach ($properties as $property => $attributes)
            {
                // $type = self :: parse_storage_unit_property_type($attributes);
                // $options = self :: parse_storage_unit_attributes($attributes);

                // $table->addColumn($property, $type, $options);

                switch ($attributes['type'])
                {
                    case 'text' :
                        if ($attributes['length'] && $attributes['length'] <= 255)
                        {
                            $attributes['type'] = 'string';
                        }
                        break;
                }

                $type = self :: parse_storage_unit_property_type($attributes);
                $options = self :: parse_storage_unit_attributes($attributes);

                $table->addColumn($property, $attributes['type'], $options);

                if ($options['autoincrement'] == true)
                {
                    $name = StorageAliasGenerator :: get_instance()->get_constraint_name(
                        $table_name,
                        $name,
                        StorageAliasGenerator :: TYPE_CONSTRAINT);
                    $table->setPrimaryKey(array($property), $name);
                }
            }

            foreach ($indexes as $index => $attributes)
            {
                $name = StorageAliasGenerator :: get_instance()->get_constraint_name(
                    $table_name,
                    $index,
                    StorageAliasGenerator :: TYPE_CONSTRAINT);

                switch ($attributes['type'])
                {
                    case 'primary' :
                        $table->setPrimaryKey(array_keys($attributes['fields']), $name);
                        break;
                    case 'unique' :
                        $table->addUniqueIndex(array_keys($attributes['fields']), $name);
                        break;
                    default :
                        $table->addIndex(array_keys($attributes['fields']), $name);
                        break;
                }
            }

            foreach ($schema->toSql($this->get_connection()->getDatabasePlatform()) as $query)
            {
                $statement = $this->get_connection()->query($query);

                if ($statement instanceof \PDOException)
                {
                    $this->error_handling($statement);
                    return false;
                }
            }

            return true;
        }
        catch (\Exception $exception)
        {
            return false;
        }
    }

    public static function parse_storage_unit_property_type($attributes)
    {
        switch ($attributes['type'])
        {
            case 'text' :
                if ($attributes['length'] && $attributes['length'] <= 255)
                {
                    return 'string';
                }
                else
                {
                    return $attributes['type'];
                }
                break;
            case 'integer' :
                if (is_null($attributes['length']))
                {
                    return 'integer';
                }
                elseif ($attributes['length'] <= 2)
                {
                    return 'smallint';
                }
                elseif ($attributes['length'] <= 9)
                {
                    return 'integer';
                }
                else
                {
                    return 'bigint';
                }
            default :
                return $attributes['type'];
                break;
        }
    }

    public static function parse_storage_unit_attributes($attributes)
    {
        $options = array();

        foreach ($attributes as $attribute => $value)
        {
            switch ($attribute)
            {
                case 'length' :
                    $options[$attribute] = (is_numeric($value) ? (int) $value : null);
                    break;
                case 'unsigned' :
                    $options[$attribute] = ($value == 1 ? true : false);
                    break;
                case 'fixed' :
                    if ($attributes['type'] != 'string')
                    {
                        $options[$attribute] = ($value == 1 ? true : false);
                    }
                    break;
                case 'notnull' :
                    $options[$attribute] = ($value == 1 ? true : false);
                    break;
                case 'default' :
                    $options[$attribute] = (! is_numeric($value) && empty($value) ? null : $value);
                    break;
                case 'autoincrement' :
                    $options[$attribute] = ($value == 'true' ? true : false);
                    break;
            }
        }

        if (! isset($options['notnull']))
        {
            $options['notnull'] = false;
        }

        return $options;
    }

    /**
     * Retrieves the next id for a given table
     *
     * @param $table_name String
     * @return Int the id
     */
    public function get_next_id($table_name)
    {
        $id = $this->connection->nextID($this->get_table_name($table_name));
        return $id;
    }

    public function get_better_next_id($table_name, $column)
    {
        return null;
    }

    /**
     *
     * @return True if creation is successfull or false
     */
    public function create($object, $auto_id = true)
    {
        if ($object instanceof CompositeDataClass)
        {
            $parent_class = $object :: parent_class_name();
            $object_table = $parent_class :: get_table_name();
        }
        else
        {
            $object_table = $object->get_table_name();
        }

        $props = array();
        foreach ($object->get_default_properties() as $key => $value)
        {
            $props[self :: escape_column_name($key)] = $value;
        }

        if ($auto_id && in_array('id', $object->get_default_property_names()))
        {
            $props[self :: escape_column_name('id')] = $this->get_better_next_id($object_table, 'id');
        }

        try
        {
            $result = $this->connection->insert($this->get_table_name($object_table), $props);
            if ($auto_id && in_array('id', $object->get_default_property_names()))
            {
                $object->set_id($this->connection->lastInsertId($this->get_table_name($object_table)));
            }

            if ($object instanceof CompositeDataClass && $object :: is_extended())
            {
                $props = array();
                foreach ($object->get_additional_properties() as $key => $value)
                {
                    $props[self :: escape_column_name($key)] = $value;
                }
                $props[self :: escape_column_name('id')] = $object->get_id();

                try
                {
                    $result = $this->connection->insert($this->get_table_name($object->get_table_name()), $props);
                }
                catch (\Exception $exception)
                {
                    $this->error_handling($exception);
                    return false;
                }
            }

            return true;
        }
        catch (\Exception $exception)
        {
            $this->error_handling($exception);
            return false;
        }
    }

    public function create_record($class_name, $record)
    {
        try
        {
            $result = $this->connection->insert($this->get_table_name($class_name :: get_table_name()), $record);
        }
        catch (\Exception $exception)
        {
            $this->error_handling($exception);
            return false;
        }

        return true;
    }

    /**
     * Update functionality (can only be used when table has an ID)
     *
     * @param $object Object the object that has to be updated
     * @param $table_name String the table name
     * @param $condition Condition The condition for the item that has to be updated
     * @return True if update is successfull
     */
    public function update($object, $condition = false)
    {
        if ($object instanceof CompositeDataClass)
        {
            $parent_class = $object :: parent_class_name();
            $object_table = $parent_class :: get_table_name();
        }
        else
        {
            $object_table = $object->get_table_name();
        }

        $query_builder = $this->connection->createQueryBuilder();
        $query_builder->update($this->get_table_name($object_table), $this->get_alias($object_table));

        foreach ($object->get_default_properties() as $key => $value)
        {
            $query_builder->set(self :: escape_column_name($key), $this->escape($value));
        }

        if ($condition)
        {
            $query_builder->where(DoctrineConditionTranslator :: render($condition));
        }
        else
        {
            throw new Exception('Cannot update records without a condition');
        }

        $statement = $this->get_connection()->query($query_builder->getSQL());

        if ($statement instanceof \PDOException)
        {
            $this->error_handling($statement);
            return false;
        }

        if ($object instanceof CompositeDataClass && $object :: is_extended())
        {
            $query_builder = $this->connection->createQueryBuilder();
            $query_builder->update($this->get_table_name($object->get_table_name()));

            $props = array();
            foreach ($object->get_additional_properties() as $key => $value)
            {
                $query_builder->set(self :: escape_column_name($key), $this->escape($value));
            }

            if ($condition)
            {
                $query_builder->where(DoctrineConditionTranslator :: render($condition));
            }
            else
            {
                throw new Exception('Cannot update records without a condition');
            }

            $statement = $this->get_connection()->query($query_builder->getSQL());

            if ($statement instanceof \PDOException)
            {
                $this->error_handling($statement);
                return false;
            }
        }

        return true;
    }

    public function update_objects($table_name, $properties = array(), $condition, $offset = null, $max_objects = null,
        $order_by = array())
    {
        if (count($properties) > 0)
        {
            $query_builder = $this->connection->createQueryBuilder();
            $query_builder->update($this->get_table_name($table_name));

            foreach ($properties as $column => $property)
            {
                $query_builder->set(self :: escape_column_name($column), $this->escape($property));
            }

            if ($condition)
            {
                $query_builder->where(DoctrineConditionTranslator :: render($condition));
            }
            else
            {
                throw new Exception('Cannot update records without a condition');
            }

            $statement = $this->get_connection()->query($query_builder->getSQL());

            if (! $statement instanceof \PDOException)
            {
                return true;
            }
            else
            {
                $this->error_handling($statement);
                return false;
            }
        }
        return true;
    }

    /**
     * Updates a given table with the given properties for a given condition, offset, count and order_by
     *
     * @param string $class
     * @param DataClassProperties $properties
     * @param Condition $condition
     *
     * @return bool
     *
     * @throws \Exception
     */
    public function updates($class, $properties, $condition)
    {
        if (count($properties->get()) > 0)
        {
            $query_builder = $this->connection->createQueryBuilder();
            $query_builder->update(
                $this->get_table_name($class :: get_table_name()),
                $this->get_alias($class :: get_table_name()));

            foreach ($properties->get() as $data_class_property)
            {
                $query_builder->set(
                    DoctrineConditionVariableTranslator :: render($data_class_property->get_property()),
                    DoctrineConditionVariableTranslator :: render($data_class_property->get_value()));
            }

            if ($condition)
            {
                $query_builder->where(DoctrineConditionTranslator :: render($condition));
            }
            else
            {
                throw new Exception('Cannot update records without a condition');
            }

            $statement = $this->get_connection()->query($query_builder->getSQL());

            if (! $statement instanceof \PDOException)
            {
                return true;
            }
            else
            {
                $this->error_handling($statement);
                return false;
            }
        }
        return true;
    }

    /**
     * Deletes an object from a table with a given condition
     *
     * @param $table_name String
     * @param $condition Condition
     * @return true if deletion is successfull
     */
    public function delete($table_name, $condition)
    {
        $query = 'DELETE FROM ' . $this->escape_table_name($table_name);

        if (isset($condition))
        {
            $query .= ' WHERE ' . DoctrineConditionTranslator :: render($condition, $this->get_table_name($table_name));
        }

        $res = $this->connection->exec($query);

        if (MDB2 :: isError($res))
        {
            $this->mdb2_error_handling($res);
            return false;
        }
        return true;
    }

    /**
     * Deletes an object from a storage unit with a given condition
     *
     * @param $class string
     * @param $condition Condition
     * @return boolean
     */
    public function new_delete($class, $condition)
    {
        $query_builder = $this->connection->createQueryBuilder();
        $query_builder->delete(
            $this->escape_table_name($class :: get_table_name()),
            $this->get_alias($class :: get_table_name()));

        if (isset($condition))
        {
            $query_builder->where(DoctrineConditionTranslator :: render($condition));
        }

        $statement = $this->get_connection()->query($query_builder->getSQL());

        if (! $statement instanceof \PDOException)
        {
            return true;
        }
        else
        {
            $this->error_handling($statement);
            return false;
        }
    }

    /**
     * Deletes the objects of a given table
     *
     * @param $table_name String
     * @param $condition Condition the condition
     * @return boolean
     */
    public function delete_objects($table_name, $condition = null)
    {
        $query = 'DELETE FROM ' . $this->escape_table_name($table_name);

        if (isset($condition))
        {
            $query .= ' WHERE ' . DoctrineConditionTranslator :: render($condition, $this->get_table_name($table_name));
        }

        $statement = $this->get_connection()->query($query);

        if (! $statement instanceof \PDOException)
        {
            return true;
        }
        else
        {
            $this->error_handling($statement);
            return false;
        }
    }

    /**
     * Drop a given storage unit
     *
     * @param $table_name String
     * @return boolean
     */
    public function drop_storage_unit($table_name)
    {
        $table_name = $this->get_table_name($table_name);

        $schema = new \Doctrine\DBAL\Schema\Schema(array(new \Doctrine\DBAL\Schema\Table($table_name)));

        $new_schema = clone $schema;
        $new_schema->dropTable($table_name);

        $sql = $schema->getMigrateToSql($new_schema, $this->get_connection()->getDatabasePlatform());

        foreach ($sql as $query)
        {
            $statement = $this->get_connection()->query($query);

            if ($statement instanceof \PDOException)
            {
                $this->error_handling($statement);
                return false;
            }
        }

        return true;
    }

    public function rename_storage_unit($old_name, $new_name)
    {
        $query = 'ALTER TABLE ' . $this->escape_table_name($old_name) . ' RENAME TO ' .
             $this->escape_table_name($new_name);

        $statement = $this->get_connection()->query($query);

        if (! $statement instanceof \PDOException)
        {
            return true;
        }
        else
        {
            $this->error_handling($statement);
            return false;
        }
    }

    public function copy_storage_unit($from_name, $to_name, $create_storage_unit = true)
    {
        if ($create_storage_unit)
        {
            $query = 'CREATE TABLE ' . $this->escape_table_name($to_name) . ' LIKE ' .
                 $this->escape_table_name($from_name) . ' ; ';
        }

        $query .= 'INSERT ' . $this->escape_table_name($to_name) . ' SELECT * FROM ' .
             $this->escape_table_name($from_name) . ';';

        $statement = $this->get_connection()->query($query);

        if (! $statement instanceof \PDOException)
        {
            return true;
        }
        else
        {
            $this->error_handling($statement);
            return false;
        }
    }

    public function move_record($old_class, $new_class, $condition)
    {
        $query_builder = $this->connection->createQueryBuilder();
        $query_builder->addSelect($this->get_alias($this->prepare_table_name($old_class)) . '.*');
        $query_builder = $this->process_condition($query_builder, $old_class, $condition);
        $query = 'INSERT ' . $this->prepare_table_name($new_class) . ' ' . $query_builder->getSQL();

        $statement = $this->get_connection()->query($query);

        if (! $statement instanceof \PDOException)
        {
            return $this->delete($old_class :: table_name(), $condition);
        }
        else
        {
            $this->error_handling($statement);
            return false;
        }
    }

    /**
     *
     * @param integer $type
     * @param string $table_name
     * @param string $property
     * @param multitype:mixed $attributes
     * @return boolean
     */
    public function alter_storage_unit($type, $table_name, $property, $attributes = array())
    {
        try
        {
            if ($type == DataManager :: ALTER_STORAGE_UNIT_DROP)
            {
                $column = new \Doctrine\DBAL\Schema\Column($property);
                $query = 'ALTER TABLE ' . $this->escape_table_name($table_name) . ' DROP COLUMN ' .
                     $column->getQuotedName($this->get_connection()->getDatabasePlatform());
            }
            else
            {
                $column = new \Doctrine\DBAL\Schema\Column(
                    $property,
                    \Doctrine\DBAL\Types\Type :: getType(self :: parse_storage_unit_property_type($attributes)),
                    self :: parse_storage_unit_attributes($attributes));

                // Column declaration translation-code more or less directly from Doctrine since it doesn't support
                // altering
                // tables (yet)
                $columns = array();
                /* @var \Doctrine\DBAL\Schema\Column $column */
                $columnData = array();

                if (isset($attributes['name']) && $type == DataManager :: ALTER_STORAGE_UNIT_CHANGE)
                {
                    $old_name = $column->getQuotedName($this->get_connection()->getDatabasePlatform());
                    $columnData['name'] = $old_name . ' ' . $attributes['name'];
                }
                elseif ($type == DataManager :: ALTER_STORAGE_UNIT_CHANGE)
                {
                    $name = $column->getQuotedName($this->get_connection()->getDatabasePlatform());
                    $columnData['name'] = $name . ' ' . $name;
                }
                elseif ($type == DataManager :: ALTER_STORAGE_UNIT_ADD)
                {
                    $name = $column->getQuotedName($this->get_connection()->getDatabasePlatform());
                    $columnData['name'] = $name;
                }

                $columnData['type'] = $column->getType();
                $columnData['length'] = $column->getLength();
                $columnData['notnull'] = $column->getNotNull();
                $columnData['fixed'] = $column->getFixed();
                $columnData['unique'] = false; // TODO: what do we do about this?
                $columnData['version'] = ($column->hasPlatformOption("version")) ? $column->getPlatformOption('version') : false;

                if (strtolower($columnData['type']) == "string" && $columnData['length'] === null)
                {
                    $columnData['length'] = 255;
                }

                $columnData['unsigned'] = $column->getUnsigned();
                $columnData['precision'] = $column->getPrecision();
                $columnData['scale'] = $column->getScale();
                $columnData['default'] = $column->getDefault();
                $columnData['columnDefinition'] = $column->getColumnDefinition();
                $columnData['autoincrement'] = $column->getAutoincrement();

                $columnData['comment'] = $column->getComment();
                if ($this->get_connection()->getDatabasePlatform()->isCommentedDoctrineType($column->getType()))
                {
                    $columnData['comment'] .= $this->get_connection()->getDatabasePlatform()->getDoctrineTypeComment(
                        $column->getType());
                }

                $columns = array($columnData['name'] => $columnData);

                $fields_query = $this->get_connection()->getDatabasePlatform()->getColumnDeclarationListSQL($columns);

                $action = $type == DataManager :: ALTER_STORAGE_UNIT_CHANGE ? 'CHANGE' : 'ADD';
                $query = 'ALTER TABLE ' . $this->escape_table_name($table_name) . ' ' . $action . ' COLUMN ' .
                     $fields_query;

                if ($type == DataManager :: ALTER_STORAGE_UNIT_ADD && $columnData['autoincrement'])
                {
                    $query .= ', ADD PRIMARY KEY(' . $columnData['name'] . ')';
                }
            }

            $statement = $this->get_connection()->query($query);

            if (! $statement instanceof \PDOException)
            {
                return true;
            }
            else
            {
                $this->error_handling($statement);
                return false;
            }
        }
        catch (\Exception $exception)
        {
            $this->error_handling($exception);
            return false;
        }
    }

    /**
     *
     * @param integer $type
     * @param string $table_name
     * @param string $name
     * @param multitype:string $columns
     * @return boolean
     */
    public function alter_storage_unit_index($type, $table_name, $name = null, $columns = array())
    {
        $query = 'ALTER TABLE ' . $this->escape_table_name($table_name) . ' ';

        switch ($type)
        {
            case DataManager :: ALTER_STORAGE_UNIT_DROP_PRIMARY_KEY :
                $query .= 'DROP PRIMARY KEY';
                break;
            case DataManager :: ALTER_STORAGE_UNIT_DROP_INDEX :

                if (is_null($name))
                {
                    return false;
                }

                $query .= 'DROP INDEX ' . $name;
                break;
            case DataManager :: ALTER_STORAGE_UNIT_ADD_PRIMARY_KEY :
                $query .= 'ADD PRIMARY KEY(' . implode(', ', array_unique($columns)) . ')';
                break;
            case DataManager :: ALTER_STORAGE_UNIT_ADD_INDEX :
                $query .= 'ADD ' . $this->get_connection()->getDatabasePlatform()->getIndexDeclarationSQL(
                    $name,
                    new \Doctrine\DBAL\Schema\Index($name, $columns, false, false));
                break;
            case DataManager :: ALTER_STORAGE_UNIT_ADD_UNIQUE :
                $query .= 'ADD ' . $this->get_connection()->getDatabasePlatform()->getIndexDeclarationSQL(
                    $name,
                    new \Doctrine\DBAL\Schema\Index($name, $columns, true, false));
                break;
        }

        $statement = $this->get_connection()->query($query);

        if (! $statement instanceof \PDOException)
        {
            return true;
        }
        else
        {
            $this->error_handling($statement);
            return false;
        }
    }

    /**
     * Counts the objects in a storage unit with a given condition
     *
     * @param $table_name string
     * @param $condition Condition
     * @return int
     *
     * @deprecated Use count() in implementations, use the PackageDataManager :: count() in applications
     */
    public function count_objects($table_name, $condition = null)
    {
        $query_builder = $this->connection->createQueryBuilder();
        $query_builder->addSelect('COUNT(1)');
        $query_builder->from($this->escape_table_name($table_name), $this->get_alias($table_name));

        return $this->count_result_set($query_builder, $table_name, $condition);
    }

    /**
     * Counts the objects in a storage unit with a given condition
     *
     * @param $class string
     * @param $parameters \common\libraries\DataClassCountParameters
     */
    public function count($class, $parameters)
    {
        $query_builder = $this->connection->createQueryBuilder();
        $query_builder->addSelect('COUNT(1)');
        $query_builder->from(
            $this->escape_table_name($this->prepare_table_name($class)),
            $this->get_alias($this->prepare_table_name($class)));

        $query_builder = $this->process_parameters($query_builder, $class, $parameters);

//         var_dump($query_builder->getSQL());

        $statement = $this->get_connection()->query($query_builder->getSQL());

        if (! $statement instanceof \PDOException)
        {
            $record = $statement->fetch(\PDO :: FETCH_NUM);
            return (int) $record[0];
        }
        else
        {
            $this->error_handling($statement);
            return false;
        }
    }

    /**
     *
     * @param $class string
     * @param $parameters \common\libraries\DataClassCountGroupedParameters
     * @return multitype:int
     */
    public function count_grouped($class, $parameters)
    {
        $query_builder = $this->connection->createQueryBuilder();
        foreach ($parameters->get_property()->get() as $property)
        {

            $query_builder->addSelect(DoctrineConditionVariableTranslator :: render($property));
        }
        $query_builder->addSelect('COUNT(1)');
        $query_builder->from(
            $this->escape_table_name($class :: get_table_name()),
            $this->get_alias($class :: get_table_name()));

        $query_builder = $this->process_parameters($query_builder, $class, $parameters);
        foreach ($parameters->get_property()->get() as $property)
        {
            $query_builder->addGroupBy(DoctrineConditionVariableTranslator :: render($property));
        }
        $query_builder->having(DoctrineConditionTranslator :: render($parameters->get_having()));
        $statement = $this->get_connection()->query($query_builder->getSQL());

        if (! $statement instanceof \PDOException)
        {
            $counts = array();
            while ($record = $statement->fetch(\PDO :: FETCH_NUM))
            {
                $counts[$record[0]] = $record[1];
            }

            $record = $statement->fetch(\PDO :: FETCH_NUM);

            return $counts;
        }
        else
        {
            $this->error_handling($statement);
            return false;
        }
    }

    /**
     *
     * @param $table_name string
     * @param $group string
     * @param $count string
     * @param $condition \common\libraries\Condition
     * @return multitype:int
     * @deprecated Use count_grouped() in implementations, use the PackageDataManager :: count_grouped() in applications
     */
    public function count_grouped_object($table_name, $group, $count, $condition = null)
    {
        $query = 'SELECT ' . self :: escape_column_name($group, $this->get_alias($table_name)) . ', COUNT(' .
             self :: escape_column_name($count, $this->get_alias($table_name)) . ') FROM ' .
             $this->escape_table_name($table_name) . ' AS ' . $this->get_alias($table_name);

        return $this->count_grouped_result_set($query, $table_name, $group, $condition);
    }

    public function count_grouped_result_set($query, $table_name, $group, $condition)
    {
        if (isset($condition))
        {
            $query .= ' WHERE ' . DoctrineConditionTranslator :: render($condition, $this->get_alias($table_name));
        }

        $query .= ' GROUP BY' . self :: escape_column_name($group, $this->get_alias($table_name));

        $res = $this->get_connection()->query($query);

        if (MDB2 :: isError($res))
        {
            $this->mdb2_error_handling($res);
            return false;
        }
        else
        {
            $counts = array();
            while ($record = $res->fetchRow(MDB2_FETCHMODE_ORDERED))
            {
                $counts[$record[0]] = $record[1];
            }
            $res->free();

            return $counts;
        }
    }

    public function count_result_set($query_builder, $table_name, $condition = null)
    {
        if (isset($condition))
        {
            $query_builder->where(DoctrineConditionTranslator :: render($condition, $this->get_alias($table_name)));
        }

        $statement = $this->get_connection()->query($query_builder->getSQL());

        if (! $statement instanceof \PDOException)
        {
            $record = $statement->fetch(\PDO :: FETCH_NUM);
            return (int) $record[0];
        }
        else
        {
            $this->error_handling($statement);
            return false;
        }
    }

    /**
     * Retrieves the objects of a given table
     *
     * @param $table_name String
     * @param $classname String The name of the class where the object has to be mapped to
     * @param $condition Condition the condition
     * @param $offset Int the starting offset
     * @param $max_objects Int the max amount of objects to be retrieved
     * @param $order_by Array(String) the list of column names that the objects have to be ordered by
     * @param $resultset String - Optional, the resultset to map the items to
     * @return ResultSet
     */
    public function retrieve_objects($table_name, $condition = null, $offset = null, $max_objects = null, $order_by = array(),
        $class_name = null)
    {
        $query_builder = $this->connection->createQueryBuilder();
        $query_builder->addSelect('*');
        $query_builder->from($this->escape_table_name($table_name), $this->get_alias($table_name));

        return $this->retrieve_object_set(
            $query_builder,
            $table_name,
            $condition,
            $offset,
            $max_objects,
            $order_by,
            $class_name);
    }

    private function process_composite_data_class_joins($query_builder, $class, $parameters)
    {
        if ($parameters->get_joins() instanceof Joins)
        {
            foreach ($parameters->get_joins()->get() as $join)
            {
                if (is_subclass_of($join->get_data_class(), CompositeDataClass :: class_name()))
                {
                    if (is_subclass_of($class, $join->get_data_class()))
                    {
                        $join_class = $join->get_data_class();
                        $data_manager = $join_class :: context() . '\DataManager';

                        $alias = $data_manager :: get_instance()->get_alias($join_class :: get_table_name());

                        $query_builder->addSelect($alias . '.*');
                    }
                }
            }
        }
    }

    /**
     * Retrieves dataclasses in a DataClassResultSet from the database
     *
     * @param $class string
     * @param DataClassRetrievesParameters $parameters
     *
     * @return \common\libraries\ResultSet
     */
    public function retrieves($class, DataClassRetrievesParameters $parameters)
    {
        return new DoctrineDataClassResultSet(
            $this,
            $this->get_records_result($this->build_retrieves_sql($class, $parameters), $class, $parameters),
            $class);
    }

    /**
     * Retrieves records from the database
     *
     * @param string $class
     * @param RecordRetrievesParameters $parameters
     *
     * @return DoctrineRecordResultSet
     */
    public function records($class, RecordRetrievesParameters $parameters)
    {
        return new DoctrineRecordResultSet(
            $this->get_records_result($this->build_records_sql($class, $parameters), $class, $parameters));
    }

    /**
     * Returns the records result of a given query
     *
     * @param $sql
     * @param $class
     * @param $parameters
     * @return mixed
     *
     * @throws DataClassNoResultException
     */
    public function get_records_result($sql, $class, $parameters)
    {
        // var_dump($sql);
        try
        {
            return $this->get_connection()->query($sql);
        }
        catch (\PDOException $exception)
        {
            $this->error_handling($exception);
            throw new DataClassNoResultException($class, $parameters);
        }
    }

    /**
     * Builds an sql statement to retrieve dataclasses
     *
     * @param string $class
     * @param DataClassRetrievesParameters $parameters
     *
     * @return string
     */
    public function build_retrieves_sql($class, DataClassRetrievesParameters $parameters)
    {
        $query_builder = $this->connection->createQueryBuilder();

        $query_builder->addSelect($this->get_alias($this->prepare_table_name($class)) . '.*');

        $this->process_composite_data_class_joins($query_builder, $class, $parameters);

        return $this->build_basic_records_sql($query_builder, $class, $parameters);
    }

    /**
     * Builds an sql statement to retrieve records
     *
     * @param string $class
     * @param RecordRetrievesParameters $parameters
     *
     * @return string
     */
    public function build_records_sql($class, RecordRetrievesParameters $parameters)
    {
        $query_builder = $this->connection->createQueryBuilder();

        $query_builder = $this->process_data_class_properties($query_builder, $class, $parameters->get_properties());
        $query_builder = $this->process_group_by($query_builder, $parameters->get_group_by());

        return $this->build_basic_records_sql($query_builder, $class, $parameters);
    }

    /**
     * Builds a basic sql statement for records retrieval
     *
     * @param QueryBuilder $query_builder
     * @param string $class
     * @param DataClassParameters $parameters
     *
     * @return string
     */
    public function build_basic_records_sql($query_builder, $class, $parameters)
    {
        $query_builder->from(
            $this->escape_table_name($this->prepare_table_name($class)),
            $this->get_alias($this->prepare_table_name($class)));

        $query_builder = $this->process_parameters($query_builder, $class, $parameters);
        $query_builder = $this->process_order_by($query_builder, $class, $parameters->get_order_by());
        $query_builder = $this->process_limit($query_builder, $parameters->get_count(), $parameters->get_offset());

        return $query_builder->getSQL();
    }

    /**
     *
     * @param $query_builder Doctrine\DBAL\Query\QueryBuilder
     * @param $table_name string
     * @param $condition Condition
     * @param $offset int
     * @param $max_objects int
     * @param $order_by multi:ObjectTableOrder
     * @return Ambigous <boolean, unknown>
     */
    public function retrieve_result($query_builder, $table_name, $condition = null, $offset = null, $max_objects = null,
        $order_by = array())
    {
        if (isset($condition))
        {
            $query_builder->where(DoctrineConditionTranslator :: render($condition, $this->get_alias($table_name)));
        }

        if (is_null($order_by))
        {
            $order_by = array();
        }
        elseif (! is_array($order_by))
        {
            $order_by = array($order_by);
        }

        foreach ($order_by as $order)
        {
            $query_builder->addOrderBy(
                self :: escape_column_name(
                    $order->get_property(),
                    ($order->alias_is_set() ? $order->get_alias() : $this->get_alias($table_name))),
                ($order->get_direction() == SORT_DESC ? 'DESC' : 'ASC'));
        }

        if (intval($max_objects) > 0)
        {
            $query_builder->setMaxResults(intval($max_objects));
        }

        if (intval($offset) > 0)
        {
            $query_builder->setFirstResult(intval($offset));
        }
        return $this->get_connection()->query($query_builder->getSQL());
    }

    public function retrieve_object_set($query_builder, $table_name, $condition = null, $offset = null, $max_objects = null,
        $order_by = array(), $class_name = null)
    {
        $statement = $this->retrieve_result($query_builder, $table_name, $condition, $offset, $max_objects, $order_by);
        if (is_null($class_name))
        {
            $called_class = get_called_class();
            $namespace = Utilities :: get_namespace_from_classname($called_class);

            $class_name = $namespace . '\\' . Utilities :: underscores_to_camelcase($table_name);
        }

        return new DoctrineDataClassResultSet($this, $statement, $class_name);
    }

    /*
     * broken function, does anyone care ? function retrieve_record_set($query, $table_name, $condition = null, $offset
     * = null, $max_objects = null, $order_by = array()) { return new RecordResultSet($this->retrieve_result($query,
     * $table_name, $condition, $offset, $max_objects, $order_by)); }
     */

    /**
     *
     * @param $table_name string
     * @param $column string
     * @param $condition \common\libraries\Condition
     * @return int
     * @deprecated Use retrieve_maximum_value() in implementations, use the PackageDataManager ::
     *             retrieve_maximum_value() in applications
     */
    public function retrieve_max_sort_value($table_name, $column, $condition = null)
    {
        $query = 'SELECT MAX(' . self :: escape_column_name($column) . ') as ' . self :: ALIAS_MAX_SORT . ' FROM ' . $this->escape_table_name(
            $table_name) . ' AS ' . $this->get_alias($table_name);

        if (isset($condition))
        {
            $query .= ' WHERE ' . DoctrineConditionTranslator :: render($condition, $this->get_alias($table_name));
        }

        $statement = $this->get_connection()->query($query);

        if (! $statement instanceof \PDOException)
        {
            $record = $statement->fetch(\PDO :: FETCH_NUM);
            return (int) $record[0];
        }
        else
        {
            $this->error_handling($statement);
            return false;
        }
    }

    /**
     *
     * @param $class string
     * @param $property string
     * @param $condition \common\libraries\Condition
     * @return int
     */
    public function retrieve_maximum_value($class, $property, $condition = null)
    {
        $query_builder = $this->connection->createQueryBuilder();
        $query_builder->addSelect(
            'MAX(' . self :: escape_column_name($property, $this->get_alias($class :: get_table_name())) . ') AS ' .
                 self :: ALIAS_MAX_SORT);
        $query_builder->from(
            $this->escape_table_name($class :: get_table_name()),
            $this->get_alias($class :: get_table_name()));

        if (isset($condition))
        {
            $query_builder->where(
                DoctrineConditionTranslator :: render($condition, $this->get_alias($class :: get_table_name())));
        }

        $statement = $this->get_connection()->query($query_builder->getSQL());

        if (! $statement instanceof \PDOException)
        {
            $record = $statement->fetch(\PDO :: FETCH_NUM);
            return (int) $record[0];
        }
        else
        {
            $this->error_handling($statement);
            return false;
        }
    }

    /**
     *
     * @param $table_name string
     * @param $column string
     * @param $condition \common\libraries\Condition
     * @return int
     * @deprecated Use PackageDataManager :: retrieve_next_value()
     */
    public function retrieve_next_sort_value($table_name, $column, $condition = null)
    {
        return $this->retrieve_max_sort_value($table_name, $column, $condition) + 1;
    }

    public function truncate_storage_unit($table_name, $optimize = true)
    {
        $query_builder = $this->connection->createQueryBuilder();
        $query_builder->delete($table_name);

        $statement = $this->get_connection()->query($query_builder->getSQL());

        if (! $statement instanceof \PDOException)
        {
            if ($optimize)
            {
                return $this->optimize_storage_unit($table_name);
            }
            else
            {
                return true;
            }
        }
        else
        {
            $this->error_handling($statement);
            return false;
        }
    }

    public function optimize_storage_unit($table_name)
    {
        return true;
    }

    public function retrieve_record($table_name, $condition = null, $order_by = array())
    {
        $query_builder = $this->connection->createQueryBuilder();
        $query_builder->addSelect('*');
        $query_builder->from($this->escape_table_name($table_name), $this->get_alias($table_name));

        return $this->retrieve_row($query_builder, $table_name, $condition, $order_by);
    }

    /**
     *
     * @param $query_builder Doctrine\DBAL\Query\QueryBuilder
     * @param $table_name string
     * @param $condition Condition
     * @param $order_by multi:ObjectTableOrder
     * @return boolean \PDOException
     */
    public function retrieve_row($query_builder, $table_name, $condition = null, $order_by = array())
    {
        if (isset($condition))
        {
            $query_builder->where(DoctrineConditionTranslator :: render($condition, $this->get_alias($table_name)));
        }

        foreach ($order_by as $order)
        {
            $query_builder->addOrderBy(
                self :: escape_column_name(
                    $order->get_property(),
                    ($order->alias_is_set() ? $order->get_alias() : $this->get_alias($table_name))),
                ($order->get_direction() == SORT_DESC ? 'DESC' : 'ASC'));
        }

        $query_builder->setFirstResult(0);
        $query_builder->setMaxResults(1);

        $statement = $this->get_connection()->query($query_builder->getSQL());

        if (! $statement instanceof \PDOException)
        {
            $record = $statement->fetch(\PDO :: FETCH_ASSOC);
        }
        else
        {
            $this->error_handling($statement);
            return false;
        }

        if ($record instanceof \PDOException)
        {
            $this->error_handling($record);
            return false;
        }

        if (is_null($record))
        {
            return false;
        }

        foreach ($record as &$field)
        {
            if (is_resource($field))
            {
                $data = '';
                while (! feof($field))
                    $data .= fread($field, 1024);
                $field = $data;
            }
        }

        if ($record)
        {
            return $record;
        }
        else
        {
            return false;
        }
    }

    /**
     *
     * @param $class string
     * @param $parameters \common\libraries\DataClassResultParameters
     * @throws DataClassNoResultException
     * @return multitype:string
     */
    public function retrieve($class, $parameters = null)
    {
        $query_builder = $this->connection->createQueryBuilder();

        $query_builder->addSelect($this->get_alias($this->prepare_table_name($class)) . '.*');

        $this->process_composite_data_class_joins($query_builder, $class, $parameters);

        return $this->fetch_record($query_builder, $class, $parameters);
    }

    private function prepare_table_name($class)
    {
        if (is_subclass_of($class, CompositeDataClass :: class_name()) &&
             get_parent_class($class) == CompositeDataClass :: class_name())
        {
            $table_name = $class :: get_table_name();
        }
        elseif (is_subclass_of($class, CompositeDataClass :: class_name()) && $class :: is_extended())
        {
            $table_name = $class :: get_table_name();
        }
        elseif (is_subclass_of($class, CompositeDataClass :: class_name()) && ! $class :: is_extended())
        {
            $parent = $class :: parent_class_name();
            $table_name = $parent :: get_table_name();
        }
        else
        {
            $table_name = $class :: get_table_name();
        }
        return $table_name;
    }

    public function record($class, $parameters = null)
    {
        $query_builder = $this->connection->createQueryBuilder();

        $group_by = $parameters->get_group_by();
        if ($group_by instanceof GroupBy)
        {
            foreach ($group_by->get_group_by() as $group_by_variable)
            {
                $query_builder->addGroupBy(DoctrineConditionVariableTranslator :: render($group_by_variable));
            }
        }

        if ($parameters->get_properties() instanceof DataClassProperties)
        {

            foreach ($parameters->get_properties()->get() as $condition_variable)
            {
                $query_builder->addSelect(DoctrineConditionVariableTranslator :: render($condition_variable));
            }
        }
        else
        {

            return $this->retrieve($class, $parameters);
        }

        return $this->fetch_record($query_builder, $class, $parameters);
    }

    private function fetch_record($query_builder, $class, $parameters)
    {
        $query_builder->from(
            $this->escape_table_name($this->prepare_table_name($class)),
            $this->get_alias($this->prepare_table_name($class)));

        $query_builder = $this->process_parameters($query_builder, $class, $parameters);

        $order_by = $parameters->get_order_by();

        foreach ($order_by as $order)
        {
            $query_builder->addOrderBy(
                self :: escape_column_name(
                    $order->get_property(),
                    ($order->alias_is_set() ? $order->get_alias() : $this->get_alias($class :: get_table_name()))),
                ($order->get_direction() == SORT_DESC ? 'DESC' : 'ASC'));
        }

        $query_builder->setFirstResult(0);
        $query_builder->setMaxResults(1);

        $statement = $this->get_connection()->query($query_builder->getSQL());

        if (! $statement instanceof \PDOException)
        {
            $record = $statement->fetch(\PDO :: FETCH_ASSOC);
        }
        else
        {

            $this->error_handling($statement);
            throw new DataClassNoResultException($class, $parameters);
        }

        if ($record instanceof \PDOException)
        {
            $this->error_handling($record);
            throw new DataClassNoResultException($class, $parameters);
        }

        if (is_null($record) || ! is_array($record) || empty($record))
        {
            throw new DataClassNoResultException($class, $parameters);
        }

        foreach ($record as &$field)
        {
            if (is_resource($field))
            {
                $data = '';
                while (! feof($field))
                    $data .= fread($field, 1024);
                $field = $data;
            }
        }
        return $record;
    }

    /**
     *
     * @param $table_name string
     * @param $condition Condition
     * @param $order_by array
     * @param $class_name string
     * @return Ambigous <\common\libraries\DataClass|boolean>
     * @deprecated Use PackageDataManager :: retrieve()
     */
    public function retrieve_object($table_name, $condition = null, $order_by = array(), $class_name = null)
    {
        $record = $this->retrieve_record($table_name, $condition, $order_by);
        if (is_null($class_name))
        {
            $called_class = get_called_class();
            $namespace = Utilities :: get_namespace_from_classname($called_class);

            $class_name = $namespace . '\\' . Utilities :: underscores_to_camelcase($table_name);
        }

        if ($record)
        {
            return self :: record_to_object($record, $class_name);
        }
        else
        {
            return false;
        }
    }

    public function retrieve_distinct($table_name, $column_name, $condition = null)
    {
        $query_builder = $this->connection->createQueryBuilder();
        $query_builder->addSelect('DISTINCT(' . self :: escape_column_name($column_name) . ')');
        $query_builder->from($this->escape_table_name($table_name), $this->get_alias($table_name));

        $query_builder = $this->process_condition($query_builder, $class, $condition);

        $statement = $this->get_connection()->query($query_builder->getSQL());

        if (! $statement instanceof \PDOException)
        {
            $distinct_elements = array();
            while ($record = $statement->fetch(\PDO :: FETCH_ASSOC))
            {
                $distinct_elements[] = $record[$column_name];
            }
            return $distinct_elements;
        }
        else
        {
            $this->error_handling($statement);
            return false;
        }
    }

    /**
     *
     * @param $class string
     * @param $property string
     * @param $condition \common\libraries\Condition
     * @return multitype:string
     */
    public function distinct($class, DataClassDistinctParameters $parameters)
    {
        $query_builder = $this->connection->createQueryBuilder();
        $query_builder->addSelect(
            'DISTINCT(' . self :: escape_column_name(
                $parameters->get_property(),
                $this->get_alias($class :: get_table_name())) . ')');
        $query_builder->from(
            $this->escape_table_name($class :: get_table_name()),
            $this->get_alias($class :: get_table_name()));

        $query_builder = $this->process_parameters($query_builder, $class, $parameters);
        $statement = $this->get_connection()->query($query_builder->getSQL());

        if (! $statement instanceof \PDOException)
        {
            $distinct_elements = array();
            while ($record = $statement->fetch(\PDO :: FETCH_NUM))
            {
                $distinct_elements[] = $record[0];
            }
            return $distinct_elements;
        }
        else
        {
            $this->error_handling($statement);
            return false;
        }
    }

    /**
     *
     * @param $class string
     * @param $parameters \common\libraries\DataClassCountDistinctParameters
     * @return int
     */
    public function new_count_distinct($class, DataClassCountDistinctParameters $parameters)
    {
        $query_builder = $this->connection->createQueryBuilder();
        $query_builder->addSelect(
            'COUNT(DISTINCT(' . self :: escape_column_name(
                $parameters->get_property(),
                $this->get_alias($class :: get_table_name())) . '))');
        $query_builder->from(
            $this->escape_table_name($class :: get_table_name()),
            $this->get_alias($class :: get_table_name()));

        $query_builder = $this->process_parameters($query_builder, $class, $parameters);
        $statement = $this->get_connection()->query($query_builder->getSQL());

        if (! $statement instanceof \PDOException)
        {
            $record = $statement->fetch(\PDO :: FETCH_NUM);
            return (int) $record[0];
        }
        else
        {
            $this->error_handling($statement);
            return false;
        }
    }

    /**
     *
     * @param $table_name string
     * @param $column_name string
     * @param $condition \common\libraries\Condition
     * @return int
     * @deprecated Use new_count_distinct() in implementations, use the PackageDataManager :: count_distinct() in
     *             applications
     */
    public function count_distinct($table_name, $column_name, $condition = null)
    {
        $query = 'SELECT COUNT(DISTINCT(' . self :: escape_column_name($column_name) . ')) FROM ' . $this->escape_table_name(
            $table_name) . ' AS ' . $this->get_alias($table_name);

        if (isset($condition))
        {
            $query .= ' WHERE ' . DoctrineConditionTranslator :: render($condition, $this->get_alias($table_name));
        }

        $res = $this->get_connection()->query($query);
        if (MDB2 :: isError($res))
        {
            $this->mdb2_error_handling($res);
            return false;
        }

        $record = $res->fetchRow(MDB2_FETCHMODE_ORDERED);
        $res->free();
        return $record[0];
    }

    /**
     * Returns the alias of the table name Please note that this table name should NOT be escaped using the
     * escape_table_name method of the Database class
     *
     * @param $table_name String
     * @return String the alias
     */
    public function get_alias($table_name)
    {
        return StorageAliasGenerator :: get_instance()->get_table_alias($table_name, $this->get_prefix());
    }

    public function get_constraint_name($name)
    {
        $possible_name = '';
        $parts = explode('_', $name);
        foreach ($parts as & $part)
        {
            $possible_name .= $part{0};
        }

        return $possible_name;
    }

    public static function quote($value, $type = null, $quote = true, $escape_wildcards = false, $connection = null)
    {
        if (is_null($connection))
        {
            $connection = DoctrineConnection :: get_instance()->get_connection();
        }

        return $connection->quote($value, $type, $quote, $escape_wildcards);
    }

    public function escape($text, $escape_wildcards = false)
    {
        return $this->connection->quote($text);
    }

    /**
     * Delegate method for the exec function Exec is used when the query does not expect a resultset but only true /
     * false
     *
     * @param $query String
     * @return boolean
     */
    public function exec($query)
    {
        return $this->connection->exec($query);
    }

    public function set_limit($limit, $offset = 0)
    {
        if ($offset && ! $limit)
            throw new Exception('Supplying an offset without a limit is not supported by MDB2!');

        $result = $this->connection->setLimit($limit, $offset);
        if (MDB2 :: isError($result))
        {
            $this->mdb2_error_handling($result);
            return false;
        }
    }

    /**
     * ************************************************************************ FUNCTIONALITY THAT ENABLES NESTED TREES
     * VIA NESTED_TREE_NODE.CLASS.PHP * ************************************************************************
     */

    /**
     * Counts the children of a tree node
     *
     * @param $node NestedTreeNode - the node
     * @param $condition Condition - additional conditions
     */
    public function nested_tree_count_children($node, $recursive = false, $condition = null)
    {
        $condition = $this->nested_tree_build_children_condition($node, $recursive, $condition);
        return $this->count_objects($node->get_table_name(), $condition);
    }

    /**
     * Retrieves the children of a tree node
     *
     * @param $node NestedTreeNode - the node
     * @param $recursive boolean - if put on true, every child will be retrieved, even those who are not directly
     *            connected with parent_id
     * @param $condition Condition - additional conditions
     */
    public function nested_tree_get_children($node, $recursive = false, $condition = null)
    {
        $condition = $this->nested_tree_build_children_condition($node, $recursive, $condition);
        return $this->retrieve_objects($node->get_table_name(), $condition, null, null, array(), get_class($node));
    }

    /**
     * Build the conditions for the get / count children methods
     *
     * @param $node NestedTreeNode - the node
     * @param $recursive boolean - use recursive checks with left / right value or not recursive checks with parent_id
     * @param $condition Condition - additional conditions
     */
    private function nested_tree_build_children_condition($node, $recursive = false, $condition = null)
    {
        $children_conditions = array();

        if ($recursive)
        {
            $children_conditions[] = new InequalityCondition(
                NestedTreeNode :: PROPERTY_LEFT_VALUE,
                InequalityCondition :: GREATER_THAN,
                $node->get_left_value());
            $children_conditions[] = new InequalityCondition(
                NestedTreeNode :: PROPERTY_RIGHT_VALUE,
                InequalityCondition :: LESS_THAN,
                $node->get_right_value());
        }
        else
        {
            $children_conditions[] = new EqualityCondition(NestedTreeNode :: PROPERTY_PARENT_ID, $node->get_id());
        }

        if ($condition)
        {
            $children_conditions[] = $condition;
        }

        return new AndCondition($children_conditions);
    }

    /**
     * Counts the parents of a tree node
     *
     * @param $node NestedTreeNode - the node
     * @param $include_object boolean - if set to true the current node will be added to the count
     * @param $condition Condition - additional conditions
     */
    public function nested_tree_count_parents($node, $include_object = false, $condition = null)
    {
        $condition = $this->nested_tree_build_parents_condition($node, true, $include_object, $condition);
        return $this->count_objects($node->get_table_name(), $condition);
    }

    /**
     * Get the parents of a tree node
     *
     * @param $node NestedTreeNode - the node
     * @param $recursive boolean - if set to true every parent will be retrieved recursivly, even those not connected
     *            with parent_id directly
     * @param $include_object boolean - if set to true the current node will be added to the parents list
     * @param $condition Condition - additional conditions
     */
    public function nested_tree_get_parents($node, $recursive = false, $include_object = false, $condition = null)
    {
        $condition = $this->nested_tree_build_parents_condition($node, $recursive, $include_object, $condition);
        $order = new ObjectTableOrder(NestedTreeNode :: PROPERTY_LEFT_VALUE, SORT_DESC);
        return $this->retrieve_objects($node->get_table_name(), $condition, null, null, $order, get_class($node));
    }

    /**
     * Build the conditions for the get / count parents methods
     *
     * @param $node NestedTreeNode - the node
     * @param $recursive boolean - use recursive checks with left / right value or not recursive checks with parent_id
     * @param $include_object boolean - if set to true the current node will be added to the parents list
     * @param $condition Condition - additional conditions
     */
    private function nested_tree_build_parents_condition($node, $recursive = false, $include_object = false, $condition = null)
    {
        $parent_conditions = array();

        if ($recursive)
        {
            if ($include_object)
            {
                $parent_conditions[] = new InequalityCondition(
                    NestedTreeNode :: PROPERTY_LEFT_VALUE,
                    InequalityCondition :: LESS_THAN_OR_EQUAL,
                    $node->get_left_value());
                $parent_conditions[] = new InequalityCondition(
                    NestedTreeNode :: PROPERTY_RIGHT_VALUE,
                    InequalityCondition :: GREATER_THAN_OR_EQUAL,
                    $node->get_right_value());
            }
            else
            {
                $parent_conditions[] = new InequalityCondition(
                    NestedTreeNode :: PROPERTY_LEFT_VALUE,
                    InequalityCondition :: LESS_THAN,
                    $node->get_left_value());
                $parent_conditions[] = new InequalityCondition(
                    NestedTreeNode :: PROPERTY_RIGHT_VALUE,
                    InequalityCondition :: GREATER_THAN,
                    $node->get_right_value());
            }
        }
        else
        {
            $parent_conditions[] = new EqualityCondition(NestedTreeNode :: PROPERTY_ID, $node->get_parent_id());
        }

        if ($condition)
        {
            $parent_conditions[] = $condition;
        }

        return new AndCondition($parent_conditions);
    }

    /**
     * Counts the sibblings of a tree node
     *
     * @param $node NestedTreeNode - the node
     * @param $include_object boolean - if set to true the current node will be added to the count
     * @param $condition Condition - additional conditions
     */
    public function nested_tree_count_sibblings($node, $include_object = false, $condition = null)
    {
        $condition = $this->nested_tree_build_sibblings_condition($node, $include_object, $condition);
        return $this->count_objects($node->get_table_name(), $condition);
    }

    /**
     * Gets the sibblings of a tree node
     *
     * @param $node NestedTreeNode - the node
     * @param $include_object boolean - if set to true the current node will be added to the sibblings list
     * @param $condition Condition - additional conditions
     */
    public function nested_tree_get_sibblings($node, $include_object = false, $condition = null)
    {
        $condition = $this->nested_tree_build_sibblings_condition($node, $include_object, $condition);
        return $this->retrieve_objects($node->get_table_name(), $condition, null, null, array(), get_class($node));
    }

    /**
     * Build the conditions for the get / count sibblings methods
     *
     * @param $node NestedTreeNode - the node
     * @param $include_object boolean - if set to true the current node will be added to the sibblings list
     * @param $condition Condition - additional conditions
     */
    private function nested_tree_build_sibblings_condition($node, $include_object = false, $condition = null)
    {
        $siblings_conditions = array();

        $siblings_conditions[] = new EqualityCondition(NestedTreeNode :: PROPERTY_PARENT_ID, $node->get_parent());

        if (! $include_object)
        {
            $siblings_conditions[] = new NotCondition(
                new EqualityCondition(NestedTreeNode :: PROPERTY_ID, $node->get_id()));
        }

        if ($condition)
        {
            $siblings_conditions[] = $condition;
        }

        return new AndCondition($siblings_conditions);
    }

    /**
     * Retrieve a node from the database
     *
     * @param $table_name String - the table name
     * @param $id int - the id of the node
     */
    public function nested_tree_retrieve_node($node, $id)
    {
        if (! isset($id))
        {
            return false;
        }

        $condition = new EqualityCondition(NestedTreeNode :: PROPERTY_ID, $id);
        return $this->retrieve_object($node->get_table_name(), $condition, array(), get_class($node));
    }

    /**
     * Retrieve a parent node from the database
     *
     * @param $table_name String - the table name
     */
    public function nested_tree_retrieve_parent_from_node($node)
    {
        $condition = new EqualityCondition(NestedTreeNode :: PROPERTY_ID, $node->get_parent_id());
        return $this->retrieve_object($node->get_table_name(), $condition, array(), get_class($node));
    }

    /**
     * Change the left/right values in the tree of every node that comes after the given node
     *
     * @param $node NestedTreeNode - the node
     * @param $previous_visited int - the previous node
     * @param $number_of_elements int - the number of elements which have to be inserted
     * @param $condition Condition - additional condition
     */
    public function nested_tree_add_nested_values($node, $previous_visited, $number_of_elements = 1, $condition = null)
    {
        // Update all necessary left-values
        $conditions = array();
        $conditions[] = new InequalityCondition(
            NestedTreeNode :: PROPERTY_LEFT_VALUE,
            InequalityCondition :: GREATER_THAN,
            $previous_visited);

        if ($condition)
        {
            $conditions[] = $condition;
        }

        $update_condition = new AndCondition($conditions);

        $properties = array(
            NestedTreeNode :: PROPERTY_LEFT_VALUE => self :: escape_column_name(NestedTreeNode :: PROPERTY_LEFT_VALUE) .
                 ' + ' . $this->quote($number_of_elements * 2));
        $res = $this->update_objects($node->get_table_name(), $properties, $update_condition);

        if (! $res)
        {
            return false;
        }

        // Update all necessary right-values
        $conditions = array();
        $conditions[] = new InequalityCondition(
            NestedTreeNode :: PROPERTY_RIGHT_VALUE,
            InequalityCondition :: GREATER_THAN,
            $previous_visited);

        if ($condition)
        {
            $conditions[] = $condition;
        }

        $update_condition = new AndCondition($conditions);

        $properties = array(
            NestedTreeNode :: PROPERTY_RIGHT_VALUE => self :: escape_column_name(NestedTreeNode :: PROPERTY_RIGHT_VALUE) .
                 ' + ' . $this->quote($number_of_elements * 2));
        $res = $this->update_objects($node->get_table_name(), $properties, $update_condition);

        if (! $res)
        {
            return false;
        }

        return true;
    }

    /**
     * Change the left/right values in the tree of every node that is infected due to a delete of the given node
     *
     * @param $node NestedTreeNode - the node
     * @param $condition Condition - additional condition
     */
    public function nested_tree_delete_nested_values($node, $condition)
    {
        $delta = $node->get_right_value() - $node->get_left_value() + 1;

        // Update all necessary nested-values
        $conditions = array();
        $conditions[] = new InequalityCondition(
            NestedTreeNode :: PROPERTY_LEFT_VALUE,
            InequalityCondition :: GREATER_THAN,
            $node->get_left_value());

        if ($condition)
        {
            $conditions[] = $condition;
        }

        $update_condition = new AndCondition($conditions);

        $properties = array();
        $properties[NestedTreeNode :: PROPERTY_LEFT_VALUE] = self :: escape_column_name(
            NestedTreeNode :: PROPERTY_LEFT_VALUE) . ' - ' . $this->quote($delta);
        $properties[NestedTreeNode :: PROPERTY_RIGHT_VALUE] = self :: escape_column_name(
            NestedTreeNode :: PROPERTY_RIGHT_VALUE) . ' - ' . $this->quote($delta);
        $res = $this->update_objects($node->get_table_name(), $properties, $update_condition);

        if (! $res)
        {
            return false;
        }

        // Update some more nested-values
        $conditions = array();
        $conditions[] = new InequalityCondition(
            NestedTreeNode :: PROPERTY_LEFT_VALUE,
            InequalityCondition :: LESS_THAN,
            $node->get_left_value());
        $conditions[] = new InequalityCondition(
            NestedTreeNode :: PROPERTY_RIGHT_VALUE,
            InequalityCondition :: GREATER_THAN,
            $node->get_right_value());

        if ($condition)
        {
            $conditions[] = $condition;
        }

        $update_condition = new AndCondition($conditions);

        $properties = array(
            NestedTreeNode :: PROPERTY_RIGHT_VALUE => self :: escape_column_name(NestedTreeNode :: PROPERTY_RIGHT_VALUE) .
                 ' - ' . $this->quote($delta));
        $res = $this->update_objects($node->get_table_name(), $properties, $update_condition);

        if (! $res)
        {
            return false;
        }

        return true;
    }

    public function __sleep()
    {
        throw new Exception("DoctrineDatabase Cannot be Serialized !");
    }

    /**
     * Get the fully qualified class name of the Mdb2Database object
     *
     * @return string
     */
    public static function class_name()
    {
        return get_called_class();
    }

    /**
     * Get the namespace of the Mdb2Database object
     *
     * @return string
     */
    public static function context()
    {
        return Utilities :: get_namespace_from_classname(get_called_class());
    }

    public function retrieve_composite_data_class_additional_properties(CompositeDataClass $object)
    {
        if (! $object->is_extended())
        {
            return array();
        }
        $array = array_map(array($this, 'escape_column_name'), $object->get_additional_property_names());

        if (count($array) == 0)
        {
            $array = array("*");
        }

        $query = 'SELECT ' . implode(',', $array) . ' FROM ' . $this->escape_table_name($object :: get_table_name()) .
             ' WHERE ' . self :: escape_column_name($object :: PROPERTY_ID) . '=' . $this->quote($object->get_id());

        $statement = $this->get_connection()->query($query);

        if (! $statement instanceof \PDOException)
        {
            $distinct_elements = array();
            $record = $statement->fetch(\PDO :: FETCH_ASSOC);

            $additional_properties = array();

            if (is_array($record))
            {
                $properties = $object->get_additional_property_names();

                if (count($properties) > 0)
                {
                    foreach ($properties as $prop)
                    {
                        if (is_resource($record[$prop]))
                        {
                            $data = '';
                            while (! feof($record[$prop]))
                                $data .= fread($record[$prop], 1024);
                            $record[$prop] = $data;
                        }
                        $additional_properties[$prop] = $record[$prop];
                    }
                }
            }
            return $additional_properties;
        }
        else
        {
            $this->error_handling($statement);
            return false;
        }
    }

    public function transactional($function)
    {
        // Rather than directly using Doctrine's version of transactional, we implement
        // an intermediate function that throws an exception if the function returns #f.
        // This mediates between Chamilo's convention of returning #f to signal failure
        // versus Doctrine's use of Exceptions.
        $throw_on_false = function ($connection) use($function)
        {
            $result = call_user_func($function, $connection);
            if (! $result)
            {
                throw new Exception();
            }
            else
            {
                return $result;
            }
        };

        try
        {
            return $this->connection->transactional($throw_on_false);
        }
        catch (Exception $e)
        {
            return false;
        }
    }

    /**
     * Processes the order by
     *
     * @param QueryBuilder $query_builder
     * @param string $class
     * @param ObjectTableOrder[] $order_by
     *
     * @return QueryBuilder
     */
    protected function process_order_by($query_builder, $class, $order_by)
    {
        if (is_null($order_by))
        {
            $order_by = array();
        }
        elseif (! is_array($order_by))
        {
            $order_by = array($order_by);
        }

        foreach ($order_by as $order)
        {
            $query_builder->addOrderBy(
                self :: escape_column_name(
                    $order->get_property(),
                    ($order->alias_is_set() ? $order->get_alias() : $this->get_alias($class :: get_table_name()))),
                ($order->get_direction() == SORT_DESC ? 'DESC' : 'ASC'));
        }

        return $query_builder;
    }

    /**
     * Processes query limit values
     *
     * @param QueryBuilder $query_builder
     * @param int $count
     * @param int $offset
     *
     * @return QueryBuilder
     */
    protected function process_limit($query_builder, $count = null, $offset = null)
    {
        if (intval($count) > 0)
        {
            $query_builder->setMaxResults(intval($count));
        }

        if (intval($offset) > 0)
        {
            $query_builder->setFirstResult(intval($offset));
        }

        return $query_builder;
    }

    /**
     * Processes the dataclass properties
     *
     * @param QueryBuilder $query_builder
     * @param string $class
     * @param DataClassProperties $properties
     *
     * @return QueryBuilder
     */
    protected function process_data_class_properties($query_builder, $class, DataClassProperties $properties)
    {
        if ($properties instanceof DataClassProperties)
        {
            foreach ($properties->get() as $condition_variable)
            {
                $query_builder->addSelect(DoctrineConditionVariableTranslator :: render($condition_variable));
            }
        }
        else
        {
            $query_builder->addSelect($this->get_alias($class :: get_table_name()) . '.*');
        }

        return $query_builder;
    }

    /**
     * Processes the group by
     *
     * @param QueryBuilder $query_builder
     * @param GroupBy $group_by
     *
     * @return QueryBuilder
     */
    protected function process_group_by($query_builder, $group_by)
    {
        if ($group_by instanceof GroupBy)
        {
            foreach ($group_by->get() as $group_by_variable)
            {
                $query_builder->addGroupBy(DoctrineConditionVariableTranslator :: render($group_by_variable));
            }
        }

        return $query_builder;
    }

    /**
     * Processes the joins into the query builder
     *
     * @param QueryBuilder $query_builder
     * @param string $class
     * @param Joins $joins
     *
     * @return QueryBuilder
     */
    protected function process_joins($query_builder, $class, $joins)
    {
        if ($joins instanceof Joins)
        {
            foreach ($joins->get() as $join)
            {
                $join_condition = DoctrineConditionTranslator :: render($join->get_condition());
                switch ($join->get_type())
                {
                    case Join :: TYPE_NORMAL :
                        $query_builder->join(
                            $this->get_alias($class :: get_table_name()),
                            $join->get_data_manager()->get_table_name($join->get_storage_unit_name()),
                            $join->get_data_manager()->get_alias($join->get_storage_unit_name()),
                            $join_condition);
                        break;
                    case Join :: TYPE_RIGHT :
                        $query_builder->rightJoin(
                            $this->get_alias($class :: get_table_name()),
                            $join->get_data_manager()->get_table_name($join->get_storage_unit_name()),
                            $join->get_data_manager()->get_alias($join->get_storage_unit_name()),
                            $join_condition);
                        break;
                    case Join :: TYPE_LEFT :
                        $query_builder->leftJoin(
                            $this->get_alias($class :: get_table_name()),
                            $join->get_data_manager()->get_table_name($join->get_storage_unit_name()),
                            $join->get_data_manager()->get_alias($join->get_storage_unit_name()),
                            $join_condition);
                        break;
                }
            }
        }
        return $query_builder;
    }

    /**
     * Processes the condition
     *
     * @param QueryBuilder $query_builder
     * @param string $class
     * @param Condition $condition
     *
     * @return QueryBuilder
     */
    protected function process_condition($query_builder, $class, $condition)
    {
        if ($condition instanceof Condition)
        {
            $query_builder->where(
                DoctrineConditionTranslator :: render($condition, $this->get_alias($this->prepare_table_name($class))));
        }

        return $query_builder;
    }

    /**
     * Processes the parameters
     *
     * @param QueryBuilder $query_builder
     * @param string $class
     * @param mixed $parameters
     *
     * @return QueryBuilder
     */
    protected function process_parameters($query_builder, $class, $parameters)
    {
        $query_builder = $this->process_joins($query_builder, $class, $parameters->get_joins());
        $query_builder = $this->process_condition($query_builder, $class, $parameters->get_condition());
        return $query_builder;
    }
}
