<?php
namespace common\libraries;

use MDB2;
use Exception;

/**
 * This class provides basic functionality for database connections Create Table, Get next id, Insert, Update, Delete,
 * Select(with use of conditions), Count(with use of conditions)
 *
 * @author Sven Vanpoucke
 * @author Hans De Bisschop
 * @package common.libraries
 */
class Mdb2Database
{
    const STORAGE_TYPE = 'mdb2';
    const ALIAS_MAX_SORT = 'max_sort';

    private $connection;
   private $prefix;
    private $aliases;

    /**
     * Static error log so we don't open the error log every time an error is written
     *
     * @var FileLogger
     */
    private static $error_log;

    /**
     * Used for debug
     *
     * @var int
     */
    private static $query_counter;

    public function __call($name, $arguments)
    {
        $class = static :: context() . '\\' . Utilities :: get_package_name_from_namespace(static :: context(), true) . 'DataManager';
        return call_user_func_array($class . '::' . $name, $arguments);
    }

    /**
     * Constructor
     */
    public function __construct($aliases = array())
    {
        $this->aliases = $aliases;
        $this->initialize();
    }

    /**
     * Initialiser, creates the connection and sets the database to UTF8
     */
    public function initialize()
    {
        $this->connection = Mdb2Connection :: get_instance()->get_connection();
        $this->connection->setOption('debug_handler', array(get_class($this), 'debug'));
        // Important: MDB2_PORTABILITY_RTRIM and
        // MDB2_PORTABILITY_EMPTY_TO_NULL should be *off*

        // We could import a complete class or other PHP file, but
        // that's overkill for this single known instance of only mysql
        // requiring special attention.
        // Feel free to refactor this if you disagree.
        $initializer = 'initialize_' . $this->connection->phptype;
        if (method_exists($this, $initializer))
            $this->$initializer();

        $package = Utilities :: get_package_name_from_namespace(static :: context());
        $this->set_prefix($package . '_');
    }

    public function initialize_mysqli()
    {
        // $this->connection->exec("SET sql_mode='TRADITIONAL'");
    }

    public function set_aliases($aliases = array())
    {
        $this->aliases = $aliases;
    }

    public function get_aliases()
    {
        return $this->aliases;
    }

    public function get_storage_type()
    {
        return self :: STORAGE_TYPE;
    }

    /**
     * Returns the prefix
     *
     * @return String the prefix
     */
    public function get_prefix()
    {
        return $this->prefix;
    }

    /**
     * Sets the prefix
     *
     * @param $prefix String
     */
    public function set_prefix($prefix)
    {
        $this->prefix = $prefix;
    }

    /**
     * Returns the connection
     *
     * @return Connection the connection
     */
    public function get_connection()
    {
        return $this->connection;
    }

    /**
     * Sets the connection
     *
     * @param $connection Connection
     */
    public function set_connection($connection)
    {
        $this->connection = $connection;
    }

    /**
     * Debug function Uncomment the lines if you want to debug
     */
    public static function debug()
    {
        $args = func_get_args();
        // Do something with the arguments
        if ($args[1] == 'query' || $args[1] == 'prepare')
        {
            // echo '<pre>';
            // echo($args[2]);
            // echo self :: $query_counter;
            // echo '</pre>';
            // self :: $query_counter++;
        }
    }

    /*
     * A workaround to the fact that MDB2 hardcodes a default PEAR_Error error mode of PEAR_ERROR_RETURN
     */
    public function mdb2_error_handling($err)
    {
        // I don't care about performance, this should not happen more than once
        // per request
        if (! self :: $error_log)
        {
            $logfile = Path :: get_log_path() . '/mdb2_errors.log';
            self :: $error_log = new FileLogger($logfile, true);
        }

        $message = "[Message: {$err->getMessage()}] [Information: {$err->getUserInfo()}]";
        self :: $error_log->log_message($message);

        // throw new Exception("[Message: ".$err->getMessage()."] [Information:
        // ".$err->getUserInfo()."]", $err->getCode());
    }

    /**
     * Escapes a column name in accordance with the database type.
     *
     * @param $name string The column name.
     * @param $table_alias String The alias of the table the coloumn is in
     * @return string The escaped column name.
     */
    public static function escape_column_name($name, $table_alias = null, $connection = null)
    {
        if (is_null($connection))
        {
	        $connection = Mdb2Connection :: get_instance()->get_connection();
	        $connection->setOption('debug_handler', array(get_class($this), 'debug'));
        }

        $quoted_name = $connection->quoteIdentifier($name);
        if (!empty($table_alias))
        {
            return $connection->quoteIdentifier($table_alias) . '.' . $quoted_name;
        }
        else
        {
            return $quoted_name;
        }
    }

    /**
     * Expands a table identifier to the real table name. Currently, this method prefixes the given table name with the
     * user-defined prefix, if any.
     *
     * @param $name string The table identifier.
     * @return string The actual table name.
     */
    public function get_table_name($name)
    {
        return $this->prefix . $name;
    }

    /**
     * Escapes a table name in accordance with the database type.
     *
     * @param $name string The table identifier as provided by the data class.
     * @return string The escaped table name INCLUDING the application prefix.
     */
    public function escape_table_name($name)
    {
        return $this->connection->quoteIdentifier($this->get_table_name($name));
    }

    /**
     * Maps a record to an object
     *
     * @param $record Record a record from the database
     * @param $class String Class to create new object
     * @return new object from type Class
     * @deprecated Use DataClass :: factory()
     */
    public function record_to_object($record, $class_name)
    {
        return DataClass :: factory($class_name, $record);
    }

    public function storage_unit_exist($name)
    {
        $name = $this->get_table_name($name);
        $this->connection->loadModule('Manager');
        $manager = $this->connection->manager;
        $tables = $manager->listTables();

        if (MDB2 :: isError($tables))
        {
            $this->mdb2_error_handling($tables);
            return false;
        }
        return in_array($name, $tables);
    }

    public function database_exists($name)
    {
        $this->connection->loadModule('Manager');
        $manager = $this->connection->manager;
        $databases = $manager->listDatabases();

        if (MDB2 :: isError($databases))
        {
            $this->mdb2_error_handling($databases);
            return false;
        }
        return in_array($name, $databases);
    }

    /**
     * Creates a storage unit in the system
     *
     * @param $name String the table name
     * @param $properties Array the table properties
     * @param $indexes Array the table indexes
     * @return true if the storage unit is succesfully created
     */
    public function create_storage_unit($name, $properties, $indexes)
    {
        $check_name = $this->prefix . $name;
        $name = $this->get_table_name($name);

        $this->connection->loadModule('Manager');
        $manager = $this->connection->manager;
        // If table allready exists -> drop it
        // @todo This should change: no automatic table drop but warning to user
        $tables = $manager->listTables();

        if (in_array($check_name, $tables))
        {
            $manager->dropTable($name);
        }
        $options['charset'] = 'utf8';
        $options['collate'] = 'utf8_unicode_ci';

        if(strncmp($this->connection->phptype, 'mysql', 5) == 0)
        {
            $options['type']  = 'innodb';
        }

        $result = $manager->createTable($name, $properties, $options);

        $constraint_table_alias = $this->get_constraint_name($name);

        if (! MDB2 :: isError($result))
        {
            foreach ($indexes as $index_name => $index_info)
            {
                if ($index_info['type'] == 'primary')
                {
                    $index_info['primary'] = 1;
                    $primary_result = $manager->createConstraint($name, StorageAliasGenerator :: get_instance()->get_constraint_name($name, $index_name, StorageAliasGenerator :: TYPE_CONSTRAINT), $index_info);
                    if (MDB2 :: isError($primary_result))
                    {
                        $this->mdb2_error_handling($primary_result);
                        return false;
                    }
                }
                elseif ($index_info['type'] == 'unique')
                {
                    $index_info['unique'] = 1;
                    $unique_result = $manager->createConstraint($name, StorageAliasGenerator :: get_instance()->get_constraint_name($name, $index_name, StorageAliasGenerator :: TYPE_CONSTRAINT), $index_info);
                    if (MDB2 :: isError($unique_result))
                    {
                        $this->mdb2_error_handling($unique_result);
                        return false;
                    }
                }
                else
                {
                    $index_result = $manager->createIndex($name, StorageAliasGenerator :: get_instance()->get_constraint_name($name, $index_name, StorageAliasGenerator :: TYPE_CONSTRAINT), $index_info);
                    if (MDB2 :: isError($index_result))
                    {
                        $this->mdb2_error_handling($index_result);
                        return false;
                    }
                }
            }
            return true;
        }
        else
        {
            $this->mdb2_error_handling($result);
            return false;
        }
    }

    /**
     * Retrieves the next id for a given table
     *
     * @param $table_name String
     * @return Int the id
     */
    public function get_next_id($table_name)
    {
        $id = $this->connection->nextID($this->get_table_name($table_name));
        return $id;
    }

    public function get_better_next_id($table_name, $column)
    {
        $this->connection->loadModule('Extended');
        return $this->connection->extended->getBeforeID($this->get_table_name($table_name), $column, true, false);
    }

    /**
     *
     * @return True if creation is successfull or false
     */
    public function create($object, $auto_id = true)
    {
        if ($object instanceof CompositeDataClass)
        {
            $parent_class = $object :: parent_class_name();
            $object_table = $parent_class :: get_table_name();
        }
        else
        {
            $object_table = $object->get_table_name();
        }

        $props = array();
        foreach ($object->get_default_properties() as $key => $value)
        {
            $props[self :: escape_column_name($key)] = $value;
        }

        if ($auto_id && in_array('id', $object->get_default_property_names()))
        {
            $props[self :: escape_column_name('id')] = $this->get_better_next_id($object_table, 'id');
        }

        $this->connection->loadModule('Extended');
        $result = $this->connection->extended->autoExecute($this->get_table_name($object_table), $props, MDB2_AUTOQUERY_INSERT);

        if (MDB2 :: isError($result))
        {
            $this->mdb2_error_handling($result);
            return false;
        }

        if ($auto_id && in_array('id', $object->get_default_property_names()))
        {
            $object->set_id($this->connection->extended->getAfterID($props[self :: escape_column_name('id')], $this->get_table_name($object_table)));
        }

        if ($object instanceof CompositeDataClass && $object :: is_extended())
        {
            $props = array();
            foreach ($object->get_additional_properties() as $key => $value)
            {
                $props[self :: escape_column_name($key)] = $value;
            }
            $props[self :: escape_column_name('id')] = $object->get_id();

            $result = $this->connection->extended->autoExecute($this->get_table_name($object->get_table_name()), $props, MDB2_AUTOQUERY_INSERT);

            if (MDB2 :: isError($result))
            {
                $this->mdb2_error_handling($result);
                return false;
            }
        }

        return true;
    }

    /**
     * Update functionality (can only be used when the storage unit has an ID property)
     *
     * @param $object DataClass
     * @param $condition Condition
     * @return boolean
     */
    public function update($object, $condition = false)
    {
        if ($object instanceof CompositeDataClass)
        {
            $parent_class = $object :: parent_class_name();
            $object_table = $parent_class :: get_table_name();
        }
        else
        {
            $object_table = $object->get_table_name();
        }

        $props = array();
        foreach ($object->get_default_properties() as $key => $value)
        {
            $props[self :: escape_column_name($key)] = $value;
        }

        if ($condition)
        {
            $condition = Mdb2ConditionTranslator :: render($condition);
        }
        else
        {
            throw new Exception('Cannot update records without a condition');
        }

        $this->connection->loadModule('Extended');
        $result = $this->connection->extended->autoExecute($this->get_table_name($object_table), $props, MDB2_AUTOQUERY_UPDATE, $condition);
        if (MDB2 :: isError($result))
        {
            $this->mdb2_error_handling($result);
            return false;
        }

        if ($object instanceof CompositeDataClass && $object :: is_extended())
        {
            $props = array();
            foreach ($object->get_additional_properties() as $key => $value)
            {
                $props[self :: escape_column_name($key)] = $value;
            }

            $condition = new EqualityCondition($object :: PROPERTY_ID, $object->get_id());

            $condition = Mdb2ConditionTranslator :: render($condition);

            $result = $this->connection->extended->autoExecute($this->get_table_name($object->get_table_name()), $props, MDB2_AUTOQUERY_UPDATE, $condition);

            if (MDB2 :: isError($result))
            {
                $this->mdb2_error_handling($result);
                return false;
            }
        }

        return true;
    }

    /**
     *
     * @param $table_name string
     * @param $properties multitype:string
     * @param $condition \common\libraries\Condition
     * @param $offset int
     * @param $max_objects int
     * @param $order_by multitype:\common\libraries\ObjectTableOrder
     * @return boolean
     * @deprecated Use updates() in implementations, use the PackageDataManager :: updates() in applications
     */
    public function update_objects($table_name, $properties = array(), $condition, $offset = null, $max_objects = null, $order_by = array())
    {
        if (count($properties) > 0)
        {
            $table_name_alias = $this->get_alias($table_name);

            $query = 'UPDATE ' . $this->escape_table_name($table_name) . ' AS ' . $table_name_alias . ' SET ';

            $updates = array();

            foreach ($properties as $column => $property)
            {
                $column    = strpos($column, '`') === 0 ? $column : self :: escape_column_name($column);
                $updates[] = $column . '=' . $property;
            }

            $query .= implode(", ", $updates);

            if (isset($condition))
            {
                $query .= ' WHERE ' . Mdb2ConditionTranslator :: render($condition, $this->get_alias($table_name));
            }

            $orders = array();

            if (is_null($order_by))
            {
                $order_by = array();
            }
            elseif (! is_array($order_by))
            {
                $order_by = array($order_by);
            }

            foreach ($order_by as $order)
            {
                if ($order)
                {
                    $orders[] = self :: escape_column_name($order->get_property(), ($order->alias_is_set() ? $order->get_alias() : $this->get_alias($table_name))) . ' ' . ($order->get_direction() == SORT_DESC ? 'DESC' : 'ASC');
                }
            }
            if (count($orders))
            {
                $query .= ' ORDER BY ' . implode(', ', $orders);
            }

            if ($max_objects > 0)
            {
                $query .= ' LIMIT ' . $max_objects;
            }

            $res = $this->connection->exec($query);

            if (MDB2 :: isError($res))
            {
                $this->mdb2_error_handling($res);
                return false;
            }
        }
        return true;
    }

    public function updates($class, $properties, $condition, $offset = null, $count = null, $order_by = array())
    {
        if ($properties instanceof DataClassProperties)
        {
            $old_style_properties = array();

            foreach ($properties->get() as $data_class_property)
            {
                $old_style_properties[Mdb2ConditionVariableTranslator :: render($data_class_property->get_property())] =
                        Mdb2ConditionVariableTranslator :: render($data_class_property->get_value());
            }

            $properties = $old_style_properties;
        }

        return $this->update_objects($class :: get_table_name(), $properties, $condition, $offset, $count, $order_by);
    }

    /**
     * Deletes an object from a table with a given condition
     *
     * @param $table_name String
     * @param $condition Condition
     * @return true if deletion is successfull
     * @deprecated Use new_delete() in implementations, use the PackageDataManager :: delete() in applications
     */
    public function delete($table_name, $condition)
    {
        $alias = $this->get_alias($table_name);
        $query = 'DELETE ' . $alias . ' FROM ' . $this->escape_table_name($table_name) . ' AS ' . $alias;

        if (isset($condition))
        {
            $query .= ' WHERE ' . Mdb2ConditionTranslator :: render($condition, $this->get_table_name($table_name));
        }

        $res = $this->connection->exec($query);

        if (MDB2 :: isError($res))
        {
            $this->mdb2_error_handling($res);
            return false;
        }
        return true;
    }

    /**
     * Deletes an object from a storage unit with a given condition
     *
     * @param $class string
     * @param $condition Condition
     * @return boolean
     */
    public function new_delete($class, $condition)
    {
        return $this->delete($class :: get_table_name(), $condition);
    }

    // /**
    // * Deletes the objects of a given table
    // *
    // * @param $table_name String
    // * @param $condition Condition
    // * the condition
    // * @return boolean
    // */
    /**
     * Deletes the objects of a given storage unit
     *
     * @param $table_name string
     * @param $condition \common\libraries\Condition
     * @return boolean
     * @deprecated Use deletes() in implementations, use the PackageDataManager :: deletes() in applications
     */
    public function delete_objects($table_name, $condition = null)
    {
        $query = 'DELETE FROM ' . $this->escape_table_name($table_name);

        if (isset($condition))
        {
            $query .= ' WHERE ' . Mdb2ConditionTranslator :: render($condition, $this->get_table_name($table_name));
        }

//         var_dump($query);

        $res = $this->connection->exec($query);

        if (MDB2 :: isError($res))
        {
            $this->mdb2_error_handling($res);
            return false;
        }
        return true;
    }

    public function deletes($class, $condition = null)
    {
        return $this->delete_objects($class :: get_table_name(), $condition);
    }

    /**
     * Drop a given storage unit
     *
     * @param $table_name String
     * @return boolean
     */
    public function drop_storage_unit($table_name)
    {
        $this->connection->loadModule('Manager');
        $manager = $this->connection->manager;

        $result = $manager->dropTable($this->escape_table_name($table_name));

        if (MDB2 :: isError($result))
        {
            $this->mdb2_error_handling($result);
            return false;
        }
        else
        {
            return true;
        }
    }

    public function rename_storage_unit($old_name, $new_name)
    {

        $query = 'ALTER TABLE ' . $this->escape_table_name($old_name) . ' RENAME TO ' .
             $this->escape_table_name($new_name);

        $res = $this->connection->exec($query);

        if (MDB2 :: isError($res))
        {
            $this->mdb2_error_handling($res);
            return false;
        }

        return true;
    }

    public function copy_storage_unit($from_name, $to_name, $create_storage_unit = true)
    {
        if ($create_storage_unit)
        {
            $query = 'CREATE TABLE ' . $this->escape_table_name($to_name) . ' LIKE ' .
                 $this->escape_table_name($from_name) . ' ; ';
        }

        $query .= 'INSERT ' . $this->escape_table_name($to_name) . ' SELECT * FROM ' .
             $this->escape_table_name($from_name) . ';';

        $res = $this->connection->exec($query);

        if (MDB2 :: isError($res))
        {
            $this->mdb2_error_handling($res);
            return false;
        }

        return true;
    }

    /**
     * @param integer $type
     * @param string $table_name
     * @param string $property
     * @param multitype:mixed $attributes
     * @return boolean
     */
    public function alter_storage_unit($type, $table_name, $property, $attributes = array())
    {
        $column_name = self :: escape_column_name($property);
        $changes = array();
        if ($type == DataManager :: ALTER_STORAGE_UNIT_DROP)
        {
            //$query = 'ALTER TABLE ' . $this->escape_table_name($table_name) . ' DROP COLUMN ' . $column_name;
            $changes['remove'] = array($column_name => array());
        }
        else
        {

            if ($type == DataManager :: ALTER_STORAGE_UNIT_CHANGE)
            {
                if(isset($attributes['name']))
                {
                    $new_column_name = self :: escape_column_name($attributes['name']);
                    $changes['rename'] = array($column_name => array('name' => $new_column_name));
                    $column_name = $new_column_name;
                }
                $changes['change'] = array($column_name => array('definition' => $attributes));
            } elseif ($type == DataManager :: ALTER_STORAGE_UNIT_ADD)
            {
                $changes['add'] = array($column_name => $attributes);
            }
        }

        //$res = $this->connection->exec($query);

        $this->connection->loadModule('Manager');
        $res = $this->connection->manager->alterTable($this->escape_table_name($table_name), $changes, false);
        if (MDB2 :: isError($res))
        {
            $this->mdb2_error_handling($res);
            return false;
        }
        return true;
    }

    /**
     * Counts the objects in a storage unit with a given condition
     *
     * @param $table_name string
     * @param $condition Condition
     * @return int
     *
     * @deprecated Use count() in implementations, use the PackageDataManager :: count() in applications
     */
    public function count_objects($table_name, $condition = null)
    {
        $query = 'SELECT COUNT(*) FROM ' . $this->escape_table_name($table_name) . ' AS ' . $this->get_alias($table_name);

        return $this->count_result_set($query, $table_name, $condition);
    }

    /**
     * Counts the objects in a storage unit with a given condition
     *
     * @param $class string
     * @param $parameters \common\libraries\DataClassCountParameters
     */
    public function count($class, $parameters)
    {
        return $this->count_objects($class :: get_table_name(), $parameters->get_condition());
    }

    /**
     *
     * @param $class string
     * @param $parameters \common\libraries\DataClassCountGroupedParameters
     * @return multitype:int
     */
    public function count_grouped($class, $parameters)
    {
        return $this->count_grouped_object($class :: get_table_name(), $parameters->get_property(), $parameters->get_property(), $parameters->get_condition());
    }

    /**
     *
     * @param $table_name string
     * @param $group string
     * @param $count string
     * @param $condition \common\libraries\Condition
     * @return multitype:int
     * @deprecated Use count_grouped() in implementations, use the PackageDataManager :: count_grouped() in applications
     */
    public function count_grouped_object($table_name, $group, $count, $condition = null)
    {
        $query = 'SELECT ' . self :: escape_column_name($group, $this->get_alias($table_name)) . ', COUNT(' . self :: escape_column_name($count, $this->get_alias($table_name)) . ') FROM ' . $this->escape_table_name($table_name) . ' AS ' . $this->get_alias($table_name);

        return $this->count_grouped_result_set($query, $table_name, $group, $condition);
    }

    public function count_grouped_result_set($query, $table_name, $group, $condition)
    {
        if (isset($condition))
        {
            $query .= ' WHERE ' . Mdb2ConditionTranslator :: render($condition, $this->get_alias($table_name));
        }

        $query .= ' GROUP BY' . self :: escape_column_name($group, $this->get_alias($table_name));

        $res = $this->query($query);

        if (MDB2 :: isError($res))
        {
            $this->mdb2_error_handling($res);
            return false;
        }
        else
        {
            $counts = array();
            while ($record = $res->fetchRow(MDB2_FETCHMODE_ORDERED))
            {
                $counts[$record[0]] = $record[1];
            }
            $res->free();

            return $counts;
        }
    }

    public function count_result_set($query, $table_name, $condition = null)
    {
        if (isset($condition))
        {
            $query .= ' WHERE ' . Mdb2ConditionTranslator :: render($condition, $this->get_alias($table_name));
        }

        $res = $this->query($query);

        if (MDB2 :: isError($res))
        {
            $this->mdb2_error_handling($res);
            return false;
        }
        else
        {
            $record = $res->fetchRow(MDB2_FETCHMODE_ORDERED);
            $res->free();
            // dump($res);
            return $record[0];
        }
    }

    /**
     * Retrieves the objects of a given table
     *
     * @param $table_name String
     * @param $classname String The name of the class where the object has to be mapped to
     * @param $condition Condition the condition
     * @param $offset Int the starting offset
     * @param $max_objects Int the max amount of objects to be retrieved
     * @param $order_by Array(String) the list of column names that the objects have to be ordered by
     * @param $resultset String - Optional, the resultset to map the items to
     * @return ResultSet
     * @deprecated Use retrieves() in implementations, use the PackageDataManager :: retrieves() in applications
     */
    public function retrieve_objects($table_name, $condition = null, $offset = null, $max_objects = null, $order_by = array(), $class_name = null)
    {
        $query = 'SELECT * FROM ' . $this->escape_table_name($table_name) . ' AS ' . $this->get_alias($table_name);
        // echo $query . '<br />';
        return $this->retrieve_object_set($query, $table_name, $condition, $offset, $max_objects, $order_by, $class_name);
    }

    /**
     *
     * @param $class string
     * @param $parameters \common\libraries\DataClassResultSetParameters
     * @return \common\libraries\ResultSet
     */
    public function retrieves($class, DataClassRetrievesParameters $parameters)
    {
        return $this->retrieve_objects($class :: get_table_name(), $parameters->get_condition(), $parameters->get_offset(), $parameters->get_count(), $parameters->get_order_by(), $class);
    }

    public function retrieve_result($query, $table_name, $condition = null, $offset = null, $max_objects = null, $order_by = array())
    {
        if (isset($condition))
        {
            $query .= ' WHERE ' . Mdb2ConditionTranslator :: render($condition, $this->get_alias($table_name));
        }

        $orders = array();

//                 print_r('<strong>Statement</strong><br />' . $query . '<br /><br
//                 /><br />');
//         dump($order_by);


        if (is_null($order_by))
        {
            $order_by = array();
        }
        elseif (! is_array($order_by))
        {
            $order_by = array($order_by);
        }

        foreach ($order_by as $order)
        {
            if ($order)
            {
                $orders[] = self :: escape_column_name($order->get_property(), ($order->alias_is_set() ? $order->get_alias() : $this->get_alias($table_name))) . ' ' . ($order->get_direction() == SORT_DESC ? 'DESC' : 'ASC');
            }
        }
        if (count($orders))
        {
            $query .= ' ORDER BY ' . implode(', ', $orders);
        }
        if ($max_objects < 0)
        {
            $max_objects = null;
        }

        $this->set_limit(intval($max_objects), intval($offset));

        return $this->query($query);
    }

    public function retrieve_object_set($query, $table_name, $condition = null, $offset = null, $max_objects = null, $order_by = array(), $class_name = null)
    {
        $res = $this->retrieve_result($query, $table_name, $condition, $offset, $max_objects, $order_by);

        if (is_null($class_name))
        {
            $called_class = get_called_class();
            $namespace = Utilities :: get_namespace_from_classname($called_class);

            $class_name = $namespace . '\\' . Utilities :: underscores_to_camelcase($table_name);
        }

          if (MDB2 :: isError($res))
        {
            $this->mdb2_error_handling($res);
            return null;
        }
        return new Mdb2DataClassResultSet($this, $res, $class_name);
    }

    /*
     * broken function, does anyone care ? function retrieve_record_set($query, $table_name, $condition = null, $offset
     * = null, $max_objects = null, $order_by = array()) { return new RecordResultSet($this->retrieve_result($query,
     * $table_name, $condition, $offset, $max_objects, $order_by)); }
     */

    /**
     *
     * @param $table_name string
     * @param $column string
     * @param $condition \common\libraries\Condition
     * @return int
     * @deprecated Use retrieve_maximum_value() in implementations, use the PackageDataManager ::
     *             retrieve_maximum_value() in applications
     */
    public function retrieve_max_sort_value($table_name, $column, $condition = null)
    {
        $query = 'SELECT MAX(' . self :: escape_column_name($column) . ') as ' . self :: ALIAS_MAX_SORT . ' FROM ' . $this->escape_table_name($table_name) . ' AS ' . $this->get_alias($table_name);

        if (isset($condition))
        {
            $query .= ' WHERE ' . Mdb2ConditionTranslator :: render($condition, $this->get_alias($table_name));
        }

        $res = $this->query($query);
        if ($res->numRows() >= 1)
        {
            $record = $res->fetchRow(MDB2_FETCHMODE_ORDERED);
            $res->free();
            return $record[0];
        }
        else
        {
            $res->free();
            return 0;
        }
    }

    /**
     *
     * @param $class string
     * @param $property string
     * @param $condition \common\libraries\Condition
     * @return int
     */
    public function retrieve_maximum_value($class, $property, $condition = null)
    {
        return $this->retrieve_max_sort_value($class :: get_table_name(), $property, $condition);
    }

    /**
     *
     * @param $table_name string
     * @param $column string
     * @param $condition \common\libraries\Condition
     * @return int
     * @deprecated Use PackageDataManager :: retrieve_next_value()
     */
    public function retrieve_next_sort_value($table_name, $column, $condition = null)
    {
        return $this->retrieve_max_sort_value($table_name, $column, $condition) + 1;
    }

    public function truncate_storage_unit($table_name, $optimize = true)
    {
        $this->connection->loadModule('Manager');
        $manager = $this->connection->manager;
        $result = $manager->truncateTable($this->escape_table_name($table_name));
        if (! MDB2 :: isError($result))
        {
            if ($optimize)
            {
                return $this->optimize_storage_unit($table_name);
            }
            else
            {
                return true;
            }
        }
        else
        {
            $this->mdb2_error_handling($result);
            return false;
        }
    }

    public function optimize_storage_unit($table_name)
    {
        $this->connection->loadModule('Manager');
        $manager = $this->connection->manager;
        if ($manager->vacuum($this->escape_table_name($table_name)))
        {
            return true;
        }
        else
        {
            return false;
        }
    }

    public function retrieve_record($table_name, $condition = null, $order_by = array())
    {
        $query = 'SELECT * FROM ' . $this->escape_table_name($table_name) . ' AS ' . $this->get_alias($table_name);

        return $this->retrieve_row($query, $table_name, $condition, $order_by);
    }

    public function retrieve_row($query, $table_name, $condition = null, $order_by = array())
    {
        if (isset($condition))
        {
            $query .= ' WHERE ' . Mdb2ConditionTranslator :: render($condition, $this->get_alias($table_name));
        }

        $orders = array();

        foreach ($order_by as $order)
        {
            if ($order)
            {
                $orders[] = self :: escape_column_name($order->get_property(), ($order->alias_is_set() ? $order->get_alias() : $this->get_alias($table_name))) . ' ' . ($order->get_direction() == SORT_DESC ? 'DESC' : 'ASC');
            }
        }
        if (count($orders))
        {
            $query .= ' ORDER BY ' . implode(', ', $orders);
        }

        $this->set_limit(1, 0);
        $res = $this->query($query);

        if (! MDB2 :: isError($res))
        {
            $record = $res->fetchRow(MDB2_FETCHMODE_ASSOC);
            $res->free();
        }
        else
        {
            $this->mdb2_error_handling($res);
            return false;
        }

        if (MDB2 :: isError($record))
        {
            $this->mdb2_error_handling($record);
            return false;
        }
        if (is_null($record))
        {
            return false;
        }

        foreach ($record as &$field)
        {
            if (is_resource($field))
            {
                $data = '';
                while (! feof($field))
                    $data .= fread($field, 1024);
                $field = $data;
            }
        }

        if ($record)
        {
            return $record;
        }
        else
        {
            return false;
        }
    }

    /**
     *
     * @param $class string
     * @param $parameters \common\libraries\DataClassResultParameters
     * @throws DataClassNoResultException
     * @return multitype:string
     */
    public function retrieve($class, $parameters = null)
    {
        $record = $this->retrieve_composite_record($class, $parameters);

        if (! is_array($record) || count($record) == 0)
        {
            throw new DataClassNoResultException($class, $parameters);
        }
        else
        {
            return $record;
        }
    }

    /**
     *
     * @param $class unknown_type
     * @param $parameters unknown_type
     */
    public function retrieve_composite_record($class, $parameters)
    {
        $condition = $parameters->get_condition();
        $order_by = $parameters->get_order_by();

        $table_name = $class :: get_table_name();
        $table_alias = $this->get_alias($table_name);

        $query = 'SELECT * FROM ' . $this->escape_table_name($table_name) . ' AS ' . $table_alias;

        if (is_subclass_of($class, CompositeDataClass :: class_name()))
        {
            $type_query = 'SELECT ' . self :: escape_column_name(CompositeDataClass :: PROPERTY_TYPE, $table_alias) . ' FROM ' . $this->escape_table_name($table_name) . ' AS ' . $table_alias;
            $record = $this->retrieve_row($type_query, $table_name, $condition);
            $type = $record[CompositeDataClass :: PROPERTY_TYPE];
            $type_alias = $this->get_alias($type :: get_table_name());

            if ($type :: is_extended())
            {
                $query .= ' JOIN ' . $this->escape_table_name($type :: get_table_name()) . ' AS ' . $type_alias . ' ON ' . self :: escape_column_name(DataClass :: PROPERTY_ID, $table_alias) . '=' . self :: escape_column_name(CompositeDataClass :: PROPERTY_ID, $type_alias);
            }
        }

        if (isset($condition))
        {
            $query .= ' WHERE ' . Mdb2ConditionTranslator :: render($condition, $this->get_alias($class :: get_table_name()));
        }

        $orders = array();

        foreach ($order_by as $order)
        {
            if ($order)
            {
                $orders[] = self :: escape_column_name($order->get_property(), ($order->alias_is_set() ? $order->get_alias() : $this->get_alias($class :: get_table_name()))) . ' ' . ($order->get_direction() == SORT_DESC ? 'DESC' : 'ASC');
            }
        }
        if (count($orders))
        {
            $query .= ' ORDER BY ' . implode(', ', $orders);
        }

        $this->set_limit(1, 0);

        $res = $this->query($query);
        if ($res && !MDB2 :: isError($res) )
        {
            $record = $res->fetchRow(MDB2_FETCHMODE_ASSOC);
            $res->free();
        }
        else
        {
            $this->mdb2_error_handling($res);
            return false;
        }

        if (MDB2 :: isError($record))
        {
            $this->mdb2_error_handling($record);
            return false;
        }
        if (is_null($record))
        {
            return false;
        }

        foreach ($record as &$field)
        {
            if (is_resource($field))
            {
                $data = '';
                while (! feof($field))
                    $data .= fread($field, 1024);
                $field = $data;
            }
        }

        if ($record)
        {
            return $record;
        }
        else
        {
            return false;
        }
    }

    /**
     *
     * @param $table_name string
     * @param $condition Condition
     * @param $order_by array
     * @param $class_name string
     * @return Ambigous <\common\libraries\DataClass|boolean>
     * @deprecated Use PackageDataManager :: retrieve()
     */
    public function retrieve_object($table_name, $condition = null, $order_by = array(), $class_name = null)
    {
        $record = $this->retrieve_record($table_name, $condition, $order_by);

        if (is_null($class_name))
        {
            $called_class = get_called_class();
            $namespace = Utilities :: get_namespace_from_classname($called_class);

            $class_name = $namespace . '\\' . Utilities :: underscores_to_camelcase($table_name);
        }

        if ($record)
        {
            return self :: record_to_object($record, $class_name);
        }
        else
        {
            return false;
        }
    }

    /**
     *
     * @param $table_name string
     * @param $column_name string
     * @param $condition \common\libraries\Condition
     * @return multitype:string
     * @deprecated Use distinct() in implementations, use the PackageDataManager :: distinct() in applications
     */
    public function retrieve_distinct($table_name, $column_name, $condition = null)
    {
        if($column_name instanceof DataClassDistinctParameters)
        {
            $column_name = $column_name->get_property();
        }

        $query = 'SELECT DISTINCT(' . self :: escape_column_name($column_name) . ') FROM ' . $this->escape_table_name($table_name) . ' AS ' . $this->get_alias($table_name);

        if (isset($condition))
        {
            $query .= ' WHERE ' . Mdb2ConditionTranslator :: render($condition, $this->get_alias($table_name));
        }

        $res = $this->query($query);
        if (MDB2 :: isError($res))
        {
            $this->mdb2_error_handling($res);
            return false;
        }

        $distinct_elements = array();
        while ($record = $res->fetchRow(MDB2_FETCHMODE_ASSOC))
        {
            $distinct_elements[] = $record[$column_name];
        }
        $res->free();
        return $distinct_elements;
    }

    /**
     *
     * @param $class string
     * @param $property string
     * @param $condition \common\libraries\Condition
     * @return multitype:string
     */
    public function distinct($class, $property, $condition = null)
    {
        return $this->retrieve_distinct($class :: get_table_name(), $property, $condition);
    }

    /**
     *
     * @param $class string
     * @param $parameters \common\libraries\DataClassCountDistinctParameters
     * @return int
     */
    public function new_count_distinct($class, DataClassCountDistinctParameters $parameters)
    {
        return $this->count_distinct($class :: get_table_name(), $parameters->get_property(), $parameters->get_condition());
    }

    /**
     *
     * @param $table_name string
     * @param $column_name string
     * @param $condition \common\libraries\Condition
     * @return int
     * @deprecated Use new_count_distinct() in implementations, use the PackageDataManager :: count_distinct() in
     *             applications
     */
    public function count_distinct($table_name, $column_name, $condition = null)
    {
        $query = 'SELECT COUNT(DISTINCT(' . self :: escape_column_name($column_name) . ')) FROM ' . $this->escape_table_name($table_name) . ' AS ' . $this->get_alias($table_name);

        if (isset($condition))
        {
            $query .= ' WHERE ' . Mdb2ConditionTranslator :: render($condition, $this->get_alias($table_name));
        }

        $res = $this->query($query);
        if (MDB2 :: isError($res))
        {
            $this->mdb2_error_handling($res);
            return false;
        }

        $record = $res->fetchRow(MDB2_FETCHMODE_ORDERED);
        $res->free();
        return $record[0];
    }

    /**
     * Returns the alias of the table name Please note that this table name should NOT be escaped using the
     * escape_table_name method of the Database class
     *
     * @param $table_name String
     * @return String the alias
     */
    public function get_alias($table_name)
    {
        return StorageAliasGenerator :: get_instance()->get_table_alias($table_name, $this->get_prefix());

        // if (!array_key_exists($table_name, $this->aliases))
        // {
        // $possible_name = substr($table_name, 0, 2) . substr($table_name, -
        // 2);
        // $index = 0;
        // while (array_key_exists($possible_name, $this->aliases))
        // {
        // $possible_name = $possible_name . $index;
        // $index = $index ++;
        // }
        // $this->aliases[$table_name] = $possible_name;
        // }
        //
        // return $this->aliases[$table_name];
    }

    public function get_constraint_name($name)
    {
        $possible_name = '';
        $parts = explode('_', $name);
        foreach ($parts as & $part)
        {
            $possible_name .= $part{0};
        }

        return $possible_name;
    }

    public static function quote($value, $type = null, $quote = true, $escape_wildcards = false, $connection = null)
    {
        if (is_null($connection))
        {
	        $connection = Mdb2Connection :: get_instance()->get_connection();
	        $connection->setOption('debug_handler', array(get_class($this), 'debug'));
        }

        return $connection->quote($value, $type, $quote, $escape_wildcards);
    }

    public function escape($text, $escape_wildcards = false)
    {
        return $this->connection->escape($text, $escape_wildcards);
    }

    public function query($query, $types = true, $result_class = true, $result_wrap_class = false)
    {
        $result = $this->connection->query($query, $types, $result_class, $result_wrap_class);
        if (MDB2 :: isError($result))
        {
            $this->mdb2_error_handling($result);
            return $result;
        }
        return $result;
    }

    /**
     * Delegate method for the exec function Exec is used when the query does not expect a resultset but only true /
     * false
     *
     * @param $query String
     * @return boolean
     */
    public function exec($query)
    {
        $result = $this->connection->exec($query);
        if (MDB2 :: isError($result))
        {
            $this->mdb2_error_handling($result);
            return false;
        }
        return $result;
    }

    public function set_limit($limit, $offset = 0)
    {
        if ($offset && ! $limit)
            throw new Exception('Supplying an offset without a limit is not supported by MDB2!');

        $result = $this->connection->setLimit($limit, $offset);
        if (MDB2 :: isError($result))
        {
            $this->mdb2_error_handling($result);
            return false;
        }
    }

    /**
     * ************************************************************************ FUNCTIONALITY THAT ENABLES NESTED TREES
     * VIA NESTED_TREE_NODE.CLASS.PHP * ************************************************************************
     */

    /**
     * Counts the children of a tree node
     *
     * @param $node NestedTreeNode - the node
     * @param $condition Condition - additional conditions
     */
    public function nested_tree_count_children($node, $recursive = false, $condition = null)
    {
        $condition = $this->nested_tree_build_children_condition($node, $recursive, $condition);
        return $this->count_objects($node->get_table_name(), $condition);
    }

    /**
     * Retrieves the children of a tree node
     *
     * @param $node NestedTreeNode - the node
     * @param $recursive boolean - if put on true, every child will be retrieved, even those who are not directly
     *            connected with parent_id
     * @param $condition Condition - additional conditions
     */
    public function nested_tree_get_children($node, $recursive = false, $condition = null)
    {
        $condition = $this->nested_tree_build_children_condition($node, $recursive, $condition);
        return $this->retrieve_objects($node->get_table_name(), $condition, null, null, array(), get_class($node));
    }

    /**
     * Build the conditions for the get / count children methods
     *
     * @param $node NestedTreeNode - the node
     * @param $recursive boolean - use recursive checks with left / right value or not recursive checks with parent_id
     * @param $condition Condition - additional conditions
     */
    private function nested_tree_build_children_condition($node, $recursive = false, $condition = null)
    {
        $children_conditions = array();

        if ($recursive)
        {
            $children_conditions[] = new InequalityCondition(NestedTreeNode :: PROPERTY_LEFT_VALUE, InequalityCondition :: GREATER_THAN, $node->get_left_value());
            $children_conditions[] = new InequalityCondition(NestedTreeNode :: PROPERTY_RIGHT_VALUE, InequalityCondition :: LESS_THAN, $node->get_right_value());
        }
        else
        {
            $children_conditions[] = new EqualityCondition(NestedTreeNode :: PROPERTY_PARENT_ID, $node->get_id());
        }

        if ($condition)
        {
            $children_conditions[] = $condition;
        }

        return new AndCondition($children_conditions);
    }

    /**
     * Counts the parents of a tree node
     *
     * @param $node NestedTreeNode - the node
     * @param $include_object boolean - if set to true the current node will be added to the count
     * @param $condition Condition - additional conditions
     */
    public function nested_tree_count_parents($node, $include_object = false, $condition = null)
    {
        $condition = $this->nested_tree_build_parents_condition($node, true, $include_object, $condition);
        return $this->count_objects($node->get_table_name(), $condition);
    }

    /**
     * Get the parents of a tree node
     *
     * @param $node NestedTreeNode - the node
     * @param $recursive boolean - if set to true every parent will be retrieved recursivly, even those not connected
     *            with parent_id directly
     * @param $include_object boolean - if set to true the current node will be added to the parents list
     * @param $condition Condition - additional conditions
     */
    public function nested_tree_get_parents($node, $recursive = false, $include_object = false, $condition = null)
    {
        $condition = $this->nested_tree_build_parents_condition($node, $recursive, $include_object, $condition);
        $order = new ObjectTableOrder(NestedTreeNode :: PROPERTY_LEFT_VALUE, SORT_DESC);
        return $this->retrieve_objects($node->get_table_name(), $condition, null, null, $order, get_class($node));
    }

    /**
     * Build the conditions for the get / count parents methods
     *
     * @param $node NestedTreeNode - the node
     * @param $recursive boolean - use recursive checks with left / right value or not recursive checks with parent_id
     * @param $include_object boolean - if set to true the current node will be added to the parents list
     * @param $condition Condition - additional conditions
     */
    private function nested_tree_build_parents_condition($node, $recursive = false, $include_object = false, $condition = null)
    {
        $parent_conditions = array();

        if ($recursive)
        {
            if ($include_object)
            {
                $parent_conditions[] = new InequalityCondition(NestedTreeNode :: PROPERTY_LEFT_VALUE, InequalityCondition :: LESS_THAN_OR_EQUAL, $node->get_left_value());
                $parent_conditions[] = new InequalityCondition(NestedTreeNode :: PROPERTY_RIGHT_VALUE, InequalityCondition :: GREATER_THAN_OR_EQUAL, $node->get_right_value());
            }
            else
            {
                $parent_conditions[] = new InequalityCondition(NestedTreeNode :: PROPERTY_LEFT_VALUE, InequalityCondition :: LESS_THAN, $node->get_left_value());
                $parent_conditions[] = new InequalityCondition(NestedTreeNode :: PROPERTY_RIGHT_VALUE, InequalityCondition :: GREATER_THAN, $node->get_right_value());
            }
        }
        else
        {
            $parent_conditions[] = new EqualityCondition(NestedTreeNode :: PROPERTY_ID, $node->get_parent_id());
        }

        if ($condition)
        {
            $parent_conditions[] = $condition;
        }

        return new AndCondition($parent_conditions);
    }

    /**
     * Counts the sibblings of a tree node
     *
     * @param $node NestedTreeNode - the node
     * @param $include_object boolean - if set to true the current node will be added to the count
     * @param $condition Condition - additional conditions
     */
    public function nested_tree_count_sibblings($node, $include_object = false, $condition = null)
    {
        $condition = $this->nested_tree_build_sibblings_condition($node, $include_object, $condition);
        return $this->count_objects($node->get_table_name(), $condition);
    }

    /**
     * Gets the sibblings of a tree node
     *
     * @param $node NestedTreeNode - the node
     * @param $include_object boolean - if set to true the current node will be added to the sibblings list
     * @param $condition Condition - additional conditions
     */
    public function nested_tree_get_sibblings($node, $include_object = false, $condition = null)
    {
        $condition = $this->nested_tree_build_sibblings_condition($node, $include_object, $condition);
        return $this->retrieve_objects($node->get_table_name(), $condition, null, null, array(), get_class($node));
    }

    /**
     * Build the conditions for the get / count sibblings methods
     *
     * @param $node NestedTreeNode - the node
     * @param $include_object boolean - if set to true the current node will be added to the sibblings list
     * @param $condition Condition - additional conditions
     */
    private function nested_tree_build_sibblings_condition($node, $include_object = false, $condition = null)
    {
        $siblings_conditions = array();

        $siblings_conditions[] = new EqualityCondition(NestedTreeNode :: PROPERTY_PARENT_ID, $node->get_parent());

        if (! $include_object)
        {
            $siblings_conditions[] = new NotCondition(new EqualityCondition(NestedTreeNode :: PROPERTY_ID, $node->get_id()));
        }

        if ($condition)
        {
            $siblings_conditions[] = $condition;
        }

        return new AndCondition($siblings_conditions);
    }

    /**
     * Retrieve a node from the database
     *
     * @param $table_name String - the table name
     * @param $id int - the id of the node
     */
    public function nested_tree_retrieve_node($node, $id)
    {
        if (! isset($id))
        {
            return false;
        }

        $condition = new EqualityCondition(NestedTreeNode :: PROPERTY_ID, $id);
        return $this->retrieve_object($node->get_table_name(), $condition, array(), get_class($node));
    }

    /**
     * Retrieve a parent node from the database
     *
     * @param $table_name String - the table name
     */
    public function nested_tree_retrieve_parent_from_node($node)
    {
        $condition = new EqualityCondition(NestedTreeNode :: PROPERTY_ID, $node->get_parent_id());
        return $this->retrieve_object($node->get_table_name(), $condition, array(), get_class($node));
    }

    /**
     * Change the left/right values in the tree of every node that comes after the given node
     *
     * @param $node NestedTreeNode - the node
     * @param $previous_visited int - the previous node
     * @param $number_of_elements int - the number of elements which have to be inserted
     * @param $condition Condition - additional condition
     */
    public function nested_tree_add_nested_values($node, $previous_visited, $number_of_elements = 1, $condition = null)
    {
        // Update all necessary left-values
        $conditions = array();
        $conditions[] = new InequalityCondition(NestedTreeNode :: PROPERTY_LEFT_VALUE, InequalityCondition :: GREATER_THAN, $previous_visited);

        if ($condition)
        {
            $conditions[] = $condition;
        }

        $update_condition = new AndCondition($conditions);

        $properties = array(
                NestedTreeNode :: PROPERTY_LEFT_VALUE => self :: escape_column_name(NestedTreeNode :: PROPERTY_LEFT_VALUE) . ' + ' . $this->quote($number_of_elements * 2));
        $res = $this->update_objects($node->get_table_name(), $properties, $update_condition);

        if (! $res)
        {
            return false;
        }

        // Update all necessary right-values
        $conditions = array();
        $conditions[] = new InequalityCondition(NestedTreeNode :: PROPERTY_RIGHT_VALUE, InequalityCondition :: GREATER_THAN, $previous_visited);

        if ($condition)
        {
            $conditions[] = $condition;
        }

        $update_condition = new AndCondition($conditions);

        $properties = array(
                NestedTreeNode :: PROPERTY_RIGHT_VALUE => self :: escape_column_name(NestedTreeNode :: PROPERTY_RIGHT_VALUE) . ' + ' . $this->quote($number_of_elements * 2));
        $res = $this->update_objects($node->get_table_name(), $properties, $update_condition);

        if (! $res)
        {
            return false;
        }

        return true;
    }

    /**
     * Change the left/right values in the tree of every node that is infected due to a delete of the given node
     *
     * @param $node NestedTreeNode - the node
     * @param $condition Condition - additional condition
     */
    public function nested_tree_delete_nested_values($node, $condition)
    {
        $delta = $node->get_right_value() - $node->get_left_value() + 1;

        // Update all necessary nested-values
        $conditions = array();
        $conditions[] = new InequalityCondition(NestedTreeNode :: PROPERTY_LEFT_VALUE, InequalityCondition :: GREATER_THAN, $node->get_left_value());

        if ($condition)
        {
            $conditions[] = $condition;
        }

        $update_condition = new AndCondition($conditions);

        $properties = array();
        $properties[NestedTreeNode :: PROPERTY_LEFT_VALUE] = self :: escape_column_name(NestedTreeNode :: PROPERTY_LEFT_VALUE) . ' - ' . $this->quote($delta);
        $properties[NestedTreeNode :: PROPERTY_RIGHT_VALUE] = self :: escape_column_name(NestedTreeNode :: PROPERTY_RIGHT_VALUE) . ' - ' . $this->quote($delta);
        $res = $this->update_objects($node->get_table_name(), $properties, $update_condition);

        if (! $res)
        {
            return false;
        }

        // Update some more nested-values
        $conditions = array();
        $conditions[] = new InequalityCondition(NestedTreeNode :: PROPERTY_LEFT_VALUE, InequalityCondition :: LESS_THAN, $node->get_left_value());
        $conditions[] = new InequalityCondition(NestedTreeNode :: PROPERTY_RIGHT_VALUE, InequalityCondition :: GREATER_THAN, $node->get_right_value());

        if ($condition)
        {
            $conditions[] = $condition;
        }

        $update_condition = new AndCondition($conditions);

        $properties = array(
                NestedTreeNode :: PROPERTY_RIGHT_VALUE => self :: escape_column_name(NestedTreeNode :: PROPERTY_RIGHT_VALUE) . ' - ' . $this->quote($delta));
        $res = $this->update_objects($node->get_table_name(), $properties, $update_condition);

        if (! $res)
        {
            return false;
        }

        return true;
    }

    /**
     * Locks the given table
     *
     * @param $table_name type
     */
    public function write_lock_table($table_name)
    {
        $query = 'LOCK TABLES ' . $this->escape_table_name($table_name) . ' WRITE, ' . $this->escape_table_name($table_name) . ' AS ' . $this->get_alias($table_name) . ' WRITE ';
        $this->exec($query);
    }

    /**
     * Unlocks all locked tables
     */
    public function unlock_tables()
    {
        $query = 'UNLOCK TABLES';
        $this->exec($query);
    }

    /**
     * Get the fully qualified class name of the Mdb2Database object
     *
     * @return string
     */
    public static function class_name()
    {
        return get_called_class();
    }

    /**
     * Get the namespace of the Mdb2Database object
     *
     * @return string
     */
    public static function context()
    {
        return Utilities :: get_namespace_from_classname(get_called_class());
    }

    public function retrieve_composite_data_class_additional_properties(CompositeDataClass $object)
    {
        if (! $object->is_extended())
        {
            return array();
        }
        $array = array_map(array($this, 'escape_column_name'), $object->get_additional_property_names());

        if (count($array) == 0)
        {
            $array = array("*");
        }

        $query = 'SELECT ' . implode(',', $array) . ' FROM ' . $this->escape_table_name($object :: get_table_name()) . ' WHERE ' . self :: escape_column_name($object :: PROPERTY_ID) . '=' . $this->quote($object->get_id());

        $this->set_limit(1, 0);
        $res = $this->query($query);
        $row = $res->fetchRow(MDB2_FETCHMODE_ASSOC);

        $properties = $object->get_additional_property_names();

        $additional_properties = array();
        if (count($properties) > 0)
        {
            foreach ($properties as $prop)
            {
                if (is_resource($row[$prop]))
                {
                    $data = '';
                    while (! feof($row[$prop]))
                        $data .= fread($row[$prop], 1024);
                    $row[$prop] = $data;
                }
                $additional_properties[$prop] = $row[$prop];
            }
        }

        $res->free();

        return $additional_properties;
    }

    /**
     * The transaction nesting level.
     *
     * @var integer
     */
    private $_transactionNestingLevel = 0;

    /**
     * Flag that indicates whether the current transaction is marked for rollback only.
     * The flag is set when nested transactions attempt to rollback, which poisons the
     * enclosing transactions, forcing them to rollback as well.
     *
     * The flag is unset as soons as an actual rollback occurs.
     *
     * @var boolean
     */
    private $_isRollbackOnly = false;

    public function transactional($function)
    {
        if ($this->connection->supports('transactions') && $this->connection->setTransactionIsolation('REPEATABLE READ', array('wait' => 'WAIT', 'rw' => 'READ WRITE')))
        {
            ++$this->_transactionNestingLevel;

            if ($this->_transactionNestingLevel == 1)
            {
                $this->connection->beginTransaction();
            }
        }
        else
        {
            return false;
        }

        $result = $function($this->connection);

        if ($this->_isRollbackOnly || MDB2 :: isError($result) || ! $result)
        {
            $result = false; // return false to signal an error.

            if ($this->connection->in_transaction)
            {
                if($this->_transactionNestingLevel == 1)
                {
                    $this->connection->rollback(); // echo 'rollback';

                    // Since a rollback was performed and we are at the top level
                    $this->_isRollbackOnly = false;
                }
                else
                {
                    // Alert the enclosing transaction that it must be rolled back
                    // because one of its nested transactions has failed.
                    $this->_isRollbackOnly = true;
                }
            }
        }
        else
        {
            if ($this->connection->in_transaction && $this->_transactionNestingLevel == 1)
            {
                $this->connection->commit(); // echo 'commit';
            }
        }

        --$this->_transactionNestingLevel;

        return $result;
    }

}
