<?php

use Behat\MinkExtension\Context\MinkContext;

/**
 * Extension on the mink context to define our own feature context for behat
 *
 * This feature context adds additional scenario steps
 *
 * @author Sven Vanpoucke - Hogeschool Gent
 */
class FeatureContext extends MinkContext
{
    /****************************************************************************************************************
     * Extensions                                                                                                   *
     ****************************************************************************************************************/

    /**
     * Checks if the admin user is logged in or logs in as an admin user
     *
     * @Given /^I am logged in$/
     */
    public function iAmLoggedIn()
    {
        $this->login('admin', 'admin');
    }

    /**
     * Checks if the given user is logged in or logs in as the given user
     *
     * @Given /^I am logged in as "(admin|teacher|student)"$/
     */
    public function iAmLoggedInAs($user)
    {
        $this->login($user, $user);
    }

    /**
     * Checks if a notification div is available
     *
     * @Then /^I should see a success box$/
     *
     * @throws Exception
     */
    public function iShouldSeeASuccessBox()
    {
        $page = $this->getSession()->getPage();

        $success_box = $page->find('xpath', '//div[contains(@class, "notification-4")]');

        if(is_null($success_box))
        {
            throw new Exception('Success box not found');
        }
    }

    /**
     * Checks if a notification div is available
     *
     * @Then /^I should see an error box$/
     *
     * @throws Exception
     */
    public function iShouldSeeAnErrorBox()
    {
        $page = $this->getSession()->getPage();

        $error_box = $page->find('xpath', '//div[contains(@class, "notification-1")]');

        if(is_null($error_box))
        {
            throw new Exception('Error box not found');
        }
    }

    /**
     * Presses a button with a given button name (bugfix for mink problem)
     *
     * @When /^I press button with name "([^"]*)"$/
     *
     * @param string $name
     *
     * @throws Exception
     */
    public function iPressButtonWithName($name)
    {
        $page = $this->getSession()->getPage();

        $button = $page->find('xpath', '//button[@name="' . $name . '"]');

        if(is_null($button))
        {
            throw new Exception('Button not found');
        }

        $button->click();
    }

    /****************************************************************************************************************
     * Hooks                                                                                                        *
     ****************************************************************************************************************/

    /**
     * Checks for debugging after a failed scenario
     *
     * @AfterScenario
     */
    public function after_scenario_hook($event)
    {
        if($event->getResult() == \Behat\Behat\Event\StepEvent :: FAILED)
        {
            //$this->printDebug('content: ' . $this->getSession()->getPage()->getContent());
        }
    }

    /****************************************************************************************************************
     * Helper Functionality                                                                                         *
     ****************************************************************************************************************/

    /**
     * Checks if the given user is logged in already or logs in as the given user with the given password
     *
     * @param string $user
     * @param string $password
     *
     * @return boolean
     */
    protected function login($user, $password)
    {
        $page = $this->getSession()->getPage();

        if($page->findLink('Logout'))
        {
            return;
        }

        $login_field = $page->findField('login');
        $login_field->setValue($user);

        $password_field = $page->findField('password');
        $password_field->setValue($password);

        $login_button = $page->findButton('Login');
        $login_button->click();
    }

}
