<?php

namespace common\libraries;

/**
 * Class to provide authentication for webservices
 *
 * Authentication uses the Authentication header that can be sent to the webservice
 * Secures the authentication through signing a combination of several parameters through HMAC-SHA512
 * The shared secret key for signing can be found in the users table
 * The combination of username / timestamp / nonce should be unique so requests can not be hijacked and executed again
 * Parameter list is username&nonce&password&timestamp&ipaddress
 *
 * Example
 * GET /rest.php?application=user&object=user HTTP/1.1
 * Host: localhost
 * Authorization: Chamilo realm="Chamilo"
 * username="admin",
 * timestamp="137131200",
 * nonce="wIjqoS",
 * signature="74KNZJeDHnMBp0EMJ9ZHt%2FXKycU%3D"
 *
 */

class ChamiloWebserviceAuthentication extends WebserviceAuthentication
{
    const PARAM_AUTHORIZATION = 'Authorization';
    
    const PARAM_USERNAME = 'username';
    const PARAM_TIMESTAMP = 'timestamp';
    const PARAM_NONCE = 'nonce';
    const PARAM_SIGNATURE = 'signature';
    
    const PARAM_REMOTE_ADDRESS = 'REMOTE_ADDR';
    
    const PARAM_HASH_ALGORITHM = 'sha512';

    /**
     * Checks if the authentication is valid
     * @return boolean
     */
    public function is_valid()
    {
        $headers = apache_request_headers();
        if (! $headers[self :: PARAM_AUTHORIZATION])
        {
            $this->add_message(Translation :: get('WebserviceNoAuthorizationHeadersFound'));
            return false;
        }
        
        $authorization = $this->parse_header($headers[self :: PARAM_AUTHORIZATION]);
        $username = $authorization[self :: PARAM_USERNAME];
        
        if (! $this->load_user_by_username($username))
        {
            $this->add_message(Translation :: get('WebserviceNoUserFoundWithGivenUsername'));
            return false;
        }
        
        $timestamp = $authorization[self :: PARAM_TIMESTAMP];
        if ($timestamp < time())
        {
            $this->add_message(Translation :: get('WebserviceTimeInPast'));
            return false;
        }
        
        $signature = $this->calculate_signature($authorization);
        
        if ($signature == $authorization[self :: PARAM_SIGNATURE])
        {
            return true;
        }
        
        $this->add_message(Translation :: get('WebserviceWrongSignature'));
        
        return false;
    }

    /**
     * Calculates the signature for the authentication
     * @param Array $authorization - authorization headers
     * @return String
     */
    private function calculate_signature($authorization)
    {
        $user = $this->get_user();
        
        $base_text = $authorization[self :: PARAM_USERNAME] . '&' . $authorization[self :: PARAM_NONCE] . '&' . $user->get_password() . '&' . $authorization[self :: PARAM_TIMESTAMP] . '&' . $_SERVER[self :: PARAM_REMOTE_ADDRESS];
        
        $secret_key = $user->get_security_token();
        
        return hash_hmac(self :: PARAM_HASH_ALGORITHM, $base_text, $secret_key);
    }

    /**
     * Parses the authorization header and moves the data to an array
     * @param String $header
     * @return Array
     */
    private function parse_header($header)
    {
        $needed_parts = array(self :: PARAM_NONCE => 1, self :: PARAM_USERNAME => 1, self :: PARAM_TIMESTAMP => 1, 
                self :: PARAM_SIGNATURE => 1);
        
        $data = array();
        $keys = implode('|', array_keys($needed_parts));
        
        preg_match_all('@(' . $keys . ')=(?:([\'"])([^\2]+?)\2|([^\s,]+))@', $header, $matches, PREG_SET_ORDER);
        
        foreach ($matches as $m)
        {
            $data[$m[1]] = $m[3] ? $m[3] : $m[4];
            unset($needed_parts[$m[1]]);
        }
        
        return $needed_parts ? false : $data;
    }

}
