<?php
namespace core\metadata;
use common\libraries\DataClass;
use common\libraries\Utilities;
use common\libraries\EqualityCondition;
use common\libraries\AndCondition;
use common\libraries\OrCondition;
use common\libraries\Translation;
/**
 * This class describes a MetadataAttribute data object
 * @author Sven Vanpoucke
 * @author Jens Vanderheyden
 */
class MetadataAttribute extends DataClass
{
    const CLASS_NAME = __CLASS__;
    
    /**
     * MetadataAttribute properties
     */
    const PROPERTY_ID = 'id';
    const PROPERTY_NAMESPACE = 'namespace';
    const PROPERTY_NS_PREFIX = 'ns_prefix';
    const PROPERTY_NAME = 'name';
    
    private $ns_prefix = false;

    /**
     * Get the default properties
     * @return array The property names.
     */
    public static function get_default_property_names()
    {
        return array(self :: PROPERTY_ID, self :: PROPERTY_NAMESPACE, self :: PROPERTY_NAME);
    }

    public function get_data_manager()
    {
        return MetadataDataManager :: get_instance();
    }

    /**
     * Returns the id of this MetadataAttribute.
     * @return the id.
     */
    public function get_id()
    {
        return $this->get_default_property(self :: PROPERTY_ID);
    }

    /**
     * Sets the id of this MetadataAttribute.
     * @param id
     */
    public function set_id($id)
    {
        $this->set_default_property(self :: PROPERTY_ID, $id);
    }

    /**
     * Returns the ns_prefix of this MetadataElement.
     * @return the namespace_id.
     */
    public function get_namespace()
    {
        return $this->get_default_property(self :: PROPERTY_NAMESPACE);
    }

    /**
     * Sets the ns_prefix of this MetadataElement.
     * @param namespace
     */
    public function set_namespace($namespace)
    {
        $this->set_default_property(self :: PROPERTY_NAMESPACE, $namespace);
    }

    /**
     * Returns the ns_prefix of this MetadataElement.
     * @return the ns_prefix.
     */
    public function get_ns_prefix()
    {
        if (! $this->ns_prefix)
        {
            $namespace = $this->get_data_manager()->retrieve_metadata_namespace($this->get_namespace());
            
            if (! $namespace)
            {
                return false;
            }
            $this->set_ns_prefix($namespace->get_ns_prefix());
        }
        return $this->ns_prefix;
    }

    /**
     * Sets the ns_prefix of this MetadataAttribute.
     * @param ns_prefix
     */
    public function set_ns_prefix($ns_prefix)
    {
        $this->ns_prefix = $ns_prefix;
    }

    /**
     * Returns the name of this MetadataAttribute.
     * @return the name.
     */
    public function get_name()
    {
        return $this->get_default_property(self :: PROPERTY_NAME);
    }

    /**
     * Sets the name of this MetadataAttribute.
     * @param name
     */
    public function set_name($name)
    {
        $this->set_default_property(self :: PROPERTY_NAME, $name);
    }

    public static function get_table_name()
    {
        return Utilities :: camelcase_to_underscores(Utilities :: get_classname_from_namespace(self :: CLASS_NAME));
    }

    public function render_name()
    {
        $pref = $this->get_ns_prefix();
        $prefix = (empty($pref)) ? '' : $this->get_ns_prefix() . ':';
        return $prefix . $this->get_name();
    }

    /*
     * object creation if name + ns_prfix are unique
     * return MetadaPropertyattributeType or false
     */
    public function create()
    {
        $condition1 = new EqualityCondition(MetadataAttribute :: PROPERTY_NAME, $this->get_name());
        $condition2 = new EqualityCondition(MetadataAttribute :: PROPERTY_NAMESPACE, $this->get_namespace());
        
        $condition = new AndCondition($condition1, $condition2);
        
        if ($this->get_data_manager()->count_metadata_attributes($condition) >= 1)
        {
            $this->add_error(Translation :: get('ObjectAlreadyExists'));
            return false;
        }
        return parent :: create();
    }

    public function delete()
    {
        //check dependencies before deleting
        $mdm = $this->get_data_manager();
        
        $condition = new EqualityCondition(MetadataAttributeValue :: PROPERTY_ATTRIBUTE_ID, $this->get_id());
        $count = $mdm->count_metadata_attributes($condition);
        
        if ($count == 0)
        {
            if (parent :: delete())
            {
                //delete associations
                $condition1 = new EqualityCondition(MetadataElementNesting :: PROPERTY_PARENT_ID, $this->get_id());
                
                $condition2 = new EqualityCondition(MetadataElementNesting :: PROPERTY_CHILD_ID, $this->get_id());
                $condition3 = new EqualityCondition(MetadataElementNesting :: PROPERTY_CHILD_TYPE, Utilities :: camelcase_to_underscores(self :: CLASS_NAME));
                $condition4 = new AndCondition($condition2, $condition3);
                
                $condition = new OrCondition($condition1, $condition4);
                
                $metadata_element_nestings = $mdm->retrieve_metadata_element_nestings($condition);
                
                while ($metadata_element_nesting = $metadata_element_nestings->next_result())
                {
                    $metadata_element_nesting->delete();
                }
                
                $condition1 = new EqualityCondition(MetadataAttributeAssociation :: PROPERTY_PARENT_ID, $this->get_id());
                
                $condition2 = new EqualityCondition(MetadataAttributeAssociation :: PROPERTY_CHILD_ID, $this->get_id());
                $condition3 = new EqualityCondition(MetadataAttributeAssociation :: PROPERTY_CHILD_TYPE, Utilities :: get_classname_from_namespace(Utilities :: camelcase_to_underscores(self :: CLASS_NAME)));
                $condition4 = new AndCondition($condition2, $condition3);
                
                $condition = new OrCondition($condition1, $condition4);
                
                $metadata_attribute_associations = $mdm->retrieve_metadata_attribute_associations($condition);
                
                while ($metadata_attribute_association = $metadata_attribute_associations->next_result())
                {
                    $metadata_attribute_association->delete();
                }
                return true;
            }
        }
        return false;
    }
}
