<?php

namespace core\migration;

use common\libraries\CommonDataManager;
use common\libraries\Translation;
use repository\RepositoryDataManager;
use common\libraries\Utilities;
use repository\content_object\calendar_event\CalendarEvent;
use application\personal_calendar\PersonalCalendarPublication;

/**
 * $Id: dokeos185_personal_agenda.class.php 221 2009-11-13 14:36:41Z vanpouckesven $
 * @package migration.platform.dokeos185
 */
require_once dirname(__FILE__) . '/../dokeos185_migration_data_class.class.php';
require_once dirname(__FILE__) . '/../dokeos185_data_manager.class.php';

/**
 * Class that represents the personal agenda data from dokeos 1.8.5
 * @author Sven Vanpoucke
 */
class Dokeos185PersonalAgenda extends Dokeos185MigrationDataClass
{
    const CLASS_NAME = __CLASS__;
    const TABLE_NAME = 'personal_agenda';
    const DATABASE_NAME = 'user_personal_database';

    /**
     * personal agenda properties
     */
    const PROPERTY_ID = 'id';
    const PROPERTY_USER = 'user';
    const PROPERTY_TITLE = 'title';
    const PROPERTY_TEXT = 'text';
    const PROPERTY_DATE = 'date';
    const PROPERTY_ENDDATE = 'enddate';
    const PROPERTY_COURSE = 'course';

    /**
     * Get the default properties of all personal agenda.
     * @return array The property names.
     */
    public static function get_default_property_names()
    {
	return array(self :: PROPERTY_ID, self :: PROPERTY_USER, self :: PROPERTY_TITLE, self :: PROPERTY_TEXT, self :: PROPERTY_DATE, self :: PROPERTY_ENDDATE, self :: PROPERTY_COURSE);
    }

    /**
     * Returns the id of this personal agenda.
     * @return int The id.
     */
    public function get_id()
    {
	return $this->get_default_property(self :: PROPERTY_ID);
    }

    /**
     * Returns the user of this personal agenda
     * @return int The user ID
     */
    public function get_user()
    {
	return $this->get_default_property(self :: PROPERTY_USER);
    }

    /**
     * Returns the date of this personal agenda
     * @return int The date
     */
    public function get_date()
    {
	return $this->get_default_property(self :: PROPERTY_DATE);
    }

    /**
     * Returns the end date of this personal agenda
     * @return int The end date
     */
    public function get_enddate()
    {
	return $this->get_default_property(self :: PROPERTY_ENDDATE);
    }

    /**
     * Returns the course of this personal agenda
     * @return int The course ID
     */
    public function get_course()
    {
	return $this->get_default_property(self :: PROPERTY_COURSE);
    }

    /**
     * Returns the title of this personal agenda
     * @return string The title
     */
    public function get_title()
    {
	return $this->get_default_property(self :: PROPERTY_TITLE);
    }

    /**
     * Returns the text of this personal agenda
     * @return string The text
     */
    public function get_text()
    {
	return $this->get_default_property(self :: PROPERTY_TEXT);
    }

    /**
     * Check if the personal agenda is valid
     * @return true if the personal agenda is valid
     */
    public function is_valid()
    {
	$owner_id = $this->get_id_reference($this->get_user(), 'main_database.user');

	if (!$owner_id || (!$this->get_title() && trim(strip_tags($this->get_text())) == '') ||
		(!$this->get_title() && !$this->get_text()) || !$this->get_date())
	{
	    if (!$owner_id)
	    {
		$this->set_message(Translation :: get('UserDeletedMessage', array('ID' => $this->get_user())));
	    }
	    else
	    {
		$this->set_message(Translation :: get('GeneralInvalidMessage', array('TYPE' => 'personal_agenda', 'ID' => $this->get_id())));
	    }
	    $this->create_failed_element($this->get_id());

	    return false;
	}
	return true;
    }

    /**
     * Convert to new personal agenda
     * @return the new personal agenda
     */
    public function convert_data()
    {
	$new_start_date = strtotime($this->get_date());
	$new_end_date = strtotime($this->get_enddate());

	// Create calendar event
	$chamilo_calendar_event = new CalendarEvent();
	$chamilo_calendar_event->set_start_date($new_start_date);

	if (!$this->get_enddate())
	{
	    $chamilo_calendar_event->set_end_date($new_start_date);
	}
	else
	{
	    $chamilo_calendar_event->set_end_date($new_end_date);
	}

	if (!$this->get_title())
	{
	    $chamilo_calendar_event->set_title(Utilities :: truncate_string($this->get_text(), 50, true));
	}
	else
	{
	    $chamilo_calendar_event->set_title($this->get_title());
	}

	if (!$this->get_text())
	{
	    $chamilo_calendar_event->set_description($this->get_title());
	}
	else
	{
	    $chamilo_calendar_event->set_description($this->get_text());
	}

	//Get owner_ID from
	$owner_id = $this->get_id_reference($this->get_user(), 'main_database.user');
	if ($owner_id)
	{
	    $chamilo_calendar_event->set_owner_id($owner_id);
	}
	$title = $chamilo_calendar_event->get_title();

	$repository_category_root_id = $this->get_repository_migration_root_category_id($owner_id);
	$chamilo_category_id = RepositoryDataManager :: get_repository_category_by_name_or_create_new($owner_id, Translation :: get('CalendarEvents'), $repository_category_root_id);
	$chamilo_calendar_event->set_parent_id($chamilo_category_id);
	$chamilo_calendar_event->create();
	$this->set_message(Translation :: get('GeneralConvertedMessage', array('TYPE' => 'calendar_event', 'OLD_ID' => $this->get_id(), 'NEW_ID' => $chamilo_calendar_event->get_id())));
	//control if the personal_agenda application exists
	$is_registered = CommonDataManager :: is_registered('application\personal_calendar');

	if ($is_registered)
	{
	    //Create personal agenda publication
	    $chamilo_personal_calendar = new PersonalCalendarPublication();
	    $chamilo_personal_calendar->set_content_object_id($chamilo_calendar_event->get_id());
	    $chamilo_personal_calendar->set_publisher($owner_id);
	    $chamilo_personal_calendar->set_published($new_start_date);
	    $chamilo_personal_calendar->create_all();

	    return $chamilo_calendar_event;
	}
    }

    public static function get_table_name()
    {
	return Utilities :: camelcase_to_underscores(substr(Utilities :: get_classname_from_namespace(__CLASS__), 9));
    }

    public static function get_class_name()
    {
	return self :: CLASS_NAME;
    }

    public function get_database_name()
    {
	return self :: DATABASE_NAME;
    }

}
