<?php
namespace group;

use common\libraries\Condition;
use common\libraries\EqualityCondition;
use common\libraries\EmptyResultSet;
use common\libraries\InCondition;
use common\libraries\InequalityCondition;
use user\UserDataManager;
use common\libraries\AndCondition;
use common\libraries\OrCondition;
use common\libraries\Joins;
use common\libraries\DataClassJoin;
use common\libraries\DataClassProperties;
use common\libraries\DataClassRetrievesParameters;
use common\libraries\RecordRetrievesParameters;
use common\libraries\PropertyConditionVariable;
use common\libraries\FunctionConditionVariable;
use common\libraries\StaticConditionVariable;
use common\libraries\DataClassRetrieveParameters;

/**
 * $Id: group_data_manager.class.php 157 2009-11-10 13:44:02Z vanpouckesven $
 *
 * @package group.lib
 */
/**
 * This is a skeleton for a data manager for the Users table. Data managers must extend this class and implement its
 * abstract methods.
 *
 * @author Hans De Bisschop
 * @author Dieter De Neef
 */
class DataManager extends \common\libraries\DataManager
{

    /**
     * Cache for the direct subscribed groups
     *
     * @var Group[]
     */
    private static $direct_subscribed_groups_cache;

    /**
     * Gets the type of DataManager to be instantiated
     *
     * @return string
     */
    public static function get_type()
    {
        return 'doctrine';
    }

    public static function get_root_group()
    {
        return static :: retrieve(
            Group :: class_name(),
            new EqualityCondition(
                new PropertyConditionVariable(Group :: class_name(), Group :: PROPERTY_PARENT_ID),
                new StaticConditionVariable(0)));
    }

    public static function retrieve_group_by_code($code)
    {
        $condition = new EqualityCondition(
            new PropertyConditionVariable(Group :: class_name(), Group :: PROPERTY_CODE),
            new StaticConditionVariable($code));

        return self :: retrieve(Group :: class_name(), $condition);
    }

    public static function retrieve_group_by_code_and_parent_id($code, $parent_id)
    {
        $conditions = array();
        $conditions[] = new EqualityCondition(
            new PropertyConditionVariable(Group :: class_name(), Group :: PROPERTY_CODE),
            new StaticConditionVariable($code));
        $conditions[] = new EqualityCondition(
            new PropertyConditionVariable(Group :: class_name(), Group :: PROPERTY_PARENT_ID),
            new StaticConditionVariable($parent_id));
        $condition = new AndCondition($conditions);

        return self :: retrieve(Group :: class_name(), new DataClassRetrieveParameters($condition));
    }

    public static function subscribe_user_to_group_by_official_code_and_group_code($official_code, $group_code)
    {
        $group_rel_user = self :: make_group_rel_user_from_official_code_and_group_code($official_code, $group_code);

        if ($group_rel_user)
        {
            return $group_rel_user->create();
        }

        return false;
    }

    public static function remove_user_from_group_by_official_code_and_group_code($official_code, $group_code)
    {
        $group_rel_user = self :: make_group_rel_user_from_official_code_and_group_code($official_code, $group_code);

        if ($group_rel_user)
        {
            return $group_rel_user->delete();
        }

        return false;
    }

    // cache to make subscribing users in batch more performant
    private static $group_rel_user_cache;

    private static function make_group_rel_user_from_official_code_and_group_code($official_code, $group_code)
    {
        if (! self :: $group_rel_user_cache[$group_code][$official_code])
        {
            $group = self :: retrieve_group_by_code($group_code);
            $user = UserDataManager :: retrieve_user_by_official_code($official_code);

            if (! $group || ! $user)
            {
                return false;
            }

            $group_rel_user = new GroupRelUser();
            $group_rel_user->set_group_id($group->get_id());
            $group_rel_user->set_user_id($user->get_id());
            self :: $group_rel_user_cache[$group_code][$official_code] = $group_rel_user;
        }

        return self :: $group_rel_user_cache[$group_code][$official_code];
    }

    public static function is_group_member($group_id, $user_id)
    {
        $conditions = array();
        $conditions[] = new EqualityCondition(GroupRelUser :: PROPERTY_GROUP_ID, $group_id);
        $conditions[] = new EqualityCondition(GroupRelUser :: PROPERTY_USER_ID, $user_id);
        $condition = new AndCondition($conditions);

        return static :: count(GroupRelUser :: class_name(), $condition) > 0;
    }

    public static function retrieve_user_groups($user_id)
    {
        $join = new DataClassJoin(
            GroupRelUser :: class_name(),
            new AndCondition(
                array(
                    new EqualityCondition(
                        new PropertyConditionVariable(GroupRelUser :: class_name(), GroupRelUser :: PROPERTY_USER_ID),
                        new StaticConditionVariable($user_id)),

                    new EqualityCondition(
                        new PropertyConditionVariable(GroupRelUser :: class_name(), GroupRelUser :: PROPERTY_GROUP_ID),
                        new PropertyConditionVariable(Group :: class_name(), Group :: PROPERTY_ID)))));

        $joins = new Joins(array($join));

        $parameters = new DataClassRetrievesParameters(null, null, null, null, $joins);
        return static :: retrieves(Group :: class_name(), $parameters);
    }

    public static function retrieve_all_groups_and_templates($user_id)
    {
        // First: retrieve an array of group ids which this user is (indirectly) subscribed to.
        $group_ids = static :: retrieve_all_subscribed_groups_array($user_id, true);

        // Second: construct a result set that that finds templates for these groups.
        if (! empty($group_ids))
        {
            $properties = new DataClassProperties();

            $properties->add(new PropertyConditionVariable(Group :: class_name(), Group :: PROPERTY_ID));
            $properties->add(new PropertyConditionVariable(Group :: class_name(), Group :: PROPERTY_PARENT_ID));
            $properties->add(
                new PropertyConditionVariable(
                    GroupRightsTemplate :: class_name(),
                    GroupRightsTemplate :: PROPERTY_RIGHTS_TEMPLATE_ID));

            $join_conditions = array();

            $join_conditions[] = new InCondition(
                new PropertyConditionVariable(
                    GroupRightsTemplate :: class_name(),
                    GroupRightsTemplate :: PROPERTY_GROUP_ID),
                $group_ids);

            $join_conditions[] = new EqualityCondition(
                new PropertyConditionVariable(
                    GroupRightsTemplate :: class_name(),
                    GroupRightsTemplate :: PROPERTY_GROUP_ID),
                new PropertyConditionVariable(Group :: class_name(), Group :: PROPERTY_ID));

            $join = new DataClassJoin(GroupRightsTemplate :: class_name(), new AndCondition($join_conditions));

            $parameters = new RecordRetrievesParameters($properties, /* $condition = */ null, /* $count = */ null, /* $offset = */ null,
                    /* $order_by = */ null, new Joins(array($join)));

            return static :: records(Group :: class_name(), $parameters)->as_array();
        }

        return new EmptyResultSet();
    }

    public static function retrieve_all_subscribed_groups_array($user_id, $only_retrieve_ids = false)
    {
        // First: retrieve the left and right values of groups the user is directly subscribed to.
        $properties = new DataClassProperties();

        $properties->add(new PropertyConditionVariable(Group :: class_name(), Group :: PROPERTY_LEFT_VALUE));
        $properties->add(new PropertyConditionVariable(Group :: class_name(), Group :: PROPERTY_RIGHT_VALUE));

        $join_conditions = array();

        $join_conditions[] = new EqualityCondition(
            new PropertyConditionVariable(GroupRelUser :: class_name(), GroupRelUser :: PROPERTY_USER_ID),
            new StaticConditionVariable($user_id));

        $join_conditions[] = new EqualityCondition(
            new PropertyConditionVariable(GroupRelUser :: class_name(), GroupRelUser :: PROPERTY_GROUP_ID),
            new PropertyConditionVariable(Group :: class_name(), Group :: PROPERTY_ID));

        $join = new DataClassJoin(GroupRelUser :: class_name(), new AndCondition($join_conditions));

        $parameters = new RecordRetrievesParameters($properties, /* $condition = */ null, /* $count = */ null, /* $offset = */ null,
                /* $order_by = */ null, new Joins(array($join)));

        $directly_subscribed_group_nesting_values = static :: records(Group :: class_name(), $parameters)->as_array();

        // Second: retrieve the (ids of) directly subscribed groups and their ancestors
        if (count($directly_subscribed_group_nesting_values) > 0)
        {
            $conditions = array();

            $or_conditions = array();

            foreach ($directly_subscribed_group_nesting_values as $descendent)
            {
                $or_conditions[] = new AndCondition(
                    array(
                        new InequalityCondition(
                            new PropertyConditionVariable(Group :: class_name(), Group :: PROPERTY_LEFT_VALUE),
                            InequalityCondition :: LESS_THAN_OR_EQUAL,
                            new StaticConditionVariable($descendent[Group :: PROPERTY_LEFT_VALUE])),

                        new InequalityCondition(
                            new PropertyConditionVariable(Group :: class_name(), Group :: PROPERTY_RIGHT_VALUE),
                            InequalityCondition :: GREATER_THAN_OR_EQUAL,
                            new StaticConditionVariable($descendent[Group :: PROPERTY_RIGHT_VALUE]))));
            }

            $conditions[] = new OrCondition($or_conditions);

            if ($only_retrieve_ids)
            {
                $properties = new DataClassProperties();

                $properties->add(
                    new FunctionConditionVariable(
                        FunctionConditionVariable :: DISTINCT,
                        new PropertyConditionVariable(Group :: class_name(), Group :: PROPERTY_ID)));

                $parameters = new RecordRetrievesParameters($properties, new AndCondition($conditions));

                $records = static :: records(Group :: class_name(), $parameters);
                $group_ids = array();

                foreach ($records->as_array() as $group)
                {
                    $group_ids[] = $group[Group :: PROPERTY_ID];
                }

                return $group_ids;
            }
            else
            {
                $parameters = new DataClassRetrievesParameters(new AndCondition($conditions));

                return static :: retrieves(Group :: class_name(), $parameters);
            }
        }

        if ($only_retrieve_ids)
        {
            return array();
        }
        else
        {
            // If the user is not a member of any group
            return new EmptyResultSet();
        }
    }

    public static function retrieve_groups_and_subgroups($group_ids, $additional_condition = null, $count = null, $offset = null,
        $order_by = array())
    {
        if (count($group_ids) == 0)
        {
            $group_ids[] = - 1;
        }

        // First: retrieve the left and right values of the groups provided by the caller.
        $properties = new DataClassProperties();

        $properties->add(new PropertyConditionVariable(Group :: class_name(), Group :: PROPERTY_LEFT_VALUE));
        $properties->add(new PropertyConditionVariable(Group :: class_name(), Group :: PROPERTY_RIGHT_VALUE));

        $condition = new InCondition(
            new PropertyConditionVariable(Group :: class_name(), Group :: PROPERTY_ID),
            $group_ids);

        $parameters = new RecordRetrievesParameters($properties, $condition);

        $group_nesting_values = static :: records(Group :: class_name(), $parameters)->as_array();

        // Second: retrieve the indicated groups and their descendents
        if (count($group_nesting_values) > 0)
        {
            $conditions = array();

            if ($additional_condition instanceof Condition)
            {
                $conditions[] = $additional_condition;
            }

            $or_conditions = array();

            foreach ($group_nesting_values as $ancestor)
            {
                $or_conditions[] = new AndCondition(
                    array(
                        new InequalityCondition(
                            new PropertyConditionVariable(Group :: class_name(), Group :: PROPERTY_LEFT_VALUE),
                            InequalityCondition :: GREATER_THAN_OR_EQUAL,
                            new StaticConditionVariable($ancestor[Group :: PROPERTY_LEFT_VALUE])),

                        new InequalityCondition(
                            new PropertyConditionVariable(Group :: class_name(), Group :: PROPERTY_RIGHT_VALUE),
                            InequalityCondition :: LESS_THAN_OR_EQUAL,
                            new StaticConditionVariable($ancestor[Group :: PROPERTY_RIGHT_VALUE]))));
            }

            $conditions[] = new OrCondition($or_conditions);

            $parameters = new DataClassRetrievesParameters(new AndCondition($conditions), $count, $offset, $order_by);

            return static :: retrieves(Group :: class_name(), $parameters);
        }

        // If the provided group_ids do not exist or were empty
        return new EmptyResultSet();
    }

    /**
     * Retrieves the direct subscribed groups as an array
     *
     * @param int $user_id
     * @param bool $only_retrieve_ids
     *
     * @return Group[] | int[]
     */
    public static function retrieve_direct_subscribed_groups_array($user_id, $only_retrieve_ids = false)
    {
        $cache_hash = md5($user_id . ':' . (int) $only_retrieve_ids);

        if (is_null(self :: $direct_subscribed_groups_cache[$cache_hash]))
        {
            $direct_subscribed_groups = self :: retrieve_direct_subscribed_groups($user_id);
            if (! $direct_subscribed_groups->is_empty())
            {
                if ($only_retrieve_ids)
                {
                    while ($direct_subscribed_group = $direct_subscribed_groups->next_result())
                    {
                        self :: $direct_subscribed_groups_cache[$cache_hash][] = $direct_subscribed_group->get_id();
                    }
                }
                else
                {
                    while ($direct_subscribed_group = $direct_subscribed_groups->next_result())
                    {
                        self :: $direct_subscribed_groups_cache[$cache_hash][] = $direct_subscribed_group;
                    }
                }
            }
        }
        return self :: $direct_subscribed_groups_cache[$cache_hash];
    }

    /**
     * Retrieves the groups a user is linked to Use the condition to limit or exclude certain groups
     *
     * @param int $user_id
     *
     * @return Mdb2ResultSet
     */
    public static function retrieve_direct_subscribed_groups($user_id)
    {
        $condition = new EqualityCondition(
            new PropertyConditionVariable(GroupRelUser :: class_name(), GroupRelUser :: PROPERTY_USER_ID),
            new StaticConditionVariable($user_id));

        $joins = array();
        $joins[] = new DataClassJoin(
            GroupRelUser :: class_name(),
            new EqualityCondition(
                new PropertyConditionVariable(GroupRelUser :: class_name(), GroupRelUser :: PROPERTY_GROUP_ID),
                new PropertyConditionVariable(Group :: class_name(), Group :: PROPERTY_ID)));
        $joins = new Joins($joins);

        $parameters = new DataClassRetrievesParameters($condition, null, null, null, $joins);
        return self :: retrieves(Group :: class_name(), $parameters);
    }
}
