<?php
namespace repository\content_object\assessment;

use common\libraries\PropertyConditionVariable;
use common\libraries\StaticConditionVariable;
use common\libraries\Utilities;
use common\libraries\EqualityCondition;
use common\libraries\ComplexContentObjectSupport;
use repository\ContentObject;
use repository\ComplexContentObjectItem;
use repository\content_object\assessment_rating_question\AssessmentRatingQuestion;
use repository\content_object\assessment_open_question\AssessmentOpenQuestion;
use repository\content_object\hotspot_question\HotspotQuestion;
use repository\content_object\fill_in_blanks_question\FillInBlanksQuestion;
use repository\content_object\assessment_multiple_choice_question\AssessmentMultipleChoiceQuestion;
use repository\content_object\assessment_matching_question\AssessmentMatchingQuestion;
use repository\content_object\assessment_select_question\AssessmentSelectQuestion;
use repository\content_object\assessment_matrix_question\AssessmentMatrixQuestion;
use repository\content_object\assessment_match_numeric_question\AssessmentMatchNumericQuestion;
use repository\content_object\assessment_match_text_question\AssessmentMatchTextQuestion;
use repository\content_object\ordering_question\OrderingQuestion;

/**
 * This class represents an assessment
 *
 * @package repository.lib.content_object.assessment
 */
class Assessment extends ContentObject implements ComplexContentObjectSupport
{
    const PROPERTY_TIMES_TAKEN = 'times_taken';
    const PROPERTY_AVERAGE_SCORE = 'average_score';
    const PROPERTY_MAXIMUM_SCORE = 'maximum_score';
    const PROPERTY_MAXIMUM_ATTEMPTS = 'max_attempts';
    const PROPERTY_QUESTIONS_PER_PAGE = 'questions_per_page';
    const PROPERTY_MAXIMUM_TIME = 'max_time';
    const PROPERTY_RANDOM_QUESTIONS = 'random_questions';
    const CLASS_NAME = __CLASS__;

    /**
     * The number of questions in this assessment
     *
     * @var int
     */
    private $question_count;

    /**
     * An ObjectResultSet containing all ComplexContentObjectItem objects for individual questions.
     *
     * @var ObjectResultSet
     */
    private $questions;

    public static function get_type_name()
    {
        return Utilities :: camelcase_to_underscores(Utilities :: get_classname_from_namespace(self :: CLASS_NAME));
    }

    public static function get_additional_property_names()
    {
        return array(
            self :: PROPERTY_MAXIMUM_ATTEMPTS,
            self :: PROPERTY_QUESTIONS_PER_PAGE,
            self :: PROPERTY_MAXIMUM_TIME,
            self :: PROPERTY_RANDOM_QUESTIONS);
    }

    public function set_assessment_type($type)
    {
        $this->set_additional_property(self :: PROPERTY_ASSESSMENT_TYPE, $type);
    }

    public function get_maximum_attempts()
    {
        return $this->get_additional_property(self :: PROPERTY_MAXIMUM_ATTEMPTS);
    }

    public function has_unlimited_attempts()
    {
        return $this->get_maximum_attempts() == 0;
    }

    public function set_maximum_attempts($value)
    {
        $this->set_additional_property(self :: PROPERTY_MAXIMUM_ATTEMPTS, $value);
    }

    public function get_maximum_time()
    {
        return $this->get_additional_property(self :: PROPERTY_MAXIMUM_TIME);
    }

    public function set_maximum_time($value)
    {
        $this->set_additional_property(self :: PROPERTY_MAXIMUM_TIME, $value);
    }

    public function get_questions_per_page()
    {
        return $this->get_additional_property(self :: PROPERTY_QUESTIONS_PER_PAGE);
    }

    public function set_questions_per_page($value)
    {
        $this->set_additional_property(self :: PROPERTY_QUESTIONS_PER_PAGE, $value);
    }

    public function get_random_questions()
    {
        return $this->get_additional_property(self :: PROPERTY_RANDOM_QUESTIONS);
    }

    public function set_random_questions($random_questions)
    {
        $this->set_additional_property(self :: PROPERTY_RANDOM_QUESTIONS, $random_questions);
    }

    public function get_allowed_types()
    {
        $allowed_types = array();
        $allowed_types[] = AssessmentRatingQuestion :: class_name();
        $allowed_types[] = AssessmentOpenQuestion :: class_name();
        $allowed_types[] = HotspotQuestion :: class_name();
        $allowed_types[] = FillInBlanksQuestion :: class_name();
        $allowed_types[] = AssessmentMultipleChoiceQuestion :: class_name();
        $allowed_types[] = AssessmentMatchingQuestion :: class_name();
        $allowed_types[] = AssessmentSelectQuestion :: class_name();
        $allowed_types[] = AssessmentMatrixQuestion :: class_name();
        $allowed_types[] = AssessmentMatchNumericQuestion :: class_name();
        $allowed_types[] = AssessmentMatchTextQuestion :: class_name();
        $allowed_types[] = OrderingQuestion :: class_name();
        return $allowed_types;
    }

    public function get_table()
    {
        return self :: get_type_name();
    }

    public function count_questions()
    {
        if (! isset($this->question_count))
        {
            $this->question_count = \repository\DataManager :: count_complex_content_object_items(
                ComplexContentObjectItem :: class_name(),
                new EqualityCondition(
                    ComplexContentObjectItem :: PROPERTY_PARENT,
                    $this->get_id(),
                    ComplexContentObjectItem :: get_table_name()));
        }

        return $this->question_count;
    }

    public function get_questions()
    {
        if (! isset($this->questions))
        {
            $condition = new EqualityCondition(
                ComplexContentObjectItem :: PROPERTY_PARENT,
                $this->get_id(),
                ComplexContentObjectItem :: get_table_name());
            $this->questions = \repository\DataManager :: retrieve_complex_content_object_items(
                ComplexContentObjectItem :: class_name(),
                $condition);
        }

        return $this->questions;
    }

    /**
     * Returns the maximum score for this assessment
     */
    public function get_maximum_score()
    {
        $condition = new EqualityCondition(
            new PropertyConditionVariable(
                ComplexContentObjectItem :: class_name(),
                ComplexContentObjectItem :: class_name(),
                ComplexContentObjectItem :: PROPERTY_PARENT),
            new StaticConditionVariable($this->get_id()));

        $clo_questions = \repository\DataManager :: retrieve_complex_content_object_items(
            $this->get_type_name(),
            ComplexContentObjectItem :: class_name(),
            $condition);

        $maxscore = 0;

        while ($clo_question = $clo_questions->next_result())
        {
            $maxscore += $clo_question->get_weight();
        }

        return $maxscore;
    }
}
