<?php
namespace repository\content_object\assignment\upgrade\v4_0_0;

use common\libraries\CommonDataManager;
use common\libraries\DataClassProperties;
use common\libraries\DataClassProperty;
use common\libraries\EqualityCondition;
use common\libraries\PropertyConditionVariable;
use common\libraries\StaticConditionVariable;
use common\libraries\Translation;
use common\libraries\Utilities;
use repository\content_object\assignment\Assignment;
use common\libraries\package\Package;
use repository\ContentObject;


/**
 *
 * Upgrades assignment tables
 * @author Pieterjan Broekaert
 */
class Upgrader extends \common\libraries\package\Upgrader
{

    protected static $CONTENT_OBJECT_CLASS = Assignment :: CLASS_NAME; // Change this value!

    protected function get_namespace()
    {
        return call_user_func(self :: $CONTENT_OBJECT_CLASS .'::context');
    }

    protected function get_class_name()
    {
        return call_user_func(self :: $CONTENT_OBJECT_CLASS .'::class_name');
    }

    protected function get_type_name()
    {
        return call_user_func(self :: $CONTENT_OBJECT_CLASS .'::get_type_name');
    }

    /**
     * Constructor
     */
    public function __construct()
    {
        parent :: __construct(\repository\content_object\assignment\DataManager :: get_instance());
    }

    /*
     * (non-PHPdoc) @see \common\libraries\package\Upgrader::run()
     */
    public function run()
    {
        $data_manager = $this->get_data_manager();
        $upgrader = $this;

        $success = $data_manager->transactional(
            function ($c) use($data_manager, $upgrader)
            {
                $success = $upgrader->upgrade_db($data_manager);

                if($success)
                {
                    $upgrader->add_message(\common\libraries\package\Upgrader :: TYPE_NORMAL, Translation :: get('UpgradeDatabaseSuccessful', null, Utilities :: COMMON_LIBRARIES));
                }
                else
                {
                    $upgrader->add_message(\common\libraries\package\Upgrader :: TYPE_ERROR, Translation :: get('UpgradeDatabaseFailed', null, Utilities :: COMMON_LIBRARIES));
                }

                if(!$success)
                {
                    return false;
                }

                $success = $upgrader->update_db_content($data_manager);

                if($success)
                {
                    $upgrader->add_message(\common\libraries\package\Upgrader :: TYPE_NORMAL, Translation :: get('UpdateDatabaseContentSuccessful', null, Utilities :: COMMON_LIBRARIES));
                }
                else
                {
                    $upgrader->add_message(\common\libraries\package\Upgrader :: TYPE_ERROR, Translation :: get('UpdateDatabaseContentFailed', null, Utilities :: COMMON_LIBRARIES));
                }

                if(!$success)
                {
                    return false;
                }

                $success = $upgrader->upgrade_registration($data_manager);

                if($success)
                {
                    $upgrader->add_message(\common\libraries\package\Upgrader :: TYPE_NORMAL, Translation :: get('UpgradeRegistrationSuccessful', null, Utilities :: COMMON_LIBRARIES));
                }
                else
                {
                    $upgrader->add_message(\common\libraries\package\Upgrader :: TYPE_ERROR, Translation :: get('UpgradeRegistrationFailed', null, Utilities :: COMMON_LIBRARIES));
                }

                return $success;

            });

        if ($success)
        {
            $this->add_message(self :: TYPE_NORMAL, Translation :: get('UpgradeDatabaseSuccessful', null, Utilities :: COMMON_LIBRARIES));
        }
        else
        {
            $this->add_message(self :: TYPE_ERROR, Translation :: get('UpgradeDatabaseFailed', null, Utilities :: COMMON_LIBRARIES));
        }

        return $success;
    }

    /**
     * Upgrade the table structures
     * @param $data_manager
     * @return bool
     */
    public function upgrade_db($data_manager)
    {
        // Change some property types for the settings storage unit
        $attributes = array('type' => 'text');
        $success = $data_manager->alter_storage_unit(
            \common\libraries\DataManager :: ALTER_STORAGE_UNIT_CHANGE,
            Assignment :: get_table_name(),
            Assignment :: PROPERTY_AUTOMATIC_FEEDBACK_TEXT,
            $attributes);

        if (! $success)
        {
            return false;
        }

        $attributes = array('type' => 'text');
        $success = $data_manager->alter_storage_unit(
            \common\libraries\DataManager :: ALTER_STORAGE_UNIT_CHANGE,
            Assignment :: get_table_name(),
            Assignment :: PROPERTY_AUTOMATIC_FEEDBACK_CO_IDS,
            $attributes);

        if (! $success)
        {
            return false;
        }

        $attributes = array('type' => 'integer', 'length' => 10);
        $success = $data_manager->alter_storage_unit(
            \common\libraries\DataManager :: ALTER_STORAGE_UNIT_CHANGE,
            Assignment :: get_table_name(),
            Assignment :: PROPERTY_VISIBILTY_FEEDBACK,
            $attributes);

        if (! $success)
        {
            return false;
        }

        $attributes = array('type' => 'text');
        $success = $data_manager->alter_storage_unit(
            \common\libraries\DataManager :: ALTER_STORAGE_UNIT_CHANGE,
            Assignment :: get_table_name(),
            Assignment :: PROPERTY_ALLOWED_TYPES,
            $attributes);

        if (! $success)
        {
            return false;
        }

        return true;
    }

    /**
     * Upgrade the content
     * @param $data_manager
     * @return bool
     */
    public function update_db_content($data_manager)
    {

        $properties = new DataClassProperties();
        $properties->add(
            new DataClassProperty(
                new PropertyConditionVariable(
                    ContentObject :: class_name(),
                    ContentObject :: PROPERTY_TYPE),
                new StaticConditionVariable($this->get_class_name()))
        );

        $condition = new EqualityCondition(
            new PropertyConditionVariable(
                ContentObject :: class_name(),
                ContentObject :: PROPERTY_TYPE),
            new StaticConditionVariable($this->get_type_name())
        );

        if (! \repository\DataManager :: updates(
            ContentObject :: class_name(),
            $properties,
            $condition))
        {
            return false;
        }

        $assignments = \repository\DataManager :: retrieve_content_objects(
            \repository\content_object\assignment\Assignment :: class_name());

        $mapping = array('document' => 'file');

        while($assignment = $assignments->next_result())
        {
            //update the allowed types to make use of namespaces
            $old_types = $assignment->get_allowed_types();

            $new_types = array();
            foreach(explode(',', $old_types) as $old_type)
            {
                if(array_key_exists($old_type, $mapping))
                {
                    $old_type = $mapping[$old_type];
                }

                $new_types[] = 'repository\content_object\\' . $old_type . '\\' . Utilities :: underscores_to_camelcase
                        ($old_type);
            }

            $assignment->set_allowed_types(implode(',', $new_types));

            if(!$assignment->update())
            {
                return false;
            }
        }

        return true;
    }

    /**
     * Upgrades the package registration.
     *
     * @return boolean
     */
    public function upgrade_registration()
    {
        $package = Package :: get($this->get_namespace());
        $registration = CommonDataManager :: get_registration($this->get_namespace());
        $registration->set_version($package->get_version());
        if (! $registration->update())
        {
            return false;
        }
        return true;
    }
}
