<?php

namespace repository\content_object\glossary;

use common\libraries\AndCondition;
use common\libraries\DataClassCountParameters;
use common\libraries\DataClassJoin;
use common\libraries\DataClassRetrievesParameters;
use common\libraries\EqualityCondition;
use common\libraries\Joins;
use common\libraries\OrCondition;
use common\libraries\PatternMatchCondition;
use common\libraries\PropertyConditionVariable;
use common\libraries\StaticConditionVariable;
use repository\ComplexContentObjectItem;
use repository\ContentObject;

/**
 * Basic class to render the glossary
 *
 * @package repository\content_object\glossary
 * @author Sven Vanpoucke - Hogeschool Gent
 */
abstract class GlossaryRenderer
{
    /**
     * The component in which this renderer runs
     *
     * @var mixed
     */
    private $component;

    /**
     * The glossary
     *
     * @var Glossary
     */
    private $glossary;

    /**
     * The search query
     *
     * @var string
     */
    private $search_query;

    /**
     * Constructor
     *
     * @param mixed $component
     * @param Glossary $glossary
     * @param string $search_query
     */
    public function __construct($component, $glossary, $search_query)
    {
        $this->component = $component;
        $this->glossary = $glossary;
        $this->search_query = $search_query;
    }

    /**
     * Retrieves the objects from the database
     *
     * @param int $offset
     * @param int $count
     * @param ObjectTableOrder[] $order_property
     *
     * @return \common\libraries\ResultSet
     */
    public function get_objects($offset = null, $count = null, $order_property = null)
    {
        $parameters = new DataClassRetrievesParameters(
            $this->get_condition(), $count, $offset, $order_property,
            $this->get_joins()
        );

        return \repository\DataManager :: retrieve_complex_content_object_items(
            ComplexContentObjectItem :: class_name(), $parameters
        );
    }

    /**
     * Counts the objects from the database
     *
     * @return int
     */
    public function count_objects()
    {
        $parameters = new DataClassCountParameters($this->get_condition(), $this->get_joins());

        return \repository\DataManager :: count_complex_content_object_items(
            ComplexContentObjectItem :: class_name(), $parameters
        );
    }

    /**
     * Returns the joins with the content object
     *
     * @return Joins
     */
    protected function get_joins()
    {
        $joins = new Joins();

        $joins->add(
            new DataClassJoin(
                ContentObject :: class_name(),
                new EqualityCondition(
                    new PropertyConditionVariable(
                        ComplexContentObjectItem :: class_name(), ComplexContentObjectItem :: PROPERTY_REF
                    ),
                    new PropertyConditionVariable(ContentObject :: class_name(), ContentObject :: PROPERTY_ID)
                )
            )
        );

        return $joins;
    }

    /**
     * Returns the condition for the children
     *
     * @return Condition
     */
    protected function get_condition()
    {
        $query = $this->search_query;

        if (isset($query) && $query != '')
        {
            $search_conditions[] = new PatternMatchCondition(
                new PropertyConditionVariable(ContentObject :: class_name(), ContentObject :: PROPERTY_TITLE),
                '*' . $query . '*'
            );

            $search_conditions[] = new PatternMatchCondition(
                new PropertyConditionVariable(ContentObject :: class_name(), ContentObject :: PROPERTY_DESCRIPTION),
                '*' . $query . '*'
            );

            $conditions[] = new OrCondition($search_conditions);
        }

        $objects = $this->glossary;

        if (! is_array($objects))
        {
            $objects = array($objects);
        }

        $co_conditions = array();

        foreach ($objects as $object)
        {
            $co_conditions[] = new EqualityCondition(
                new PropertyConditionVariable(
                    ComplexContentObjectItem :: class_name(), ComplexContentObjectItem :: PROPERTY_PARENT
                ),
                new StaticConditionVariable($object->get_id())
            );
        }

        $conditions[] = new OrCondition($co_conditions);

        $condition = new AndCondition($conditions);

        return $condition;
    }

    /**
     * Returns the glossary
     *
     * @return Glossary
     */
    public function get_glossary()
    {
        return $this->glossary;
    }

    /**
     * Returns the search query
     *
     * @return string
     */
    public function get_search_query()
    {
        return $this->search_query;
    }

    /**
     * Returns the component
     *
     * @return mixed
     */
    public function get_component()
    {
        return $this->component;
    }

    /**
     * Renders the glossary
     *
     * @return string
     */
    abstract public function render();

}