<?php

namespace repository\content_object\glossary_item\upgrade\v4_0_0;

use common\libraries\CommonDataManager;
use common\libraries\package\Package;
use common\libraries\Translation;
use common\libraries\Utilities;
use repository\content_object\glossary_item\GlossaryItem;
use common\libraries\DataClassProperties;
use common\libraries\DataClassProperty;
use common\libraries\EqualityCondition;
use common\libraries\PropertyConditionVariable;
use common\libraries\StaticConditionVariable;
use repository\ContentObject;
use repository\ComplexContentObjectItem;
use common\libraries\SubselectCondition;

/**
 * Database schema & content upgrader for Chamilo 3.x to 4.0.0.
 *
 * Instead of putting everything in the run() method, this class makes calls to
 * upgrade_db() and upgrade_registration(). Since the only thing that really differs
 * across Upgraders is upgrade_db() implementation, the other methods could/should
 * be moved to the \common\libraries\package\Upgrader.
 */
class Upgrader extends \common\libraries\package\Upgrader
{

    protected static $CONTENT_OBJECT_CLASS = GlossaryItem :: CLASS_NAME; // Change this value!

    protected function get_namespace()
    {
        return call_user_func(self :: $CONTENT_OBJECT_CLASS .'::context');
    }

    protected function get_classname()
    {
        return call_user_func(self :: $CONTENT_OBJECT_CLASS .'::class_name');
    }

    protected function get_type_name()
    {
        return call_user_func(self :: $CONTENT_OBJECT_CLASS .'::get_type_name');
    }

    /**
     * Constructor
     */
    public function __construct()
    {
        parent :: __construct(\repository\content_object\glossary_item\DataManager :: get_instance());
    }

    /*
     * (non-PHPdoc) @see \common\libraries\package\Upgrader::run()
     */
    public function run()
    {
        $data_manager = $this->get_data_manager();
        $upgrader = $this;

        $success = $data_manager->transactional(
            function ($c) use($data_manager, $upgrader)
            {
                $success = $upgrader->upgrade_db($data_manager);

                if($success)
                {
                    $upgrader->add_message(\common\libraries\package\Upgrader :: TYPE_NORMAL, Translation :: get('UpgradeDatabaseSuccessful', null, Utilities :: COMMON_LIBRARIES));
                }
                else
                {
                    $upgrader->add_message(\common\libraries\package\Upgrader :: TYPE_ERROR, Translation :: get('UpgradeDatabaseFailed', null, Utilities :: COMMON_LIBRARIES));
                }

                if(!$success)
                {
                    return false;
                }

                $success = $upgrader->update_db_content($data_manager);

                if($success)
                {
                    $upgrader->add_message(\common\libraries\package\Upgrader :: TYPE_NORMAL, Translation :: get('UpdateDatabaseContentSuccessful', null, Utilities :: COMMON_LIBRARIES));
                }
                else
                {
                    $upgrader->add_message(\common\libraries\package\Upgrader :: TYPE_ERROR, Translation :: get('UpdateDatabaseContentFailed', null, Utilities :: COMMON_LIBRARIES));
                }

                if(!$success)
                {
                    return false;
                }

                $success = $upgrader->upgrade_registration($data_manager);

                if($success)
                {
                    $upgrader->add_message(\common\libraries\package\Upgrader :: TYPE_NORMAL, Translation :: get('UpgradeRegistrationSuccessful', null, Utilities :: COMMON_LIBRARIES));
                }
                else
                {
                    $upgrader->add_message(\common\libraries\package\Upgrader :: TYPE_ERROR, Translation :: get('UpgradeRegistrationFailed', null, Utilities :: COMMON_LIBRARIES));
                }

                return $success;
            }
        );

        return $success;
    }

    /**
     * Performs the database upgrade.
     *
     * @param type $data_manager
     * @return boolean
     */
    public function upgrade_db($data_manager) // Needs to be public, otherwise can't be called from a transactional context.
    {
        // No changes
        return true;
    }

    /**
     * Performs the database content upgrade.
     *
     * @param type $data_manager
     * @return boolean
     */
    public function update_db_content($data_manager)
    {
        $properties = new DataClassProperties();
        $properties->add(
            new DataClassProperty(
                new PropertyConditionVariable(
                    ContentObject :: class_name(),
                    ContentObject :: PROPERTY_TYPE),
                new StaticConditionVariable($this->get_classname()))
        );

        $condition = new EqualityCondition(
            new PropertyConditionVariable(
                ContentObject :: class_name(),
                ContentObject :: PROPERTY_TYPE),
            new StaticConditionVariable($this->get_type_name())
        );

        if (! \repository\DataManager :: updates(
            ContentObject :: class_name(),
            $properties,
            $condition))
        {
            return false;
        }

        // Complex type
        $co_type = $this->get_classname();
        $namespace = $this->get_namespace();
        $cco_type = str_replace($namespace . '\\', $namespace . '\\Complex', $co_type);

        $properties = new DataClassProperties();
        $properties->add(
            new DataClassProperty(
                new PropertyConditionVariable(
                    ComplexContentObjectItem :: class_name(),
                    ComplexContentObjectItem :: PROPERTY_TYPE),
                new StaticConditionVariable($cco_type)));

        $subselect_condition = new EqualityCondition(
            new PropertyConditionVariable(
                ContentObject :: class_name(),
                ContentObject :: PROPERTY_TYPE),
            new StaticConditionVariable($co_type));

        $condition = new SubselectCondition(
            new PropertyConditionVariable(
                ComplexContentObjectItem :: class_name(),
                ComplexContentObjectItem :: PROPERTY_REF),
            new PropertyConditionVariable(
                ContentObject :: class_name(),
                ContentObject :: PROPERTY_ID),
            null,
            $subselect_condition);

        if (! \repository\DataManager :: updates(
            ComplexContentObjectItem :: class_name(),
            $properties,
            $condition))
        {
            return false;
        }

        return true;
    }

    /**
     * Upgrades the package registration.
     *
     * @param type $data_manager
     * @return boolean
     */
    public function upgrade_registration($data_manager)
    {
        $package = Package :: get($this->get_namespace());
        $registration = CommonDataManager :: get_registration($this->get_namespace());
        $registration->set_version($package->get_version());
        if (! $registration->update())
        {
            return false;
        }
        return true;
    }
}
