<?php
namespace repository;

use common\libraries\ObjectTable;
use common\libraries\ComplexMenuSupport;
use common\libraries\Request;
use common\libraries\Translation;
use common\libraries\Utilities;
use common\libraries\EqualityCondition;
use common\libraries\SubManager;
use common\libraries\ObjectNotExistException;
use common\libraries\NotAllowedException;

/**
 * $Id: complex_builder.class.php 200 2009-11-13 12:30:04Z kariboe $
 *
 * @package repository.lib.complex_builder
 */

/**
 * This class represents a basic complex builder structure. When a builder is needed for a certain type of complex
 * object an extension should be written. We will make use of the repoviewer for selection, creation of objects
 *
 * @author Sven Vanpoucke
 */
abstract class ComplexBuilder extends SubManager
{
    const PARAM_BUILDER_ACTION = 'builder_action';
    const PARAM_COMPLEX_CONTENT_OBJECT_ITEM_ID = 'cloi';
    const PARAM_SELECTED_COMPLEX_CONTENT_OBJECT_ITEM_ID = 'selected_cloi';
    const PARAM_DELETE_SELECTED_COMPLEX_CONTENT_OBJECT_ITEM = 'delete_selected_cloi';
    const PARAM_MOVE_SELECTED_COMPLEX_CONTENT_OBJECT_ITEM = 'move_selected_cloi';
    const PARAM_TYPE = 'type';
    const PARAM_DIRECTION = 'direction';
    const PARAM_ATTACHMENT_ID = 'attachment_id';
    const ACTION_BROWSE = 'browser';
    const ACTION_DELETE_COMPLEX_CONTENT_OBJECT_ITEM = 'deleter';
    const ACTION_VIEW_COMPLEX_CONTENT_OBJECT_ITEM = 'viewer';
    const ACTION_UPDATE_COMPLEX_CONTENT_OBJECT_ITEM = 'updater';
    const ACTION_CREATE_COMPLEX_CONTENT_OBJECT_ITEM = 'creator';
    const ACTION_MOVE_COMPLEX_CONTENT_OBJECT_ITEM = 'mover';
    const ACTION_CHANGE_PARENT = 'parent_changer';
    const ACTION_VIEW_ATTACHMENT = 'attachment_viewer';
    const ACTION_PREVIEW = 'preview';
    const DEFAULT_ACTION = self :: ACTION_BROWSE;

    protected $menu;

    /**
     * The current item in treemenu to determine where we are in the structure
     *
     * @var ComplexContentObjectItem
     */
    private $complex_content_object_item;

    /**
     * The item we select to execute an action like update / delete / move etc
     *
     * @var ComplexContentObjectItem
     */
    private $selected_complex_content_object_item;

    /**
     * The selected parent content object
     *
     * @var ContentObject
     */
    private $parent_content_object;

    public function __construct($parent)
    {
        parent :: __construct($parent);

        $complex_content_object_item_id = Request :: get(self :: PARAM_COMPLEX_CONTENT_OBJECT_ITEM_ID);
        if ($complex_content_object_item_id)
        {
            $this->complex_content_object_item = $this->get_complex_content_object_by_id($complex_content_object_item_id);
        }

        $selected_complex_content_object_item_id = Request :: get(self :: PARAM_SELECTED_COMPLEX_CONTENT_OBJECT_ITEM_ID);

        if ($selected_complex_content_object_item_id)
        {
            $this->set_parameter(
                self :: PARAM_SELECTED_COMPLEX_CONTENT_OBJECT_ITEM_ID,
                $selected_complex_content_object_item_id);

            if (! is_array($selected_complex_content_object_item_id))
            {
                $this->selected_complex_content_object_item = $this->get_complex_content_object_by_id(
                    $selected_complex_content_object_item_id);
            }
            else
            {
                $this->selected_complex_content_object_item = array();

                foreach ($selected_complex_content_object_item_id as $id)
                {
                    $this->selected_complex_content_object_item[] = $this->get_complex_content_object_by_id($id);
                }
            }
        }

        $this->set_action(Request :: get(self :: PARAM_BUILDER_ACTION));
        $this->parse_input_from_table();
    }

    // Singleton
    private static $instance;

    public static function factory($type, $application)
    {
        $name_space = __NAMESPACE__ . '\\' . 'content_object\\' . $type . '\\';
        $class = $name_space . Utilities :: underscores_to_camelcase($type) . 'Builder';
        return new $class($application);
    }

    /**
     *
     * @param $type string
     * @param $application Application
     */
    public static function launch($type, $application)
    {
        $classname = static :: class_name();
        if (is_subclass_of($classname, __CLASS__))
        {
            parent :: launch(null, $application);
        }
        else
        {
            $class = $type . 'Builder';
            $class :: launch(null, $application);
        }
    }

    /**
     * Retrieves and validates a complex content object with a given id
     *
     * @param $complex_content_object_item_id int
     *
     * @return ComplexContentObjectItem
     */
    protected function get_complex_content_object_by_id($complex_content_object_item_id)
    {
        $complex_content_object_item = DataManager :: retrieve_complex_content_object_item(
            $complex_content_object_item_id);

        if (is_null($complex_content_object_item))
        {
            throw new ObjectNotExistException(
                Translation :: get('ComplexContentObjectItem'),
                $complex_content_object_item_id);
        }

        if (! DataManager :: is_child_of_content_object(
            $this->get_root_content_object_id(),
            $complex_content_object_item->get_ref()))
        {
            throw new NotAllowedException();
        }

        return $complex_content_object_item;
    }

    protected function parse_input_from_table()
    {
        if (isset($_POST['action']))
        {
            $selected_ids = $_POST[RepositoryBrowserTable :: DEFAULT_NAME . ObjectTable :: CHECKBOX_NAME_SUFFIX];
            if (empty($selected_ids))
            {
                $selected_ids = array();
            }
            elseif (! is_array($selected_ids))
            {
                $selected_ids = array($selected_ids);
            }
            switch ($_POST['action'])
            {
                case self :: PARAM_DELETE_SELECTED_COMPLEX_CONTENT_OBJECT_ITEM :
                    $this->set_action(self :: ACTION_DELETE_COMPLEX_CONTENT_OBJECT_ITEM);
                    Request :: set_get(self :: PARAM_SELECTED_COMPLEX_CONTENT_OBJECT_ITEM_ID, $selected_ids);
                    break;
                case self :: PARAM_MOVE_SELECTED_COMPLEX_CONTENT_OBJECT_ITEM :
                    $this->set_action(self :: ACTION_CHANGE_PARENT);
                    Request :: set_get(self :: PARAM_SELECTED_COMPLEX_CONTENT_OBJECT_ITEM_ID, $selected_ids);
                    break;
            }
        }
    }

    public function get_action()
    {
        return $this->get_parameter(self :: PARAM_BUILDER_ACTION);
    }

    public function set_action($action)
    {
        $this->set_parameter(self :: PARAM_BUILDER_ACTION, $action);
    }

    public function get_root_content_object()
    {
        return $this->get_parent()->get_root_content_object();
    }

    public function get_complex_content_object_item()
    {
        return $this->complex_content_object_item;
    }

    public function get_selected_complex_content_object_item()
    {
        return $this->selected_complex_content_object_item;
    }

    public function get_root_content_object_id()
    {
        return $this->get_parent()->get_root_content_object()->get_id();
    }

    public function get_complex_content_object_item_id()
    {
        if ($this->complex_content_object_item)
        {
            return $this->complex_content_object_item->get_id();
        }
    }

    public function get_selected_complex_content_object_item_id()
    {
        if ($this->selected_complex_content_object_item)
        {
            return $this->selected_complex_content_object_item->get_id();
        }
    }

    public function get_parent_content_object_id()
    {
        if ($this->complex_content_object_item)
        {
            return $this->get_complex_content_object_item()->get_ref();
        }

        return $this->get_root_content_object_id();
    }

    public function get_parent_content_object()
    {
        if (! $this->parent_content_object)
        {
            $this->parent_content_object = \repository\DataManager :: retrieve_content_object(
                $this->get_parent_content_object_id());
        }
        return $this->parent_content_object;
    }

    /**
     * Common functionality
     */
    public function get_complex_content_object_table_html($show_subitems_column = true, $model = null, $renderer = null)
    {
        $parameters = $this->get_parameters();
        $parameters[self :: PARAM_COMPLEX_CONTENT_OBJECT_ITEM_ID] = $this->get_complex_content_object_item_id();

        $table = new ComplexBrowserTable(
            $this,
            $parameters,
            $this->get_complex_content_object_table_condition(),
            $show_subitems_column,
            $model,
            $renderer);
        return $table->as_html();
    }

    public function get_complex_content_object_table_condition()
    {
        if ($this->get_complex_content_object_item())
        {
            return new EqualityCondition(
                ComplexContentObjectItem :: PROPERTY_PARENT,
                $this->get_complex_content_object_item()->get_ref(),
                ComplexContentObjectItem :: get_table_name());
        }
        return new EqualityCondition(
            ComplexContentObjectItem :: PROPERTY_PARENT,
            $this->get_root_content_object_id(),
            ComplexContentObjectItem :: get_table_name());
    }

    public function get_complex_content_object_menu()
    {
        if (is_null($this->menu))
        {
            $this->build_complex_content_object_menu();
        }
        return $this->menu->render_as_tree();
    }

    public function get_complex_content_object_breadcrumbs()
    {
        if (is_null($this->menu))
        {
            $this->build_complex_content_object_menu();
        }
        return $this->menu->get_breadcrumbs();
    }

    protected function build_complex_content_object_menu()
    {
        $this->menu = new ComplexMenu(
            $this->get_root_content_object(),
            $this->get_complex_content_object_item(),
            $this->get_url(array(self :: PARAM_BUILDER_ACTION => self :: ACTION_BROWSE)));
    }

    // url building
    public function get_complex_content_object_item_edit_url($selected_content_object_item_id)
    {
        return $this->get_url(
            array(
                self :: PARAM_BUILDER_ACTION => self :: ACTION_UPDATE_COMPLEX_CONTENT_OBJECT_ITEM,
                self :: PARAM_SELECTED_COMPLEX_CONTENT_OBJECT_ITEM_ID => $selected_content_object_item_id,
                self :: PARAM_COMPLEX_CONTENT_OBJECT_ITEM_ID => $this->get_complex_content_object_item_id()));
    }

    public function get_complex_content_object_item_delete_url($selected_content_object_item_id)
    {
        return $this->get_url(
            array(
                self :: PARAM_BUILDER_ACTION => self :: ACTION_DELETE_COMPLEX_CONTENT_OBJECT_ITEM,
                self :: PARAM_SELECTED_COMPLEX_CONTENT_OBJECT_ITEM_ID => $selected_content_object_item_id,
                self :: PARAM_COMPLEX_CONTENT_OBJECT_ITEM_ID => $this->get_complex_content_object_item_id()));
    }

    public function get_complex_content_object_item_view_url($selected_content_object_item_id)
    {
        return $this->get_url(
            array(
                self :: PARAM_BUILDER_ACTION => self :: ACTION_VIEW_COMPLEX_CONTENT_OBJECT_ITEM,
                self :: PARAM_SELECTED_COMPLEX_CONTENT_OBJECT_ITEM_ID => $selected_content_object_item_id,
                self :: PARAM_COMPLEX_CONTENT_OBJECT_ITEM_ID => $this->get_complex_content_object_item_id()));
    }

    public function get_complex_content_object_item_move_url($selected_content_object_item_id, $direction)
    {
        return $this->get_url(
            array(
                self :: PARAM_BUILDER_ACTION => self :: ACTION_MOVE_COMPLEX_CONTENT_OBJECT_ITEM,
                self :: PARAM_SELECTED_COMPLEX_CONTENT_OBJECT_ITEM_ID => $selected_content_object_item_id,
                self :: PARAM_COMPLEX_CONTENT_OBJECT_ITEM_ID => $this->get_complex_content_object_item_id(),
                self :: PARAM_DIRECTION => $direction));
    }

    public function get_complex_content_object_parent_changer_url($selected_content_object_item_id)
    {
        return $this->get_url(
            array(
                self :: PARAM_BUILDER_ACTION => self :: ACTION_CHANGE_PARENT,
                self :: PARAM_SELECTED_COMPLEX_CONTENT_OBJECT_ITEM_ID => $selected_content_object_item_id,
                self :: PARAM_COMPLEX_CONTENT_OBJECT_ITEM_ID => $this->get_complex_content_object_item_id()));
    }

    public function get_browse_url()
    {
        return $this->get_url(
            array(
                self :: PARAM_BUILDER_ACTION => self :: ACTION_BROWSE,
                self :: PARAM_COMPLEX_CONTENT_OBJECT_ITEM_ID => $this->get_complex_content_object_item_id()));
    }

    public function get_create_complex_content_object_item_url()
    {
        return $this->get_url(
            array(
                self :: PARAM_BUILDER_ACTION => self :: ACTION_CREATE_COMPLEX_CONTENT_OBJECT_ITEM,
                self :: PARAM_COMPLEX_CONTENT_OBJECT_ITEM_ID => $this->get_complex_content_object_item_id()));
    }

    public function get_additional_links()
    {
        return array();
    }

    public function get_content_object_type_creation_url($type)
    {
        return $this->get_url(
            array(
                self :: PARAM_BUILDER_ACTION => self :: ACTION_CREATE_COMPLEX_CONTENT_OBJECT_ITEM,
                self :: PARAM_TYPE => $type,
                self :: PARAM_COMPLEX_CONTENT_OBJECT_ITEM_ID => $this->get_complex_content_object_item_id()));
    }

    /**
     * Returns the url to the preview component
     *
     * @return string
     */
    public function get_preview_content_object_url()
    {
        return $this->get_url(array(self :: PARAM_BUILDER_ACTION => self :: ACTION_PREVIEW));
    }

    public function get_creation_links($content_object, $types = array())
    {
        if (count($types) == 0)
        {
            $types = $content_object->get_allowed_types();
        }

        $type_selector = new ContentObjectTypeSelector(
            $this,
            $types,
            $this->get_additional_links(),
            null,
            $this->get_url(
                array(
                    self :: PARAM_BUILDER_ACTION => self :: ACTION_CREATE_COMPLEX_CONTENT_OBJECT_ITEM,
                    self :: PARAM_COMPLEX_CONTENT_OBJECT_ITEM_ID => $this->get_complex_content_object_item_id())));
        $html[] = $type_selector->as_html();

        return implode("\n", $html);
    }

    public function get_content_object_display_attachment_url($attachment)
    {
        return $this->get_url(
            array(
                self :: PARAM_BUILDER_ACTION => self :: ACTION_VIEW_ATTACHMENT,
                self :: PARAM_ATTACHMENT_ID => $attachment->get_id()));
    }

    /**
     *
     * @return boolean
     * @deprecated Use instanceof ComplexMenuSupport directly from now on
     */
    public function show_menu()
    {
        return $this instanceof ComplexMenuSupport;
    }

    public function redirect_away_from_complex_builder($message, $error_message)
    {
        $this->get_parent()->redirect_away_from_complex_builder($message, $error_message);
    }

    public function get_action_bar(ContentObject $content_object)
    {
        return '';
    }

    public static function get_action_parameter()
    {
        return self :: PARAM_BUILDER_ACTION;
    }

    public static function get_default_action()
    {
        return self :: DEFAULT_ACTION;
    }
}
