<?php
namespace repository;

use common\libraries\Translation;
use common\libraries\Path;
use common\libraries\Utilities;
use common\libraries\Theme;
use common\libraries\AttachmentSupport;
use common\libraries\BbcodeParser;

/**
 *
 * @package repository.lib A class to display a ContentObject.
 */
abstract class ContentObjectDisplay
{
    const TITLE_MARKER = '<!-- /title -->';
    const DESCRIPTION_MARKER = '<!-- /description -->';

    /**
     * The content object.
     *
     * @var ContentObject
     */
    private $content_object;

    /**
     * The URL format.
     *
     * @var string
     */
    private $url_format;

    /**
     * Link target. I.e. if we need to open a link into the same windows, a new one, etc.
     *
     * @var string
     */
    private $link_target = '';

    /**
     * The context from which the ContentObjectDisplay will be called
     *
     * @var ContentObjectDisplayContext
     */
    private $context;

    /**
     * Constructor
     *
     * @param $content_object ContentObject The content object to display
     * @param $url_format string A pattern to pass to sprintf(), representing the format for URLs that link to other
     *            objects. The first parameter will be replaced with the ID of the other object. By default, an attempt
     *            is made to extract the ID of the current object from the query string, and replace it.
     * @param $context ContentObjectDisplayContext The application or submanager context the ContentObjectDisplay is
     *            being instantiated from
     */
    public function __construct($content_object, $url_format = null, ContentObjectDisplayContext $context)
    {

        $this->content_object = $content_object;

        if (! isset($url_format))
        {
            $pairs = explode('&', $_SERVER['QUERY_STRING']);
            $new_pairs = array();
            foreach ($pairs as $pair)
            {
                list($name, $value) = explode('=', $pair, 2);
                if ($value == $content_object->get_id())
                {
                    $new_pairs[] = $name . '=%d';
                }
                else
                {
                    $new_pairs[] = $pair;
                }
            }
            $url_format = $_SERVER['PHP_SELF'] . '?' . implode('&', $new_pairs);
        }
        $this->url_format = $url_format;

        if (! isset($context))
        {
            $context = new LegacyContentObjectDisplayContext();
        }
        $this->context = $context;
    }

    /**
     * Link target. I.e. if we need to open a link into the same windows, a new one, etc. One of the usual HTML values:
     * _blank, ...
     *
     * @return string
     */
    public function get_link_target()
    {
        return $this->link_target;
    }

    public function set_link_target($value)
    {
        $this->link_target = $value;
    }

    /**
     * Returns the object associated with this object.
     *
     * @return ContentObject The object.
     */
    protected function get_content_object()
    {
        return $this->content_object;
    }

    /**
     * Returns a full HTML view of the object.
     *
     * @return string The HTML.
     */
    public function get_full_html($buttons = null)
    {
        // TODO: split this into several methods, don't use marker
        $object = $this->get_content_object();
        $html = array();
        $html[] = '<div class="content_object" style="background-image: url(' . $object->get_icon_path() . ');">';
        $html[] = '<div class="title">' . $object->get_title() . '</div>';
        $html[] = self :: TITLE_MARKER;
        // $html[] = '<div class="description" style="overflow: auto;">';
        $html[] = $this->get_description();
        // $html[] = '</div>';
        // $html[] = '<div class="clear"></div>';
        $html[] = '</div>';
        $html[] = '<div class="clear"></div>';

        return implode("\n", $html);
    }

    /**
     * Returns a reduced HTML view of the object.
     *
     * @return string The HTML.
     */
    public function get_short_html()
    {
        $object = $this->get_content_object();
        return '<span class="content_object">' . htmlentities($object->get_title()) . '</span>';
    }

    public function get_preview($is_thumbnail = false)
    {
        if ($is_thumbnail)
        {
            $class = 'no_thumbnail';
            $image = Theme :: get_common_image('thumbnail');
            $text = '<h3>' . Translation :: get('NoThumbnailAvailable') . '</h3>';
        }
        else
        {
            $class = 'no_preview';
            $image = Theme :: get_common_image('preview');
            $text = '<h1>' . Translation :: get('NoPreviewAvailable') . '</h1>';
        }

        $html = array();
        $html[] = '<div class="' . $class . '">';
        $html[] = '<div class="background">';
        $html[] = $image;
        $html[] = $text;
        $html[] = '</div>';
        $html[] = '<div class="clear"></div>';
        $html[] = '</div>';
        return implode("\n", $html);
    }

    /**
     * Returns a HTML view of the description
     *
     * @return string The HTML.
     */
    public function get_description()
    {
        $description = $this->get_content_object()->get_description();
        $parsed_description = BbcodeParser :: get_instance()->parse($description);

        $html[] = '<div class="description" style="overflow: auto;">';
        $html[] = $parsed_description;
        $html[] = '<div class="clear"></div>';
        $html[] = self :: DESCRIPTION_MARKER;
        if (isset($buttons))
        {
            $html[] = '<div class="publication_actions">';
            if (is_array($buttons))
            {
                foreach ($buttons as $button)
                {
                    // echo "erin";
                    $html[] = $button;
                }
            }
            else
            {
                $html[] = $buttons;
            }
            $html[] = '</div>';
        }
        $html[] = $this->get_attached_content_objects_as_html();
        $html[] = '</div>';

        return implode("\n", $html);
    }

    public function get_context()
    {
        return $this->context;
    }

    /**
     * Returns a HTML view of the objects attached to the object.
     *
     * @return string The HTML.
     */
    public function get_attached_content_objects_as_html()
    {
        $object = $this->get_content_object();
        if ($object instanceof AttachmentSupport)
        {
            $attachments = $object->get_attachments();
            if (count($attachments))
            {
                /*
                 * $html = array(); $html[] = '<div class="attachments" style="margin-top: 1em;">'; $html[] = '<div
                 * class="attachments_title">'.htmlentities(Translation :: get('Attachments')).'</div>'; $html[] = '<ul
                 * class="attachments_list">'; Utilities :: order_content_objects_by_title($attachments); foreach
                 * ($attachments as $attachment) { $disp = self :: factory($attachment); $html[] = '<li><img
                 * src="'.Theme :: get_common_image_path().'treemenu_types/'.$attachment->get_type().'.png"
                 * alt="'.htmlentities(Translation :: get('TypeName', null, ContentObject ::
                 * get_content_object_type_namespace($attachment->get_type()))).'"/> '.$disp->get_short_html().'</li>';
                 * } $html[] = '</ul>'; $html[] = '</div>'; return implode("\n", $html);
                 */

                // $html[] = '<h4>Attachments</h4>';
                $html[] = '<div class="attachments" style="margin-top: 1em;">';
                $html[] = '<div class="attachments_title">' . htmlentities(Translation :: get('Attachments')) . '</div>';
                Utilities :: order_content_objects_by_title($attachments);
                $html[] = '<ul class="attachments_list">';
                foreach ($attachments as $attachment)
                {
                    $url = $this->context->get_content_object_display_attachment_url($attachment);
                    $url = 'javascript:openPopup(\'' . $url . '\'); return false;';
                    $html[] = '<li><a href="#" onClick="' . $url . '"><img src="' . Theme :: get_image_path(
                            ContentObject :: get_content_object_type_namespace($attachment->get_type())) . 'logo/' . Theme :: ICON_MINI . '.png" alt="' . htmlentities(
                            Translation :: get('TypeName', null,
                                    ContentObject :: get_content_object_type_namespace($attachment->get_type()))) . '"/> ' . $attachment->get_title() . '</a></li>';
                }
                $html[] = '</ul>';
                $html[] = '</div>';
                return implode("\n", $html);
            }
        }
        return '';
    }

    /**
     * Returns a HTML view of the versions of the object.
     *
     * @return string The HTML.
     */
    public function get_version_quota_as_html()
    {
        $object = $this->get_content_object();

        $html = array();
        if ($object->is_latest_version())
        {
            $html[] = '<div class="version_stats">';
        }
        else
        {
            $html[] = '<div class="version_stats_na">';
        }
        $html[] = '<div class="version_stats_title">' . htmlentities(Translation :: get('VersionQuota')) . '</div>';

        $percent = $object->get_version_count() / ($object->get_version_count() + $object->get_available_version_count()) * 100;
        $status = $object->get_version_count() . ' / ' . ($object->get_version_count() + $object->get_available_version_count());

        $html[] = self :: get_bar($percent, $status);
        $html[] = '</div>';
        return implode("\n", $html);
    }

    /**
     * Build a bar-view of the used quota.
     *
     * @param $percent float The percentage of the bar that is in use
     * @param $status string A status message which will be displayed below the bar.
     * @return string HTML representation of the requested bar.
     */
    private function get_bar($percent, $status)
    {
        $html = array();
        $html[] = '<div class="usage_information">';
        $html[] = '<div class="usage_bar">';
        for($i = 0; $i < 100; $i ++)
        {
            if ($percent > $i)
            {
                if ($i >= 90)
                {
                    $class = 'very_critical';
                }
                elseif ($i >= 80)
                {
                    $class = 'critical';
                }
                else
                {
                    $class = 'used';
                }
            }
            else
            {
                $class = '';
            }
            $html[] = '<div class="' . $class . '"></div>';
        }
        $html[] = '</div>';
        $html[] = '<div class="usage_status">' . $status . ' &ndash; ' . round($percent, 2) . ' %</div>';
        $html[] = '</div>';
        return implode("\n", $html);
    }

    /**
     * Returns the URL where the given learning object may be viewed.
     *
     * @param $content_object ContentObject The object.
     * @return string The URL.
     */
    protected function get_content_object_url($content_object)
    {
        return sprintf($this->url_format, $content_object->get_id());
    }

    /**
     * Returns the URL format for linked objects.
     *
     * @return string The URL, ready to pass to sprintf() with the object ID.
     */
    protected function get_content_object_url_format()
    {
        return $this->url_format;
    }

    /**
     * Creates an object that can display the given object in a standardized fashion.
     *
     * @param $object ContentObject The object to display. * @param ContentObjectDisplayContext $context
     * @return ContentObjectDisplay
     */
    public static function factory(&$object, ContentObjectDisplayContext $context)
    {
        $type = $object->get_type();

        $class = $type . 'Display';
        return new $class($object, null, $context);
    }

    public function get_path($path_type)
    {
        return Path :: get($path_type);
    }
}
