<?php
namespace repository;

use common\libraries\Utilities;
use common\libraries\EqualityCondition;
use common\libraries\Path;
use common\libraries\DataClass;
use DOMDocument;

/**
 *
 * @author Hans De Bisschop
 */
class ExternalSetting extends DataClass
{
    const CLASS_NAME = __CLASS__;
    const PROPERTY_EXTERNAL_ID = 'external_id';
    const PROPERTY_VARIABLE = 'variable';
    const PROPERTY_VALUE = 'value';
    const PROPERTY_USER_SETTING = 'user_setting';

    /**
     * A static array containing all settings of external repository instances
     * 
     * @var array
     */
    private static $settings;

    /**
     * Get the default properties of all settings.
     * 
     * @return array The property names.
     */
    /**
     *
     * @return array:
     */
    public static function get_default_property_names()
    {
        return parent :: get_default_property_names(
                array(self :: PROPERTY_EXTERNAL_ID, self :: PROPERTY_VARIABLE, self :: PROPERTY_VALUE, 
                        self :: PROPERTY_USER_SETTING));
    }

    /**
     *
     * @return string the external repository id
     */
    public function get_external_id()
    {
        return $this->get_default_property(self :: PROPERTY_EXTERNAL_ID);
    }

    /**
     * Returns the variable of this setting object
     * 
     * @return string the variable
     */
    public function get_variable()
    {
        return $this->get_default_property(self :: PROPERTY_VARIABLE);
    }

    /**
     * Returns the value of this setting object
     * 
     * @return string the value
     */
    public function get_value()
    {
        return $this->get_default_property(self :: PROPERTY_VALUE);
    }

    /**
     *
     * @param $external_repository_id string
     */
    public function set_external_id($external_id)
    {
        $this->set_default_property(self :: PROPERTY_EXTERNAL_ID, $external_id);
    }

    /**
     * Sets the variable of this setting.
     * 
     * @param $variable string the variable.
     */
    public function set_variable($variable)
    {
        $this->set_default_property(self :: PROPERTY_VARIABLE, $variable);
    }

    /**
     * Sets the value of this setting.
     * 
     * @param $value string the value.
     */
    public function set_value($value)
    {
        $this->set_default_property(self :: PROPERTY_VALUE, $value);
    }

    /**
     * Returns the user_setting of this setting object
     * 
     * @return string the user_setting
     */
    public function get_user_setting()
    {
        return $this->get_default_property(self :: PROPERTY_USER_SETTING);
    }

    /**
     * Sets the user_setting of this setting.
     * 
     * @param $user_setting string the user_setting.
     */
    public function set_user_setting($user_setting)
    {
        $this->set_default_property(self :: PROPERTY_USER_SETTING, $user_setting);
    }

    /**
     *
     * @return string
     */
    public static function get_table_name()
    {
        return Utilities :: get_classname_from_namespace(self :: CLASS_NAME, true);
    }

    public static function get_class_name()
    {
        return self :: CLASS_NAME;
    }

    public static function initialize(ExternalInstance $external_instance)
    {
        $settings_file = Path :: namespace_to_full_path($external_instance->get_type()) . '/php/settings/settings.xml';
        
        $doc = new DOMDocument();
        
        $doc->load($settings_file);
        $object = $doc->getElementsByTagname('application')->item(0);
        $settings = $doc->getElementsByTagname('setting');
        
        foreach ($settings as $index => $setting)
        {
            $external_setting = new ExternalSetting();
            $external_setting->set_external_id($external_instance->get_id());
            $external_setting->set_variable($setting->getAttribute('name'));
            $external_setting->set_value($setting->getAttribute('default'));
            
            $user_setting = $setting->getAttribute('user_setting');
            if ($user_setting)
            {
                $external_setting->set_user_setting($user_setting);
            }
            else
            {
                $external_setting->set_user_setting(0);
            }
            
            if (! $external_setting->create())
            {
                return false;
            }
        }
        
        return true;
    }

    public function delete()
    {
        if (! parent :: delete())
        {
            return false;
        }
        else
        {
            if ($this->get_user_setting())
            {
                $condition = new EqualityCondition(ExternalUserSetting :: PROPERTY_SETTING_ID, $this->get_id());
                
                if (! DataManager :: deletes(ExternalUserSetting :: class_name(), $condition))
                {
                    return false;
                }
                else
                {
                    return true;
                }
            }
            else
            {
                return true;
            }
        }
    }

    /**
     *
     * @param $variable string
     * @param $external_repository_id int
     * @return mixed
     */
    public static function get($variable, $external_id)
    {
        if (! isset(self :: $settings[$external_id]))
        {
            self :: load($external_id);
        }
        
        return (isset(self :: $settings[$external_id][$variable]) ? self :: $settings[$external_id][$variable] : null);
    }

    public static function get_all($external_id)
    {
        if (! isset(self :: $settings[$external_id]))
        {
            self :: load($external_id);
        }
        
        return self :: $settings[$external_id];
    }

    /**
     *
     * @param $external_repository_id int
     */
    public static function load($external_id)
    {
        $condition = new EqualityCondition(self :: PROPERTY_EXTERNAL_ID, $external_id);
        $settings = \repository\DataManager :: retrieves(ExternalSetting :: class_name(), $condition);
        
        while ($setting = $settings->next_result())
        {
            self :: $settings[$external_id][$setting->get_variable()] = $setting->get_value();
        }
    }
}
