<?php
namespace repository\viewer;

use repository\RepositoryRights;
use common\libraries\ComplexContentObjectSupport;
use common\libraries\Toolbar;
use common\libraries\ToolbarItem;
use common\libraries\Translation;
use common\libraries\Theme;
use common\libraries\Request;
use common\libraries\FormValidator;
use common\libraries\Utilities;
use common\libraries\BreadcrumbTrail;

class BrowserComponent extends Manager
{
    const SHARED_BROWSER = 'shared';
    const SHARED_BROWSER_ALLOWED = 'allow_shared_browser';
    const PROPERTY_CATEGORY = 'category';

    /**
     * The search form
     *
     * @var \common\libraries\FormValidator
     */
    private $form;

    /**
     * The renderer for the search form
     */
    private $renderer;

    public function __construct($parent)
    {
        parent :: __construct($parent);

        $form_parameters = $this->get_parameter();
        $form_parameters[self :: PARAM_ACTION] = self :: ACTION_BROWSER;

        if ($this->is_shared_object_browser())
        {
            $form_parameters[self :: SHARED_BROWSER] = 1;
        }

        $this->set_form(
            new FormValidator('search', 'post', $this->get_url($form_parameters), '', array('id' => 'search'), false));
        $this->get_form()->addElement(
            'text',
            self :: PARAM_QUERY,
            Translation :: get('Search', null, Utilities :: COMMON_LIBRARIES),
            'size="30" class="search_query"');
        $this->get_form()->addElement('style_submit_button', 'submit', null, array('class' => 'search'));
        $this->get_form()->setDefaults(array(self :: PARAM_QUERY => $this->get_query()));
    }

    /*
     * Inherited
     */
    public function run()
    {
        $this->renderer = clone $this->form->defaultRenderer();
        $this->renderer->setElementTemplate('<span>{element}</span> ');
        $this->form->accept($this->renderer);

        $html = array();
        $html[] = '<div class="search_form" style="float: right; margin: 0px 0px 5px 0px;">';
        $html[] = '<div class="simple_search">';
        $html[] = $this->renderer->toHTML();
        $html[] = '</div>';
        $html[] = '</div>';

        if ($this->get_maximum_select() > self :: SELECT_SINGLE)
        {
            $html[] = '<b>' . sprintf(
                Translation :: get('SelectMaximumNumberOfContentObjects'),
                $this->get_maximum_select()) . '</b><br />';
        }

        $menu = $this->get_menu();

        if ($this->is_allowed_shared_object_browsing())
        {
            $shared_menu = $this->get_shared_menu();
        }

        $table = $this->get_object_table();

        $html[] = '<br />';

        $html[] = '<div style="width: 15%; overflow: auto; float:left">';
        $html[] = $menu->render_as_tree();
        $html[] = '<br />';
        $html[] = $shared_menu ? $shared_menu->render_as_tree() : '';
        $html[] = '</div>';

        $html[] = '<div style="width: 83%; float: right;">';
        $html[] = $table->as_html();
        $html[] = '</div>';
        $html[] = '<div class="clear">&nbsp;</div>';

        $this->display_header();
        echo implode("\n", $html);
        $this->display_footer();
    }

    /**
     *
     * @return \repository\viewer\ContentObjectTable
     */
    protected function get_object_table()
    {
        return new ContentObjectTable($this, $this->get_user(), $this->get_types(), $this->get_query());
    }

    /**
     *
     * @return string NULL
     */
    protected function get_query()
    {
        if ($this->get_form()->validate())
        {

            return $this->get_form()->exportValue(self :: PARAM_QUERY);
        }

        if (Request :: get(self :: PARAM_QUERY))
        {
            return Request :: get(self :: PARAM_QUERY);
        }

        return null;
    }

    /**
     *
     * @return \common\libraries\FormValidator
     */
    public function get_form()
    {
        return $this->form;
    }

    /**
     *
     * @param \common\libraries\FormValidator $form
     */
    public function set_form($form)
    {
        $this->form = $form;
    }

    /**
     *
     * @param boolean $allow_shared
     * @return \repository\RepositoryCategoryMenu
     */
    public function get_menu($allow_shared = true)
    {
        $url = $this->get_url($this->get_parameters()) . '&' . self :: PROPERTY_CATEGORY . '=%s';

        $extra = array();

        if ($this->get_query())
        {
            $search_url = '#';
            $search = array();

            if ($this->is_shared_object_browser())
            {
                $search['title'] = Translation :: get('SharedSearchResults');
            }
            else
            {
                $search['title'] = Translation :: get('SearchResults', null, Utilities :: COMMON_LIBRARIES);
            }

            $search['url'] = $search_url;
            $search['class'] = 'search_results';
            $extra[] = $search;
        }
        else
        {
            $search_url = null;
        }

        $menu = new RepositoryCategoryMenu(
            $this,
            $this->get_user_id(),
            Request :: get(self :: PROPERTY_CATEGORY) ? Request :: get(self :: PROPERTY_CATEGORY) : 0,
            $url,
            $extra,
            $this->get_types());

        if ($search_url)
        {
            $menu->forceCurrentUrl($search_url);
        }

        return $menu;
    }

    /**
     *
     * @param int $category_id
     * @return string
     */
    public function get_category_url($category_id)
    {
        return $this->get_url(array(self :: PROPERTY_CATEGORY => $category_id));
    }

    /**
     *
     * @return \repository\RepositorySharedCategoryMenu
     */
    public function get_shared_menu()
    {
        return new RepositorySharedCategoryMenu($this);
    }

    /**
     *
     * @param \repository\ContentObject $content_object
     * @return \common\libraries\Toolbar
     */
    public function get_default_browser_actions($content_object)
    {
        $toolbar = new Toolbar(Toolbar :: TYPE_HORIZONTAL);

        if ($content_object->has_right(RepositoryRights :: USE_RIGHT, $this->get_user_id()))
        {
            $toolbar->add_item(
                new ToolbarItem(
                    Translation :: get('Publish', null, Utilities :: COMMON_LIBRARIES),
                    Theme :: get_common_image_path() . 'action_publish.png',
                    $this->get_url(
                        array_merge(
                            $this->get_parameters(),
                            array(
                                self :: PARAM_ACTION => self :: ACTION_PUBLISHER,
                                self :: PARAM_ID => $content_object->get_id())),
                        false),
                    ToolbarItem :: DISPLAY_ICON));
        }

        if ($content_object->has_right(RepositoryRights :: VIEW_RIGHT, $this->get_user_id()))
        {
            $toolbar->add_item(
                new ToolbarItem(
                    Translation :: get('Preview'),
                    Theme :: get_common_image_path() . 'action_browser.png',
                    $this->get_url(
                        array_merge(
                            $this->get_parameters(),
                            array(
                                self :: PARAM_ACTION => self :: ACTION_VIEWER,
                                self :: PARAM_ID => $content_object->get_id())),
                        false),
                    ToolbarItem :: DISPLAY_ICON));
        }

        if ($content_object->has_right(RepositoryRights :: COLLABORATE_RIGHT, $this->get_user_id()))
        {
            $toolbar->add_item(
                new ToolbarItem(
                    Translation :: get('EditAndPublish'),
                    Theme :: get_common_image_path() . 'action_editpublish.png',
                    $this->get_url(
                        array_merge(
                            $this->get_parameters(),
                            array(
                                self :: PARAM_ACTION => self :: ACTION_CREATOR,
                                self :: PARAM_EDIT_ID => $content_object->get_id())),
                        false),
                    ToolbarItem :: DISPLAY_ICON));
        }

        if ($content_object instanceof ComplexContentObjectSupport)
        {

            $preview_url = \repository\RepositoryManager :: get_preview_complex_content_object_url($content_object);
            $onclick = '" onclick="javascript:openPopup(\'' . $preview_url . '\'); return false;';
            $toolbar->add_item(
                new ToolbarItem(
                    Translation :: get('Preview', null, Utilities :: COMMON_LIBRARIES),
                    Theme :: get_common_image_path() . 'action_preview.png',
                    $preview_url,
                    ToolbarItem :: DISPLAY_ICON,
                    false,
                    $onclick,
                    '_blank'));
        }

        return $toolbar;
    }

    /**
     *
     * @return boolean
     */
    public function is_shared_object_browser()
    {
        return (Request :: get(self :: SHARED_BROWSER) == 1);
    }

    public function add_additional_breadcrumbs(BreadcrumbTrail $breadcrumbtrail)
    {
        $breadcrumbtrail->add_help('repo_viewer_browser');
    }

    /**
     *
     * @return boolean
     */
    public function is_allowed_shared_object_browsing()
    {
        $parameters = $this->get_parameters();
        $allowed = $parameters[self :: SHARED_BROWSER_ALLOWED];
        if ($allowed === false)
        {
            return false;
        }
        else
        {
            return true;
        }
    }
}
