<?php
namespace rights;

use common\libraries\Theme;
use common\libraries\DataClassRetrievesParameters;
use common\libraries\Path;
use common\libraries\Translation;
use common\libraries\Utilities;
use common\libraries\AdvancedElementFinderElement;
use common\libraries\AdvancedElementFinderElementType;
use common\libraries\DataClassCountParameters;

use group\DataManager as GroupDataManager;
use group\Group;

/**
 * Class that describes the platform groups for the rights editor
 *
 * @author Sven Vanpoucke
 */
class PlatformGroupEntity implements NestedRightsEntity
{
    const ENTITY_NAME = 'group';
    const ENTITY_TYPE = 2;

    private $platform_group_cache;
    private static $instance;

    public static function get_instance()
    {
        if (! isset(self :: $instance))
        {
            self :: $instance = new static();
        }
        return self :: $instance;
    }

    /**
     * Retrieves the items for this entity
     *
     * @param $condition Condition
     * @param $offset int
     * @param $count int
     * @param $order_property Array
     * @return ObjectResultSet
     */
    public function retrieve_entity_items($condition = null, $offset = null, $count = null, $order_property = null)
    {
        $condition = $this->get_condition($condition);

        return GroupDataManager :: retrieves(
            Group :: class_name(),
            new DataClassRetrievesParameters($condition, $count, $offset, $order_property)
        );
    }

    /**
     * Retrieves the entity item ids relevant for a given user (direct subscribed platformgroups and their parents)
     *
     * @param $user_id integer
     * @return array
     */
    public function retrieve_entity_item_ids_linked_to_user($user_id)
    {
        if (is_null($this->platform_group_cache[$user_id]))
        {
            $this->platform_group_cache[$user_id] =
                GroupDataManager :: retrieve_all_subscribed_groups_array($user_id, true);
        }
        return $this->platform_group_cache[$user_id];
    }

    /**
     * Counts the items for this entity
     *
     * @param $condition Condition
     * @return int
     */
    public function count_entity_items($condition = null)
    {
        $condition = $this->get_condition($condition);
        return GroupDataManager :: count(Group :: class_name(), new DataClassCountParameters($condition));
    }

    /**
     * Returns the name of this entity
     *
     * @return String
     */
    public function get_entity_name()
    {
        return self :: ENTITY_NAME;
    }

    /**
     * Returns the translated name of this entiry for displaying purposes only!
     *
     * @return String Translated name of the entity
     */
    public function get_entity_translated_name()
    {
        return Translation :: get(Utilities :: underscores_to_camelcase(self :: ENTITY_NAME));
    }

    /**
     * Returns the type of this entity
     *
     * @return int
     */
    public function get_entity_type()
    {
        return self :: ENTITY_TYPE;
    }

    /**
     * Returns the path to the icon of the entity
     *
     * @return String
     */
    public function get_entity_icon()
    {
        return Theme :: get_image_path('common\extensions\rights_editor_manager') . 'place_group.png';
    }

    /**
     * Returns the cell renderer of this entity
     *
     * @param $browser Application
     * @return LocationPlatformGroupBrowserTableCellRenderer
     */
    public function get_entity_cell_renderer($browser)
    {
        return new LocationPlatformGroupBrowserTableCellRenderer($browser);
    }

    /**
     * Returns the column model of this entity
     *
     * @param $browser Application
     * @return LocationPlatformGroupBrowserTableColumnModel
     */
    public function get_entity_column_model($browser)
    {
        return new LocationPlatformGroupBrowserTableColumnModel($browser);
    }

    /**
     * Returns the property for the ID column of this entity
     *
     * @return String
     */
    public function get_id_property()
    {
        return Group :: PROPERTY_ID;
    }

    /**
     * Returns the property for the PARENT column of this entity
     *
     * @return String
     */
    public function get_parent_property()
    {
        return Group :: PROPERTY_PARENT_ID;
    }

    /**
     * Returns the property for the TITLE column of this entity
     *
     * @return String
     */
    public function get_title_property()
    {
        return Group :: PROPERTY_NAME;
    }

    /**
     * Returns the root ids for this entity
     *
     * @return Array<int>
     */
    public function get_root_ids()
    {
        return array(GroupDataManager :: get_root_group()->get_id());
    }

    /**
     * Returns the properties on which can be searched
     *
     * @return Array
     */
    public function get_search_properties()
    {
        return array(Group :: PROPERTY_NAME, Group :: PROPERTY_CODE);
    }

    /**
     * Returns the xml feed which will be used for the element finder and for the ajax tree menu
     */
    public function get_xml_feed()
    {
        return Path :: get(WEB_PATH) . 'group/php/xml_feeds/xml_group_menu_feed.php';
    }

    /**
     * Function that can be filled in extensions of this class to limit the platform groups
     *
     * @return Condition
     */
    public function get_condition($condition)
    {
        return $condition;
    }

    /**
     * Retrieves the type for the advanced element finder for the simple rights editor
     */
    public function get_element_finder_type()
    {
        return new AdvancedElementFinderElementType('platform_groups', Translation :: get('PlatformGroups'),
            RightsManager :: APPLICATION_NAME, 'platform_group_entity_feed', array());
    }

    /**
     * Retrieves an element for the advanced element finder for the simple rights editor with the given id
     */
    public function get_element_finder_element($id)
    {
        $group = GroupDataManager :: retrieve_by_id(Group :: class_name(), $id);
        if (! $group)
        {
            return null;
        }

        return new AdvancedElementFinderElement(self :: ENTITY_TYPE . '_' . $id, 'type type_group', $group->get_name(),
            $group->get_code());
    }

    /**
     * Returns the class name of the data class that is used for this entity
     *
     * @return string
     */
    public static function data_class_class_name()
    {
        return Group :: class_name();
    }
}
