<?php
namespace rights;

use common\libraries\Translation;
use common\libraries\PropertyConditionVariable;
use common\libraries\StaticConditionVariable;
use common\libraries\DataClassRetrieveParameters;

use common\libraries\Utilities;
use common\libraries\EqualityCondition;
use common\libraries\AndCondition;
use common\libraries\InequalityCondition;
use common\libraries\ObjectTableOrder;
use common\libraries\NestedTreeNode;
use common\libraries\DataClassRetrievesParameters;

/**
 * $Id: location.class.php 214 2009-11-13 13:57:37Z vanpouckesven $
 * @package rights.lib
 * @author Hans de Bisschop
 */

class RightsLocation extends NestedTreeNode
{
    //Keep track of the context so we know which table to call
    private $context;

    const CLASS_NAME = __CLASS__;

    const PROPERTY_TREE_IDENTIFIER = 'tree_identifier';
    const PROPERTY_TREE_TYPE = 'tree_type';
    const PROPERTY_TYPE = 'type';
    const PROPERTY_IDENTIFIER = 'identifier';
    const PROPERTY_INHERIT = 'inherit';
    const PROPERTY_LOCKED = 'locked';

    /**
     * Get the default properties of all users.
     * @return array The property names.
     */
    public static function get_default_property_names()
    {
        return parent :: get_default_property_names(array(self :: PROPERTY_TYPE, self :: PROPERTY_IDENTIFIER,
                self :: PROPERTY_TREE_IDENTIFIER, self :: PROPERTY_TREE_TYPE, self :: PROPERTY_INHERIT,
                self :: PROPERTY_LOCKED));
    }

    public function get_context()
    {
        if(! isset($this->context)) {
          // Default to the namespace of the rights location subclass
          $classname_parts = explode('\\', $this->class_name());
          array_pop($classname_parts);
          $this->context = implode('\\', $classname_parts);
        }
          
        return $this->context;
    }

    public function set_context($context)
    {
        $this->context = $context;
    }

    public function get_type()
    {
        return $this->get_default_property(self :: PROPERTY_TYPE);
    }

    public function set_type($type)
    {
        $this->set_default_property(self :: PROPERTY_TYPE, $type);
    }

    public function set_type_from_object($object)
    {
        $this->set_type(Utilities :: get_classname_from_object($object, true));
    }

    public function get_identifier()
    {
        return $this->get_default_property(self :: PROPERTY_IDENTIFIER);
    }

    public function set_identifier($identifier)
    {
        $this->set_default_property(self :: PROPERTY_IDENTIFIER, $identifier);
    }

    public function get_inherit()
    {
        return $this->get_default_property(self :: PROPERTY_INHERIT);
    }

    public function set_inherit($inherit)
    {
        $this->set_default_property(self :: PROPERTY_INHERIT, $inherit);
    }

    public function get_tree_identifier()
    {
        return $this->get_default_property(self :: PROPERTY_TREE_IDENTIFIER);
    }

    public function set_tree_identifier($tree_identifier)
    {
        $this->set_default_property(self :: PROPERTY_TREE_IDENTIFIER, $tree_identifier);
    }

    public function get_tree_type()
    {
        return $this->get_default_property(self :: PROPERTY_TREE_TYPE);
    }

    public function set_tree_type($tree_type)
    {
        $this->set_default_property(self :: PROPERTY_TREE_TYPE, $tree_type);
    }

    public function inherits()
    {
        return $this->get_inherit();
    }

    public function switch_inherit()
    {
        if ($this->inherits())
        {
            $this->set_inherit(false);
        }
        else
        {
            $this->set_inherit(true);
        }
    }

    public function inherit()
    {
        $this->set_inherit(true);
    }

    public function disinherit()
    {
        $this->set_inherit(false);
    }

    public function get_locked()
    {
        return $this->get_default_property(self :: PROPERTY_LOCKED);
    }

    public function set_locked($locked)
    {
        $this->set_default_property(self :: PROPERTY_LOCKED, $locked);
    }

    public function is_locked()
    {
        return $this->get_locked();
    }

    public function lock()
    {
        $this->set_locked(true);
    }

    public function unlock()
    {
        $this->set_locked(false);
    }

    public function switch_lock()
    {
        if ($this->is_locked())
        {
            $this->unlock();
        }
        else
        {
            $this->lock();
        }
    }

    public function is_root()
    {
        $parent = $this->get_parent();
        return ($parent == 0);
    }

    public function get_locked_parent()
    {
        $locked_parent_conditions = $this->get_nested_set_condition_array();

        $locked_parent_conditions[] = new EqualityCondition(
            new PropertyConditionVariable(self :: class_name(), self :: PROPERTY_LOCKED),
            new StaticConditionVariable(true)
        );

        $locked_parent_conditions[] = new InequalityCondition(
            new PropertyConditionVariable(self :: class_name(), self :: PROPERTY_LEFT_VALUE),
            InequalityCondition :: LESS_THAN,
            new StaticConditionVariable($this->get_left_value())
        );

        $locked_parent_conditions[] = new InequalityCondition(
            new PropertyConditionVariable(self :: class_name(), self :: PROPERTY_RIGHT_VALUE),
            InequalityCondition :: GREATER_THAN,
            new StaticConditionVariable($this->get_right_value())
        );

        $locked_parent_condition = new AndCondition($locked_parent_conditions);
        $order = array(new ObjectTableOrder(RightsLocation :: PROPERTY_LEFT_VALUE, SORT_DESC));

        $datamanager = $this->determine_data_manager();
        return $datamanager :: retrieve(
            self :: class_name(), new DataClassRetrieveParameters($locked_parent_condition, $order)
        );

    }

    /**  on this location
     */
    public function clear_rights()
    {
        return DataManager :: delete_rights_location_entity_rights($this);
    }

    public function clear_right($right_id)
    {
        return DataManager :: delete_rights_location_entity_rights($this, null, null, $right_id);
    }

    /**
     * Retrieves the rights entities linked to this location
     *
     * @param int $right_id - [OPTIONAL] default null
     *
     * @return ResultSet
     */
    public function get_rights_entities($right_id = null)
    {
        $conditions = array();

        $conditions[] = new EqualityCondition(RightsLocationEntityRight :: PROPERTY_LOCATION_ID, $this->get_id());

        if (! is_null($right_id))
        {
            $conditions[] = new EqualityCondition(RightsLocationEntityRight :: PROPERTY_RIGHT_ID, $right_id);
        }

        $condition = new AndCondition($conditions);

        $entity_rights = DataManager :: retrieve_rights_location_rights($this->get_context(), $condition);
        if ($entity_rights)
        {
            return $entity_rights->as_array();
        }
        else
        {
            throw new \Exception(Translation :: get('InvalidDataRetrievedFromDatabase'));
        }

    }

//    // C(R)UD methods that take into account that this class may be used in different contexts
//    // and thus they need to use the correct table prefix.
//
//    public function create($previous_id = 0, $create_in_batch = false)
//    {
////        $rldm         = $this->get_data_manager();
////        $orig_context = $rldm->get_prefix();
////        $context      = $this->get_context();
////
////        $rldm->set_prefix($context . '_');
//
//        $result       = parent :: create($previous_id, $create_in_batch);
//
////        $rldm->set_prefix($orig_context);
//
//        return $result;
//    }
//
//    public function update()
//    {
//        $rldm         = $this->get_data_manager();
//        $orig_context = $rldm->get_prefix();
//        $context      = $this->get_context();
//
//        $rldm->set_prefix($context . '_');
//
//        $result       = parent :: update();
//
//        $rldm->set_prefix($orig_context);
//
//        return $result;
//    }
//
//    public function delete()
//    {
//        $rldm         = $this->get_data_manager();
//        $orig_context = $rldm->get_prefix();
//        $context      = $this->get_context();
//
//        $rldm->set_prefix($context . '_');
//
//        $result       = parent :: delete();
//
//        $rldm->set_prefix($orig_context);
//
//        return $result;
//    }

    public function delete_related_content() {
        return $this->clear_rights();
    }
    
    /**
     * Inherited method which specifies how to identify the tree this location is situated in.
     * Should be used as the basic set of condition whenever one makes a query.
     */
    public function get_nested_set_condition_array()
    {
        $conditions   = parent :: get_nested_set_condition_array();
        $conditions[] = new EqualityCondition(self :: PROPERTY_TREE_TYPE, $this->get_tree_type());
        $conditions[] = new EqualityCondition(self :: PROPERTY_TREE_IDENTIFIER, $this->get_tree_identifier());

        return $conditions;
    }

//    public function move($new_parent_id = 0, $new_previous_id = 0)
//    {
//        $rldm         = $this->get_data_manager();
//        $orig_context = $rldm->get_prefix();
//        $context      = $this->get_context();
//
//        $rldm->set_prefix($context . '_');
//
//        $result       = parent :: move($new_parent_id, $new_previous_id);
//
//        $rldm->set_prefix($orig_context);
//
//        return $result;
//    }

}
