<?php

if (isset($_SERVER['argv']) && isset($_SERVER['argv'][1]))
{
    $old_database_file = $_SERVER['argv'][1];
}

if (isset($_SERVER['argv']) && isset($_SERVER['argv'][2]))
{
    $new_database_file = $_SERVER['argv'][2];
}

$comparer = new DatabaseComparer($old_database_file, $new_database_file);
$comparer->run();

/**
 * SELECT `TABLE_NAME` as table_name, `ORDINAL_POSITION`, `COLUMN_NAME`, `DATA_TYPE` AS column_type, `COLUMN_DEFAULT`,
 * `IS_NULLABLE` AS column_allow_null, IF( `CHARACTER_MAXIMUM_LENGTH` IS NULL , `NUMERIC_SCALE`, `CHARACTER_MAXIMUM_LENGTH` )
 * AS column_length, COLUMN_KEY FROM COLUMNS WHERE `TABLE_SCHEMA` LIKE 'chamilo_3_structure'
 *
 * This class compares the structure of two databases by providing the structure of the database as csv files.
 *
 * The structure of a database is read from a csv file with the following headers:
 * table_name;column_position;column_name;column_type;column_default;column_allow_null;column_length;column_key
 *
 * The script is called with the paths of the database files as arguments. The first argument is the old database
 * structure. The second argument is the new database structure.
 *
 * php database_comparer.class.php old_database.csv new_database.csv
 *
 * @author Sven Vanpoucke - Hogeschool Gent
 */
class DatabaseComparer
{
    /**
     * The path to the old database file
     *
     * @var string
     */
    private $old_database_file_path;

    /**
     * The path to the new database file
     *
     * @var string
     */
    private $new_database_file_path;

    /**
     * Constructor
     *
     * @param string $old_database_file_path
     * @param string $new_database_file_path
     *
     * @throws Exception
     */
    public function __construct($old_database_file_path, $new_database_file_path)
    {
        if (! file_exists($old_database_file_path))
        {
            throw new \Exception('The path to the old database file must be valid');
        }

        if (! file_exists($new_database_file_path))
        {
            throw new \Exception('The path to the new database file must be valid');
        }

        $this->old_database_file_path = $old_database_file_path;
        $this->new_database_file_path = $new_database_file_path;
    }

    /**
     * Runs the comparer script
     */
    public function run()
    {
        $old_database_structure = $this->read_database_structure_from_csv($this->old_database_file_path);
        $new_database_structure = $this->read_database_structure_from_csv($this->new_database_file_path);

        $this->check_table_changes($old_database_structure, $new_database_structure);
        $this->check_table_column_changes($old_database_structure, $new_database_structure);
    }

    /**
     * Checks the table changes between two database structures
     *
     * @param array $old_database_structure
     * @param array $new_database_structure
     */
    protected function check_table_changes($old_database_structure, $new_database_structure)
    {
        $removed_tables = array_diff_key($old_database_structure, $new_database_structure);
        $added_tables = array_diff_key($new_database_structure, $old_database_structure);

        $this->write_csv_file(__DIR__ . DIRECTORY_SEPARATOR . 'removed_tables.csv', array_keys($removed_tables));
        $this->write_csv_file(__DIR__ . DIRECTORY_SEPARATOR . 'added_tables.csv', array_keys($added_tables));
    }

    /**
     * Checks the table column changes between two database structures
     *
     * @param array $old_database_structure
     * @param array $new_database_structure
     */
    protected function check_table_column_changes($old_database_structure, $new_database_structure)
    {
        $changed_columns = array();

        $changed_column_headers = array('Table Name', 'Column Name', 'Status', 'Remarks');
        $changed_columns[] = $changed_column_headers;

        $existing_old_tables = array_intersect_key($old_database_structure, $new_database_structure);

        foreach($existing_old_tables as $table_name => $old_columns)
        {
            $new_columns = $new_database_structure[$table_name];

            $changed_columns = array_merge(
                $changed_columns, $this->get_changed_columns($table_name, $old_columns, $new_columns)
            );

            $changed_columns = array_merge(
                $changed_columns, $this->get_added_columns($table_name, $old_columns, $new_columns)
            );

            $changed_columns = array_merge(
                $changed_columns, $this->get_deleted_columns($table_name, $old_columns, $new_columns)
            );
        }

        $this->write_csv_file(__DIR__ . DIRECTORY_SEPARATOR . 'column_changes.csv', $changed_columns);
    }

    /**
     * Returns the columns which content have been changed
     *
     * @param string $table_name
     * @param array $old_columns
     * @param array $new_columns
     *
     * @return array
     */
    protected function get_changed_columns($table_name, $old_columns, $new_columns)
    {
        $changed_columns = array();

        $existing_old_columns = array_intersect_key($old_columns, $new_columns);

        if(is_array($existing_old_columns))
        {
            foreach($existing_old_columns as $column_name => $old_column_information)
            {
                $column_changes = $this->get_column_changes($old_column_information, $new_columns[$column_name]);

                if(count($column_changes) > 0)
                {
                    $changed_column = array();

                    $changed_column[] = $table_name;
                    $changed_column[] = $column_name;
                    $changed_column[] = 'CHANGED';
                    $changed_column[] = implode(', ', $column_changes);

                    $changed_columns[] = $changed_column;
                }
            }
        }

        return $changed_columns;
    }

    /**
     * Returns the added columns for a given table
     *
     * @param string $table_name
     * @param array $old_columns
     * @param array $new_columns
     *
     * @return array
     */
    protected function get_added_columns($table_name, $old_columns, $new_columns)
    {
        $added_columns = array_diff_key($new_columns, $old_columns);
        return $this->get_changed_column_array($table_name, array_keys($added_columns), 'ADDED');
    }

    /**
     * Returns the added columns for a given table
     *
     * @param string $table_name
     * @param array $old_columns
     * @param array $new_columns
     *
     * @return array
     */
    protected function get_deleted_columns($table_name, $old_columns, $new_columns)
    {
        $deleted_columns = array_diff_key($old_columns, $new_columns);
        return $this->get_changed_column_array($table_name, array_keys($deleted_columns), 'DELETED');
    }

    /**
     * Returns the changed column array for a table with an array of columns and a fixed status
     *
     * @param string $table_name
     * @param array $columns
     * @param string $status
     *
     * @return array
     */
    protected function get_changed_column_array($table_name, $columns, $status)
    {
        $changed_columns = array();

        foreach($columns as $column_name)
        {
            $changed_column = array();

            $changed_column[] = $table_name;
            $changed_column[] = $column_name;
            $changed_column[] = $status;

            $changed_columns[] = $changed_column;
        }

        return $changed_columns;
    }

    /**
     * Returns the changes between two columns by checking the information arrays
     *
     * @param array $old_column_information
     * @param array $new_column_information
     *
     * @return array
     */
    protected function get_column_changes($old_column_information, $new_column_information)
    {
        $column_changes = array();

        foreach($old_column_information as $information_header => $old_information_value)
        {
            if($old_information_value != $new_column_information[$information_header])
            {
                $column_changes[] = $information_header;
            }
        }

        return $column_changes;
    }

    /**
     * Reads the database structure from a csv
     *
     * @param string $csv_file_path
     *
     * @return array
     */
    protected function read_database_structure_from_csv($csv_file_path)
    {
        $database_structure = array();

        $handle = fopen($csv_file_path, 'r');
        $headers = fgetcsv($handle);

        while ($line = fgetcsv($handle))
        {
            $column_info = array();

            $column_info['position'] = $line[1];
            $column_info['type'] = $line[3];
            $column_info['default'] = $line[4];
            $column_info['allow_null'] = $line[5];
            $column_info['length'] = $line[6];
            $column_info['key'] = $line[7];

            $database_structure[$line[0]][$line[2]] = $column_info;
        }

        fclose($handle);

        return $database_structure;
    }

    /**
     * Writes content to a csv file
     *
     * @param string $csv_file_path
     * @param mixed $data_rows
     */
    protected function write_csv_file($csv_file_path, $data_rows)
    {
        $handle = fopen($csv_file_path, 'w+');

        foreach($data_rows as $row)
        {
            if(!is_array($row))
            {
                $row = array($row);
            }

            fputcsv($handle, $row);
        }

        fclose($handle);
    }
}