<?php

use application\weblcms\CourseTool;
use common\libraries\DataClassCache;
use common\libraries\FileLogger;
use application\weblcms\integration\tracking\CourseVisit;
use user\integration\tracking\Visit;

require_once __DIR__ . '/../../../common/global.inc.php';

$upgrader = new VisitTrackerUpgrade();
$upgrader->run();

/**
 * Upgrades the visit tracker table into the course visit tracker table. This script has been separated from the
 * normal upgrade procedure because of the amount of data and time it takes to finish this upgrade.
 *
 * @author Sven Vanpoucke - Hogeschool Gent
 */
class VisitTrackerUpgrade
{
    /**
     * The logger object
     *
     * @var FileLogger
     */
    private $logger;

    /**
     * The tool id's by tool name
     *
     * @var int[string]
     */
    private $tool_ids_by_name;

    /**
     * A chamilo DM to execute the queries. We use this DM because the credentials are stored in Chamilo and
     * we can reuse them
     *
     * @var mixed
     */
    private $dm;

    /**
     * Initializes the upgrader and executes the upgrade
     */
    public function run()
    {
        $this->logger = new FileLogger(__DIR__ . '/upgrader.log');
        $this->dm = \user\integration\tracking\DataManager :: get_instance();
        $this->intialize_tool_ids_by_name();

        try
        {
            $this->upgrade_course_visit_tracker();
        }
        catch(\Exception $ex)
        {
            var_dump($ex->getMessage());
        }
    }

    /**
     * Initializes the tool_ids by name
     */
    protected function intialize_tool_ids_by_name()
    {
        $query = 'SELECT id, name FROM weblcms_course_tool';
        $result = $this->dm->get_connection()->query($query);
        while ($course_tool = $result->fetch(\PDO :: FETCH_ASSOC))
        {
            $this->tool_ids_by_name[$course_tool[CourseTool::PROPERTY_NAME]] = $course_tool[CourseTool::PROPERTY_ID];
        }
    }

    /**
     * Upgrades the course visit tracker
     *
     * @return bool
     */
    protected function upgrade_course_visit_tracker()
    {
        $pattern = '%application=weblcms%go=course_viewer%';
        $offset = 0;
        $count = 100000;

        $base_query = 'SELECT * FROM tracking_user_visit WHERE location LIKE "' . $pattern . '" LIMIT ';

        do
        {
            $query = $base_query . $offset . ', ' . $count;

            $row_counter = 0;

            $result = $this->dm->get_connection()->query($query);
            while ($visit_tracker_row = $result->fetch(\PDO :: FETCH_ASSOC))
            {
                $this->handle_visit_tracker($visit_tracker_row);
                $row_counter++;
            }

            $offset += $count;
            $this->logger->log_message('Upgraded ' . ($offset + $row_counter) . ' records');

        }while($row_counter == $count);
    }

    /**
     * Handles a single visit tracker
     *
     * @param Visit $visit_tracker
     *
     * @return bool
     */
    protected function handle_visit_tracker($visit_tracker)
    {
        $location = $visit_tracker[Visit::PROPERTY_LOCATION];
        $user_id = $visit_tracker[Visit::PROPERTY_USER_ID];

        $query = array();

        $url_parts = parse_url($location);
        parse_str($url_parts['query'], $query);

        $course_id = $query['course'];
        $tool_name = $query['tool'];
        $publication_id = $query['publication'];
        $category_id = $query['category'];

        /**
         * When multiple publications are used, it's mostly due to table actions and the publication is not really visited
         */
        if(is_array($publication_id) || is_array($category_id))
        {
            return false;
        }

        if(!$tool_name)
        {
            $tool_name = 'home';
        }

        $course_tool_id = $this->tool_ids_by_name[$tool_name];

        if(!$course_tool_id)
        {
            return false;
        }

        $course_visit = \application\weblcms\integration\tracking\DataManager ::
            retrieve_course_visit_by_user_and_course_data(
                $user_id, $course_id, $course_tool_id, $category_id, $publication_id
            );

        DataClassCache :: truncate(CourseVisit::class_name());

        if(!$course_visit)
        {
            $course_visit = new CourseVisit();

            $course_visit->set_user_id($user_id);
            $course_visit->set_course_id($course_id);
            $course_visit->set_tool_id($course_tool_id);
            $course_visit->set_category_id($category_id);
            $course_visit->set_publication_id($publication_id);
            $course_visit->set_first_access_date($visit_tracker[Visit::PROPERTY_ENTER_DATE]);
        }

        $course_visit->set_last_access_date($visit_tracker[Visit::PROPERTY_ENTER_DATE]);

        $course_visit->set_total_time(
            $course_visit->get_total_time() +
                ($visit_tracker[Visit::PROPERTY_LEAVE_DATE] - $visit_tracker[Visit::PROPERTY_ENTER_DATE])
        );

        $course_visit->set_total_number_of_access($course_visit->get_total_number_of_access() + 1);


        return $course_visit->save();
    }
}